// This code conforms with the UFC specification version 1.0
// and was automatically generated by FFC version 0.4.4.

#ifndef __VECTORLAPLACEGRADCURL_H
#define __VECTORLAPLACEGRADCURL_H

#include <cmath>
#include <stdexcept>
#include <ufc.h>

/// This class defines the interface for a finite element.

class VectorLaplaceGradCurlBilinearForm_finite_element_0_0: public ufc::finite_element
{
public:

  /// Constructor
  VectorLaplaceGradCurlBilinearForm_finite_element_0_0() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~VectorLaplaceGradCurlBilinearForm_finite_element_0_0()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "Lagrange finite element of degree 1 on a tetrahedron";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::tetrahedron;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 4;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 0;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 1;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-09)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-09)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Reset values
    *values = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
    const double scalings_z_0 = 1;
    const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    const double psitilde_a_1 = x;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    const double psitilde_bs_0_1 = 1.5*y + 0.5;
    const double psitilde_bs_1_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    const double psitilde_cs_00_1 = 2*z + 1;
    const double psitilde_cs_01_0 = 1;
    const double psitilde_cs_10_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.8660254038*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    const double basisvalue1 = 2.738612788*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
    const double basisvalue2 = 1.58113883*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
    const double basisvalue3 = 1.118033989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    
    // Table(s) of coefficients
    const static double coefficients0[4][4] = \
    {{0.2886751346, -0.1825741858, -0.1054092553, -0.07453559925},
    {0.2886751346, 0.1825741858, -0.1054092553, -0.07453559925},
    {0.2886751346, 0, 0.2108185107, -0.07453559925},
    {0.2886751346, 0, 0, 0.2236067977}};
    
    // Extract relevant coefficients
    const double coeff0_0 = coefficients0[dof][0];
    const double coeff0_1 = coefficients0[dof][1];
    const double coeff0_2 = coefficients0[dof][2];
    const double coeff0_3 = coefficients0[dof][3];
    
    // Compute value(s)
    *values = coeff0_0*basisvalue0 + coeff0_1*basisvalue1 + coeff0_2*basisvalue2 + coeff0_3*basisvalue3;
  }

  /// Evaluate all basis functions at given point in cell
  virtual void evaluate_basis_all(double* values,
                                  const double* coordinates,
                                  const ufc::cell& c) const
  {
    throw std::runtime_error("The vectorised version of evaluate_basis() is not yet implemented.");
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-09)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-09)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Compute number of derivatives
    unsigned int num_derivatives = 1;
    
    for (unsigned int j = 0; j < n; j++)
      num_derivatives *= 3;
    
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      combinations[j] = new unsigned int [n];
      for (unsigned int k = 0; k < n; k++)
        combinations[j][k] = 0;
    }
        
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 2)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian
    const double Jinv[3][3] ={{d00 / detJ, d10 / detJ, d20 / detJ}, {d01 / detJ, d11 / detJ, d21 / detJ}, {d02 / detJ, d12 / detJ, d22 / detJ}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values
    for (unsigned int j = 0; j < 1*num_derivatives; j++)
      values[j] = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
    const double scalings_z_0 = 1;
    const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    const double psitilde_a_1 = x;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    const double psitilde_bs_0_1 = 1.5*y + 0.5;
    const double psitilde_bs_1_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    const double psitilde_cs_00_1 = 2*z + 1;
    const double psitilde_cs_01_0 = 1;
    const double psitilde_cs_10_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.8660254038*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    const double basisvalue1 = 2.738612788*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
    const double basisvalue2 = 1.58113883*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
    const double basisvalue3 = 1.118033989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    
    // Table(s) of coefficients
    const static double coefficients0[4][4] = \
    {{0.2886751346, -0.1825741858, -0.1054092553, -0.07453559925},
    {0.2886751346, 0.1825741858, -0.1054092553, -0.07453559925},
    {0.2886751346, 0, 0.2108185107, -0.07453559925},
    {0.2886751346, 0, 0, 0.2236067977}};
    
    // Interesting (new) part
    // Tables of derivatives of the polynomial base (transpose)
    const static double dmats0[4][4] = \
    {{0, 0, 0, 0},
    {6.32455532, 0, 0, 0},
    {0, 0, 0, 0},
    {0, 0, 0, 0}};
    
    const static double dmats1[4][4] = \
    {{0, 0, 0, 0},
    {3.16227766, 0, 0, 0},
    {5.477225575, 0, 0, 0},
    {0, 0, 0, 0}};
    
    const static double dmats2[4][4] = \
    {{0, 0, 0, 0},
    {3.16227766, 0, 0, 0},
    {1.825741858, 0, 0, 0},
    {5.163977795, 0, 0, 0}};
    
    // Compute reference derivatives
    // Declare pointer to array of derivatives on FIAT element
    double *derivatives = new double [num_derivatives];
    
    // Declare coefficients
    double coeff0_0 = 0;
    double coeff0_1 = 0;
    double coeff0_2 = 0;
    double coeff0_3 = 0;
    
    // Declare new coefficients
    double new_coeff0_0 = 0;
    double new_coeff0_1 = 0;
    double new_coeff0_2 = 0;
    double new_coeff0_3 = 0;
    
    // Loop possible derivatives
    for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
    {
      // Get values from coefficients array
      new_coeff0_0 = coefficients0[dof][0];
      new_coeff0_1 = coefficients0[dof][1];
      new_coeff0_2 = coefficients0[dof][2];
      new_coeff0_3 = coefficients0[dof][3];
    
      // Loop derivative order
      for (unsigned int j = 0; j < n; j++)
      {
        // Update old coefficients
        coeff0_0 = new_coeff0_0;
        coeff0_1 = new_coeff0_1;
        coeff0_2 = new_coeff0_2;
        coeff0_3 = new_coeff0_3;
    
        if(combinations[deriv_num][j] == 0)
        {
          new_coeff0_0 = coeff0_0*dmats0[0][0] + coeff0_1*dmats0[1][0] + coeff0_2*dmats0[2][0] + coeff0_3*dmats0[3][0];
          new_coeff0_1 = coeff0_0*dmats0[0][1] + coeff0_1*dmats0[1][1] + coeff0_2*dmats0[2][1] + coeff0_3*dmats0[3][1];
          new_coeff0_2 = coeff0_0*dmats0[0][2] + coeff0_1*dmats0[1][2] + coeff0_2*dmats0[2][2] + coeff0_3*dmats0[3][2];
          new_coeff0_3 = coeff0_0*dmats0[0][3] + coeff0_1*dmats0[1][3] + coeff0_2*dmats0[2][3] + coeff0_3*dmats0[3][3];
        }
        if(combinations[deriv_num][j] == 1)
        {
          new_coeff0_0 = coeff0_0*dmats1[0][0] + coeff0_1*dmats1[1][0] + coeff0_2*dmats1[2][0] + coeff0_3*dmats1[3][0];
          new_coeff0_1 = coeff0_0*dmats1[0][1] + coeff0_1*dmats1[1][1] + coeff0_2*dmats1[2][1] + coeff0_3*dmats1[3][1];
          new_coeff0_2 = coeff0_0*dmats1[0][2] + coeff0_1*dmats1[1][2] + coeff0_2*dmats1[2][2] + coeff0_3*dmats1[3][2];
          new_coeff0_3 = coeff0_0*dmats1[0][3] + coeff0_1*dmats1[1][3] + coeff0_2*dmats1[2][3] + coeff0_3*dmats1[3][3];
        }
        if(combinations[deriv_num][j] == 2)
        {
          new_coeff0_0 = coeff0_0*dmats2[0][0] + coeff0_1*dmats2[1][0] + coeff0_2*dmats2[2][0] + coeff0_3*dmats2[3][0];
          new_coeff0_1 = coeff0_0*dmats2[0][1] + coeff0_1*dmats2[1][1] + coeff0_2*dmats2[2][1] + coeff0_3*dmats2[3][1];
          new_coeff0_2 = coeff0_0*dmats2[0][2] + coeff0_1*dmats2[1][2] + coeff0_2*dmats2[2][2] + coeff0_3*dmats2[3][2];
          new_coeff0_3 = coeff0_0*dmats2[0][3] + coeff0_1*dmats2[1][3] + coeff0_2*dmats2[2][3] + coeff0_3*dmats2[3][3];
        }
    
      }
      // Compute derivatives on reference element as dot product of coefficients and basisvalues
      derivatives[deriv_num] = new_coeff0_0*basisvalue0 + new_coeff0_1*basisvalue1 + new_coeff0_2*basisvalue2 + new_coeff0_3*basisvalue3;
    }
    
    // Transform derivatives back to physical element
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        values[row] += transform[row][col]*derivatives[col];
      }
    }
    // Delete pointer to array of derivatives on FIAT element
    delete [] derivatives;
    
    // Delete pointer to array of combinations of derivatives and transform
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      delete [] combinations[row];
      delete [] transform[row];
    }
    
    delete [] combinations;
    delete [] transform;
  }

  /// Evaluate order n derivatives of all basis functions at given point in cell
  virtual void evaluate_basis_derivatives_all(unsigned int n,
                                              double* values,
                                              const double* coordinates,
                                              const ufc::cell& c) const
  {
    throw std::runtime_error("The vectorised version of evaluate_basis_derivatives() is not yet implemented.");
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    // The reference points, direction and weights:
    const static double X[4][1][3] = {{{0, 0, 0}}, {{1, 0, 0}}, {{0, 1, 0}}, {{0, 0, 1}}};
    const static double W[4][1] = {{1}, {1}, {1}, {1}};
    const static double D[4][1][1] = {{{1}}, {{1}}, {{1}}, {{1}}};
    
    const double * const * x = c.coordinates;
    double result = 0.0;
    // Iterate over the points:
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0][0] - X[i][0][1] - X[i][0][2];
    const double w1 = X[i][0][0];
    const double w2 = X[i][0][1];
    const double w3 = X[i][0][2];
    
    // Compute affine mapping y = F(X)
    double y[3];
    y[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0] + w3*x[3][0];
    y[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1] + w3*x[3][1];
    y[2] = w0*x[0][2] + w1*x[1][2] + w2*x[2][2] + w3*x[3][2];
    
    // Evaluate function at physical points
    double values[1];
    f.evaluate(values, y, c);
    
    // Map function values using appropriate mapping
    // Affine map: Do nothing
    
    // Note that we do not map the weights (yet).
    
    // Take directional components
    for(int k = 0; k < 1; k++)
      result += values[k]*D[i][0][k];
    // Multiply by weights 
    result *= W[i][0];
    
    return result;
  }

  /// Evaluate linear functionals for all dofs on the function f
  virtual void evaluate_dofs(double* values,
                             const ufc::function& f,
                             const ufc::cell& c) const
  {
    throw std::runtime_error("Not implemented (introduced in UFC v1.1).");
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    // Evaluate at vertices and use affine mapping
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[1];
    vertex_values[2] = dof_values[2];
    vertex_values[3] = dof_values[3];
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 1;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    return new VectorLaplaceGradCurlBilinearForm_finite_element_0_0();
  }

};

/// This class defines the interface for a finite element.

class VectorLaplaceGradCurlBilinearForm_finite_element_0_1: public ufc::finite_element
{
public:

  /// Constructor
  VectorLaplaceGradCurlBilinearForm_finite_element_0_1() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~VectorLaplaceGradCurlBilinearForm_finite_element_0_1()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "Nedelec finite element of degree 1 on a tetrahedron";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::tetrahedron;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 6;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 1;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 3;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    const double Jinv_00 = d00 / detJ;
    const double Jinv_01 = d10 / detJ;
    const double Jinv_02 = d20 / detJ;
    const double Jinv_10 = d01 / detJ;
    const double Jinv_11 = d11 / detJ;
    const double Jinv_12 = d21 / detJ;
    const double Jinv_20 = d02 / detJ;
    const double Jinv_21 = d12 / detJ;
    const double Jinv_22 = d22 / detJ;
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-09)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-09)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Reset values
    values[0] = 0;
    values[1] = 0;
    values[2] = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
    const double scalings_z_0 = 1;
    const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    const double psitilde_a_1 = x;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    const double psitilde_bs_0_1 = 1.5*y + 0.5;
    const double psitilde_bs_1_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    const double psitilde_cs_00_1 = 2*z + 1;
    const double psitilde_cs_01_0 = 1;
    const double psitilde_cs_10_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.8660254038*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    const double basisvalue1 = 2.738612788*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
    const double basisvalue2 = 1.58113883*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
    const double basisvalue3 = 1.118033989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    
    // Table(s) of coefficients
    const static double coefficients0[6][4] = \
    {{0, 0, 0, 0},
    {-0.2886751346, 0, 0, -0.2236067977},
    {-0.2886751346, 0, -0.2108185107, 0.07453559925},
    {0.2886751346, 0, 0, 0.2236067977},
    {0.2886751346, 0, 0.2108185107, -0.07453559925},
    {0.5773502692, 0, -0.2108185107, -0.1490711985}};
    
    const static double coefficients1[6][4] = \
    {{-0.2886751346, 0, 0, -0.2236067977},
    {0, 0, 0, 0},
    {0.2886751346, 0.1825741858, -0.1054092553, -0.07453559925},
    {0.2886751346, 0, 0, 0.2236067977},
    {0.5773502692, -0.1825741858, 0.1054092553, -0.1490711985},
    {0.2886751346, 0.1825741858, -0.1054092553, -0.07453559925}};
    
    const static double coefficients2[6][4] = \
    {{0.2886751346, 0, 0.2108185107, -0.07453559925},
    {0.2886751346, 0.1825741858, -0.1054092553, -0.07453559925},
    {0, 0, 0, 0},
    {0.5773502692, -0.1825741858, -0.1054092553, 0.1490711985},
    {0.2886751346, 0, 0.2108185107, -0.07453559925},
    {0.2886751346, 0.1825741858, -0.1054092553, -0.07453559925}};
    
    // Extract relevant coefficients
    const double coeff0_0 = coefficients0[dof][0];
    const double coeff0_1 = coefficients0[dof][1];
    const double coeff0_2 = coefficients0[dof][2];
    const double coeff0_3 = coefficients0[dof][3];
    const double coeff1_0 = coefficients1[dof][0];
    const double coeff1_1 = coefficients1[dof][1];
    const double coeff1_2 = coefficients1[dof][2];
    const double coeff1_3 = coefficients1[dof][3];
    const double coeff2_0 = coefficients2[dof][0];
    const double coeff2_1 = coefficients2[dof][1];
    const double coeff2_2 = coefficients2[dof][2];
    const double coeff2_3 = coefficients2[dof][3];
    
    // Compute value(s)
    const double tmp0_0 = coeff0_0*basisvalue0 + coeff0_1*basisvalue1 + coeff0_2*basisvalue2 + coeff0_3*basisvalue3;
    const double tmp0_1 = coeff1_0*basisvalue0 + coeff1_1*basisvalue1 + coeff1_2*basisvalue2 + coeff1_3*basisvalue3;
    const double tmp0_2 = coeff2_0*basisvalue0 + coeff2_1*basisvalue1 + coeff2_2*basisvalue2 + coeff2_3*basisvalue3;
    // Using covariant Piola transform to map values back to the physical element
    values[0] = (Jinv_00*tmp0_0 + Jinv_10*tmp0_1 + Jinv_20*tmp0_2);
    values[1] = (Jinv_01*tmp0_0 + Jinv_11*tmp0_1 + Jinv_21*tmp0_2);
    values[2] = (Jinv_02*tmp0_0 + Jinv_12*tmp0_1 + Jinv_22*tmp0_2);
  }

  /// Evaluate all basis functions at given point in cell
  virtual void evaluate_basis_all(double* values,
                                  const double* coordinates,
                                  const ufc::cell& c) const
  {
    throw std::runtime_error("The vectorised version of evaluate_basis() is not yet implemented.");
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    const double Jinv_00 = d00 / detJ;
    const double Jinv_01 = d10 / detJ;
    const double Jinv_02 = d20 / detJ;
    const double Jinv_10 = d01 / detJ;
    const double Jinv_11 = d11 / detJ;
    const double Jinv_12 = d21 / detJ;
    const double Jinv_20 = d02 / detJ;
    const double Jinv_21 = d12 / detJ;
    const double Jinv_22 = d22 / detJ;
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-09)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-09)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Compute number of derivatives
    unsigned int num_derivatives = 1;
    
    for (unsigned int j = 0; j < n; j++)
      num_derivatives *= 3;
    
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      combinations[j] = new unsigned int [n];
      for (unsigned int k = 0; k < n; k++)
        combinations[j][k] = 0;
    }
        
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 2)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian
    const double Jinv[3][3] ={{d00 / detJ, d10 / detJ, d20 / detJ}, {d01 / detJ, d11 / detJ, d21 / detJ}, {d02 / detJ, d12 / detJ, d22 / detJ}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values
    for (unsigned int j = 0; j < 3*num_derivatives; j++)
      values[j] = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
    const double scalings_z_0 = 1;
    const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    const double psitilde_a_1 = x;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    const double psitilde_bs_0_1 = 1.5*y + 0.5;
    const double psitilde_bs_1_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    const double psitilde_cs_00_1 = 2*z + 1;
    const double psitilde_cs_01_0 = 1;
    const double psitilde_cs_10_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.8660254038*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    const double basisvalue1 = 2.738612788*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
    const double basisvalue2 = 1.58113883*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
    const double basisvalue3 = 1.118033989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    
    // Table(s) of coefficients
    const static double coefficients0[6][4] = \
    {{0, 0, 0, 0},
    {-0.2886751346, 0, 0, -0.2236067977},
    {-0.2886751346, 0, -0.2108185107, 0.07453559925},
    {0.2886751346, 0, 0, 0.2236067977},
    {0.2886751346, 0, 0.2108185107, -0.07453559925},
    {0.5773502692, 0, -0.2108185107, -0.1490711985}};
    
    const static double coefficients1[6][4] = \
    {{-0.2886751346, 0, 0, -0.2236067977},
    {0, 0, 0, 0},
    {0.2886751346, 0.1825741858, -0.1054092553, -0.07453559925},
    {0.2886751346, 0, 0, 0.2236067977},
    {0.5773502692, -0.1825741858, 0.1054092553, -0.1490711985},
    {0.2886751346, 0.1825741858, -0.1054092553, -0.07453559925}};
    
    const static double coefficients2[6][4] = \
    {{0.2886751346, 0, 0.2108185107, -0.07453559925},
    {0.2886751346, 0.1825741858, -0.1054092553, -0.07453559925},
    {0, 0, 0, 0},
    {0.5773502692, -0.1825741858, -0.1054092553, 0.1490711985},
    {0.2886751346, 0, 0.2108185107, -0.07453559925},
    {0.2886751346, 0.1825741858, -0.1054092553, -0.07453559925}};
    
    // Interesting (new) part
    // Tables of derivatives of the polynomial base (transpose)
    const static double dmats0[4][4] = \
    {{0, 0, 0, 0},
    {6.32455532, 0, 0, 0},
    {0, 0, 0, 0},
    {0, 0, 0, 0}};
    
    const static double dmats1[4][4] = \
    {{0, 0, 0, 0},
    {3.16227766, 0, 0, 0},
    {5.477225575, 0, 0, 0},
    {0, 0, 0, 0}};
    
    const static double dmats2[4][4] = \
    {{0, 0, 0, 0},
    {3.16227766, 0, 0, 0},
    {1.825741858, 0, 0, 0},
    {5.163977795, 0, 0, 0}};
    
    // Compute reference derivatives
    // Declare pointer to array of derivatives on FIAT element
    double *derivatives = new double [3*num_derivatives];
    
    // Declare coefficients
    double coeff0_0 = 0;
    double coeff0_1 = 0;
    double coeff0_2 = 0;
    double coeff0_3 = 0;
    double coeff1_0 = 0;
    double coeff1_1 = 0;
    double coeff1_2 = 0;
    double coeff1_3 = 0;
    double coeff2_0 = 0;
    double coeff2_1 = 0;
    double coeff2_2 = 0;
    double coeff2_3 = 0;
    
    // Declare new coefficients
    double new_coeff0_0 = 0;
    double new_coeff0_1 = 0;
    double new_coeff0_2 = 0;
    double new_coeff0_3 = 0;
    double new_coeff1_0 = 0;
    double new_coeff1_1 = 0;
    double new_coeff1_2 = 0;
    double new_coeff1_3 = 0;
    double new_coeff2_0 = 0;
    double new_coeff2_1 = 0;
    double new_coeff2_2 = 0;
    double new_coeff2_3 = 0;
    
    // Loop possible derivatives
    for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
    {
      // Get values from coefficients array
      new_coeff0_0 = coefficients0[dof][0];
      new_coeff0_1 = coefficients0[dof][1];
      new_coeff0_2 = coefficients0[dof][2];
      new_coeff0_3 = coefficients0[dof][3];
      new_coeff1_0 = coefficients1[dof][0];
      new_coeff1_1 = coefficients1[dof][1];
      new_coeff1_2 = coefficients1[dof][2];
      new_coeff1_3 = coefficients1[dof][3];
      new_coeff2_0 = coefficients2[dof][0];
      new_coeff2_1 = coefficients2[dof][1];
      new_coeff2_2 = coefficients2[dof][2];
      new_coeff2_3 = coefficients2[dof][3];
    
      // Loop derivative order
      for (unsigned int j = 0; j < n; j++)
      {
        // Update old coefficients
        coeff0_0 = new_coeff0_0;
        coeff0_1 = new_coeff0_1;
        coeff0_2 = new_coeff0_2;
        coeff0_3 = new_coeff0_3;
        coeff1_0 = new_coeff1_0;
        coeff1_1 = new_coeff1_1;
        coeff1_2 = new_coeff1_2;
        coeff1_3 = new_coeff1_3;
        coeff2_0 = new_coeff2_0;
        coeff2_1 = new_coeff2_1;
        coeff2_2 = new_coeff2_2;
        coeff2_3 = new_coeff2_3;
    
        if(combinations[deriv_num][j] == 0)
        {
          new_coeff0_0 = coeff0_0*dmats0[0][0] + coeff0_1*dmats0[1][0] + coeff0_2*dmats0[2][0] + coeff0_3*dmats0[3][0];
          new_coeff0_1 = coeff0_0*dmats0[0][1] + coeff0_1*dmats0[1][1] + coeff0_2*dmats0[2][1] + coeff0_3*dmats0[3][1];
          new_coeff0_2 = coeff0_0*dmats0[0][2] + coeff0_1*dmats0[1][2] + coeff0_2*dmats0[2][2] + coeff0_3*dmats0[3][2];
          new_coeff0_3 = coeff0_0*dmats0[0][3] + coeff0_1*dmats0[1][3] + coeff0_2*dmats0[2][3] + coeff0_3*dmats0[3][3];
          new_coeff1_0 = coeff1_0*dmats0[0][0] + coeff1_1*dmats0[1][0] + coeff1_2*dmats0[2][0] + coeff1_3*dmats0[3][0];
          new_coeff1_1 = coeff1_0*dmats0[0][1] + coeff1_1*dmats0[1][1] + coeff1_2*dmats0[2][1] + coeff1_3*dmats0[3][1];
          new_coeff1_2 = coeff1_0*dmats0[0][2] + coeff1_1*dmats0[1][2] + coeff1_2*dmats0[2][2] + coeff1_3*dmats0[3][2];
          new_coeff1_3 = coeff1_0*dmats0[0][3] + coeff1_1*dmats0[1][3] + coeff1_2*dmats0[2][3] + coeff1_3*dmats0[3][3];
          new_coeff2_0 = coeff2_0*dmats0[0][0] + coeff2_1*dmats0[1][0] + coeff2_2*dmats0[2][0] + coeff2_3*dmats0[3][0];
          new_coeff2_1 = coeff2_0*dmats0[0][1] + coeff2_1*dmats0[1][1] + coeff2_2*dmats0[2][1] + coeff2_3*dmats0[3][1];
          new_coeff2_2 = coeff2_0*dmats0[0][2] + coeff2_1*dmats0[1][2] + coeff2_2*dmats0[2][2] + coeff2_3*dmats0[3][2];
          new_coeff2_3 = coeff2_0*dmats0[0][3] + coeff2_1*dmats0[1][3] + coeff2_2*dmats0[2][3] + coeff2_3*dmats0[3][3];
        }
        if(combinations[deriv_num][j] == 1)
        {
          new_coeff0_0 = coeff0_0*dmats1[0][0] + coeff0_1*dmats1[1][0] + coeff0_2*dmats1[2][0] + coeff0_3*dmats1[3][0];
          new_coeff0_1 = coeff0_0*dmats1[0][1] + coeff0_1*dmats1[1][1] + coeff0_2*dmats1[2][1] + coeff0_3*dmats1[3][1];
          new_coeff0_2 = coeff0_0*dmats1[0][2] + coeff0_1*dmats1[1][2] + coeff0_2*dmats1[2][2] + coeff0_3*dmats1[3][2];
          new_coeff0_3 = coeff0_0*dmats1[0][3] + coeff0_1*dmats1[1][3] + coeff0_2*dmats1[2][3] + coeff0_3*dmats1[3][3];
          new_coeff1_0 = coeff1_0*dmats1[0][0] + coeff1_1*dmats1[1][0] + coeff1_2*dmats1[2][0] + coeff1_3*dmats1[3][0];
          new_coeff1_1 = coeff1_0*dmats1[0][1] + coeff1_1*dmats1[1][1] + coeff1_2*dmats1[2][1] + coeff1_3*dmats1[3][1];
          new_coeff1_2 = coeff1_0*dmats1[0][2] + coeff1_1*dmats1[1][2] + coeff1_2*dmats1[2][2] + coeff1_3*dmats1[3][2];
          new_coeff1_3 = coeff1_0*dmats1[0][3] + coeff1_1*dmats1[1][3] + coeff1_2*dmats1[2][3] + coeff1_3*dmats1[3][3];
          new_coeff2_0 = coeff2_0*dmats1[0][0] + coeff2_1*dmats1[1][0] + coeff2_2*dmats1[2][0] + coeff2_3*dmats1[3][0];
          new_coeff2_1 = coeff2_0*dmats1[0][1] + coeff2_1*dmats1[1][1] + coeff2_2*dmats1[2][1] + coeff2_3*dmats1[3][1];
          new_coeff2_2 = coeff2_0*dmats1[0][2] + coeff2_1*dmats1[1][2] + coeff2_2*dmats1[2][2] + coeff2_3*dmats1[3][2];
          new_coeff2_3 = coeff2_0*dmats1[0][3] + coeff2_1*dmats1[1][3] + coeff2_2*dmats1[2][3] + coeff2_3*dmats1[3][3];
        }
        if(combinations[deriv_num][j] == 2)
        {
          new_coeff0_0 = coeff0_0*dmats2[0][0] + coeff0_1*dmats2[1][0] + coeff0_2*dmats2[2][0] + coeff0_3*dmats2[3][0];
          new_coeff0_1 = coeff0_0*dmats2[0][1] + coeff0_1*dmats2[1][1] + coeff0_2*dmats2[2][1] + coeff0_3*dmats2[3][1];
          new_coeff0_2 = coeff0_0*dmats2[0][2] + coeff0_1*dmats2[1][2] + coeff0_2*dmats2[2][2] + coeff0_3*dmats2[3][2];
          new_coeff0_3 = coeff0_0*dmats2[0][3] + coeff0_1*dmats2[1][3] + coeff0_2*dmats2[2][3] + coeff0_3*dmats2[3][3];
          new_coeff1_0 = coeff1_0*dmats2[0][0] + coeff1_1*dmats2[1][0] + coeff1_2*dmats2[2][0] + coeff1_3*dmats2[3][0];
          new_coeff1_1 = coeff1_0*dmats2[0][1] + coeff1_1*dmats2[1][1] + coeff1_2*dmats2[2][1] + coeff1_3*dmats2[3][1];
          new_coeff1_2 = coeff1_0*dmats2[0][2] + coeff1_1*dmats2[1][2] + coeff1_2*dmats2[2][2] + coeff1_3*dmats2[3][2];
          new_coeff1_3 = coeff1_0*dmats2[0][3] + coeff1_1*dmats2[1][3] + coeff1_2*dmats2[2][3] + coeff1_3*dmats2[3][3];
          new_coeff2_0 = coeff2_0*dmats2[0][0] + coeff2_1*dmats2[1][0] + coeff2_2*dmats2[2][0] + coeff2_3*dmats2[3][0];
          new_coeff2_1 = coeff2_0*dmats2[0][1] + coeff2_1*dmats2[1][1] + coeff2_2*dmats2[2][1] + coeff2_3*dmats2[3][1];
          new_coeff2_2 = coeff2_0*dmats2[0][2] + coeff2_1*dmats2[1][2] + coeff2_2*dmats2[2][2] + coeff2_3*dmats2[3][2];
          new_coeff2_3 = coeff2_0*dmats2[0][3] + coeff2_1*dmats2[1][3] + coeff2_2*dmats2[2][3] + coeff2_3*dmats2[3][3];
        }
    
      }
      // Compute derivatives on reference element as dot product of coefficients and basisvalues
      // Correct values by the covariant Piola transform
      const double tmp0_0 = new_coeff0_0*basisvalue0 + new_coeff0_1*basisvalue1 + new_coeff0_2*basisvalue2 + new_coeff0_3*basisvalue3;
      const double tmp0_1 = new_coeff1_0*basisvalue0 + new_coeff1_1*basisvalue1 + new_coeff1_2*basisvalue2 + new_coeff1_3*basisvalue3;
      const double tmp0_2 = new_coeff2_0*basisvalue0 + new_coeff2_1*basisvalue1 + new_coeff2_2*basisvalue2 + new_coeff2_3*basisvalue3;
      derivatives[deriv_num] = (Jinv_00*tmp0_0 + Jinv_10*tmp0_1 + Jinv_20*tmp0_2);
      derivatives[num_derivatives + deriv_num] = (Jinv_01*tmp0_0 + Jinv_11*tmp0_1 + Jinv_21*tmp0_2);
      derivatives[2*num_derivatives + deriv_num] = (Jinv_02*tmp0_0 + Jinv_12*tmp0_1 + Jinv_22*tmp0_2);
    }
    
    // Transform derivatives back to physical element
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        values[row] += transform[row][col]*derivatives[col];
        values[num_derivatives + row] += transform[row][col]*derivatives[num_derivatives + col];
        values[2*num_derivatives + row] += transform[row][col]*derivatives[2*num_derivatives + col];
      }
    }
    // Delete pointer to array of derivatives on FIAT element
    delete [] derivatives;
    
    // Delete pointer to array of combinations of derivatives and transform
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      delete [] combinations[row];
      delete [] transform[row];
    }
    
    delete [] combinations;
    delete [] transform;
  }

  /// Evaluate order n derivatives of all basis functions at given point in cell
  virtual void evaluate_basis_derivatives_all(unsigned int n,
                                              double* values,
                                              const double* coordinates,
                                              const ufc::cell& c) const
  {
    throw std::runtime_error("The vectorised version of evaluate_basis_derivatives() is not yet implemented.");
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    // The reference points, direction and weights:
    const static double X[6][1][3] = {{{0, 0.5, 0.5}}, {{0.5, 0, 0.5}}, {{0.5, 0.5, 0}}, {{0, 0, 0.5}}, {{0, 0.5, 0}}, {{0.5, 0, 0}}};
    const static double W[6][1] = {{1}, {1}, {1}, {1}, {1}, {1}};
    const static double D[6][1][3] = {{{0, -1, 1}}, {{-1, 0, 1}}, {{-1, 1, 0}}, {{0, 0, 1}}, {{0, 1, 0}}, {{1, 0, 0}}};
    
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = x[1][0] - x[0][0];
    const double J_01 = x[2][0] - x[0][0];
    const double J_02 = x[3][0] - x[0][0];
    const double J_10 = x[1][1] - x[0][1];
    const double J_11 = x[2][1] - x[0][1];
    const double J_12 = x[3][1] - x[0][1];
    const double J_20 = x[1][2] - x[0][2];
    const double J_21 = x[2][2] - x[0][2];
    const double J_22 = x[3][2] - x[0][2];
      
    // Compute sub determinants
    const double d_00 = J_11*J_22 - J_12*J_21;
    const double d_01 = J_12*J_20 - J_10*J_22;
    const double d_02 = J_10*J_21 - J_11*J_20;
    
    const double d_10 = J_02*J_21 - J_01*J_22;
    const double d_11 = J_00*J_22 - J_02*J_20;
    const double d_12 = J_01*J_20 - J_00*J_21;
    
    const double d_20 = J_01*J_12 - J_02*J_11;
    const double d_21 = J_02*J_10 - J_00*J_12;
    const double d_22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d_00 + J_10*d_10 + J_20*d_20;
      
    // Compute inverse of Jacobian
    const double Jinv_00 = d_00 / detJ;
    const double Jinv_01 = d_10 / detJ;
    const double Jinv_02 = d_20 / detJ;
    const double Jinv_10 = d_01 / detJ;
    const double Jinv_11 = d_11 / detJ;
    const double Jinv_12 = d_21 / detJ;
    const double Jinv_20 = d_02 / detJ;
    const double Jinv_21 = d_12 / detJ;
    const double Jinv_22 = d_22 / detJ;
    
    double copyofvalues[3];
    double result = 0.0;
    // Iterate over the points:
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0][0] - X[i][0][1] - X[i][0][2];
    const double w1 = X[i][0][0];
    const double w2 = X[i][0][1];
    const double w3 = X[i][0][2];
    
    // Compute affine mapping y = F(X)
    double y[3];
    y[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0] + w3*x[3][0];
    y[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1] + w3*x[3][1];
    y[2] = w0*x[0][2] + w1*x[1][2] + w2*x[2][2] + w3*x[3][2];
    
    // Evaluate function at physical points
    double values[3];
    f.evaluate(values, y, c);
    
    // Map function values using appropriate mapping
    // Copy old values:
    copyofvalues[0] = values[0];
    copyofvalues[1] = values[1];
    copyofvalues[2] = values[2];
    // Do the inverse of curl piola 
    values[0] = J_00*copyofvalues[0]+J_10*copyofvalues[1]+J_20*copyofvalues[2];
    values[1] = J_01*copyofvalues[0]+J_11*copyofvalues[1]+J_21*copyofvalues[2];
    values[2] = J_02*copyofvalues[0]+J_12*copyofvalues[1]+J_22*copyofvalues[2];
    
    // Note that we do not map the weights (yet).
    
    // Take directional components
    for(int k = 0; k < 3; k++)
      result += values[k]*D[i][0][k];
    // Multiply by weights 
    result *= W[i][0];
    
    return result;
  }

  /// Evaluate linear functionals for all dofs on the function f
  virtual void evaluate_dofs(double* values,
                             const ufc::function& f,
                             const ufc::cell& c) const
  {
    throw std::runtime_error("Not implemented (introduced in UFC v1.1).");
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = x[1][0] - x[0][0];
    const double J_01 = x[2][0] - x[0][0];
    const double J_02 = x[3][0] - x[0][0];
    const double J_10 = x[1][1] - x[0][1];
    const double J_11 = x[2][1] - x[0][1];
    const double J_12 = x[3][1] - x[0][1];
    const double J_20 = x[1][2] - x[0][2];
    const double J_21 = x[2][2] - x[0][2];
    const double J_22 = x[3][2] - x[0][2];
      
    // Compute sub determinants
    const double d_00 = J_11*J_22 - J_12*J_21;
    const double d_01 = J_12*J_20 - J_10*J_22;
    const double d_02 = J_10*J_21 - J_11*J_20;
    
    const double d_10 = J_02*J_21 - J_01*J_22;
    const double d_11 = J_00*J_22 - J_02*J_20;
    const double d_12 = J_01*J_20 - J_00*J_21;
    
    const double d_20 = J_01*J_12 - J_02*J_11;
    const double d_21 = J_02*J_10 - J_00*J_12;
    const double d_22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d_00 + J_10*d_10 + J_20*d_20;
      
    // Compute inverse of Jacobian
    const double Jinv_00 = d_00 / detJ;
    const double Jinv_01 = d_10 / detJ;
    const double Jinv_02 = d_20 / detJ;
    const double Jinv_10 = d_01 / detJ;
    const double Jinv_11 = d_11 / detJ;
    const double Jinv_12 = d_21 / detJ;
    const double Jinv_20 = d_02 / detJ;
    const double Jinv_21 = d_12 / detJ;
    const double Jinv_22 = d_22 / detJ;
    // Evaluate at vertices and use Piola mapping
    vertex_values[0] = (dof_values[3]*Jinv_20 + dof_values[4]*Jinv_10 + dof_values[5]*Jinv_00);
    vertex_values[1] = (dof_values[1]*Jinv_20 + dof_values[2]*Jinv_10 + dof_values[5]*(Jinv_00 + Jinv_10 + Jinv_20));
    vertex_values[2] = (dof_values[0]*Jinv_20 + dof_values[2]*Jinv_00 + dof_values[4]*(Jinv_00 + Jinv_10 + Jinv_20));
    vertex_values[3] = (dof_values[0]*Jinv_10 + dof_values[1]*Jinv_00 + dof_values[3]*(Jinv_00 + Jinv_10 + Jinv_20));
    vertex_values[4] = (dof_values[3]*Jinv_21 + dof_values[4]*Jinv_11 + dof_values[5]*Jinv_01);
    vertex_values[5] = (dof_values[1]*Jinv_21 + dof_values[2]*Jinv_11 + dof_values[5]*(Jinv_01 + Jinv_11 + Jinv_21));
    vertex_values[6] = (dof_values[0]*Jinv_21 + dof_values[2]*Jinv_01 + dof_values[4]*(Jinv_01 + Jinv_11 + Jinv_21));
    vertex_values[7] = (dof_values[0]*Jinv_11 + dof_values[1]*Jinv_01 + dof_values[3]*(Jinv_01 + Jinv_11 + Jinv_21));
    vertex_values[8] = (dof_values[3]*Jinv_22 + dof_values[4]*Jinv_12 + dof_values[5]*Jinv_02);
    vertex_values[9] = (dof_values[1]*Jinv_22 + dof_values[2]*Jinv_12 + dof_values[5]*(Jinv_02 + Jinv_12 + Jinv_22));
    vertex_values[10] = (dof_values[0]*Jinv_22 + dof_values[2]*Jinv_02 + dof_values[4]*(Jinv_02 + Jinv_12 + Jinv_22));
    vertex_values[11] = (dof_values[0]*Jinv_12 + dof_values[1]*Jinv_02 + dof_values[3]*(Jinv_02 + Jinv_12 + Jinv_22));
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 1;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    return new VectorLaplaceGradCurlBilinearForm_finite_element_0_1();
  }

};

/// This class defines the interface for a finite element.

class VectorLaplaceGradCurlBilinearForm_finite_element_0: public ufc::finite_element
{
public:

  /// Constructor
  VectorLaplaceGradCurlBilinearForm_finite_element_0() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~VectorLaplaceGradCurlBilinearForm_finite_element_0()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "Mixed finite element: [Lagrange finite element of degree 1 on a tetrahedron, Nedelec finite element of degree 1 on a tetrahedron]";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::tetrahedron;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 10;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 1;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 4;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    const double Jinv_00 = d00 / detJ;
    const double Jinv_01 = d10 / detJ;
    const double Jinv_02 = d20 / detJ;
    const double Jinv_10 = d01 / detJ;
    const double Jinv_11 = d11 / detJ;
    const double Jinv_12 = d21 / detJ;
    const double Jinv_20 = d02 / detJ;
    const double Jinv_21 = d12 / detJ;
    const double Jinv_22 = d22 / detJ;
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-09)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-09)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Reset values
    values[0] = 0;
    values[1] = 0;
    values[2] = 0;
    values[3] = 0;
    
    if (0 <= i && i <= 3)
    {
      // Map degree of freedom to element degree of freedom
      const unsigned int dof = i;
    
      // Generate scalings
      const double scalings_y_0 = 1;
      const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
      const double scalings_z_0 = 1;
      const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    
      // Compute psitilde_a
      const double psitilde_a_0 = 1;
      const double psitilde_a_1 = x;
    
      // Compute psitilde_bs
      const double psitilde_bs_0_0 = 1;
      const double psitilde_bs_0_1 = 1.5*y + 0.5;
      const double psitilde_bs_1_0 = 1;
    
      // Compute psitilde_cs
      const double psitilde_cs_00_0 = 1;
      const double psitilde_cs_00_1 = 2*z + 1;
      const double psitilde_cs_01_0 = 1;
      const double psitilde_cs_10_0 = 1;
    
      // Compute basisvalues
      const double basisvalue0 = 0.8660254038*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
      const double basisvalue1 = 2.738612788*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
      const double basisvalue2 = 1.58113883*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
      const double basisvalue3 = 1.118033989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    
      // Table(s) of coefficients
      const static double coefficients0[4][4] =   \
      {{0.2886751346, -0.1825741858, -0.1054092553, -0.07453559925},
      {0.2886751346, 0.1825741858, -0.1054092553, -0.07453559925},
      {0.2886751346, 0, 0.2108185107, -0.07453559925},
      {0.2886751346, 0, 0, 0.2236067977}};
    
      // Extract relevant coefficients
      const double coeff0_0 =   coefficients0[dof][0];
      const double coeff0_1 =   coefficients0[dof][1];
      const double coeff0_2 =   coefficients0[dof][2];
      const double coeff0_3 =   coefficients0[dof][3];
    
      // Compute value(s)
      values[0] = coeff0_0*basisvalue0 + coeff0_1*basisvalue1 + coeff0_2*basisvalue2 + coeff0_3*basisvalue3;
    }
    
    if (4 <= i && i <= 9)
    {
      // Map degree of freedom to element degree of freedom
      const unsigned int dof = i - 4;
    
      // Generate scalings
      const double scalings_y_0 = 1;
      const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
      const double scalings_z_0 = 1;
      const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    
      // Compute psitilde_a
      const double psitilde_a_0 = 1;
      const double psitilde_a_1 = x;
    
      // Compute psitilde_bs
      const double psitilde_bs_0_0 = 1;
      const double psitilde_bs_0_1 = 1.5*y + 0.5;
      const double psitilde_bs_1_0 = 1;
    
      // Compute psitilde_cs
      const double psitilde_cs_00_0 = 1;
      const double psitilde_cs_00_1 = 2*z + 1;
      const double psitilde_cs_01_0 = 1;
      const double psitilde_cs_10_0 = 1;
    
      // Compute basisvalues
      const double basisvalue0 = 0.8660254038*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
      const double basisvalue1 = 2.738612788*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
      const double basisvalue2 = 1.58113883*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
      const double basisvalue3 = 1.118033989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    
      // Table(s) of coefficients
      const static double coefficients0[6][4] =   \
      {{0, 0, 0, 0},
      {-0.2886751346, 0, 0, -0.2236067977},
      {-0.2886751346, 0, -0.2108185107, 0.07453559925},
      {0.2886751346, 0, 0, 0.2236067977},
      {0.2886751346, 0, 0.2108185107, -0.07453559925},
      {0.5773502692, 0, -0.2108185107, -0.1490711985}};
    
      const static double coefficients1[6][4] =   \
      {{-0.2886751346, 0, 0, -0.2236067977},
      {0, 0, 0, 0},
      {0.2886751346, 0.1825741858, -0.1054092553, -0.07453559925},
      {0.2886751346, 0, 0, 0.2236067977},
      {0.5773502692, -0.1825741858, 0.1054092553, -0.1490711985},
      {0.2886751346, 0.1825741858, -0.1054092553, -0.07453559925}};
    
      const static double coefficients2[6][4] =   \
      {{0.2886751346, 0, 0.2108185107, -0.07453559925},
      {0.2886751346, 0.1825741858, -0.1054092553, -0.07453559925},
      {0, 0, 0, 0},
      {0.5773502692, -0.1825741858, -0.1054092553, 0.1490711985},
      {0.2886751346, 0, 0.2108185107, -0.07453559925},
      {0.2886751346, 0.1825741858, -0.1054092553, -0.07453559925}};
    
      // Extract relevant coefficients
      const double coeff0_0 =   coefficients0[dof][0];
      const double coeff0_1 =   coefficients0[dof][1];
      const double coeff0_2 =   coefficients0[dof][2];
      const double coeff0_3 =   coefficients0[dof][3];
      const double coeff1_0 =   coefficients1[dof][0];
      const double coeff1_1 =   coefficients1[dof][1];
      const double coeff1_2 =   coefficients1[dof][2];
      const double coeff1_3 =   coefficients1[dof][3];
      const double coeff2_0 =   coefficients2[dof][0];
      const double coeff2_1 =   coefficients2[dof][1];
      const double coeff2_2 =   coefficients2[dof][2];
      const double coeff2_3 =   coefficients2[dof][3];
    
      // Compute value(s)
      const double tmp0_0 = coeff0_0*basisvalue0 + coeff0_1*basisvalue1 + coeff0_2*basisvalue2 + coeff0_3*basisvalue3;
      const double tmp0_1 = coeff1_0*basisvalue0 + coeff1_1*basisvalue1 + coeff1_2*basisvalue2 + coeff1_3*basisvalue3;
      const double tmp0_2 = coeff2_0*basisvalue0 + coeff2_1*basisvalue1 + coeff2_2*basisvalue2 + coeff2_3*basisvalue3;
      // Using covariant Piola transform to map values back to the physical element
      values[1] = (Jinv_00*tmp0_0 + Jinv_10*tmp0_1 + Jinv_20*tmp0_2);
      values[2] = (Jinv_01*tmp0_0 + Jinv_11*tmp0_1 + Jinv_21*tmp0_2);
      values[3] = (Jinv_02*tmp0_0 + Jinv_12*tmp0_1 + Jinv_22*tmp0_2);
    }
    
  }

  /// Evaluate all basis functions at given point in cell
  virtual void evaluate_basis_all(double* values,
                                  const double* coordinates,
                                  const ufc::cell& c) const
  {
    throw std::runtime_error("The vectorised version of evaluate_basis() is not yet implemented.");
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    const double Jinv_00 = d00 / detJ;
    const double Jinv_01 = d10 / detJ;
    const double Jinv_02 = d20 / detJ;
    const double Jinv_10 = d01 / detJ;
    const double Jinv_11 = d11 / detJ;
    const double Jinv_12 = d21 / detJ;
    const double Jinv_20 = d02 / detJ;
    const double Jinv_21 = d12 / detJ;
    const double Jinv_22 = d22 / detJ;
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-09)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-09)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Compute number of derivatives
    unsigned int num_derivatives = 1;
    
    for (unsigned int j = 0; j < n; j++)
      num_derivatives *= 3;
    
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      combinations[j] = new unsigned int [n];
      for (unsigned int k = 0; k < n; k++)
        combinations[j][k] = 0;
    }
        
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 2)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian
    const double Jinv[3][3] ={{d00 / detJ, d10 / detJ, d20 / detJ}, {d01 / detJ, d11 / detJ, d21 / detJ}, {d02 / detJ, d12 / detJ, d22 / detJ}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values
    for (unsigned int j = 0; j < 4*num_derivatives; j++)
      values[j] = 0;
    
    if (0 <= i && i <= 3)
    {
      // Map degree of freedom to element degree of freedom
      const unsigned int dof = i;
    
      // Generate scalings
      const double scalings_y_0 = 1;
      const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
      const double scalings_z_0 = 1;
      const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    
      // Compute psitilde_a
      const double psitilde_a_0 = 1;
      const double psitilde_a_1 = x;
    
      // Compute psitilde_bs
      const double psitilde_bs_0_0 = 1;
      const double psitilde_bs_0_1 = 1.5*y + 0.5;
      const double psitilde_bs_1_0 = 1;
    
      // Compute psitilde_cs
      const double psitilde_cs_00_0 = 1;
      const double psitilde_cs_00_1 = 2*z + 1;
      const double psitilde_cs_01_0 = 1;
      const double psitilde_cs_10_0 = 1;
    
      // Compute basisvalues
      const double basisvalue0 = 0.8660254038*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
      const double basisvalue1 = 2.738612788*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
      const double basisvalue2 = 1.58113883*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
      const double basisvalue3 = 1.118033989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    
      // Table(s) of coefficients
      const static double coefficients0[4][4] =   \
      {{0.2886751346, -0.1825741858, -0.1054092553, -0.07453559925},
      {0.2886751346, 0.1825741858, -0.1054092553, -0.07453559925},
      {0.2886751346, 0, 0.2108185107, -0.07453559925},
      {0.2886751346, 0, 0, 0.2236067977}};
    
      // Interesting (new) part
      // Tables of derivatives of the polynomial base (transpose)
      const static double dmats0[4][4] =   \
      {{0, 0, 0, 0},
      {6.32455532, 0, 0, 0},
      {0, 0, 0, 0},
      {0, 0, 0, 0}};
    
      const static double dmats1[4][4] =   \
      {{0, 0, 0, 0},
      {3.16227766, 0, 0, 0},
      {5.477225575, 0, 0, 0},
      {0, 0, 0, 0}};
    
      const static double dmats2[4][4] =   \
      {{0, 0, 0, 0},
      {3.16227766, 0, 0, 0},
      {1.825741858, 0, 0, 0},
      {5.163977795, 0, 0, 0}};
    
      // Compute reference derivatives
      // Declare pointer to array of derivatives on FIAT element
      double *derivatives = new double [num_derivatives];
    
      // Declare coefficients
      double coeff0_0 = 0;
      double coeff0_1 = 0;
      double coeff0_2 = 0;
      double coeff0_3 = 0;
    
      // Declare new coefficients
      double new_coeff0_0 = 0;
      double new_coeff0_1 = 0;
      double new_coeff0_2 = 0;
      double new_coeff0_3 = 0;
    
      // Loop possible derivatives
      for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
      {
        // Get values from coefficients array
        new_coeff0_0 = coefficients0[dof][0];
        new_coeff0_1 = coefficients0[dof][1];
        new_coeff0_2 = coefficients0[dof][2];
        new_coeff0_3 = coefficients0[dof][3];
    
        // Loop derivative order
        for (unsigned int j = 0; j < n; j++)
        {
          // Update old coefficients
          coeff0_0 = new_coeff0_0;
          coeff0_1 = new_coeff0_1;
          coeff0_2 = new_coeff0_2;
          coeff0_3 = new_coeff0_3;
    
          if(combinations[deriv_num][j] == 0)
          {
            new_coeff0_0 = coeff0_0*dmats0[0][0] + coeff0_1*dmats0[1][0] + coeff0_2*dmats0[2][0] + coeff0_3*dmats0[3][0];
            new_coeff0_1 = coeff0_0*dmats0[0][1] + coeff0_1*dmats0[1][1] + coeff0_2*dmats0[2][1] + coeff0_3*dmats0[3][1];
            new_coeff0_2 = coeff0_0*dmats0[0][2] + coeff0_1*dmats0[1][2] + coeff0_2*dmats0[2][2] + coeff0_3*dmats0[3][2];
            new_coeff0_3 = coeff0_0*dmats0[0][3] + coeff0_1*dmats0[1][3] + coeff0_2*dmats0[2][3] + coeff0_3*dmats0[3][3];
          }
          if(combinations[deriv_num][j] == 1)
          {
            new_coeff0_0 = coeff0_0*dmats1[0][0] + coeff0_1*dmats1[1][0] + coeff0_2*dmats1[2][0] + coeff0_3*dmats1[3][0];
            new_coeff0_1 = coeff0_0*dmats1[0][1] + coeff0_1*dmats1[1][1] + coeff0_2*dmats1[2][1] + coeff0_3*dmats1[3][1];
            new_coeff0_2 = coeff0_0*dmats1[0][2] + coeff0_1*dmats1[1][2] + coeff0_2*dmats1[2][2] + coeff0_3*dmats1[3][2];
            new_coeff0_3 = coeff0_0*dmats1[0][3] + coeff0_1*dmats1[1][3] + coeff0_2*dmats1[2][3] + coeff0_3*dmats1[3][3];
          }
          if(combinations[deriv_num][j] == 2)
          {
            new_coeff0_0 = coeff0_0*dmats2[0][0] + coeff0_1*dmats2[1][0] + coeff0_2*dmats2[2][0] + coeff0_3*dmats2[3][0];
            new_coeff0_1 = coeff0_0*dmats2[0][1] + coeff0_1*dmats2[1][1] + coeff0_2*dmats2[2][1] + coeff0_3*dmats2[3][1];
            new_coeff0_2 = coeff0_0*dmats2[0][2] + coeff0_1*dmats2[1][2] + coeff0_2*dmats2[2][2] + coeff0_3*dmats2[3][2];
            new_coeff0_3 = coeff0_0*dmats2[0][3] + coeff0_1*dmats2[1][3] + coeff0_2*dmats2[2][3] + coeff0_3*dmats2[3][3];
          }
    
        }
        // Compute derivatives on reference element as dot product of coefficients and basisvalues
        derivatives[deriv_num] = new_coeff0_0*basisvalue0 + new_coeff0_1*basisvalue1 + new_coeff0_2*basisvalue2 + new_coeff0_3*basisvalue3;
      }
    
      // Transform derivatives back to physical element
      for (unsigned int row = 0; row < num_derivatives; row++)
      {
        for (unsigned int col = 0; col < num_derivatives; col++)
        {
          values[row] += transform[row][col]*derivatives[col];
        }
      }
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
    
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int row = 0; row < num_derivatives; row++)
      {
        delete [] combinations[row];
        delete [] transform[row];
      }
    
      delete [] combinations;
      delete [] transform;
    }
    
    if (4 <= i && i <= 9)
    {
      // Map degree of freedom to element degree of freedom
      const unsigned int dof = i - 4;
    
      // Generate scalings
      const double scalings_y_0 = 1;
      const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
      const double scalings_z_0 = 1;
      const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    
      // Compute psitilde_a
      const double psitilde_a_0 = 1;
      const double psitilde_a_1 = x;
    
      // Compute psitilde_bs
      const double psitilde_bs_0_0 = 1;
      const double psitilde_bs_0_1 = 1.5*y + 0.5;
      const double psitilde_bs_1_0 = 1;
    
      // Compute psitilde_cs
      const double psitilde_cs_00_0 = 1;
      const double psitilde_cs_00_1 = 2*z + 1;
      const double psitilde_cs_01_0 = 1;
      const double psitilde_cs_10_0 = 1;
    
      // Compute basisvalues
      const double basisvalue0 = 0.8660254038*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
      const double basisvalue1 = 2.738612788*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
      const double basisvalue2 = 1.58113883*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
      const double basisvalue3 = 1.118033989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    
      // Table(s) of coefficients
      const static double coefficients0[6][4] =   \
      {{0, 0, 0, 0},
      {-0.2886751346, 0, 0, -0.2236067977},
      {-0.2886751346, 0, -0.2108185107, 0.07453559925},
      {0.2886751346, 0, 0, 0.2236067977},
      {0.2886751346, 0, 0.2108185107, -0.07453559925},
      {0.5773502692, 0, -0.2108185107, -0.1490711985}};
    
      const static double coefficients1[6][4] =   \
      {{-0.2886751346, 0, 0, -0.2236067977},
      {0, 0, 0, 0},
      {0.2886751346, 0.1825741858, -0.1054092553, -0.07453559925},
      {0.2886751346, 0, 0, 0.2236067977},
      {0.5773502692, -0.1825741858, 0.1054092553, -0.1490711985},
      {0.2886751346, 0.1825741858, -0.1054092553, -0.07453559925}};
    
      const static double coefficients2[6][4] =   \
      {{0.2886751346, 0, 0.2108185107, -0.07453559925},
      {0.2886751346, 0.1825741858, -0.1054092553, -0.07453559925},
      {0, 0, 0, 0},
      {0.5773502692, -0.1825741858, -0.1054092553, 0.1490711985},
      {0.2886751346, 0, 0.2108185107, -0.07453559925},
      {0.2886751346, 0.1825741858, -0.1054092553, -0.07453559925}};
    
      // Interesting (new) part
      // Tables of derivatives of the polynomial base (transpose)
      const static double dmats0[4][4] =   \
      {{0, 0, 0, 0},
      {6.32455532, 0, 0, 0},
      {0, 0, 0, 0},
      {0, 0, 0, 0}};
    
      const static double dmats1[4][4] =   \
      {{0, 0, 0, 0},
      {3.16227766, 0, 0, 0},
      {5.477225575, 0, 0, 0},
      {0, 0, 0, 0}};
    
      const static double dmats2[4][4] =   \
      {{0, 0, 0, 0},
      {3.16227766, 0, 0, 0},
      {1.825741858, 0, 0, 0},
      {5.163977795, 0, 0, 0}};
    
      // Compute reference derivatives
      // Declare pointer to array of derivatives on FIAT element
      double *derivatives = new double [3*num_derivatives];
    
      // Declare coefficients
      double coeff0_0 = 0;
      double coeff0_1 = 0;
      double coeff0_2 = 0;
      double coeff0_3 = 0;
      double coeff1_0 = 0;
      double coeff1_1 = 0;
      double coeff1_2 = 0;
      double coeff1_3 = 0;
      double coeff2_0 = 0;
      double coeff2_1 = 0;
      double coeff2_2 = 0;
      double coeff2_3 = 0;
    
      // Declare new coefficients
      double new_coeff0_0 = 0;
      double new_coeff0_1 = 0;
      double new_coeff0_2 = 0;
      double new_coeff0_3 = 0;
      double new_coeff1_0 = 0;
      double new_coeff1_1 = 0;
      double new_coeff1_2 = 0;
      double new_coeff1_3 = 0;
      double new_coeff2_0 = 0;
      double new_coeff2_1 = 0;
      double new_coeff2_2 = 0;
      double new_coeff2_3 = 0;
    
      // Loop possible derivatives
      for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
      {
        // Get values from coefficients array
        new_coeff0_0 = coefficients0[dof][0];
        new_coeff0_1 = coefficients0[dof][1];
        new_coeff0_2 = coefficients0[dof][2];
        new_coeff0_3 = coefficients0[dof][3];
        new_coeff1_0 = coefficients1[dof][0];
        new_coeff1_1 = coefficients1[dof][1];
        new_coeff1_2 = coefficients1[dof][2];
        new_coeff1_3 = coefficients1[dof][3];
        new_coeff2_0 = coefficients2[dof][0];
        new_coeff2_1 = coefficients2[dof][1];
        new_coeff2_2 = coefficients2[dof][2];
        new_coeff2_3 = coefficients2[dof][3];
    
        // Loop derivative order
        for (unsigned int j = 0; j < n; j++)
        {
          // Update old coefficients
          coeff0_0 = new_coeff0_0;
          coeff0_1 = new_coeff0_1;
          coeff0_2 = new_coeff0_2;
          coeff0_3 = new_coeff0_3;
          coeff1_0 = new_coeff1_0;
          coeff1_1 = new_coeff1_1;
          coeff1_2 = new_coeff1_2;
          coeff1_3 = new_coeff1_3;
          coeff2_0 = new_coeff2_0;
          coeff2_1 = new_coeff2_1;
          coeff2_2 = new_coeff2_2;
          coeff2_3 = new_coeff2_3;
    
          if(combinations[deriv_num][j] == 0)
          {
            new_coeff0_0 = coeff0_0*dmats0[0][0] + coeff0_1*dmats0[1][0] + coeff0_2*dmats0[2][0] + coeff0_3*dmats0[3][0];
            new_coeff0_1 = coeff0_0*dmats0[0][1] + coeff0_1*dmats0[1][1] + coeff0_2*dmats0[2][1] + coeff0_3*dmats0[3][1];
            new_coeff0_2 = coeff0_0*dmats0[0][2] + coeff0_1*dmats0[1][2] + coeff0_2*dmats0[2][2] + coeff0_3*dmats0[3][2];
            new_coeff0_3 = coeff0_0*dmats0[0][3] + coeff0_1*dmats0[1][3] + coeff0_2*dmats0[2][3] + coeff0_3*dmats0[3][3];
            new_coeff1_0 = coeff1_0*dmats0[0][0] + coeff1_1*dmats0[1][0] + coeff1_2*dmats0[2][0] + coeff1_3*dmats0[3][0];
            new_coeff1_1 = coeff1_0*dmats0[0][1] + coeff1_1*dmats0[1][1] + coeff1_2*dmats0[2][1] + coeff1_3*dmats0[3][1];
            new_coeff1_2 = coeff1_0*dmats0[0][2] + coeff1_1*dmats0[1][2] + coeff1_2*dmats0[2][2] + coeff1_3*dmats0[3][2];
            new_coeff1_3 = coeff1_0*dmats0[0][3] + coeff1_1*dmats0[1][3] + coeff1_2*dmats0[2][3] + coeff1_3*dmats0[3][3];
            new_coeff2_0 = coeff2_0*dmats0[0][0] + coeff2_1*dmats0[1][0] + coeff2_2*dmats0[2][0] + coeff2_3*dmats0[3][0];
            new_coeff2_1 = coeff2_0*dmats0[0][1] + coeff2_1*dmats0[1][1] + coeff2_2*dmats0[2][1] + coeff2_3*dmats0[3][1];
            new_coeff2_2 = coeff2_0*dmats0[0][2] + coeff2_1*dmats0[1][2] + coeff2_2*dmats0[2][2] + coeff2_3*dmats0[3][2];
            new_coeff2_3 = coeff2_0*dmats0[0][3] + coeff2_1*dmats0[1][3] + coeff2_2*dmats0[2][3] + coeff2_3*dmats0[3][3];
          }
          if(combinations[deriv_num][j] == 1)
          {
            new_coeff0_0 = coeff0_0*dmats1[0][0] + coeff0_1*dmats1[1][0] + coeff0_2*dmats1[2][0] + coeff0_3*dmats1[3][0];
            new_coeff0_1 = coeff0_0*dmats1[0][1] + coeff0_1*dmats1[1][1] + coeff0_2*dmats1[2][1] + coeff0_3*dmats1[3][1];
            new_coeff0_2 = coeff0_0*dmats1[0][2] + coeff0_1*dmats1[1][2] + coeff0_2*dmats1[2][2] + coeff0_3*dmats1[3][2];
            new_coeff0_3 = coeff0_0*dmats1[0][3] + coeff0_1*dmats1[1][3] + coeff0_2*dmats1[2][3] + coeff0_3*dmats1[3][3];
            new_coeff1_0 = coeff1_0*dmats1[0][0] + coeff1_1*dmats1[1][0] + coeff1_2*dmats1[2][0] + coeff1_3*dmats1[3][0];
            new_coeff1_1 = coeff1_0*dmats1[0][1] + coeff1_1*dmats1[1][1] + coeff1_2*dmats1[2][1] + coeff1_3*dmats1[3][1];
            new_coeff1_2 = coeff1_0*dmats1[0][2] + coeff1_1*dmats1[1][2] + coeff1_2*dmats1[2][2] + coeff1_3*dmats1[3][2];
            new_coeff1_3 = coeff1_0*dmats1[0][3] + coeff1_1*dmats1[1][3] + coeff1_2*dmats1[2][3] + coeff1_3*dmats1[3][3];
            new_coeff2_0 = coeff2_0*dmats1[0][0] + coeff2_1*dmats1[1][0] + coeff2_2*dmats1[2][0] + coeff2_3*dmats1[3][0];
            new_coeff2_1 = coeff2_0*dmats1[0][1] + coeff2_1*dmats1[1][1] + coeff2_2*dmats1[2][1] + coeff2_3*dmats1[3][1];
            new_coeff2_2 = coeff2_0*dmats1[0][2] + coeff2_1*dmats1[1][2] + coeff2_2*dmats1[2][2] + coeff2_3*dmats1[3][2];
            new_coeff2_3 = coeff2_0*dmats1[0][3] + coeff2_1*dmats1[1][3] + coeff2_2*dmats1[2][3] + coeff2_3*dmats1[3][3];
          }
          if(combinations[deriv_num][j] == 2)
          {
            new_coeff0_0 = coeff0_0*dmats2[0][0] + coeff0_1*dmats2[1][0] + coeff0_2*dmats2[2][0] + coeff0_3*dmats2[3][0];
            new_coeff0_1 = coeff0_0*dmats2[0][1] + coeff0_1*dmats2[1][1] + coeff0_2*dmats2[2][1] + coeff0_3*dmats2[3][1];
            new_coeff0_2 = coeff0_0*dmats2[0][2] + coeff0_1*dmats2[1][2] + coeff0_2*dmats2[2][2] + coeff0_3*dmats2[3][2];
            new_coeff0_3 = coeff0_0*dmats2[0][3] + coeff0_1*dmats2[1][3] + coeff0_2*dmats2[2][3] + coeff0_3*dmats2[3][3];
            new_coeff1_0 = coeff1_0*dmats2[0][0] + coeff1_1*dmats2[1][0] + coeff1_2*dmats2[2][0] + coeff1_3*dmats2[3][0];
            new_coeff1_1 = coeff1_0*dmats2[0][1] + coeff1_1*dmats2[1][1] + coeff1_2*dmats2[2][1] + coeff1_3*dmats2[3][1];
            new_coeff1_2 = coeff1_0*dmats2[0][2] + coeff1_1*dmats2[1][2] + coeff1_2*dmats2[2][2] + coeff1_3*dmats2[3][2];
            new_coeff1_3 = coeff1_0*dmats2[0][3] + coeff1_1*dmats2[1][3] + coeff1_2*dmats2[2][3] + coeff1_3*dmats2[3][3];
            new_coeff2_0 = coeff2_0*dmats2[0][0] + coeff2_1*dmats2[1][0] + coeff2_2*dmats2[2][0] + coeff2_3*dmats2[3][0];
            new_coeff2_1 = coeff2_0*dmats2[0][1] + coeff2_1*dmats2[1][1] + coeff2_2*dmats2[2][1] + coeff2_3*dmats2[3][1];
            new_coeff2_2 = coeff2_0*dmats2[0][2] + coeff2_1*dmats2[1][2] + coeff2_2*dmats2[2][2] + coeff2_3*dmats2[3][2];
            new_coeff2_3 = coeff2_0*dmats2[0][3] + coeff2_1*dmats2[1][3] + coeff2_2*dmats2[2][3] + coeff2_3*dmats2[3][3];
          }
    
        }
        // Compute derivatives on reference element as dot product of coefficients and basisvalues
        // Correct values by the covariant Piola transform
        const double tmp0_0 = new_coeff0_0*basisvalue0 + new_coeff0_1*basisvalue1 + new_coeff0_2*basisvalue2 + new_coeff0_3*basisvalue3;
        const double tmp0_1 = new_coeff1_0*basisvalue0 + new_coeff1_1*basisvalue1 + new_coeff1_2*basisvalue2 + new_coeff1_3*basisvalue3;
        const double tmp0_2 = new_coeff2_0*basisvalue0 + new_coeff2_1*basisvalue1 + new_coeff2_2*basisvalue2 + new_coeff2_3*basisvalue3;
        derivatives[deriv_num] = (Jinv_00*tmp0_0 + Jinv_10*tmp0_1 + Jinv_20*tmp0_2);
        derivatives[num_derivatives + deriv_num] = (Jinv_01*tmp0_0 + Jinv_11*tmp0_1 + Jinv_21*tmp0_2);
        derivatives[2*num_derivatives + deriv_num] = (Jinv_02*tmp0_0 + Jinv_12*tmp0_1 + Jinv_22*tmp0_2);
      }
    
      // Transform derivatives back to physical element
      for (unsigned int row = 0; row < num_derivatives; row++)
      {
        for (unsigned int col = 0; col < num_derivatives; col++)
        {
          values[num_derivatives + row] += transform[row][col]*derivatives[col];
          values[2*num_derivatives + row] += transform[row][col]*derivatives[num_derivatives + col];
          values[3*num_derivatives + row] += transform[row][col]*derivatives[2*num_derivatives + col];
        }
      }
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
    
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int row = 0; row < num_derivatives; row++)
      {
        delete [] combinations[row];
        delete [] transform[row];
      }
    
      delete [] combinations;
      delete [] transform;
    }
    
  }

  /// Evaluate order n derivatives of all basis functions at given point in cell
  virtual void evaluate_basis_derivatives_all(unsigned int n,
                                              double* values,
                                              const double* coordinates,
                                              const ufc::cell& c) const
  {
    throw std::runtime_error("The vectorised version of evaluate_basis_derivatives() is not yet implemented.");
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    // The reference points, direction and weights:
    const static double X[10][1][3] = {{{0, 0, 0}}, {{1, 0, 0}}, {{0, 1, 0}}, {{0, 0, 1}}, {{0, 0.5, 0.5}}, {{0.5, 0, 0.5}}, {{0.5, 0.5, 0}}, {{0, 0, 0.5}}, {{0, 0.5, 0}}, {{0.5, 0, 0}}};
    const static double W[10][1] = {{1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}};
    const static double D[10][1][4] = {{{1, 0, 0, 0}}, {{1, 0, 0, 0}}, {{1, 0, 0, 0}}, {{1, 0, 0, 0}}, {{0, 0, -1, 1}}, {{0, -1, 0, 1}}, {{0, -1, 1, 0}}, {{0, 0, 0, 1}}, {{0, 0, 1, 0}}, {{0, 1, 0, 0}}};
    
    static const unsigned int mappings[10] = {0, 0, 0, 0, 2, 2, 2, 2, 2, 2};
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = x[1][0] - x[0][0];
    const double J_01 = x[2][0] - x[0][0];
    const double J_02 = x[3][0] - x[0][0];
    const double J_10 = x[1][1] - x[0][1];
    const double J_11 = x[2][1] - x[0][1];
    const double J_12 = x[3][1] - x[0][1];
    const double J_20 = x[1][2] - x[0][2];
    const double J_21 = x[2][2] - x[0][2];
    const double J_22 = x[3][2] - x[0][2];
      
    // Compute sub determinants
    const double d_00 = J_11*J_22 - J_12*J_21;
    const double d_01 = J_12*J_20 - J_10*J_22;
    const double d_02 = J_10*J_21 - J_11*J_20;
    
    const double d_10 = J_02*J_21 - J_01*J_22;
    const double d_11 = J_00*J_22 - J_02*J_20;
    const double d_12 = J_01*J_20 - J_00*J_21;
    
    const double d_20 = J_01*J_12 - J_02*J_11;
    const double d_21 = J_02*J_10 - J_00*J_12;
    const double d_22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d_00 + J_10*d_10 + J_20*d_20;
      
    // Compute inverse of Jacobian
    const double Jinv_00 = d_00 / detJ;
    const double Jinv_01 = d_10 / detJ;
    const double Jinv_02 = d_20 / detJ;
    const double Jinv_10 = d_01 / detJ;
    const double Jinv_11 = d_11 / detJ;
    const double Jinv_12 = d_21 / detJ;
    const double Jinv_20 = d_02 / detJ;
    const double Jinv_21 = d_12 / detJ;
    const double Jinv_22 = d_22 / detJ;
    
    double copyofvalues[4];
    const int offsets[10] = {0, 0, 0, 0, 1, 1, 1, 1, 1, 1};
    double result = 0.0;
    // Iterate over the points:
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0][0] - X[i][0][1] - X[i][0][2];
    const double w1 = X[i][0][0];
    const double w2 = X[i][0][1];
    const double w3 = X[i][0][2];
    
    // Compute affine mapping y = F(X)
    double y[3];
    y[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0] + w3*x[3][0];
    y[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1] + w3*x[3][1];
    y[2] = w0*x[0][2] + w1*x[1][2] + w2*x[2][2] + w3*x[3][2];
    
    // Evaluate function at physical points
    double values[4];
    f.evaluate(values, y, c);
    
    // Map function values using appropriate mapping
    if (mappings[i] == 0) { 
      // Affine map: Do nothing
    } else if (mappings[i] == 2) {
       // Copy old values:
      copyofvalues[offsets[i] + 0] = values[offsets[i] + 0];
      copyofvalues[offsets[i] + 1] = values[offsets[i] + 1];
      copyofvalues[offsets[i] + 2] = values[offsets[i] + 2];
      // Do the inverse of curl piola 
      values[offsets[i] + 0] = J_00*copyofvalues[offsets[i] + 0]+J_10*copyofvalues[offsets[i] + 1]+J_20*copyofvalues[offsets[i] + 2];
      values[offsets[i] + 1] = J_01*copyofvalues[offsets[i] + 0]+J_11*copyofvalues[offsets[i] + 1]+J_21*copyofvalues[offsets[i] + 2];
      values[offsets[i] + 2] = J_02*copyofvalues[offsets[i] + 0]+J_12*copyofvalues[offsets[i] + 1]+J_22*copyofvalues[offsets[i] + 2]; 
    } else { 
       // Other mappings not applicable. 
    }
    
    // Note that we do not map the weights (yet).
    
    // Take directional components
    for(int k = 0; k < 4; k++)
      result += values[k]*D[i][0][k];
    // Multiply by weights 
    result *= W[i][0];
    
    return result;
  }

  /// Evaluate linear functionals for all dofs on the function f
  virtual void evaluate_dofs(double* values,
                             const ufc::function& f,
                             const ufc::cell& c) const
  {
    throw std::runtime_error("Not implemented (introduced in UFC v1.1).");
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = x[1][0] - x[0][0];
    const double J_01 = x[2][0] - x[0][0];
    const double J_02 = x[3][0] - x[0][0];
    const double J_10 = x[1][1] - x[0][1];
    const double J_11 = x[2][1] - x[0][1];
    const double J_12 = x[3][1] - x[0][1];
    const double J_20 = x[1][2] - x[0][2];
    const double J_21 = x[2][2] - x[0][2];
    const double J_22 = x[3][2] - x[0][2];
      
    // Compute sub determinants
    const double d_00 = J_11*J_22 - J_12*J_21;
    const double d_01 = J_12*J_20 - J_10*J_22;
    const double d_02 = J_10*J_21 - J_11*J_20;
    
    const double d_10 = J_02*J_21 - J_01*J_22;
    const double d_11 = J_00*J_22 - J_02*J_20;
    const double d_12 = J_01*J_20 - J_00*J_21;
    
    const double d_20 = J_01*J_12 - J_02*J_11;
    const double d_21 = J_02*J_10 - J_00*J_12;
    const double d_22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d_00 + J_10*d_10 + J_20*d_20;
      
    // Compute inverse of Jacobian
    const double Jinv_00 = d_00 / detJ;
    const double Jinv_01 = d_10 / detJ;
    const double Jinv_02 = d_20 / detJ;
    const double Jinv_10 = d_01 / detJ;
    const double Jinv_11 = d_11 / detJ;
    const double Jinv_12 = d_21 / detJ;
    const double Jinv_20 = d_02 / detJ;
    const double Jinv_21 = d_12 / detJ;
    const double Jinv_22 = d_22 / detJ;
    // Evaluate at vertices and use affine mapping
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[1];
    vertex_values[2] = dof_values[2];
    vertex_values[3] = dof_values[3];
    // Evaluate at vertices and use Piola mapping
    vertex_values[4] = (dof_values[7]*Jinv_20 + dof_values[8]*Jinv_10 + dof_values[9]*Jinv_00);
    vertex_values[5] = (dof_values[5]*Jinv_20 + dof_values[6]*Jinv_10 + dof_values[9]*(Jinv_00 + Jinv_10 + Jinv_20));
    vertex_values[6] = (dof_values[4]*Jinv_20 + dof_values[6]*Jinv_00 + dof_values[8]*(Jinv_00 + Jinv_10 + Jinv_20));
    vertex_values[7] = (dof_values[4]*Jinv_10 + dof_values[5]*Jinv_00 + dof_values[7]*(Jinv_00 + Jinv_10 + Jinv_20));
    vertex_values[8] = (dof_values[7]*Jinv_21 + dof_values[8]*Jinv_11 + dof_values[9]*Jinv_01);
    vertex_values[9] = (dof_values[5]*Jinv_21 + dof_values[6]*Jinv_11 + dof_values[9]*(Jinv_01 + Jinv_11 + Jinv_21));
    vertex_values[10] = (dof_values[4]*Jinv_21 + dof_values[6]*Jinv_01 + dof_values[8]*(Jinv_01 + Jinv_11 + Jinv_21));
    vertex_values[11] = (dof_values[4]*Jinv_11 + dof_values[5]*Jinv_01 + dof_values[7]*(Jinv_01 + Jinv_11 + Jinv_21));
    vertex_values[12] = (dof_values[7]*Jinv_22 + dof_values[8]*Jinv_12 + dof_values[9]*Jinv_02);
    vertex_values[13] = (dof_values[5]*Jinv_22 + dof_values[6]*Jinv_12 + dof_values[9]*(Jinv_02 + Jinv_12 + Jinv_22));
    vertex_values[14] = (dof_values[4]*Jinv_22 + dof_values[6]*Jinv_02 + dof_values[8]*(Jinv_02 + Jinv_12 + Jinv_22));
    vertex_values[15] = (dof_values[4]*Jinv_12 + dof_values[5]*Jinv_02 + dof_values[7]*(Jinv_02 + Jinv_12 + Jinv_22));
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 2;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    switch ( i )
    {
    case 0:
      return new VectorLaplaceGradCurlBilinearForm_finite_element_0_0();
      break;
    case 1:
      return new VectorLaplaceGradCurlBilinearForm_finite_element_0_1();
      break;
    }
    return 0;
  }

};

/// This class defines the interface for a finite element.

class VectorLaplaceGradCurlBilinearForm_finite_element_1_0: public ufc::finite_element
{
public:

  /// Constructor
  VectorLaplaceGradCurlBilinearForm_finite_element_1_0() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~VectorLaplaceGradCurlBilinearForm_finite_element_1_0()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "Lagrange finite element of degree 1 on a tetrahedron";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::tetrahedron;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 4;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 0;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 1;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-09)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-09)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Reset values
    *values = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
    const double scalings_z_0 = 1;
    const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    const double psitilde_a_1 = x;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    const double psitilde_bs_0_1 = 1.5*y + 0.5;
    const double psitilde_bs_1_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    const double psitilde_cs_00_1 = 2*z + 1;
    const double psitilde_cs_01_0 = 1;
    const double psitilde_cs_10_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.8660254038*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    const double basisvalue1 = 2.738612788*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
    const double basisvalue2 = 1.58113883*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
    const double basisvalue3 = 1.118033989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    
    // Table(s) of coefficients
    const static double coefficients0[4][4] = \
    {{0.2886751346, -0.1825741858, -0.1054092553, -0.07453559925},
    {0.2886751346, 0.1825741858, -0.1054092553, -0.07453559925},
    {0.2886751346, 0, 0.2108185107, -0.07453559925},
    {0.2886751346, 0, 0, 0.2236067977}};
    
    // Extract relevant coefficients
    const double coeff0_0 = coefficients0[dof][0];
    const double coeff0_1 = coefficients0[dof][1];
    const double coeff0_2 = coefficients0[dof][2];
    const double coeff0_3 = coefficients0[dof][3];
    
    // Compute value(s)
    *values = coeff0_0*basisvalue0 + coeff0_1*basisvalue1 + coeff0_2*basisvalue2 + coeff0_3*basisvalue3;
  }

  /// Evaluate all basis functions at given point in cell
  virtual void evaluate_basis_all(double* values,
                                  const double* coordinates,
                                  const ufc::cell& c) const
  {
    throw std::runtime_error("The vectorised version of evaluate_basis() is not yet implemented.");
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-09)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-09)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Compute number of derivatives
    unsigned int num_derivatives = 1;
    
    for (unsigned int j = 0; j < n; j++)
      num_derivatives *= 3;
    
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      combinations[j] = new unsigned int [n];
      for (unsigned int k = 0; k < n; k++)
        combinations[j][k] = 0;
    }
        
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 2)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian
    const double Jinv[3][3] ={{d00 / detJ, d10 / detJ, d20 / detJ}, {d01 / detJ, d11 / detJ, d21 / detJ}, {d02 / detJ, d12 / detJ, d22 / detJ}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values
    for (unsigned int j = 0; j < 1*num_derivatives; j++)
      values[j] = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
    const double scalings_z_0 = 1;
    const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    const double psitilde_a_1 = x;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    const double psitilde_bs_0_1 = 1.5*y + 0.5;
    const double psitilde_bs_1_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    const double psitilde_cs_00_1 = 2*z + 1;
    const double psitilde_cs_01_0 = 1;
    const double psitilde_cs_10_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.8660254038*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    const double basisvalue1 = 2.738612788*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
    const double basisvalue2 = 1.58113883*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
    const double basisvalue3 = 1.118033989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    
    // Table(s) of coefficients
    const static double coefficients0[4][4] = \
    {{0.2886751346, -0.1825741858, -0.1054092553, -0.07453559925},
    {0.2886751346, 0.1825741858, -0.1054092553, -0.07453559925},
    {0.2886751346, 0, 0.2108185107, -0.07453559925},
    {0.2886751346, 0, 0, 0.2236067977}};
    
    // Interesting (new) part
    // Tables of derivatives of the polynomial base (transpose)
    const static double dmats0[4][4] = \
    {{0, 0, 0, 0},
    {6.32455532, 0, 0, 0},
    {0, 0, 0, 0},
    {0, 0, 0, 0}};
    
    const static double dmats1[4][4] = \
    {{0, 0, 0, 0},
    {3.16227766, 0, 0, 0},
    {5.477225575, 0, 0, 0},
    {0, 0, 0, 0}};
    
    const static double dmats2[4][4] = \
    {{0, 0, 0, 0},
    {3.16227766, 0, 0, 0},
    {1.825741858, 0, 0, 0},
    {5.163977795, 0, 0, 0}};
    
    // Compute reference derivatives
    // Declare pointer to array of derivatives on FIAT element
    double *derivatives = new double [num_derivatives];
    
    // Declare coefficients
    double coeff0_0 = 0;
    double coeff0_1 = 0;
    double coeff0_2 = 0;
    double coeff0_3 = 0;
    
    // Declare new coefficients
    double new_coeff0_0 = 0;
    double new_coeff0_1 = 0;
    double new_coeff0_2 = 0;
    double new_coeff0_3 = 0;
    
    // Loop possible derivatives
    for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
    {
      // Get values from coefficients array
      new_coeff0_0 = coefficients0[dof][0];
      new_coeff0_1 = coefficients0[dof][1];
      new_coeff0_2 = coefficients0[dof][2];
      new_coeff0_3 = coefficients0[dof][3];
    
      // Loop derivative order
      for (unsigned int j = 0; j < n; j++)
      {
        // Update old coefficients
        coeff0_0 = new_coeff0_0;
        coeff0_1 = new_coeff0_1;
        coeff0_2 = new_coeff0_2;
        coeff0_3 = new_coeff0_3;
    
        if(combinations[deriv_num][j] == 0)
        {
          new_coeff0_0 = coeff0_0*dmats0[0][0] + coeff0_1*dmats0[1][0] + coeff0_2*dmats0[2][0] + coeff0_3*dmats0[3][0];
          new_coeff0_1 = coeff0_0*dmats0[0][1] + coeff0_1*dmats0[1][1] + coeff0_2*dmats0[2][1] + coeff0_3*dmats0[3][1];
          new_coeff0_2 = coeff0_0*dmats0[0][2] + coeff0_1*dmats0[1][2] + coeff0_2*dmats0[2][2] + coeff0_3*dmats0[3][2];
          new_coeff0_3 = coeff0_0*dmats0[0][3] + coeff0_1*dmats0[1][3] + coeff0_2*dmats0[2][3] + coeff0_3*dmats0[3][3];
        }
        if(combinations[deriv_num][j] == 1)
        {
          new_coeff0_0 = coeff0_0*dmats1[0][0] + coeff0_1*dmats1[1][0] + coeff0_2*dmats1[2][0] + coeff0_3*dmats1[3][0];
          new_coeff0_1 = coeff0_0*dmats1[0][1] + coeff0_1*dmats1[1][1] + coeff0_2*dmats1[2][1] + coeff0_3*dmats1[3][1];
          new_coeff0_2 = coeff0_0*dmats1[0][2] + coeff0_1*dmats1[1][2] + coeff0_2*dmats1[2][2] + coeff0_3*dmats1[3][2];
          new_coeff0_3 = coeff0_0*dmats1[0][3] + coeff0_1*dmats1[1][3] + coeff0_2*dmats1[2][3] + coeff0_3*dmats1[3][3];
        }
        if(combinations[deriv_num][j] == 2)
        {
          new_coeff0_0 = coeff0_0*dmats2[0][0] + coeff0_1*dmats2[1][0] + coeff0_2*dmats2[2][0] + coeff0_3*dmats2[3][0];
          new_coeff0_1 = coeff0_0*dmats2[0][1] + coeff0_1*dmats2[1][1] + coeff0_2*dmats2[2][1] + coeff0_3*dmats2[3][1];
          new_coeff0_2 = coeff0_0*dmats2[0][2] + coeff0_1*dmats2[1][2] + coeff0_2*dmats2[2][2] + coeff0_3*dmats2[3][2];
          new_coeff0_3 = coeff0_0*dmats2[0][3] + coeff0_1*dmats2[1][3] + coeff0_2*dmats2[2][3] + coeff0_3*dmats2[3][3];
        }
    
      }
      // Compute derivatives on reference element as dot product of coefficients and basisvalues
      derivatives[deriv_num] = new_coeff0_0*basisvalue0 + new_coeff0_1*basisvalue1 + new_coeff0_2*basisvalue2 + new_coeff0_3*basisvalue3;
    }
    
    // Transform derivatives back to physical element
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        values[row] += transform[row][col]*derivatives[col];
      }
    }
    // Delete pointer to array of derivatives on FIAT element
    delete [] derivatives;
    
    // Delete pointer to array of combinations of derivatives and transform
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      delete [] combinations[row];
      delete [] transform[row];
    }
    
    delete [] combinations;
    delete [] transform;
  }

  /// Evaluate order n derivatives of all basis functions at given point in cell
  virtual void evaluate_basis_derivatives_all(unsigned int n,
                                              double* values,
                                              const double* coordinates,
                                              const ufc::cell& c) const
  {
    throw std::runtime_error("The vectorised version of evaluate_basis_derivatives() is not yet implemented.");
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    // The reference points, direction and weights:
    const static double X[4][1][3] = {{{0, 0, 0}}, {{1, 0, 0}}, {{0, 1, 0}}, {{0, 0, 1}}};
    const static double W[4][1] = {{1}, {1}, {1}, {1}};
    const static double D[4][1][1] = {{{1}}, {{1}}, {{1}}, {{1}}};
    
    const double * const * x = c.coordinates;
    double result = 0.0;
    // Iterate over the points:
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0][0] - X[i][0][1] - X[i][0][2];
    const double w1 = X[i][0][0];
    const double w2 = X[i][0][1];
    const double w3 = X[i][0][2];
    
    // Compute affine mapping y = F(X)
    double y[3];
    y[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0] + w3*x[3][0];
    y[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1] + w3*x[3][1];
    y[2] = w0*x[0][2] + w1*x[1][2] + w2*x[2][2] + w3*x[3][2];
    
    // Evaluate function at physical points
    double values[1];
    f.evaluate(values, y, c);
    
    // Map function values using appropriate mapping
    // Affine map: Do nothing
    
    // Note that we do not map the weights (yet).
    
    // Take directional components
    for(int k = 0; k < 1; k++)
      result += values[k]*D[i][0][k];
    // Multiply by weights 
    result *= W[i][0];
    
    return result;
  }

  /// Evaluate linear functionals for all dofs on the function f
  virtual void evaluate_dofs(double* values,
                             const ufc::function& f,
                             const ufc::cell& c) const
  {
    throw std::runtime_error("Not implemented (introduced in UFC v1.1).");
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    // Evaluate at vertices and use affine mapping
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[1];
    vertex_values[2] = dof_values[2];
    vertex_values[3] = dof_values[3];
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 1;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    return new VectorLaplaceGradCurlBilinearForm_finite_element_1_0();
  }

};

/// This class defines the interface for a finite element.

class VectorLaplaceGradCurlBilinearForm_finite_element_1_1: public ufc::finite_element
{
public:

  /// Constructor
  VectorLaplaceGradCurlBilinearForm_finite_element_1_1() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~VectorLaplaceGradCurlBilinearForm_finite_element_1_1()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "Nedelec finite element of degree 1 on a tetrahedron";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::tetrahedron;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 6;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 1;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 3;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    const double Jinv_00 = d00 / detJ;
    const double Jinv_01 = d10 / detJ;
    const double Jinv_02 = d20 / detJ;
    const double Jinv_10 = d01 / detJ;
    const double Jinv_11 = d11 / detJ;
    const double Jinv_12 = d21 / detJ;
    const double Jinv_20 = d02 / detJ;
    const double Jinv_21 = d12 / detJ;
    const double Jinv_22 = d22 / detJ;
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-09)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-09)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Reset values
    values[0] = 0;
    values[1] = 0;
    values[2] = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
    const double scalings_z_0 = 1;
    const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    const double psitilde_a_1 = x;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    const double psitilde_bs_0_1 = 1.5*y + 0.5;
    const double psitilde_bs_1_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    const double psitilde_cs_00_1 = 2*z + 1;
    const double psitilde_cs_01_0 = 1;
    const double psitilde_cs_10_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.8660254038*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    const double basisvalue1 = 2.738612788*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
    const double basisvalue2 = 1.58113883*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
    const double basisvalue3 = 1.118033989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    
    // Table(s) of coefficients
    const static double coefficients0[6][4] = \
    {{0, 0, 0, 0},
    {-0.2886751346, 0, 0, -0.2236067977},
    {-0.2886751346, 0, -0.2108185107, 0.07453559925},
    {0.2886751346, 0, 0, 0.2236067977},
    {0.2886751346, 0, 0.2108185107, -0.07453559925},
    {0.5773502692, 0, -0.2108185107, -0.1490711985}};
    
    const static double coefficients1[6][4] = \
    {{-0.2886751346, 0, 0, -0.2236067977},
    {0, 0, 0, 0},
    {0.2886751346, 0.1825741858, -0.1054092553, -0.07453559925},
    {0.2886751346, 0, 0, 0.2236067977},
    {0.5773502692, -0.1825741858, 0.1054092553, -0.1490711985},
    {0.2886751346, 0.1825741858, -0.1054092553, -0.07453559925}};
    
    const static double coefficients2[6][4] = \
    {{0.2886751346, 0, 0.2108185107, -0.07453559925},
    {0.2886751346, 0.1825741858, -0.1054092553, -0.07453559925},
    {0, 0, 0, 0},
    {0.5773502692, -0.1825741858, -0.1054092553, 0.1490711985},
    {0.2886751346, 0, 0.2108185107, -0.07453559925},
    {0.2886751346, 0.1825741858, -0.1054092553, -0.07453559925}};
    
    // Extract relevant coefficients
    const double coeff0_0 = coefficients0[dof][0];
    const double coeff0_1 = coefficients0[dof][1];
    const double coeff0_2 = coefficients0[dof][2];
    const double coeff0_3 = coefficients0[dof][3];
    const double coeff1_0 = coefficients1[dof][0];
    const double coeff1_1 = coefficients1[dof][1];
    const double coeff1_2 = coefficients1[dof][2];
    const double coeff1_3 = coefficients1[dof][3];
    const double coeff2_0 = coefficients2[dof][0];
    const double coeff2_1 = coefficients2[dof][1];
    const double coeff2_2 = coefficients2[dof][2];
    const double coeff2_3 = coefficients2[dof][3];
    
    // Compute value(s)
    const double tmp0_0 = coeff0_0*basisvalue0 + coeff0_1*basisvalue1 + coeff0_2*basisvalue2 + coeff0_3*basisvalue3;
    const double tmp0_1 = coeff1_0*basisvalue0 + coeff1_1*basisvalue1 + coeff1_2*basisvalue2 + coeff1_3*basisvalue3;
    const double tmp0_2 = coeff2_0*basisvalue0 + coeff2_1*basisvalue1 + coeff2_2*basisvalue2 + coeff2_3*basisvalue3;
    // Using covariant Piola transform to map values back to the physical element
    values[0] = (Jinv_00*tmp0_0 + Jinv_10*tmp0_1 + Jinv_20*tmp0_2);
    values[1] = (Jinv_01*tmp0_0 + Jinv_11*tmp0_1 + Jinv_21*tmp0_2);
    values[2] = (Jinv_02*tmp0_0 + Jinv_12*tmp0_1 + Jinv_22*tmp0_2);
  }

  /// Evaluate all basis functions at given point in cell
  virtual void evaluate_basis_all(double* values,
                                  const double* coordinates,
                                  const ufc::cell& c) const
  {
    throw std::runtime_error("The vectorised version of evaluate_basis() is not yet implemented.");
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    const double Jinv_00 = d00 / detJ;
    const double Jinv_01 = d10 / detJ;
    const double Jinv_02 = d20 / detJ;
    const double Jinv_10 = d01 / detJ;
    const double Jinv_11 = d11 / detJ;
    const double Jinv_12 = d21 / detJ;
    const double Jinv_20 = d02 / detJ;
    const double Jinv_21 = d12 / detJ;
    const double Jinv_22 = d22 / detJ;
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-09)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-09)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Compute number of derivatives
    unsigned int num_derivatives = 1;
    
    for (unsigned int j = 0; j < n; j++)
      num_derivatives *= 3;
    
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      combinations[j] = new unsigned int [n];
      for (unsigned int k = 0; k < n; k++)
        combinations[j][k] = 0;
    }
        
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 2)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian
    const double Jinv[3][3] ={{d00 / detJ, d10 / detJ, d20 / detJ}, {d01 / detJ, d11 / detJ, d21 / detJ}, {d02 / detJ, d12 / detJ, d22 / detJ}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values
    for (unsigned int j = 0; j < 3*num_derivatives; j++)
      values[j] = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
    const double scalings_z_0 = 1;
    const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    const double psitilde_a_1 = x;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    const double psitilde_bs_0_1 = 1.5*y + 0.5;
    const double psitilde_bs_1_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    const double psitilde_cs_00_1 = 2*z + 1;
    const double psitilde_cs_01_0 = 1;
    const double psitilde_cs_10_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.8660254038*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    const double basisvalue1 = 2.738612788*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
    const double basisvalue2 = 1.58113883*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
    const double basisvalue3 = 1.118033989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    
    // Table(s) of coefficients
    const static double coefficients0[6][4] = \
    {{0, 0, 0, 0},
    {-0.2886751346, 0, 0, -0.2236067977},
    {-0.2886751346, 0, -0.2108185107, 0.07453559925},
    {0.2886751346, 0, 0, 0.2236067977},
    {0.2886751346, 0, 0.2108185107, -0.07453559925},
    {0.5773502692, 0, -0.2108185107, -0.1490711985}};
    
    const static double coefficients1[6][4] = \
    {{-0.2886751346, 0, 0, -0.2236067977},
    {0, 0, 0, 0},
    {0.2886751346, 0.1825741858, -0.1054092553, -0.07453559925},
    {0.2886751346, 0, 0, 0.2236067977},
    {0.5773502692, -0.1825741858, 0.1054092553, -0.1490711985},
    {0.2886751346, 0.1825741858, -0.1054092553, -0.07453559925}};
    
    const static double coefficients2[6][4] = \
    {{0.2886751346, 0, 0.2108185107, -0.07453559925},
    {0.2886751346, 0.1825741858, -0.1054092553, -0.07453559925},
    {0, 0, 0, 0},
    {0.5773502692, -0.1825741858, -0.1054092553, 0.1490711985},
    {0.2886751346, 0, 0.2108185107, -0.07453559925},
    {0.2886751346, 0.1825741858, -0.1054092553, -0.07453559925}};
    
    // Interesting (new) part
    // Tables of derivatives of the polynomial base (transpose)
    const static double dmats0[4][4] = \
    {{0, 0, 0, 0},
    {6.32455532, 0, 0, 0},
    {0, 0, 0, 0},
    {0, 0, 0, 0}};
    
    const static double dmats1[4][4] = \
    {{0, 0, 0, 0},
    {3.16227766, 0, 0, 0},
    {5.477225575, 0, 0, 0},
    {0, 0, 0, 0}};
    
    const static double dmats2[4][4] = \
    {{0, 0, 0, 0},
    {3.16227766, 0, 0, 0},
    {1.825741858, 0, 0, 0},
    {5.163977795, 0, 0, 0}};
    
    // Compute reference derivatives
    // Declare pointer to array of derivatives on FIAT element
    double *derivatives = new double [3*num_derivatives];
    
    // Declare coefficients
    double coeff0_0 = 0;
    double coeff0_1 = 0;
    double coeff0_2 = 0;
    double coeff0_3 = 0;
    double coeff1_0 = 0;
    double coeff1_1 = 0;
    double coeff1_2 = 0;
    double coeff1_3 = 0;
    double coeff2_0 = 0;
    double coeff2_1 = 0;
    double coeff2_2 = 0;
    double coeff2_3 = 0;
    
    // Declare new coefficients
    double new_coeff0_0 = 0;
    double new_coeff0_1 = 0;
    double new_coeff0_2 = 0;
    double new_coeff0_3 = 0;
    double new_coeff1_0 = 0;
    double new_coeff1_1 = 0;
    double new_coeff1_2 = 0;
    double new_coeff1_3 = 0;
    double new_coeff2_0 = 0;
    double new_coeff2_1 = 0;
    double new_coeff2_2 = 0;
    double new_coeff2_3 = 0;
    
    // Loop possible derivatives
    for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
    {
      // Get values from coefficients array
      new_coeff0_0 = coefficients0[dof][0];
      new_coeff0_1 = coefficients0[dof][1];
      new_coeff0_2 = coefficients0[dof][2];
      new_coeff0_3 = coefficients0[dof][3];
      new_coeff1_0 = coefficients1[dof][0];
      new_coeff1_1 = coefficients1[dof][1];
      new_coeff1_2 = coefficients1[dof][2];
      new_coeff1_3 = coefficients1[dof][3];
      new_coeff2_0 = coefficients2[dof][0];
      new_coeff2_1 = coefficients2[dof][1];
      new_coeff2_2 = coefficients2[dof][2];
      new_coeff2_3 = coefficients2[dof][3];
    
      // Loop derivative order
      for (unsigned int j = 0; j < n; j++)
      {
        // Update old coefficients
        coeff0_0 = new_coeff0_0;
        coeff0_1 = new_coeff0_1;
        coeff0_2 = new_coeff0_2;
        coeff0_3 = new_coeff0_3;
        coeff1_0 = new_coeff1_0;
        coeff1_1 = new_coeff1_1;
        coeff1_2 = new_coeff1_2;
        coeff1_3 = new_coeff1_3;
        coeff2_0 = new_coeff2_0;
        coeff2_1 = new_coeff2_1;
        coeff2_2 = new_coeff2_2;
        coeff2_3 = new_coeff2_3;
    
        if(combinations[deriv_num][j] == 0)
        {
          new_coeff0_0 = coeff0_0*dmats0[0][0] + coeff0_1*dmats0[1][0] + coeff0_2*dmats0[2][0] + coeff0_3*dmats0[3][0];
          new_coeff0_1 = coeff0_0*dmats0[0][1] + coeff0_1*dmats0[1][1] + coeff0_2*dmats0[2][1] + coeff0_3*dmats0[3][1];
          new_coeff0_2 = coeff0_0*dmats0[0][2] + coeff0_1*dmats0[1][2] + coeff0_2*dmats0[2][2] + coeff0_3*dmats0[3][2];
          new_coeff0_3 = coeff0_0*dmats0[0][3] + coeff0_1*dmats0[1][3] + coeff0_2*dmats0[2][3] + coeff0_3*dmats0[3][3];
          new_coeff1_0 = coeff1_0*dmats0[0][0] + coeff1_1*dmats0[1][0] + coeff1_2*dmats0[2][0] + coeff1_3*dmats0[3][0];
          new_coeff1_1 = coeff1_0*dmats0[0][1] + coeff1_1*dmats0[1][1] + coeff1_2*dmats0[2][1] + coeff1_3*dmats0[3][1];
          new_coeff1_2 = coeff1_0*dmats0[0][2] + coeff1_1*dmats0[1][2] + coeff1_2*dmats0[2][2] + coeff1_3*dmats0[3][2];
          new_coeff1_3 = coeff1_0*dmats0[0][3] + coeff1_1*dmats0[1][3] + coeff1_2*dmats0[2][3] + coeff1_3*dmats0[3][3];
          new_coeff2_0 = coeff2_0*dmats0[0][0] + coeff2_1*dmats0[1][0] + coeff2_2*dmats0[2][0] + coeff2_3*dmats0[3][0];
          new_coeff2_1 = coeff2_0*dmats0[0][1] + coeff2_1*dmats0[1][1] + coeff2_2*dmats0[2][1] + coeff2_3*dmats0[3][1];
          new_coeff2_2 = coeff2_0*dmats0[0][2] + coeff2_1*dmats0[1][2] + coeff2_2*dmats0[2][2] + coeff2_3*dmats0[3][2];
          new_coeff2_3 = coeff2_0*dmats0[0][3] + coeff2_1*dmats0[1][3] + coeff2_2*dmats0[2][3] + coeff2_3*dmats0[3][3];
        }
        if(combinations[deriv_num][j] == 1)
        {
          new_coeff0_0 = coeff0_0*dmats1[0][0] + coeff0_1*dmats1[1][0] + coeff0_2*dmats1[2][0] + coeff0_3*dmats1[3][0];
          new_coeff0_1 = coeff0_0*dmats1[0][1] + coeff0_1*dmats1[1][1] + coeff0_2*dmats1[2][1] + coeff0_3*dmats1[3][1];
          new_coeff0_2 = coeff0_0*dmats1[0][2] + coeff0_1*dmats1[1][2] + coeff0_2*dmats1[2][2] + coeff0_3*dmats1[3][2];
          new_coeff0_3 = coeff0_0*dmats1[0][3] + coeff0_1*dmats1[1][3] + coeff0_2*dmats1[2][3] + coeff0_3*dmats1[3][3];
          new_coeff1_0 = coeff1_0*dmats1[0][0] + coeff1_1*dmats1[1][0] + coeff1_2*dmats1[2][0] + coeff1_3*dmats1[3][0];
          new_coeff1_1 = coeff1_0*dmats1[0][1] + coeff1_1*dmats1[1][1] + coeff1_2*dmats1[2][1] + coeff1_3*dmats1[3][1];
          new_coeff1_2 = coeff1_0*dmats1[0][2] + coeff1_1*dmats1[1][2] + coeff1_2*dmats1[2][2] + coeff1_3*dmats1[3][2];
          new_coeff1_3 = coeff1_0*dmats1[0][3] + coeff1_1*dmats1[1][3] + coeff1_2*dmats1[2][3] + coeff1_3*dmats1[3][3];
          new_coeff2_0 = coeff2_0*dmats1[0][0] + coeff2_1*dmats1[1][0] + coeff2_2*dmats1[2][0] + coeff2_3*dmats1[3][0];
          new_coeff2_1 = coeff2_0*dmats1[0][1] + coeff2_1*dmats1[1][1] + coeff2_2*dmats1[2][1] + coeff2_3*dmats1[3][1];
          new_coeff2_2 = coeff2_0*dmats1[0][2] + coeff2_1*dmats1[1][2] + coeff2_2*dmats1[2][2] + coeff2_3*dmats1[3][2];
          new_coeff2_3 = coeff2_0*dmats1[0][3] + coeff2_1*dmats1[1][3] + coeff2_2*dmats1[2][3] + coeff2_3*dmats1[3][3];
        }
        if(combinations[deriv_num][j] == 2)
        {
          new_coeff0_0 = coeff0_0*dmats2[0][0] + coeff0_1*dmats2[1][0] + coeff0_2*dmats2[2][0] + coeff0_3*dmats2[3][0];
          new_coeff0_1 = coeff0_0*dmats2[0][1] + coeff0_1*dmats2[1][1] + coeff0_2*dmats2[2][1] + coeff0_3*dmats2[3][1];
          new_coeff0_2 = coeff0_0*dmats2[0][2] + coeff0_1*dmats2[1][2] + coeff0_2*dmats2[2][2] + coeff0_3*dmats2[3][2];
          new_coeff0_3 = coeff0_0*dmats2[0][3] + coeff0_1*dmats2[1][3] + coeff0_2*dmats2[2][3] + coeff0_3*dmats2[3][3];
          new_coeff1_0 = coeff1_0*dmats2[0][0] + coeff1_1*dmats2[1][0] + coeff1_2*dmats2[2][0] + coeff1_3*dmats2[3][0];
          new_coeff1_1 = coeff1_0*dmats2[0][1] + coeff1_1*dmats2[1][1] + coeff1_2*dmats2[2][1] + coeff1_3*dmats2[3][1];
          new_coeff1_2 = coeff1_0*dmats2[0][2] + coeff1_1*dmats2[1][2] + coeff1_2*dmats2[2][2] + coeff1_3*dmats2[3][2];
          new_coeff1_3 = coeff1_0*dmats2[0][3] + coeff1_1*dmats2[1][3] + coeff1_2*dmats2[2][3] + coeff1_3*dmats2[3][3];
          new_coeff2_0 = coeff2_0*dmats2[0][0] + coeff2_1*dmats2[1][0] + coeff2_2*dmats2[2][0] + coeff2_3*dmats2[3][0];
          new_coeff2_1 = coeff2_0*dmats2[0][1] + coeff2_1*dmats2[1][1] + coeff2_2*dmats2[2][1] + coeff2_3*dmats2[3][1];
          new_coeff2_2 = coeff2_0*dmats2[0][2] + coeff2_1*dmats2[1][2] + coeff2_2*dmats2[2][2] + coeff2_3*dmats2[3][2];
          new_coeff2_3 = coeff2_0*dmats2[0][3] + coeff2_1*dmats2[1][3] + coeff2_2*dmats2[2][3] + coeff2_3*dmats2[3][3];
        }
    
      }
      // Compute derivatives on reference element as dot product of coefficients and basisvalues
      // Correct values by the covariant Piola transform
      const double tmp0_0 = new_coeff0_0*basisvalue0 + new_coeff0_1*basisvalue1 + new_coeff0_2*basisvalue2 + new_coeff0_3*basisvalue3;
      const double tmp0_1 = new_coeff1_0*basisvalue0 + new_coeff1_1*basisvalue1 + new_coeff1_2*basisvalue2 + new_coeff1_3*basisvalue3;
      const double tmp0_2 = new_coeff2_0*basisvalue0 + new_coeff2_1*basisvalue1 + new_coeff2_2*basisvalue2 + new_coeff2_3*basisvalue3;
      derivatives[deriv_num] = (Jinv_00*tmp0_0 + Jinv_10*tmp0_1 + Jinv_20*tmp0_2);
      derivatives[num_derivatives + deriv_num] = (Jinv_01*tmp0_0 + Jinv_11*tmp0_1 + Jinv_21*tmp0_2);
      derivatives[2*num_derivatives + deriv_num] = (Jinv_02*tmp0_0 + Jinv_12*tmp0_1 + Jinv_22*tmp0_2);
    }
    
    // Transform derivatives back to physical element
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        values[row] += transform[row][col]*derivatives[col];
        values[num_derivatives + row] += transform[row][col]*derivatives[num_derivatives + col];
        values[2*num_derivatives + row] += transform[row][col]*derivatives[2*num_derivatives + col];
      }
    }
    // Delete pointer to array of derivatives on FIAT element
    delete [] derivatives;
    
    // Delete pointer to array of combinations of derivatives and transform
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      delete [] combinations[row];
      delete [] transform[row];
    }
    
    delete [] combinations;
    delete [] transform;
  }

  /// Evaluate order n derivatives of all basis functions at given point in cell
  virtual void evaluate_basis_derivatives_all(unsigned int n,
                                              double* values,
                                              const double* coordinates,
                                              const ufc::cell& c) const
  {
    throw std::runtime_error("The vectorised version of evaluate_basis_derivatives() is not yet implemented.");
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    // The reference points, direction and weights:
    const static double X[6][1][3] = {{{0, 0.5, 0.5}}, {{0.5, 0, 0.5}}, {{0.5, 0.5, 0}}, {{0, 0, 0.5}}, {{0, 0.5, 0}}, {{0.5, 0, 0}}};
    const static double W[6][1] = {{1}, {1}, {1}, {1}, {1}, {1}};
    const static double D[6][1][3] = {{{0, -1, 1}}, {{-1, 0, 1}}, {{-1, 1, 0}}, {{0, 0, 1}}, {{0, 1, 0}}, {{1, 0, 0}}};
    
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = x[1][0] - x[0][0];
    const double J_01 = x[2][0] - x[0][0];
    const double J_02 = x[3][0] - x[0][0];
    const double J_10 = x[1][1] - x[0][1];
    const double J_11 = x[2][1] - x[0][1];
    const double J_12 = x[3][1] - x[0][1];
    const double J_20 = x[1][2] - x[0][2];
    const double J_21 = x[2][2] - x[0][2];
    const double J_22 = x[3][2] - x[0][2];
      
    // Compute sub determinants
    const double d_00 = J_11*J_22 - J_12*J_21;
    const double d_01 = J_12*J_20 - J_10*J_22;
    const double d_02 = J_10*J_21 - J_11*J_20;
    
    const double d_10 = J_02*J_21 - J_01*J_22;
    const double d_11 = J_00*J_22 - J_02*J_20;
    const double d_12 = J_01*J_20 - J_00*J_21;
    
    const double d_20 = J_01*J_12 - J_02*J_11;
    const double d_21 = J_02*J_10 - J_00*J_12;
    const double d_22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d_00 + J_10*d_10 + J_20*d_20;
      
    // Compute inverse of Jacobian
    const double Jinv_00 = d_00 / detJ;
    const double Jinv_01 = d_10 / detJ;
    const double Jinv_02 = d_20 / detJ;
    const double Jinv_10 = d_01 / detJ;
    const double Jinv_11 = d_11 / detJ;
    const double Jinv_12 = d_21 / detJ;
    const double Jinv_20 = d_02 / detJ;
    const double Jinv_21 = d_12 / detJ;
    const double Jinv_22 = d_22 / detJ;
    
    double copyofvalues[3];
    double result = 0.0;
    // Iterate over the points:
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0][0] - X[i][0][1] - X[i][0][2];
    const double w1 = X[i][0][0];
    const double w2 = X[i][0][1];
    const double w3 = X[i][0][2];
    
    // Compute affine mapping y = F(X)
    double y[3];
    y[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0] + w3*x[3][0];
    y[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1] + w3*x[3][1];
    y[2] = w0*x[0][2] + w1*x[1][2] + w2*x[2][2] + w3*x[3][2];
    
    // Evaluate function at physical points
    double values[3];
    f.evaluate(values, y, c);
    
    // Map function values using appropriate mapping
    // Copy old values:
    copyofvalues[0] = values[0];
    copyofvalues[1] = values[1];
    copyofvalues[2] = values[2];
    // Do the inverse of curl piola 
    values[0] = J_00*copyofvalues[0]+J_10*copyofvalues[1]+J_20*copyofvalues[2];
    values[1] = J_01*copyofvalues[0]+J_11*copyofvalues[1]+J_21*copyofvalues[2];
    values[2] = J_02*copyofvalues[0]+J_12*copyofvalues[1]+J_22*copyofvalues[2];
    
    // Note that we do not map the weights (yet).
    
    // Take directional components
    for(int k = 0; k < 3; k++)
      result += values[k]*D[i][0][k];
    // Multiply by weights 
    result *= W[i][0];
    
    return result;
  }

  /// Evaluate linear functionals for all dofs on the function f
  virtual void evaluate_dofs(double* values,
                             const ufc::function& f,
                             const ufc::cell& c) const
  {
    throw std::runtime_error("Not implemented (introduced in UFC v1.1).");
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = x[1][0] - x[0][0];
    const double J_01 = x[2][0] - x[0][0];
    const double J_02 = x[3][0] - x[0][0];
    const double J_10 = x[1][1] - x[0][1];
    const double J_11 = x[2][1] - x[0][1];
    const double J_12 = x[3][1] - x[0][1];
    const double J_20 = x[1][2] - x[0][2];
    const double J_21 = x[2][2] - x[0][2];
    const double J_22 = x[3][2] - x[0][2];
      
    // Compute sub determinants
    const double d_00 = J_11*J_22 - J_12*J_21;
    const double d_01 = J_12*J_20 - J_10*J_22;
    const double d_02 = J_10*J_21 - J_11*J_20;
    
    const double d_10 = J_02*J_21 - J_01*J_22;
    const double d_11 = J_00*J_22 - J_02*J_20;
    const double d_12 = J_01*J_20 - J_00*J_21;
    
    const double d_20 = J_01*J_12 - J_02*J_11;
    const double d_21 = J_02*J_10 - J_00*J_12;
    const double d_22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d_00 + J_10*d_10 + J_20*d_20;
      
    // Compute inverse of Jacobian
    const double Jinv_00 = d_00 / detJ;
    const double Jinv_01 = d_10 / detJ;
    const double Jinv_02 = d_20 / detJ;
    const double Jinv_10 = d_01 / detJ;
    const double Jinv_11 = d_11 / detJ;
    const double Jinv_12 = d_21 / detJ;
    const double Jinv_20 = d_02 / detJ;
    const double Jinv_21 = d_12 / detJ;
    const double Jinv_22 = d_22 / detJ;
    // Evaluate at vertices and use Piola mapping
    vertex_values[0] = (dof_values[3]*Jinv_20 + dof_values[4]*Jinv_10 + dof_values[5]*Jinv_00);
    vertex_values[1] = (dof_values[1]*Jinv_20 + dof_values[2]*Jinv_10 + dof_values[5]*(Jinv_00 + Jinv_10 + Jinv_20));
    vertex_values[2] = (dof_values[0]*Jinv_20 + dof_values[2]*Jinv_00 + dof_values[4]*(Jinv_00 + Jinv_10 + Jinv_20));
    vertex_values[3] = (dof_values[0]*Jinv_10 + dof_values[1]*Jinv_00 + dof_values[3]*(Jinv_00 + Jinv_10 + Jinv_20));
    vertex_values[4] = (dof_values[3]*Jinv_21 + dof_values[4]*Jinv_11 + dof_values[5]*Jinv_01);
    vertex_values[5] = (dof_values[1]*Jinv_21 + dof_values[2]*Jinv_11 + dof_values[5]*(Jinv_01 + Jinv_11 + Jinv_21));
    vertex_values[6] = (dof_values[0]*Jinv_21 + dof_values[2]*Jinv_01 + dof_values[4]*(Jinv_01 + Jinv_11 + Jinv_21));
    vertex_values[7] = (dof_values[0]*Jinv_11 + dof_values[1]*Jinv_01 + dof_values[3]*(Jinv_01 + Jinv_11 + Jinv_21));
    vertex_values[8] = (dof_values[3]*Jinv_22 + dof_values[4]*Jinv_12 + dof_values[5]*Jinv_02);
    vertex_values[9] = (dof_values[1]*Jinv_22 + dof_values[2]*Jinv_12 + dof_values[5]*(Jinv_02 + Jinv_12 + Jinv_22));
    vertex_values[10] = (dof_values[0]*Jinv_22 + dof_values[2]*Jinv_02 + dof_values[4]*(Jinv_02 + Jinv_12 + Jinv_22));
    vertex_values[11] = (dof_values[0]*Jinv_12 + dof_values[1]*Jinv_02 + dof_values[3]*(Jinv_02 + Jinv_12 + Jinv_22));
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 1;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    return new VectorLaplaceGradCurlBilinearForm_finite_element_1_1();
  }

};

/// This class defines the interface for a finite element.

class VectorLaplaceGradCurlBilinearForm_finite_element_1: public ufc::finite_element
{
public:

  /// Constructor
  VectorLaplaceGradCurlBilinearForm_finite_element_1() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~VectorLaplaceGradCurlBilinearForm_finite_element_1()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "Mixed finite element: [Lagrange finite element of degree 1 on a tetrahedron, Nedelec finite element of degree 1 on a tetrahedron]";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::tetrahedron;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 10;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 1;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 4;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    const double Jinv_00 = d00 / detJ;
    const double Jinv_01 = d10 / detJ;
    const double Jinv_02 = d20 / detJ;
    const double Jinv_10 = d01 / detJ;
    const double Jinv_11 = d11 / detJ;
    const double Jinv_12 = d21 / detJ;
    const double Jinv_20 = d02 / detJ;
    const double Jinv_21 = d12 / detJ;
    const double Jinv_22 = d22 / detJ;
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-09)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-09)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Reset values
    values[0] = 0;
    values[1] = 0;
    values[2] = 0;
    values[3] = 0;
    
    if (0 <= i && i <= 3)
    {
      // Map degree of freedom to element degree of freedom
      const unsigned int dof = i;
    
      // Generate scalings
      const double scalings_y_0 = 1;
      const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
      const double scalings_z_0 = 1;
      const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    
      // Compute psitilde_a
      const double psitilde_a_0 = 1;
      const double psitilde_a_1 = x;
    
      // Compute psitilde_bs
      const double psitilde_bs_0_0 = 1;
      const double psitilde_bs_0_1 = 1.5*y + 0.5;
      const double psitilde_bs_1_0 = 1;
    
      // Compute psitilde_cs
      const double psitilde_cs_00_0 = 1;
      const double psitilde_cs_00_1 = 2*z + 1;
      const double psitilde_cs_01_0 = 1;
      const double psitilde_cs_10_0 = 1;
    
      // Compute basisvalues
      const double basisvalue0 = 0.8660254038*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
      const double basisvalue1 = 2.738612788*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
      const double basisvalue2 = 1.58113883*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
      const double basisvalue3 = 1.118033989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    
      // Table(s) of coefficients
      const static double coefficients0[4][4] =   \
      {{0.2886751346, -0.1825741858, -0.1054092553, -0.07453559925},
      {0.2886751346, 0.1825741858, -0.1054092553, -0.07453559925},
      {0.2886751346, 0, 0.2108185107, -0.07453559925},
      {0.2886751346, 0, 0, 0.2236067977}};
    
      // Extract relevant coefficients
      const double coeff0_0 =   coefficients0[dof][0];
      const double coeff0_1 =   coefficients0[dof][1];
      const double coeff0_2 =   coefficients0[dof][2];
      const double coeff0_3 =   coefficients0[dof][3];
    
      // Compute value(s)
      values[0] = coeff0_0*basisvalue0 + coeff0_1*basisvalue1 + coeff0_2*basisvalue2 + coeff0_3*basisvalue3;
    }
    
    if (4 <= i && i <= 9)
    {
      // Map degree of freedom to element degree of freedom
      const unsigned int dof = i - 4;
    
      // Generate scalings
      const double scalings_y_0 = 1;
      const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
      const double scalings_z_0 = 1;
      const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    
      // Compute psitilde_a
      const double psitilde_a_0 = 1;
      const double psitilde_a_1 = x;
    
      // Compute psitilde_bs
      const double psitilde_bs_0_0 = 1;
      const double psitilde_bs_0_1 = 1.5*y + 0.5;
      const double psitilde_bs_1_0 = 1;
    
      // Compute psitilde_cs
      const double psitilde_cs_00_0 = 1;
      const double psitilde_cs_00_1 = 2*z + 1;
      const double psitilde_cs_01_0 = 1;
      const double psitilde_cs_10_0 = 1;
    
      // Compute basisvalues
      const double basisvalue0 = 0.8660254038*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
      const double basisvalue1 = 2.738612788*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
      const double basisvalue2 = 1.58113883*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
      const double basisvalue3 = 1.118033989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    
      // Table(s) of coefficients
      const static double coefficients0[6][4] =   \
      {{0, 0, 0, 0},
      {-0.2886751346, 0, 0, -0.2236067977},
      {-0.2886751346, 0, -0.2108185107, 0.07453559925},
      {0.2886751346, 0, 0, 0.2236067977},
      {0.2886751346, 0, 0.2108185107, -0.07453559925},
      {0.5773502692, 0, -0.2108185107, -0.1490711985}};
    
      const static double coefficients1[6][4] =   \
      {{-0.2886751346, 0, 0, -0.2236067977},
      {0, 0, 0, 0},
      {0.2886751346, 0.1825741858, -0.1054092553, -0.07453559925},
      {0.2886751346, 0, 0, 0.2236067977},
      {0.5773502692, -0.1825741858, 0.1054092553, -0.1490711985},
      {0.2886751346, 0.1825741858, -0.1054092553, -0.07453559925}};
    
      const static double coefficients2[6][4] =   \
      {{0.2886751346, 0, 0.2108185107, -0.07453559925},
      {0.2886751346, 0.1825741858, -0.1054092553, -0.07453559925},
      {0, 0, 0, 0},
      {0.5773502692, -0.1825741858, -0.1054092553, 0.1490711985},
      {0.2886751346, 0, 0.2108185107, -0.07453559925},
      {0.2886751346, 0.1825741858, -0.1054092553, -0.07453559925}};
    
      // Extract relevant coefficients
      const double coeff0_0 =   coefficients0[dof][0];
      const double coeff0_1 =   coefficients0[dof][1];
      const double coeff0_2 =   coefficients0[dof][2];
      const double coeff0_3 =   coefficients0[dof][3];
      const double coeff1_0 =   coefficients1[dof][0];
      const double coeff1_1 =   coefficients1[dof][1];
      const double coeff1_2 =   coefficients1[dof][2];
      const double coeff1_3 =   coefficients1[dof][3];
      const double coeff2_0 =   coefficients2[dof][0];
      const double coeff2_1 =   coefficients2[dof][1];
      const double coeff2_2 =   coefficients2[dof][2];
      const double coeff2_3 =   coefficients2[dof][3];
    
      // Compute value(s)
      const double tmp0_0 = coeff0_0*basisvalue0 + coeff0_1*basisvalue1 + coeff0_2*basisvalue2 + coeff0_3*basisvalue3;
      const double tmp0_1 = coeff1_0*basisvalue0 + coeff1_1*basisvalue1 + coeff1_2*basisvalue2 + coeff1_3*basisvalue3;
      const double tmp0_2 = coeff2_0*basisvalue0 + coeff2_1*basisvalue1 + coeff2_2*basisvalue2 + coeff2_3*basisvalue3;
      // Using covariant Piola transform to map values back to the physical element
      values[1] = (Jinv_00*tmp0_0 + Jinv_10*tmp0_1 + Jinv_20*tmp0_2);
      values[2] = (Jinv_01*tmp0_0 + Jinv_11*tmp0_1 + Jinv_21*tmp0_2);
      values[3] = (Jinv_02*tmp0_0 + Jinv_12*tmp0_1 + Jinv_22*tmp0_2);
    }
    
  }

  /// Evaluate all basis functions at given point in cell
  virtual void evaluate_basis_all(double* values,
                                  const double* coordinates,
                                  const ufc::cell& c) const
  {
    throw std::runtime_error("The vectorised version of evaluate_basis() is not yet implemented.");
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    const double Jinv_00 = d00 / detJ;
    const double Jinv_01 = d10 / detJ;
    const double Jinv_02 = d20 / detJ;
    const double Jinv_10 = d01 / detJ;
    const double Jinv_11 = d11 / detJ;
    const double Jinv_12 = d21 / detJ;
    const double Jinv_20 = d02 / detJ;
    const double Jinv_21 = d12 / detJ;
    const double Jinv_22 = d22 / detJ;
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-09)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-09)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Compute number of derivatives
    unsigned int num_derivatives = 1;
    
    for (unsigned int j = 0; j < n; j++)
      num_derivatives *= 3;
    
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      combinations[j] = new unsigned int [n];
      for (unsigned int k = 0; k < n; k++)
        combinations[j][k] = 0;
    }
        
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 2)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian
    const double Jinv[3][3] ={{d00 / detJ, d10 / detJ, d20 / detJ}, {d01 / detJ, d11 / detJ, d21 / detJ}, {d02 / detJ, d12 / detJ, d22 / detJ}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values
    for (unsigned int j = 0; j < 4*num_derivatives; j++)
      values[j] = 0;
    
    if (0 <= i && i <= 3)
    {
      // Map degree of freedom to element degree of freedom
      const unsigned int dof = i;
    
      // Generate scalings
      const double scalings_y_0 = 1;
      const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
      const double scalings_z_0 = 1;
      const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    
      // Compute psitilde_a
      const double psitilde_a_0 = 1;
      const double psitilde_a_1 = x;
    
      // Compute psitilde_bs
      const double psitilde_bs_0_0 = 1;
      const double psitilde_bs_0_1 = 1.5*y + 0.5;
      const double psitilde_bs_1_0 = 1;
    
      // Compute psitilde_cs
      const double psitilde_cs_00_0 = 1;
      const double psitilde_cs_00_1 = 2*z + 1;
      const double psitilde_cs_01_0 = 1;
      const double psitilde_cs_10_0 = 1;
    
      // Compute basisvalues
      const double basisvalue0 = 0.8660254038*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
      const double basisvalue1 = 2.738612788*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
      const double basisvalue2 = 1.58113883*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
      const double basisvalue3 = 1.118033989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    
      // Table(s) of coefficients
      const static double coefficients0[4][4] =   \
      {{0.2886751346, -0.1825741858, -0.1054092553, -0.07453559925},
      {0.2886751346, 0.1825741858, -0.1054092553, -0.07453559925},
      {0.2886751346, 0, 0.2108185107, -0.07453559925},
      {0.2886751346, 0, 0, 0.2236067977}};
    
      // Interesting (new) part
      // Tables of derivatives of the polynomial base (transpose)
      const static double dmats0[4][4] =   \
      {{0, 0, 0, 0},
      {6.32455532, 0, 0, 0},
      {0, 0, 0, 0},
      {0, 0, 0, 0}};
    
      const static double dmats1[4][4] =   \
      {{0, 0, 0, 0},
      {3.16227766, 0, 0, 0},
      {5.477225575, 0, 0, 0},
      {0, 0, 0, 0}};
    
      const static double dmats2[4][4] =   \
      {{0, 0, 0, 0},
      {3.16227766, 0, 0, 0},
      {1.825741858, 0, 0, 0},
      {5.163977795, 0, 0, 0}};
    
      // Compute reference derivatives
      // Declare pointer to array of derivatives on FIAT element
      double *derivatives = new double [num_derivatives];
    
      // Declare coefficients
      double coeff0_0 = 0;
      double coeff0_1 = 0;
      double coeff0_2 = 0;
      double coeff0_3 = 0;
    
      // Declare new coefficients
      double new_coeff0_0 = 0;
      double new_coeff0_1 = 0;
      double new_coeff0_2 = 0;
      double new_coeff0_3 = 0;
    
      // Loop possible derivatives
      for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
      {
        // Get values from coefficients array
        new_coeff0_0 = coefficients0[dof][0];
        new_coeff0_1 = coefficients0[dof][1];
        new_coeff0_2 = coefficients0[dof][2];
        new_coeff0_3 = coefficients0[dof][3];
    
        // Loop derivative order
        for (unsigned int j = 0; j < n; j++)
        {
          // Update old coefficients
          coeff0_0 = new_coeff0_0;
          coeff0_1 = new_coeff0_1;
          coeff0_2 = new_coeff0_2;
          coeff0_3 = new_coeff0_3;
    
          if(combinations[deriv_num][j] == 0)
          {
            new_coeff0_0 = coeff0_0*dmats0[0][0] + coeff0_1*dmats0[1][0] + coeff0_2*dmats0[2][0] + coeff0_3*dmats0[3][0];
            new_coeff0_1 = coeff0_0*dmats0[0][1] + coeff0_1*dmats0[1][1] + coeff0_2*dmats0[2][1] + coeff0_3*dmats0[3][1];
            new_coeff0_2 = coeff0_0*dmats0[0][2] + coeff0_1*dmats0[1][2] + coeff0_2*dmats0[2][2] + coeff0_3*dmats0[3][2];
            new_coeff0_3 = coeff0_0*dmats0[0][3] + coeff0_1*dmats0[1][3] + coeff0_2*dmats0[2][3] + coeff0_3*dmats0[3][3];
          }
          if(combinations[deriv_num][j] == 1)
          {
            new_coeff0_0 = coeff0_0*dmats1[0][0] + coeff0_1*dmats1[1][0] + coeff0_2*dmats1[2][0] + coeff0_3*dmats1[3][0];
            new_coeff0_1 = coeff0_0*dmats1[0][1] + coeff0_1*dmats1[1][1] + coeff0_2*dmats1[2][1] + coeff0_3*dmats1[3][1];
            new_coeff0_2 = coeff0_0*dmats1[0][2] + coeff0_1*dmats1[1][2] + coeff0_2*dmats1[2][2] + coeff0_3*dmats1[3][2];
            new_coeff0_3 = coeff0_0*dmats1[0][3] + coeff0_1*dmats1[1][3] + coeff0_2*dmats1[2][3] + coeff0_3*dmats1[3][3];
          }
          if(combinations[deriv_num][j] == 2)
          {
            new_coeff0_0 = coeff0_0*dmats2[0][0] + coeff0_1*dmats2[1][0] + coeff0_2*dmats2[2][0] + coeff0_3*dmats2[3][0];
            new_coeff0_1 = coeff0_0*dmats2[0][1] + coeff0_1*dmats2[1][1] + coeff0_2*dmats2[2][1] + coeff0_3*dmats2[3][1];
            new_coeff0_2 = coeff0_0*dmats2[0][2] + coeff0_1*dmats2[1][2] + coeff0_2*dmats2[2][2] + coeff0_3*dmats2[3][2];
            new_coeff0_3 = coeff0_0*dmats2[0][3] + coeff0_1*dmats2[1][3] + coeff0_2*dmats2[2][3] + coeff0_3*dmats2[3][3];
          }
    
        }
        // Compute derivatives on reference element as dot product of coefficients and basisvalues
        derivatives[deriv_num] = new_coeff0_0*basisvalue0 + new_coeff0_1*basisvalue1 + new_coeff0_2*basisvalue2 + new_coeff0_3*basisvalue3;
      }
    
      // Transform derivatives back to physical element
      for (unsigned int row = 0; row < num_derivatives; row++)
      {
        for (unsigned int col = 0; col < num_derivatives; col++)
        {
          values[row] += transform[row][col]*derivatives[col];
        }
      }
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
    
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int row = 0; row < num_derivatives; row++)
      {
        delete [] combinations[row];
        delete [] transform[row];
      }
    
      delete [] combinations;
      delete [] transform;
    }
    
    if (4 <= i && i <= 9)
    {
      // Map degree of freedom to element degree of freedom
      const unsigned int dof = i - 4;
    
      // Generate scalings
      const double scalings_y_0 = 1;
      const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
      const double scalings_z_0 = 1;
      const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    
      // Compute psitilde_a
      const double psitilde_a_0 = 1;
      const double psitilde_a_1 = x;
    
      // Compute psitilde_bs
      const double psitilde_bs_0_0 = 1;
      const double psitilde_bs_0_1 = 1.5*y + 0.5;
      const double psitilde_bs_1_0 = 1;
    
      // Compute psitilde_cs
      const double psitilde_cs_00_0 = 1;
      const double psitilde_cs_00_1 = 2*z + 1;
      const double psitilde_cs_01_0 = 1;
      const double psitilde_cs_10_0 = 1;
    
      // Compute basisvalues
      const double basisvalue0 = 0.8660254038*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
      const double basisvalue1 = 2.738612788*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
      const double basisvalue2 = 1.58113883*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
      const double basisvalue3 = 1.118033989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    
      // Table(s) of coefficients
      const static double coefficients0[6][4] =   \
      {{0, 0, 0, 0},
      {-0.2886751346, 0, 0, -0.2236067977},
      {-0.2886751346, 0, -0.2108185107, 0.07453559925},
      {0.2886751346, 0, 0, 0.2236067977},
      {0.2886751346, 0, 0.2108185107, -0.07453559925},
      {0.5773502692, 0, -0.2108185107, -0.1490711985}};
    
      const static double coefficients1[6][4] =   \
      {{-0.2886751346, 0, 0, -0.2236067977},
      {0, 0, 0, 0},
      {0.2886751346, 0.1825741858, -0.1054092553, -0.07453559925},
      {0.2886751346, 0, 0, 0.2236067977},
      {0.5773502692, -0.1825741858, 0.1054092553, -0.1490711985},
      {0.2886751346, 0.1825741858, -0.1054092553, -0.07453559925}};
    
      const static double coefficients2[6][4] =   \
      {{0.2886751346, 0, 0.2108185107, -0.07453559925},
      {0.2886751346, 0.1825741858, -0.1054092553, -0.07453559925},
      {0, 0, 0, 0},
      {0.5773502692, -0.1825741858, -0.1054092553, 0.1490711985},
      {0.2886751346, 0, 0.2108185107, -0.07453559925},
      {0.2886751346, 0.1825741858, -0.1054092553, -0.07453559925}};
    
      // Interesting (new) part
      // Tables of derivatives of the polynomial base (transpose)
      const static double dmats0[4][4] =   \
      {{0, 0, 0, 0},
      {6.32455532, 0, 0, 0},
      {0, 0, 0, 0},
      {0, 0, 0, 0}};
    
      const static double dmats1[4][4] =   \
      {{0, 0, 0, 0},
      {3.16227766, 0, 0, 0},
      {5.477225575, 0, 0, 0},
      {0, 0, 0, 0}};
    
      const static double dmats2[4][4] =   \
      {{0, 0, 0, 0},
      {3.16227766, 0, 0, 0},
      {1.825741858, 0, 0, 0},
      {5.163977795, 0, 0, 0}};
    
      // Compute reference derivatives
      // Declare pointer to array of derivatives on FIAT element
      double *derivatives = new double [3*num_derivatives];
    
      // Declare coefficients
      double coeff0_0 = 0;
      double coeff0_1 = 0;
      double coeff0_2 = 0;
      double coeff0_3 = 0;
      double coeff1_0 = 0;
      double coeff1_1 = 0;
      double coeff1_2 = 0;
      double coeff1_3 = 0;
      double coeff2_0 = 0;
      double coeff2_1 = 0;
      double coeff2_2 = 0;
      double coeff2_3 = 0;
    
      // Declare new coefficients
      double new_coeff0_0 = 0;
      double new_coeff0_1 = 0;
      double new_coeff0_2 = 0;
      double new_coeff0_3 = 0;
      double new_coeff1_0 = 0;
      double new_coeff1_1 = 0;
      double new_coeff1_2 = 0;
      double new_coeff1_3 = 0;
      double new_coeff2_0 = 0;
      double new_coeff2_1 = 0;
      double new_coeff2_2 = 0;
      double new_coeff2_3 = 0;
    
      // Loop possible derivatives
      for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
      {
        // Get values from coefficients array
        new_coeff0_0 = coefficients0[dof][0];
        new_coeff0_1 = coefficients0[dof][1];
        new_coeff0_2 = coefficients0[dof][2];
        new_coeff0_3 = coefficients0[dof][3];
        new_coeff1_0 = coefficients1[dof][0];
        new_coeff1_1 = coefficients1[dof][1];
        new_coeff1_2 = coefficients1[dof][2];
        new_coeff1_3 = coefficients1[dof][3];
        new_coeff2_0 = coefficients2[dof][0];
        new_coeff2_1 = coefficients2[dof][1];
        new_coeff2_2 = coefficients2[dof][2];
        new_coeff2_3 = coefficients2[dof][3];
    
        // Loop derivative order
        for (unsigned int j = 0; j < n; j++)
        {
          // Update old coefficients
          coeff0_0 = new_coeff0_0;
          coeff0_1 = new_coeff0_1;
          coeff0_2 = new_coeff0_2;
          coeff0_3 = new_coeff0_3;
          coeff1_0 = new_coeff1_0;
          coeff1_1 = new_coeff1_1;
          coeff1_2 = new_coeff1_2;
          coeff1_3 = new_coeff1_3;
          coeff2_0 = new_coeff2_0;
          coeff2_1 = new_coeff2_1;
          coeff2_2 = new_coeff2_2;
          coeff2_3 = new_coeff2_3;
    
          if(combinations[deriv_num][j] == 0)
          {
            new_coeff0_0 = coeff0_0*dmats0[0][0] + coeff0_1*dmats0[1][0] + coeff0_2*dmats0[2][0] + coeff0_3*dmats0[3][0];
            new_coeff0_1 = coeff0_0*dmats0[0][1] + coeff0_1*dmats0[1][1] + coeff0_2*dmats0[2][1] + coeff0_3*dmats0[3][1];
            new_coeff0_2 = coeff0_0*dmats0[0][2] + coeff0_1*dmats0[1][2] + coeff0_2*dmats0[2][2] + coeff0_3*dmats0[3][2];
            new_coeff0_3 = coeff0_0*dmats0[0][3] + coeff0_1*dmats0[1][3] + coeff0_2*dmats0[2][3] + coeff0_3*dmats0[3][3];
            new_coeff1_0 = coeff1_0*dmats0[0][0] + coeff1_1*dmats0[1][0] + coeff1_2*dmats0[2][0] + coeff1_3*dmats0[3][0];
            new_coeff1_1 = coeff1_0*dmats0[0][1] + coeff1_1*dmats0[1][1] + coeff1_2*dmats0[2][1] + coeff1_3*dmats0[3][1];
            new_coeff1_2 = coeff1_0*dmats0[0][2] + coeff1_1*dmats0[1][2] + coeff1_2*dmats0[2][2] + coeff1_3*dmats0[3][2];
            new_coeff1_3 = coeff1_0*dmats0[0][3] + coeff1_1*dmats0[1][3] + coeff1_2*dmats0[2][3] + coeff1_3*dmats0[3][3];
            new_coeff2_0 = coeff2_0*dmats0[0][0] + coeff2_1*dmats0[1][0] + coeff2_2*dmats0[2][0] + coeff2_3*dmats0[3][0];
            new_coeff2_1 = coeff2_0*dmats0[0][1] + coeff2_1*dmats0[1][1] + coeff2_2*dmats0[2][1] + coeff2_3*dmats0[3][1];
            new_coeff2_2 = coeff2_0*dmats0[0][2] + coeff2_1*dmats0[1][2] + coeff2_2*dmats0[2][2] + coeff2_3*dmats0[3][2];
            new_coeff2_3 = coeff2_0*dmats0[0][3] + coeff2_1*dmats0[1][3] + coeff2_2*dmats0[2][3] + coeff2_3*dmats0[3][3];
          }
          if(combinations[deriv_num][j] == 1)
          {
            new_coeff0_0 = coeff0_0*dmats1[0][0] + coeff0_1*dmats1[1][0] + coeff0_2*dmats1[2][0] + coeff0_3*dmats1[3][0];
            new_coeff0_1 = coeff0_0*dmats1[0][1] + coeff0_1*dmats1[1][1] + coeff0_2*dmats1[2][1] + coeff0_3*dmats1[3][1];
            new_coeff0_2 = coeff0_0*dmats1[0][2] + coeff0_1*dmats1[1][2] + coeff0_2*dmats1[2][2] + coeff0_3*dmats1[3][2];
            new_coeff0_3 = coeff0_0*dmats1[0][3] + coeff0_1*dmats1[1][3] + coeff0_2*dmats1[2][3] + coeff0_3*dmats1[3][3];
            new_coeff1_0 = coeff1_0*dmats1[0][0] + coeff1_1*dmats1[1][0] + coeff1_2*dmats1[2][0] + coeff1_3*dmats1[3][0];
            new_coeff1_1 = coeff1_0*dmats1[0][1] + coeff1_1*dmats1[1][1] + coeff1_2*dmats1[2][1] + coeff1_3*dmats1[3][1];
            new_coeff1_2 = coeff1_0*dmats1[0][2] + coeff1_1*dmats1[1][2] + coeff1_2*dmats1[2][2] + coeff1_3*dmats1[3][2];
            new_coeff1_3 = coeff1_0*dmats1[0][3] + coeff1_1*dmats1[1][3] + coeff1_2*dmats1[2][3] + coeff1_3*dmats1[3][3];
            new_coeff2_0 = coeff2_0*dmats1[0][0] + coeff2_1*dmats1[1][0] + coeff2_2*dmats1[2][0] + coeff2_3*dmats1[3][0];
            new_coeff2_1 = coeff2_0*dmats1[0][1] + coeff2_1*dmats1[1][1] + coeff2_2*dmats1[2][1] + coeff2_3*dmats1[3][1];
            new_coeff2_2 = coeff2_0*dmats1[0][2] + coeff2_1*dmats1[1][2] + coeff2_2*dmats1[2][2] + coeff2_3*dmats1[3][2];
            new_coeff2_3 = coeff2_0*dmats1[0][3] + coeff2_1*dmats1[1][3] + coeff2_2*dmats1[2][3] + coeff2_3*dmats1[3][3];
          }
          if(combinations[deriv_num][j] == 2)
          {
            new_coeff0_0 = coeff0_0*dmats2[0][0] + coeff0_1*dmats2[1][0] + coeff0_2*dmats2[2][0] + coeff0_3*dmats2[3][0];
            new_coeff0_1 = coeff0_0*dmats2[0][1] + coeff0_1*dmats2[1][1] + coeff0_2*dmats2[2][1] + coeff0_3*dmats2[3][1];
            new_coeff0_2 = coeff0_0*dmats2[0][2] + coeff0_1*dmats2[1][2] + coeff0_2*dmats2[2][2] + coeff0_3*dmats2[3][2];
            new_coeff0_3 = coeff0_0*dmats2[0][3] + coeff0_1*dmats2[1][3] + coeff0_2*dmats2[2][3] + coeff0_3*dmats2[3][3];
            new_coeff1_0 = coeff1_0*dmats2[0][0] + coeff1_1*dmats2[1][0] + coeff1_2*dmats2[2][0] + coeff1_3*dmats2[3][0];
            new_coeff1_1 = coeff1_0*dmats2[0][1] + coeff1_1*dmats2[1][1] + coeff1_2*dmats2[2][1] + coeff1_3*dmats2[3][1];
            new_coeff1_2 = coeff1_0*dmats2[0][2] + coeff1_1*dmats2[1][2] + coeff1_2*dmats2[2][2] + coeff1_3*dmats2[3][2];
            new_coeff1_3 = coeff1_0*dmats2[0][3] + coeff1_1*dmats2[1][3] + coeff1_2*dmats2[2][3] + coeff1_3*dmats2[3][3];
            new_coeff2_0 = coeff2_0*dmats2[0][0] + coeff2_1*dmats2[1][0] + coeff2_2*dmats2[2][0] + coeff2_3*dmats2[3][0];
            new_coeff2_1 = coeff2_0*dmats2[0][1] + coeff2_1*dmats2[1][1] + coeff2_2*dmats2[2][1] + coeff2_3*dmats2[3][1];
            new_coeff2_2 = coeff2_0*dmats2[0][2] + coeff2_1*dmats2[1][2] + coeff2_2*dmats2[2][2] + coeff2_3*dmats2[3][2];
            new_coeff2_3 = coeff2_0*dmats2[0][3] + coeff2_1*dmats2[1][3] + coeff2_2*dmats2[2][3] + coeff2_3*dmats2[3][3];
          }
    
        }
        // Compute derivatives on reference element as dot product of coefficients and basisvalues
        // Correct values by the covariant Piola transform
        const double tmp0_0 = new_coeff0_0*basisvalue0 + new_coeff0_1*basisvalue1 + new_coeff0_2*basisvalue2 + new_coeff0_3*basisvalue3;
        const double tmp0_1 = new_coeff1_0*basisvalue0 + new_coeff1_1*basisvalue1 + new_coeff1_2*basisvalue2 + new_coeff1_3*basisvalue3;
        const double tmp0_2 = new_coeff2_0*basisvalue0 + new_coeff2_1*basisvalue1 + new_coeff2_2*basisvalue2 + new_coeff2_3*basisvalue3;
        derivatives[deriv_num] = (Jinv_00*tmp0_0 + Jinv_10*tmp0_1 + Jinv_20*tmp0_2);
        derivatives[num_derivatives + deriv_num] = (Jinv_01*tmp0_0 + Jinv_11*tmp0_1 + Jinv_21*tmp0_2);
        derivatives[2*num_derivatives + deriv_num] = (Jinv_02*tmp0_0 + Jinv_12*tmp0_1 + Jinv_22*tmp0_2);
      }
    
      // Transform derivatives back to physical element
      for (unsigned int row = 0; row < num_derivatives; row++)
      {
        for (unsigned int col = 0; col < num_derivatives; col++)
        {
          values[num_derivatives + row] += transform[row][col]*derivatives[col];
          values[2*num_derivatives + row] += transform[row][col]*derivatives[num_derivatives + col];
          values[3*num_derivatives + row] += transform[row][col]*derivatives[2*num_derivatives + col];
        }
      }
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
    
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int row = 0; row < num_derivatives; row++)
      {
        delete [] combinations[row];
        delete [] transform[row];
      }
    
      delete [] combinations;
      delete [] transform;
    }
    
  }

  /// Evaluate order n derivatives of all basis functions at given point in cell
  virtual void evaluate_basis_derivatives_all(unsigned int n,
                                              double* values,
                                              const double* coordinates,
                                              const ufc::cell& c) const
  {
    throw std::runtime_error("The vectorised version of evaluate_basis_derivatives() is not yet implemented.");
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    // The reference points, direction and weights:
    const static double X[10][1][3] = {{{0, 0, 0}}, {{1, 0, 0}}, {{0, 1, 0}}, {{0, 0, 1}}, {{0, 0.5, 0.5}}, {{0.5, 0, 0.5}}, {{0.5, 0.5, 0}}, {{0, 0, 0.5}}, {{0, 0.5, 0}}, {{0.5, 0, 0}}};
    const static double W[10][1] = {{1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}};
    const static double D[10][1][4] = {{{1, 0, 0, 0}}, {{1, 0, 0, 0}}, {{1, 0, 0, 0}}, {{1, 0, 0, 0}}, {{0, 0, -1, 1}}, {{0, -1, 0, 1}}, {{0, -1, 1, 0}}, {{0, 0, 0, 1}}, {{0, 0, 1, 0}}, {{0, 1, 0, 0}}};
    
    static const unsigned int mappings[10] = {0, 0, 0, 0, 2, 2, 2, 2, 2, 2};
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = x[1][0] - x[0][0];
    const double J_01 = x[2][0] - x[0][0];
    const double J_02 = x[3][0] - x[0][0];
    const double J_10 = x[1][1] - x[0][1];
    const double J_11 = x[2][1] - x[0][1];
    const double J_12 = x[3][1] - x[0][1];
    const double J_20 = x[1][2] - x[0][2];
    const double J_21 = x[2][2] - x[0][2];
    const double J_22 = x[3][2] - x[0][2];
      
    // Compute sub determinants
    const double d_00 = J_11*J_22 - J_12*J_21;
    const double d_01 = J_12*J_20 - J_10*J_22;
    const double d_02 = J_10*J_21 - J_11*J_20;
    
    const double d_10 = J_02*J_21 - J_01*J_22;
    const double d_11 = J_00*J_22 - J_02*J_20;
    const double d_12 = J_01*J_20 - J_00*J_21;
    
    const double d_20 = J_01*J_12 - J_02*J_11;
    const double d_21 = J_02*J_10 - J_00*J_12;
    const double d_22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d_00 + J_10*d_10 + J_20*d_20;
      
    // Compute inverse of Jacobian
    const double Jinv_00 = d_00 / detJ;
    const double Jinv_01 = d_10 / detJ;
    const double Jinv_02 = d_20 / detJ;
    const double Jinv_10 = d_01 / detJ;
    const double Jinv_11 = d_11 / detJ;
    const double Jinv_12 = d_21 / detJ;
    const double Jinv_20 = d_02 / detJ;
    const double Jinv_21 = d_12 / detJ;
    const double Jinv_22 = d_22 / detJ;
    
    double copyofvalues[4];
    const int offsets[10] = {0, 0, 0, 0, 1, 1, 1, 1, 1, 1};
    double result = 0.0;
    // Iterate over the points:
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0][0] - X[i][0][1] - X[i][0][2];
    const double w1 = X[i][0][0];
    const double w2 = X[i][0][1];
    const double w3 = X[i][0][2];
    
    // Compute affine mapping y = F(X)
    double y[3];
    y[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0] + w3*x[3][0];
    y[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1] + w3*x[3][1];
    y[2] = w0*x[0][2] + w1*x[1][2] + w2*x[2][2] + w3*x[3][2];
    
    // Evaluate function at physical points
    double values[4];
    f.evaluate(values, y, c);
    
    // Map function values using appropriate mapping
    if (mappings[i] == 0) { 
      // Affine map: Do nothing
    } else if (mappings[i] == 2) {
       // Copy old values:
      copyofvalues[offsets[i] + 0] = values[offsets[i] + 0];
      copyofvalues[offsets[i] + 1] = values[offsets[i] + 1];
      copyofvalues[offsets[i] + 2] = values[offsets[i] + 2];
      // Do the inverse of curl piola 
      values[offsets[i] + 0] = J_00*copyofvalues[offsets[i] + 0]+J_10*copyofvalues[offsets[i] + 1]+J_20*copyofvalues[offsets[i] + 2];
      values[offsets[i] + 1] = J_01*copyofvalues[offsets[i] + 0]+J_11*copyofvalues[offsets[i] + 1]+J_21*copyofvalues[offsets[i] + 2];
      values[offsets[i] + 2] = J_02*copyofvalues[offsets[i] + 0]+J_12*copyofvalues[offsets[i] + 1]+J_22*copyofvalues[offsets[i] + 2]; 
    } else { 
       // Other mappings not applicable. 
    }
    
    // Note that we do not map the weights (yet).
    
    // Take directional components
    for(int k = 0; k < 4; k++)
      result += values[k]*D[i][0][k];
    // Multiply by weights 
    result *= W[i][0];
    
    return result;
  }

  /// Evaluate linear functionals for all dofs on the function f
  virtual void evaluate_dofs(double* values,
                             const ufc::function& f,
                             const ufc::cell& c) const
  {
    throw std::runtime_error("Not implemented (introduced in UFC v1.1).");
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = x[1][0] - x[0][0];
    const double J_01 = x[2][0] - x[0][0];
    const double J_02 = x[3][0] - x[0][0];
    const double J_10 = x[1][1] - x[0][1];
    const double J_11 = x[2][1] - x[0][1];
    const double J_12 = x[3][1] - x[0][1];
    const double J_20 = x[1][2] - x[0][2];
    const double J_21 = x[2][2] - x[0][2];
    const double J_22 = x[3][2] - x[0][2];
      
    // Compute sub determinants
    const double d_00 = J_11*J_22 - J_12*J_21;
    const double d_01 = J_12*J_20 - J_10*J_22;
    const double d_02 = J_10*J_21 - J_11*J_20;
    
    const double d_10 = J_02*J_21 - J_01*J_22;
    const double d_11 = J_00*J_22 - J_02*J_20;
    const double d_12 = J_01*J_20 - J_00*J_21;
    
    const double d_20 = J_01*J_12 - J_02*J_11;
    const double d_21 = J_02*J_10 - J_00*J_12;
    const double d_22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d_00 + J_10*d_10 + J_20*d_20;
      
    // Compute inverse of Jacobian
    const double Jinv_00 = d_00 / detJ;
    const double Jinv_01 = d_10 / detJ;
    const double Jinv_02 = d_20 / detJ;
    const double Jinv_10 = d_01 / detJ;
    const double Jinv_11 = d_11 / detJ;
    const double Jinv_12 = d_21 / detJ;
    const double Jinv_20 = d_02 / detJ;
    const double Jinv_21 = d_12 / detJ;
    const double Jinv_22 = d_22 / detJ;
    // Evaluate at vertices and use affine mapping
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[1];
    vertex_values[2] = dof_values[2];
    vertex_values[3] = dof_values[3];
    // Evaluate at vertices and use Piola mapping
    vertex_values[4] = (dof_values[7]*Jinv_20 + dof_values[8]*Jinv_10 + dof_values[9]*Jinv_00);
    vertex_values[5] = (dof_values[5]*Jinv_20 + dof_values[6]*Jinv_10 + dof_values[9]*(Jinv_00 + Jinv_10 + Jinv_20));
    vertex_values[6] = (dof_values[4]*Jinv_20 + dof_values[6]*Jinv_00 + dof_values[8]*(Jinv_00 + Jinv_10 + Jinv_20));
    vertex_values[7] = (dof_values[4]*Jinv_10 + dof_values[5]*Jinv_00 + dof_values[7]*(Jinv_00 + Jinv_10 + Jinv_20));
    vertex_values[8] = (dof_values[7]*Jinv_21 + dof_values[8]*Jinv_11 + dof_values[9]*Jinv_01);
    vertex_values[9] = (dof_values[5]*Jinv_21 + dof_values[6]*Jinv_11 + dof_values[9]*(Jinv_01 + Jinv_11 + Jinv_21));
    vertex_values[10] = (dof_values[4]*Jinv_21 + dof_values[6]*Jinv_01 + dof_values[8]*(Jinv_01 + Jinv_11 + Jinv_21));
    vertex_values[11] = (dof_values[4]*Jinv_11 + dof_values[5]*Jinv_01 + dof_values[7]*(Jinv_01 + Jinv_11 + Jinv_21));
    vertex_values[12] = (dof_values[7]*Jinv_22 + dof_values[8]*Jinv_12 + dof_values[9]*Jinv_02);
    vertex_values[13] = (dof_values[5]*Jinv_22 + dof_values[6]*Jinv_12 + dof_values[9]*(Jinv_02 + Jinv_12 + Jinv_22));
    vertex_values[14] = (dof_values[4]*Jinv_22 + dof_values[6]*Jinv_02 + dof_values[8]*(Jinv_02 + Jinv_12 + Jinv_22));
    vertex_values[15] = (dof_values[4]*Jinv_12 + dof_values[5]*Jinv_02 + dof_values[7]*(Jinv_02 + Jinv_12 + Jinv_22));
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 2;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    switch ( i )
    {
    case 0:
      return new VectorLaplaceGradCurlBilinearForm_finite_element_1_0();
      break;
    case 1:
      return new VectorLaplaceGradCurlBilinearForm_finite_element_1_1();
      break;
    }
    return 0;
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class VectorLaplaceGradCurlBilinearForm_dof_map_0_0: public ufc::dof_map
{
private:

  unsigned int __global_dimension;

public:

  /// Constructor
  VectorLaplaceGradCurlBilinearForm_dof_map_0_0() : ufc::dof_map()
  {
    __global_dimension = 0;
  }

  /// Destructor
  virtual ~VectorLaplaceGradCurlBilinearForm_dof_map_0_0()
  {
    // Do nothing
  }

  /// Return a string identifying the dof map
  virtual const char* signature() const
  {
    return "FFC dof map for Lagrange finite element of degree 1 on a tetrahedron";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch ( d )
    {
    case 0:
      return true;
      break;
    case 1:
      return false;
      break;
    case 2:
      return false;
      break;
    case 3:
      return false;
      break;
    }
    return false;
  }

  /// Initialize dof map for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    __global_dimension = m.num_entities[0];
    return false;
  }

  /// Initialize dof map for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dof map for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return __global_dimension;
  }

  /// Return the dimension of the local finite element function space
  virtual unsigned int local_dimension() const
  {
    return 4;
  }

  // Return the geometric dimension of the coordinates this dof map provides
  virtual unsigned int geometric_dimension() const
  {
    throw std::runtime_error("Not implemented (introduced in UFC v1.1).");
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 3;
  }

  /// Return the number of dofs associated with each cell entity of dimension d
  virtual unsigned int num_entity_dofs(unsigned int d) const
  {
    throw std::runtime_error("Not implemented (introduced in UFC v1.1).");
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    dofs[0] = c.entity_indices[0][0];
    dofs[1] = c.entity_indices[0][1];
    dofs[2] = c.entity_indices[0][2];
    dofs[3] = c.entity_indices[0][3];
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch ( facet )
    {
    case 0:
      dofs[0] = 1;
      dofs[1] = 2;
      dofs[2] = 3;
      break;
    case 1:
      dofs[0] = 0;
      dofs[1] = 2;
      dofs[2] = 3;
      break;
    case 2:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 3;
      break;
    case 3:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 2;
      break;
    }
  }

  /// Tabulate the local-to-local mapping of dofs on entity (d, i)
  virtual void tabulate_entity_dofs(unsigned int* dofs,
                                    unsigned int d, unsigned int i) const
  {
    throw std::runtime_error("Not implemented (introduced in UFC v1.1).");
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    coordinates[0][0] = x[0][0];
    coordinates[0][1] = x[0][1];
    coordinates[0][2] = x[0][2];
    coordinates[1][0] = x[1][0];
    coordinates[1][1] = x[1][1];
    coordinates[1][2] = x[1][2];
    coordinates[2][0] = x[2][0];
    coordinates[2][1] = x[2][1];
    coordinates[2][2] = x[2][2];
    coordinates[3][0] = x[3][0];
    coordinates[3][1] = x[3][1];
    coordinates[3][2] = x[3][2];
  }

  /// Return the number of sub dof maps (for a mixed element)
  virtual unsigned int num_sub_dof_maps() const
  {
    return 1;
  }

  /// Create a new dof_map for sub dof map i (for a mixed element)
  virtual ufc::dof_map* create_sub_dof_map(unsigned int i) const
  {
    return new VectorLaplaceGradCurlBilinearForm_dof_map_0_0();
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class VectorLaplaceGradCurlBilinearForm_dof_map_0_1: public ufc::dof_map
{
private:

  unsigned int __global_dimension;

public:

  /// Constructor
  VectorLaplaceGradCurlBilinearForm_dof_map_0_1() : ufc::dof_map()
  {
    __global_dimension = 0;
  }

  /// Destructor
  virtual ~VectorLaplaceGradCurlBilinearForm_dof_map_0_1()
  {
    // Do nothing
  }

  /// Return a string identifying the dof map
  virtual const char* signature() const
  {
    return "FFC dof map for Nedelec finite element of degree 1 on a tetrahedron";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch ( d )
    {
    case 0:
      return false;
      break;
    case 1:
      return true;
      break;
    case 2:
      return false;
      break;
    case 3:
      return false;
      break;
    }
    return false;
  }

  /// Initialize dof map for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    __global_dimension = m.num_entities[1];
    return false;
  }

  /// Initialize dof map for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dof map for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return __global_dimension;
  }

  /// Return the dimension of the local finite element function space
  virtual unsigned int local_dimension() const
  {
    return 6;
  }

  // Return the geometric dimension of the coordinates this dof map provides
  virtual unsigned int geometric_dimension() const
  {
    throw std::runtime_error("Not implemented (introduced in UFC v1.1).");
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 3;
  }

  /// Return the number of dofs associated with each cell entity of dimension d
  virtual unsigned int num_entity_dofs(unsigned int d) const
  {
    throw std::runtime_error("Not implemented (introduced in UFC v1.1).");
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    dofs[0] = c.entity_indices[1][0];
    dofs[1] = c.entity_indices[1][1];
    dofs[2] = c.entity_indices[1][2];
    dofs[3] = c.entity_indices[1][3];
    dofs[4] = c.entity_indices[1][4];
    dofs[5] = c.entity_indices[1][5];
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch ( facet )
    {
    case 0:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 2;
      break;
    case 1:
      dofs[0] = 0;
      dofs[1] = 3;
      dofs[2] = 4;
      break;
    case 2:
      dofs[0] = 1;
      dofs[1] = 3;
      dofs[2] = 5;
      break;
    case 3:
      dofs[0] = 2;
      dofs[1] = 4;
      dofs[2] = 5;
      break;
    }
  }

  /// Tabulate the local-to-local mapping of dofs on entity (d, i)
  virtual void tabulate_entity_dofs(unsigned int* dofs,
                                    unsigned int d, unsigned int i) const
  {
    throw std::runtime_error("Not implemented (introduced in UFC v1.1).");
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    coordinates[0][0] = 0.5*x[2][0] + 0.5*x[3][0];
    coordinates[0][1] = 0.5*x[2][1] + 0.5*x[3][1];
    coordinates[0][2] = 0.5*x[2][2] + 0.5*x[3][2];
    coordinates[1][0] = 0.5*x[1][0] + 0.5*x[3][0];
    coordinates[1][1] = 0.5*x[1][1] + 0.5*x[3][1];
    coordinates[1][2] = 0.5*x[1][2] + 0.5*x[3][2];
    coordinates[2][0] = 0.5*x[1][0] + 0.5*x[2][0];
    coordinates[2][1] = 0.5*x[1][1] + 0.5*x[2][1];
    coordinates[2][2] = 0.5*x[1][2] + 0.5*x[2][2];
    coordinates[3][0] = 0.5*x[0][0] + 0.5*x[3][0];
    coordinates[3][1] = 0.5*x[0][1] + 0.5*x[3][1];
    coordinates[3][2] = 0.5*x[0][2] + 0.5*x[3][2];
    coordinates[4][0] = 0.5*x[0][0] + 0.5*x[2][0];
    coordinates[4][1] = 0.5*x[0][1] + 0.5*x[2][1];
    coordinates[4][2] = 0.5*x[0][2] + 0.5*x[2][2];
    coordinates[5][0] = 0.5*x[0][0] + 0.5*x[1][0];
    coordinates[5][1] = 0.5*x[0][1] + 0.5*x[1][1];
    coordinates[5][2] = 0.5*x[0][2] + 0.5*x[1][2];
  }

  /// Return the number of sub dof maps (for a mixed element)
  virtual unsigned int num_sub_dof_maps() const
  {
    return 1;
  }

  /// Create a new dof_map for sub dof map i (for a mixed element)
  virtual ufc::dof_map* create_sub_dof_map(unsigned int i) const
  {
    return new VectorLaplaceGradCurlBilinearForm_dof_map_0_1();
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class VectorLaplaceGradCurlBilinearForm_dof_map_0: public ufc::dof_map
{
private:

  unsigned int __global_dimension;

public:

  /// Constructor
  VectorLaplaceGradCurlBilinearForm_dof_map_0() : ufc::dof_map()
  {
    __global_dimension = 0;
  }

  /// Destructor
  virtual ~VectorLaplaceGradCurlBilinearForm_dof_map_0()
  {
    // Do nothing
  }

  /// Return a string identifying the dof map
  virtual const char* signature() const
  {
    return "FFC dof map for Mixed finite element: [Lagrange finite element of degree 1 on a tetrahedron, Nedelec finite element of degree 1 on a tetrahedron]";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch ( d )
    {
    case 0:
      return true;
      break;
    case 1:
      return true;
      break;
    case 2:
      return false;
      break;
    case 3:
      return false;
      break;
    }
    return false;
  }

  /// Initialize dof map for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    __global_dimension = m.num_entities[0] + m.num_entities[1];
    return false;
  }

  /// Initialize dof map for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dof map for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return __global_dimension;
  }

  /// Return the dimension of the local finite element function space
  virtual unsigned int local_dimension() const
  {
    return 10;
  }

  // Return the geometric dimension of the coordinates this dof map provides
  virtual unsigned int geometric_dimension() const
  {
    throw std::runtime_error("Not implemented (introduced in UFC v1.1).");
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 6;
  }

  /// Return the number of dofs associated with each cell entity of dimension d
  virtual unsigned int num_entity_dofs(unsigned int d) const
  {
    throw std::runtime_error("Not implemented (introduced in UFC v1.1).");
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    dofs[0] = c.entity_indices[0][0];
    dofs[1] = c.entity_indices[0][1];
    dofs[2] = c.entity_indices[0][2];
    dofs[3] = c.entity_indices[0][3];
    unsigned int offset = m.num_entities[0];
    dofs[4] = offset + c.entity_indices[1][0];
    dofs[5] = offset + c.entity_indices[1][1];
    dofs[6] = offset + c.entity_indices[1][2];
    dofs[7] = offset + c.entity_indices[1][3];
    dofs[8] = offset + c.entity_indices[1][4];
    dofs[9] = offset + c.entity_indices[1][5];
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch ( facet )
    {
    case 0:
      dofs[0] = 1;
      dofs[1] = 2;
      dofs[2] = 3;
      dofs[3] = 4;
      dofs[4] = 5;
      dofs[5] = 6;
      break;
    case 1:
      dofs[0] = 0;
      dofs[1] = 2;
      dofs[2] = 3;
      dofs[3] = 4;
      dofs[4] = 7;
      dofs[5] = 8;
      break;
    case 2:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 3;
      dofs[3] = 5;
      dofs[4] = 7;
      dofs[5] = 9;
      break;
    case 3:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 2;
      dofs[3] = 6;
      dofs[4] = 8;
      dofs[5] = 9;
      break;
    }
  }

  /// Tabulate the local-to-local mapping of dofs on entity (d, i)
  virtual void tabulate_entity_dofs(unsigned int* dofs,
                                    unsigned int d, unsigned int i) const
  {
    throw std::runtime_error("Not implemented (introduced in UFC v1.1).");
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    coordinates[0][0] = x[0][0];
    coordinates[0][1] = x[0][1];
    coordinates[0][2] = x[0][2];
    coordinates[1][0] = x[1][0];
    coordinates[1][1] = x[1][1];
    coordinates[1][2] = x[1][2];
    coordinates[2][0] = x[2][0];
    coordinates[2][1] = x[2][1];
    coordinates[2][2] = x[2][2];
    coordinates[3][0] = x[3][0];
    coordinates[3][1] = x[3][1];
    coordinates[3][2] = x[3][2];
    coordinates[4][0] = 0.5*x[2][0] + 0.5*x[3][0];
    coordinates[4][1] = 0.5*x[2][1] + 0.5*x[3][1];
    coordinates[4][2] = 0.5*x[2][2] + 0.5*x[3][2];
    coordinates[5][0] = 0.5*x[1][0] + 0.5*x[3][0];
    coordinates[5][1] = 0.5*x[1][1] + 0.5*x[3][1];
    coordinates[5][2] = 0.5*x[1][2] + 0.5*x[3][2];
    coordinates[6][0] = 0.5*x[1][0] + 0.5*x[2][0];
    coordinates[6][1] = 0.5*x[1][1] + 0.5*x[2][1];
    coordinates[6][2] = 0.5*x[1][2] + 0.5*x[2][2];
    coordinates[7][0] = 0.5*x[0][0] + 0.5*x[3][0];
    coordinates[7][1] = 0.5*x[0][1] + 0.5*x[3][1];
    coordinates[7][2] = 0.5*x[0][2] + 0.5*x[3][2];
    coordinates[8][0] = 0.5*x[0][0] + 0.5*x[2][0];
    coordinates[8][1] = 0.5*x[0][1] + 0.5*x[2][1];
    coordinates[8][2] = 0.5*x[0][2] + 0.5*x[2][2];
    coordinates[9][0] = 0.5*x[0][0] + 0.5*x[1][0];
    coordinates[9][1] = 0.5*x[0][1] + 0.5*x[1][1];
    coordinates[9][2] = 0.5*x[0][2] + 0.5*x[1][2];
  }

  /// Return the number of sub dof maps (for a mixed element)
  virtual unsigned int num_sub_dof_maps() const
  {
    return 2;
  }

  /// Create a new dof_map for sub dof map i (for a mixed element)
  virtual ufc::dof_map* create_sub_dof_map(unsigned int i) const
  {
    switch ( i )
    {
    case 0:
      return new VectorLaplaceGradCurlBilinearForm_dof_map_0_0();
      break;
    case 1:
      return new VectorLaplaceGradCurlBilinearForm_dof_map_0_1();
      break;
    }
    return 0;
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class VectorLaplaceGradCurlBilinearForm_dof_map_1_0: public ufc::dof_map
{
private:

  unsigned int __global_dimension;

public:

  /// Constructor
  VectorLaplaceGradCurlBilinearForm_dof_map_1_0() : ufc::dof_map()
  {
    __global_dimension = 0;
  }

  /// Destructor
  virtual ~VectorLaplaceGradCurlBilinearForm_dof_map_1_0()
  {
    // Do nothing
  }

  /// Return a string identifying the dof map
  virtual const char* signature() const
  {
    return "FFC dof map for Lagrange finite element of degree 1 on a tetrahedron";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch ( d )
    {
    case 0:
      return true;
      break;
    case 1:
      return false;
      break;
    case 2:
      return false;
      break;
    case 3:
      return false;
      break;
    }
    return false;
  }

  /// Initialize dof map for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    __global_dimension = m.num_entities[0];
    return false;
  }

  /// Initialize dof map for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dof map for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return __global_dimension;
  }

  /// Return the dimension of the local finite element function space
  virtual unsigned int local_dimension() const
  {
    return 4;
  }

  // Return the geometric dimension of the coordinates this dof map provides
  virtual unsigned int geometric_dimension() const
  {
    throw std::runtime_error("Not implemented (introduced in UFC v1.1).");
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 3;
  }

  /// Return the number of dofs associated with each cell entity of dimension d
  virtual unsigned int num_entity_dofs(unsigned int d) const
  {
    throw std::runtime_error("Not implemented (introduced in UFC v1.1).");
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    dofs[0] = c.entity_indices[0][0];
    dofs[1] = c.entity_indices[0][1];
    dofs[2] = c.entity_indices[0][2];
    dofs[3] = c.entity_indices[0][3];
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch ( facet )
    {
    case 0:
      dofs[0] = 1;
      dofs[1] = 2;
      dofs[2] = 3;
      break;
    case 1:
      dofs[0] = 0;
      dofs[1] = 2;
      dofs[2] = 3;
      break;
    case 2:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 3;
      break;
    case 3:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 2;
      break;
    }
  }

  /// Tabulate the local-to-local mapping of dofs on entity (d, i)
  virtual void tabulate_entity_dofs(unsigned int* dofs,
                                    unsigned int d, unsigned int i) const
  {
    throw std::runtime_error("Not implemented (introduced in UFC v1.1).");
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    coordinates[0][0] = x[0][0];
    coordinates[0][1] = x[0][1];
    coordinates[0][2] = x[0][2];
    coordinates[1][0] = x[1][0];
    coordinates[1][1] = x[1][1];
    coordinates[1][2] = x[1][2];
    coordinates[2][0] = x[2][0];
    coordinates[2][1] = x[2][1];
    coordinates[2][2] = x[2][2];
    coordinates[3][0] = x[3][0];
    coordinates[3][1] = x[3][1];
    coordinates[3][2] = x[3][2];
  }

  /// Return the number of sub dof maps (for a mixed element)
  virtual unsigned int num_sub_dof_maps() const
  {
    return 1;
  }

  /// Create a new dof_map for sub dof map i (for a mixed element)
  virtual ufc::dof_map* create_sub_dof_map(unsigned int i) const
  {
    return new VectorLaplaceGradCurlBilinearForm_dof_map_1_0();
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class VectorLaplaceGradCurlBilinearForm_dof_map_1_1: public ufc::dof_map
{
private:

  unsigned int __global_dimension;

public:

  /// Constructor
  VectorLaplaceGradCurlBilinearForm_dof_map_1_1() : ufc::dof_map()
  {
    __global_dimension = 0;
  }

  /// Destructor
  virtual ~VectorLaplaceGradCurlBilinearForm_dof_map_1_1()
  {
    // Do nothing
  }

  /// Return a string identifying the dof map
  virtual const char* signature() const
  {
    return "FFC dof map for Nedelec finite element of degree 1 on a tetrahedron";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch ( d )
    {
    case 0:
      return false;
      break;
    case 1:
      return true;
      break;
    case 2:
      return false;
      break;
    case 3:
      return false;
      break;
    }
    return false;
  }

  /// Initialize dof map for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    __global_dimension = m.num_entities[1];
    return false;
  }

  /// Initialize dof map for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dof map for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return __global_dimension;
  }

  /// Return the dimension of the local finite element function space
  virtual unsigned int local_dimension() const
  {
    return 6;
  }

  // Return the geometric dimension of the coordinates this dof map provides
  virtual unsigned int geometric_dimension() const
  {
    throw std::runtime_error("Not implemented (introduced in UFC v1.1).");
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 3;
  }

  /// Return the number of dofs associated with each cell entity of dimension d
  virtual unsigned int num_entity_dofs(unsigned int d) const
  {
    throw std::runtime_error("Not implemented (introduced in UFC v1.1).");
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    dofs[0] = c.entity_indices[1][0];
    dofs[1] = c.entity_indices[1][1];
    dofs[2] = c.entity_indices[1][2];
    dofs[3] = c.entity_indices[1][3];
    dofs[4] = c.entity_indices[1][4];
    dofs[5] = c.entity_indices[1][5];
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch ( facet )
    {
    case 0:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 2;
      break;
    case 1:
      dofs[0] = 0;
      dofs[1] = 3;
      dofs[2] = 4;
      break;
    case 2:
      dofs[0] = 1;
      dofs[1] = 3;
      dofs[2] = 5;
      break;
    case 3:
      dofs[0] = 2;
      dofs[1] = 4;
      dofs[2] = 5;
      break;
    }
  }

  /// Tabulate the local-to-local mapping of dofs on entity (d, i)
  virtual void tabulate_entity_dofs(unsigned int* dofs,
                                    unsigned int d, unsigned int i) const
  {
    throw std::runtime_error("Not implemented (introduced in UFC v1.1).");
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    coordinates[0][0] = 0.5*x[2][0] + 0.5*x[3][0];
    coordinates[0][1] = 0.5*x[2][1] + 0.5*x[3][1];
    coordinates[0][2] = 0.5*x[2][2] + 0.5*x[3][2];
    coordinates[1][0] = 0.5*x[1][0] + 0.5*x[3][0];
    coordinates[1][1] = 0.5*x[1][1] + 0.5*x[3][1];
    coordinates[1][2] = 0.5*x[1][2] + 0.5*x[3][2];
    coordinates[2][0] = 0.5*x[1][0] + 0.5*x[2][0];
    coordinates[2][1] = 0.5*x[1][1] + 0.5*x[2][1];
    coordinates[2][2] = 0.5*x[1][2] + 0.5*x[2][2];
    coordinates[3][0] = 0.5*x[0][0] + 0.5*x[3][0];
    coordinates[3][1] = 0.5*x[0][1] + 0.5*x[3][1];
    coordinates[3][2] = 0.5*x[0][2] + 0.5*x[3][2];
    coordinates[4][0] = 0.5*x[0][0] + 0.5*x[2][0];
    coordinates[4][1] = 0.5*x[0][1] + 0.5*x[2][1];
    coordinates[4][2] = 0.5*x[0][2] + 0.5*x[2][2];
    coordinates[5][0] = 0.5*x[0][0] + 0.5*x[1][0];
    coordinates[5][1] = 0.5*x[0][1] + 0.5*x[1][1];
    coordinates[5][2] = 0.5*x[0][2] + 0.5*x[1][2];
  }

  /// Return the number of sub dof maps (for a mixed element)
  virtual unsigned int num_sub_dof_maps() const
  {
    return 1;
  }

  /// Create a new dof_map for sub dof map i (for a mixed element)
  virtual ufc::dof_map* create_sub_dof_map(unsigned int i) const
  {
    return new VectorLaplaceGradCurlBilinearForm_dof_map_1_1();
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class VectorLaplaceGradCurlBilinearForm_dof_map_1: public ufc::dof_map
{
private:

  unsigned int __global_dimension;

public:

  /// Constructor
  VectorLaplaceGradCurlBilinearForm_dof_map_1() : ufc::dof_map()
  {
    __global_dimension = 0;
  }

  /// Destructor
  virtual ~VectorLaplaceGradCurlBilinearForm_dof_map_1()
  {
    // Do nothing
  }

  /// Return a string identifying the dof map
  virtual const char* signature() const
  {
    return "FFC dof map for Mixed finite element: [Lagrange finite element of degree 1 on a tetrahedron, Nedelec finite element of degree 1 on a tetrahedron]";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch ( d )
    {
    case 0:
      return true;
      break;
    case 1:
      return true;
      break;
    case 2:
      return false;
      break;
    case 3:
      return false;
      break;
    }
    return false;
  }

  /// Initialize dof map for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    __global_dimension = m.num_entities[0] + m.num_entities[1];
    return false;
  }

  /// Initialize dof map for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dof map for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return __global_dimension;
  }

  /// Return the dimension of the local finite element function space
  virtual unsigned int local_dimension() const
  {
    return 10;
  }

  // Return the geometric dimension of the coordinates this dof map provides
  virtual unsigned int geometric_dimension() const
  {
    throw std::runtime_error("Not implemented (introduced in UFC v1.1).");
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 6;
  }

  /// Return the number of dofs associated with each cell entity of dimension d
  virtual unsigned int num_entity_dofs(unsigned int d) const
  {
    throw std::runtime_error("Not implemented (introduced in UFC v1.1).");
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    dofs[0] = c.entity_indices[0][0];
    dofs[1] = c.entity_indices[0][1];
    dofs[2] = c.entity_indices[0][2];
    dofs[3] = c.entity_indices[0][3];
    unsigned int offset = m.num_entities[0];
    dofs[4] = offset + c.entity_indices[1][0];
    dofs[5] = offset + c.entity_indices[1][1];
    dofs[6] = offset + c.entity_indices[1][2];
    dofs[7] = offset + c.entity_indices[1][3];
    dofs[8] = offset + c.entity_indices[1][4];
    dofs[9] = offset + c.entity_indices[1][5];
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch ( facet )
    {
    case 0:
      dofs[0] = 1;
      dofs[1] = 2;
      dofs[2] = 3;
      dofs[3] = 4;
      dofs[4] = 5;
      dofs[5] = 6;
      break;
    case 1:
      dofs[0] = 0;
      dofs[1] = 2;
      dofs[2] = 3;
      dofs[3] = 4;
      dofs[4] = 7;
      dofs[5] = 8;
      break;
    case 2:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 3;
      dofs[3] = 5;
      dofs[4] = 7;
      dofs[5] = 9;
      break;
    case 3:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 2;
      dofs[3] = 6;
      dofs[4] = 8;
      dofs[5] = 9;
      break;
    }
  }

  /// Tabulate the local-to-local mapping of dofs on entity (d, i)
  virtual void tabulate_entity_dofs(unsigned int* dofs,
                                    unsigned int d, unsigned int i) const
  {
    throw std::runtime_error("Not implemented (introduced in UFC v1.1).");
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    coordinates[0][0] = x[0][0];
    coordinates[0][1] = x[0][1];
    coordinates[0][2] = x[0][2];
    coordinates[1][0] = x[1][0];
    coordinates[1][1] = x[1][1];
    coordinates[1][2] = x[1][2];
    coordinates[2][0] = x[2][0];
    coordinates[2][1] = x[2][1];
    coordinates[2][2] = x[2][2];
    coordinates[3][0] = x[3][0];
    coordinates[3][1] = x[3][1];
    coordinates[3][2] = x[3][2];
    coordinates[4][0] = 0.5*x[2][0] + 0.5*x[3][0];
    coordinates[4][1] = 0.5*x[2][1] + 0.5*x[3][1];
    coordinates[4][2] = 0.5*x[2][2] + 0.5*x[3][2];
    coordinates[5][0] = 0.5*x[1][0] + 0.5*x[3][0];
    coordinates[5][1] = 0.5*x[1][1] + 0.5*x[3][1];
    coordinates[5][2] = 0.5*x[1][2] + 0.5*x[3][2];
    coordinates[6][0] = 0.5*x[1][0] + 0.5*x[2][0];
    coordinates[6][1] = 0.5*x[1][1] + 0.5*x[2][1];
    coordinates[6][2] = 0.5*x[1][2] + 0.5*x[2][2];
    coordinates[7][0] = 0.5*x[0][0] + 0.5*x[3][0];
    coordinates[7][1] = 0.5*x[0][1] + 0.5*x[3][1];
    coordinates[7][2] = 0.5*x[0][2] + 0.5*x[3][2];
    coordinates[8][0] = 0.5*x[0][0] + 0.5*x[2][0];
    coordinates[8][1] = 0.5*x[0][1] + 0.5*x[2][1];
    coordinates[8][2] = 0.5*x[0][2] + 0.5*x[2][2];
    coordinates[9][0] = 0.5*x[0][0] + 0.5*x[1][0];
    coordinates[9][1] = 0.5*x[0][1] + 0.5*x[1][1];
    coordinates[9][2] = 0.5*x[0][2] + 0.5*x[1][2];
  }

  /// Return the number of sub dof maps (for a mixed element)
  virtual unsigned int num_sub_dof_maps() const
  {
    return 2;
  }

  /// Create a new dof_map for sub dof map i (for a mixed element)
  virtual ufc::dof_map* create_sub_dof_map(unsigned int i) const
  {
    switch ( i )
    {
    case 0:
      return new VectorLaplaceGradCurlBilinearForm_dof_map_1_0();
      break;
    case 1:
      return new VectorLaplaceGradCurlBilinearForm_dof_map_1_1();
      break;
    }
    return 0;
  }

};

/// This class defines the interface for the tabulation of the cell
/// tensor corresponding to the local contribution to a form from
/// the integral over a cell.

class VectorLaplaceGradCurlBilinearForm_cell_integral_0: public ufc::cell_integral
{
public:

  /// Constructor
  VectorLaplaceGradCurlBilinearForm_cell_integral_0() : ufc::cell_integral()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~VectorLaplaceGradCurlBilinearForm_cell_integral_0()
  {
    // Do nothing
  }

  /// Tabulate the tensor for the contribution from a local cell
  virtual void tabulate_tensor(double* A,
                               const double * const * w,
                               const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = x[1][0] - x[0][0];
    const double J_01 = x[2][0] - x[0][0];
    const double J_02 = x[3][0] - x[0][0];
    const double J_10 = x[1][1] - x[0][1];
    const double J_11 = x[2][1] - x[0][1];
    const double J_12 = x[3][1] - x[0][1];
    const double J_20 = x[1][2] - x[0][2];
    const double J_21 = x[2][2] - x[0][2];
    const double J_22 = x[3][2] - x[0][2];
      
    // Compute sub determinants
    const double d_00 = J_11*J_22 - J_12*J_21;
    const double d_01 = J_12*J_20 - J_10*J_22;
    const double d_02 = J_10*J_21 - J_11*J_20;
    
    const double d_10 = J_02*J_21 - J_01*J_22;
    const double d_11 = J_00*J_22 - J_02*J_20;
    const double d_12 = J_01*J_20 - J_00*J_21;
    
    const double d_20 = J_01*J_12 - J_02*J_11;
    const double d_21 = J_02*J_10 - J_00*J_12;
    const double d_22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d_00 + J_10*d_10 + J_20*d_20;
      
    // Compute inverse of Jacobian
    const double Jinv_00 = d_00 / detJ;
    const double Jinv_01 = d_10 / detJ;
    const double Jinv_02 = d_20 / detJ;
    const double Jinv_10 = d_01 / detJ;
    const double Jinv_11 = d_11 / detJ;
    const double Jinv_12 = d_21 / detJ;
    const double Jinv_20 = d_02 / detJ;
    const double Jinv_21 = d_12 / detJ;
    const double Jinv_22 = d_22 / detJ;
    
    // Set scale factor
    const double det = std::abs(detJ);
    
    // Compute geometry tensors
    const double G0_ = det;
    const double G1_0_0 = det*(Jinv_00*Jinv_00 + Jinv_01*Jinv_01 + Jinv_02*Jinv_02);
    const double G1_0_1 = det*(Jinv_00*Jinv_10 + Jinv_01*Jinv_11 + Jinv_02*Jinv_12);
    const double G1_0_2 = det*(Jinv_00*Jinv_20 + Jinv_01*Jinv_21 + Jinv_02*Jinv_22);
    const double G1_1_0 = det*(Jinv_10*Jinv_00 + Jinv_11*Jinv_01 + Jinv_12*Jinv_02);
    const double G1_1_1 = det*(Jinv_10*Jinv_10 + Jinv_11*Jinv_11 + Jinv_12*Jinv_12);
    const double G1_1_2 = det*(Jinv_10*Jinv_20 + Jinv_11*Jinv_21 + Jinv_12*Jinv_22);
    const double G1_2_0 = det*(Jinv_20*Jinv_00 + Jinv_21*Jinv_01 + Jinv_22*Jinv_02);
    const double G1_2_1 = det*(Jinv_20*Jinv_10 + Jinv_21*Jinv_11 + Jinv_22*Jinv_12);
    const double G1_2_2 = det*(Jinv_20*Jinv_20 + Jinv_21*Jinv_21 + Jinv_22*Jinv_22);
    const double G2_0_0 = det*(Jinv_00*Jinv_00 + Jinv_01*Jinv_01 + Jinv_02*Jinv_02);
    const double G2_0_1 = det*(Jinv_00*Jinv_10 + Jinv_01*Jinv_11 + Jinv_02*Jinv_12);
    const double G2_0_2 = det*(Jinv_00*Jinv_20 + Jinv_01*Jinv_21 + Jinv_02*Jinv_22);
    const double G2_1_0 = det*(Jinv_10*Jinv_00 + Jinv_11*Jinv_01 + Jinv_12*Jinv_02);
    const double G2_1_1 = det*(Jinv_10*Jinv_10 + Jinv_11*Jinv_11 + Jinv_12*Jinv_12);
    const double G2_1_2 = det*(Jinv_10*Jinv_20 + Jinv_11*Jinv_21 + Jinv_12*Jinv_22);
    const double G2_2_0 = det*(Jinv_20*Jinv_00 + Jinv_21*Jinv_01 + Jinv_22*Jinv_02);
    const double G2_2_1 = det*(Jinv_20*Jinv_10 + Jinv_21*Jinv_11 + Jinv_22*Jinv_12);
    const double G2_2_2 = det*(Jinv_20*Jinv_20 + Jinv_21*Jinv_21 + Jinv_22*Jinv_22);
    const double G3_0_1_0_1 = det*(Jinv_02*Jinv_02*(Jinv_10*Jinv_10 + Jinv_11*Jinv_11) + Jinv_12*Jinv_12*(Jinv_00*Jinv_00 + Jinv_01*Jinv_01) + Jinv_01*Jinv_10*Jinv_01*Jinv_10 + Jinv_00*Jinv_11*Jinv_00*Jinv_11);
    const double G3_0_1_0_2 = det*(Jinv_02*Jinv_02*(Jinv_10*Jinv_20 + Jinv_11*Jinv_21) + Jinv_12*Jinv_22*(Jinv_00*Jinv_00 + Jinv_01*Jinv_01) + Jinv_01*Jinv_10*Jinv_01*Jinv_20 + Jinv_00*Jinv_11*Jinv_00*Jinv_21);
    const double G3_0_1_1_0 = det*(Jinv_02*Jinv_12*(Jinv_10*Jinv_00 + Jinv_11*Jinv_01) + Jinv_12*Jinv_02*(Jinv_00*Jinv_10 + Jinv_01*Jinv_11) + Jinv_01*Jinv_10*Jinv_11*Jinv_00 + Jinv_00*Jinv_11*Jinv_10*Jinv_01);
    const double G3_0_1_1_2 = det*(Jinv_02*Jinv_12*(Jinv_10*Jinv_20 + Jinv_11*Jinv_21) + Jinv_12*Jinv_22*(Jinv_00*Jinv_10 + Jinv_01*Jinv_11) + Jinv_01*Jinv_10*Jinv_11*Jinv_20 + Jinv_00*Jinv_11*Jinv_10*Jinv_21);
    const double G3_0_1_2_0 = det*(Jinv_02*Jinv_22*(Jinv_10*Jinv_00 + Jinv_11*Jinv_01) + Jinv_12*Jinv_02*(Jinv_00*Jinv_20 + Jinv_01*Jinv_21) + Jinv_01*Jinv_10*Jinv_21*Jinv_00 + Jinv_00*Jinv_11*Jinv_20*Jinv_01);
    const double G3_0_1_2_1 = det*(Jinv_02*Jinv_22*(Jinv_10*Jinv_10 + Jinv_11*Jinv_11) + Jinv_12*Jinv_12*(Jinv_00*Jinv_20 + Jinv_01*Jinv_21) + Jinv_01*Jinv_10*Jinv_21*Jinv_10 + Jinv_00*Jinv_11*Jinv_20*Jinv_11);
    const double G3_0_2_0_1 = det*(Jinv_02*Jinv_02*(Jinv_20*Jinv_10 + Jinv_21*Jinv_11) + Jinv_22*Jinv_12*(Jinv_00*Jinv_00 + Jinv_01*Jinv_01) + Jinv_01*Jinv_20*Jinv_01*Jinv_10 + Jinv_00*Jinv_21*Jinv_00*Jinv_11);
    const double G3_0_2_0_2 = det*(Jinv_02*Jinv_02*(Jinv_20*Jinv_20 + Jinv_21*Jinv_21) + Jinv_22*Jinv_22*(Jinv_00*Jinv_00 + Jinv_01*Jinv_01) + Jinv_01*Jinv_20*Jinv_01*Jinv_20 + Jinv_00*Jinv_21*Jinv_00*Jinv_21);
    const double G3_0_2_1_0 = det*(Jinv_02*Jinv_12*(Jinv_20*Jinv_00 + Jinv_21*Jinv_01) + Jinv_22*Jinv_02*(Jinv_00*Jinv_10 + Jinv_01*Jinv_11) + Jinv_01*Jinv_20*Jinv_11*Jinv_00 + Jinv_00*Jinv_21*Jinv_10*Jinv_01);
    const double G3_0_2_1_2 = det*(Jinv_02*Jinv_12*(Jinv_20*Jinv_20 + Jinv_21*Jinv_21) + Jinv_22*Jinv_22*(Jinv_00*Jinv_10 + Jinv_01*Jinv_11) + Jinv_01*Jinv_20*Jinv_11*Jinv_20 + Jinv_00*Jinv_21*Jinv_10*Jinv_21);
    const double G3_0_2_2_0 = det*(Jinv_02*Jinv_22*(Jinv_20*Jinv_00 + Jinv_21*Jinv_01) + Jinv_22*Jinv_02*(Jinv_00*Jinv_20 + Jinv_01*Jinv_21) + Jinv_01*Jinv_20*Jinv_21*Jinv_00 + Jinv_00*Jinv_21*Jinv_20*Jinv_01);
    const double G3_0_2_2_1 = det*(Jinv_02*Jinv_22*(Jinv_20*Jinv_10 + Jinv_21*Jinv_11) + Jinv_22*Jinv_12*(Jinv_00*Jinv_20 + Jinv_01*Jinv_21) + Jinv_01*Jinv_20*Jinv_21*Jinv_10 + Jinv_00*Jinv_21*Jinv_20*Jinv_11);
    const double G3_1_0_0_1 = det*(Jinv_12*Jinv_02*(Jinv_00*Jinv_10 + Jinv_01*Jinv_11) + Jinv_02*Jinv_12*(Jinv_10*Jinv_00 + Jinv_11*Jinv_01) + Jinv_11*Jinv_00*Jinv_01*Jinv_10 + Jinv_10*Jinv_01*Jinv_00*Jinv_11);
    const double G3_1_0_0_2 = det*(Jinv_12*Jinv_02*(Jinv_00*Jinv_20 + Jinv_01*Jinv_21) + Jinv_02*Jinv_22*(Jinv_10*Jinv_00 + Jinv_11*Jinv_01) + Jinv_11*Jinv_00*Jinv_01*Jinv_20 + Jinv_10*Jinv_01*Jinv_00*Jinv_21);
    const double G3_1_0_1_0 = det*(Jinv_12*Jinv_12*(Jinv_00*Jinv_00 + Jinv_01*Jinv_01) + Jinv_02*Jinv_02*(Jinv_10*Jinv_10 + Jinv_11*Jinv_11) + Jinv_11*Jinv_00*Jinv_11*Jinv_00 + Jinv_10*Jinv_01*Jinv_10*Jinv_01);
    const double G3_1_0_1_2 = det*(Jinv_12*Jinv_12*(Jinv_00*Jinv_20 + Jinv_01*Jinv_21) + Jinv_02*Jinv_22*(Jinv_10*Jinv_10 + Jinv_11*Jinv_11) + Jinv_11*Jinv_00*Jinv_11*Jinv_20 + Jinv_10*Jinv_01*Jinv_10*Jinv_21);
    const double G3_1_0_2_0 = det*(Jinv_12*Jinv_22*(Jinv_00*Jinv_00 + Jinv_01*Jinv_01) + Jinv_02*Jinv_02*(Jinv_10*Jinv_20 + Jinv_11*Jinv_21) + Jinv_11*Jinv_00*Jinv_21*Jinv_00 + Jinv_10*Jinv_01*Jinv_20*Jinv_01);
    const double G3_1_0_2_1 = det*(Jinv_12*Jinv_22*(Jinv_00*Jinv_10 + Jinv_01*Jinv_11) + Jinv_02*Jinv_12*(Jinv_10*Jinv_20 + Jinv_11*Jinv_21) + Jinv_11*Jinv_00*Jinv_21*Jinv_10 + Jinv_10*Jinv_01*Jinv_20*Jinv_11);
    const double G3_1_2_0_1 = det*(Jinv_12*Jinv_02*(Jinv_20*Jinv_10 + Jinv_21*Jinv_11) + Jinv_22*Jinv_12*(Jinv_10*Jinv_00 + Jinv_11*Jinv_01) + Jinv_11*Jinv_20*Jinv_01*Jinv_10 + Jinv_10*Jinv_21*Jinv_00*Jinv_11);
    const double G3_1_2_0_2 = det*(Jinv_12*Jinv_02*(Jinv_20*Jinv_20 + Jinv_21*Jinv_21) + Jinv_22*Jinv_22*(Jinv_10*Jinv_00 + Jinv_11*Jinv_01) + Jinv_11*Jinv_20*Jinv_01*Jinv_20 + Jinv_10*Jinv_21*Jinv_00*Jinv_21);
    const double G3_1_2_1_0 = det*(Jinv_12*Jinv_12*(Jinv_20*Jinv_00 + Jinv_21*Jinv_01) + Jinv_22*Jinv_02*(Jinv_10*Jinv_10 + Jinv_11*Jinv_11) + Jinv_11*Jinv_20*Jinv_11*Jinv_00 + Jinv_10*Jinv_21*Jinv_10*Jinv_01);
    const double G3_1_2_1_2 = det*(Jinv_12*Jinv_12*(Jinv_20*Jinv_20 + Jinv_21*Jinv_21) + Jinv_22*Jinv_22*(Jinv_10*Jinv_10 + Jinv_11*Jinv_11) + Jinv_11*Jinv_20*Jinv_11*Jinv_20 + Jinv_10*Jinv_21*Jinv_10*Jinv_21);
    const double G3_1_2_2_0 = det*(Jinv_12*Jinv_22*(Jinv_20*Jinv_00 + Jinv_21*Jinv_01) + Jinv_22*Jinv_02*(Jinv_10*Jinv_20 + Jinv_11*Jinv_21) + Jinv_11*Jinv_20*Jinv_21*Jinv_00 + Jinv_10*Jinv_21*Jinv_20*Jinv_01);
    const double G3_1_2_2_1 = det*(Jinv_12*Jinv_22*(Jinv_20*Jinv_10 + Jinv_21*Jinv_11) + Jinv_22*Jinv_12*(Jinv_10*Jinv_20 + Jinv_11*Jinv_21) + Jinv_11*Jinv_20*Jinv_21*Jinv_10 + Jinv_10*Jinv_21*Jinv_20*Jinv_11);
    const double G3_2_0_0_1 = det*(Jinv_22*Jinv_02*(Jinv_00*Jinv_10 + Jinv_01*Jinv_11) + Jinv_02*Jinv_12*(Jinv_20*Jinv_00 + Jinv_21*Jinv_01) + Jinv_21*Jinv_00*Jinv_01*Jinv_10 + Jinv_20*Jinv_01*Jinv_00*Jinv_11);
    const double G3_2_0_0_2 = det*(Jinv_22*Jinv_02*(Jinv_00*Jinv_20 + Jinv_01*Jinv_21) + Jinv_02*Jinv_22*(Jinv_20*Jinv_00 + Jinv_21*Jinv_01) + Jinv_21*Jinv_00*Jinv_01*Jinv_20 + Jinv_20*Jinv_01*Jinv_00*Jinv_21);
    const double G3_2_0_1_0 = det*(Jinv_22*Jinv_12*(Jinv_00*Jinv_00 + Jinv_01*Jinv_01) + Jinv_02*Jinv_02*(Jinv_20*Jinv_10 + Jinv_21*Jinv_11) + Jinv_21*Jinv_00*Jinv_11*Jinv_00 + Jinv_20*Jinv_01*Jinv_10*Jinv_01);
    const double G3_2_0_1_2 = det*(Jinv_22*Jinv_12*(Jinv_00*Jinv_20 + Jinv_01*Jinv_21) + Jinv_02*Jinv_22*(Jinv_20*Jinv_10 + Jinv_21*Jinv_11) + Jinv_21*Jinv_00*Jinv_11*Jinv_20 + Jinv_20*Jinv_01*Jinv_10*Jinv_21);
    const double G3_2_0_2_0 = det*(Jinv_22*Jinv_22*(Jinv_00*Jinv_00 + Jinv_01*Jinv_01) + Jinv_02*Jinv_02*(Jinv_20*Jinv_20 + Jinv_21*Jinv_21) + Jinv_21*Jinv_00*Jinv_21*Jinv_00 + Jinv_20*Jinv_01*Jinv_20*Jinv_01);
    const double G3_2_0_2_1 = det*(Jinv_22*Jinv_22*(Jinv_00*Jinv_10 + Jinv_01*Jinv_11) + Jinv_02*Jinv_12*(Jinv_20*Jinv_20 + Jinv_21*Jinv_21) + Jinv_21*Jinv_00*Jinv_21*Jinv_10 + Jinv_20*Jinv_01*Jinv_20*Jinv_11);
    const double G3_2_1_0_1 = det*(Jinv_22*Jinv_02*(Jinv_10*Jinv_10 + Jinv_11*Jinv_11) + Jinv_12*Jinv_12*(Jinv_20*Jinv_00 + Jinv_21*Jinv_01) + Jinv_21*Jinv_10*Jinv_01*Jinv_10 + Jinv_20*Jinv_11*Jinv_00*Jinv_11);
    const double G3_2_1_0_2 = det*(Jinv_22*Jinv_02*(Jinv_10*Jinv_20 + Jinv_11*Jinv_21) + Jinv_12*Jinv_22*(Jinv_20*Jinv_00 + Jinv_21*Jinv_01) + Jinv_21*Jinv_10*Jinv_01*Jinv_20 + Jinv_20*Jinv_11*Jinv_00*Jinv_21);
    const double G3_2_1_1_0 = det*(Jinv_22*Jinv_12*(Jinv_10*Jinv_00 + Jinv_11*Jinv_01) + Jinv_12*Jinv_02*(Jinv_20*Jinv_10 + Jinv_21*Jinv_11) + Jinv_21*Jinv_10*Jinv_11*Jinv_00 + Jinv_20*Jinv_11*Jinv_10*Jinv_01);
    const double G3_2_1_1_2 = det*(Jinv_22*Jinv_12*(Jinv_10*Jinv_20 + Jinv_11*Jinv_21) + Jinv_12*Jinv_22*(Jinv_20*Jinv_10 + Jinv_21*Jinv_11) + Jinv_21*Jinv_10*Jinv_11*Jinv_20 + Jinv_20*Jinv_11*Jinv_10*Jinv_21);
    const double G3_2_1_2_0 = det*(Jinv_22*Jinv_22*(Jinv_10*Jinv_00 + Jinv_11*Jinv_01) + Jinv_12*Jinv_02*(Jinv_20*Jinv_20 + Jinv_21*Jinv_21) + Jinv_21*Jinv_10*Jinv_21*Jinv_00 + Jinv_20*Jinv_11*Jinv_20*Jinv_01);
    const double G3_2_1_2_1 = det*(Jinv_22*Jinv_22*(Jinv_10*Jinv_10 + Jinv_11*Jinv_11) + Jinv_12*Jinv_12*(Jinv_20*Jinv_20 + Jinv_21*Jinv_21) + Jinv_21*Jinv_10*Jinv_21*Jinv_10 + Jinv_20*Jinv_11*Jinv_20*Jinv_11);
    const double G4_0_1_0_1 = det*(Jinv_02*Jinv_12*(Jinv_10*Jinv_00 + Jinv_11*Jinv_01) + Jinv_12*Jinv_02*(Jinv_00*Jinv_10 + Jinv_01*Jinv_11) + Jinv_01*Jinv_10*Jinv_00*Jinv_11 + Jinv_00*Jinv_11*Jinv_01*Jinv_10);
    const double G4_0_1_0_2 = det*(Jinv_02*Jinv_22*(Jinv_10*Jinv_00 + Jinv_11*Jinv_01) + Jinv_12*Jinv_02*(Jinv_00*Jinv_20 + Jinv_01*Jinv_21) + Jinv_01*Jinv_10*Jinv_00*Jinv_21 + Jinv_00*Jinv_11*Jinv_01*Jinv_20);
    const double G4_0_1_1_0 = det*(Jinv_02*Jinv_02*(Jinv_10*Jinv_10 + Jinv_11*Jinv_11) + Jinv_12*Jinv_12*(Jinv_00*Jinv_00 + Jinv_01*Jinv_01) + Jinv_01*Jinv_10*Jinv_10*Jinv_01 + Jinv_00*Jinv_11*Jinv_11*Jinv_00);
    const double G4_0_1_1_2 = det*(Jinv_02*Jinv_22*(Jinv_10*Jinv_10 + Jinv_11*Jinv_11) + Jinv_12*Jinv_12*(Jinv_00*Jinv_20 + Jinv_01*Jinv_21) + Jinv_01*Jinv_10*Jinv_10*Jinv_21 + Jinv_00*Jinv_11*Jinv_11*Jinv_20);
    const double G4_0_1_2_0 = det*(Jinv_02*Jinv_02*(Jinv_10*Jinv_20 + Jinv_11*Jinv_21) + Jinv_12*Jinv_22*(Jinv_00*Jinv_00 + Jinv_01*Jinv_01) + Jinv_01*Jinv_10*Jinv_20*Jinv_01 + Jinv_00*Jinv_11*Jinv_21*Jinv_00);
    const double G4_0_1_2_1 = det*(Jinv_02*Jinv_12*(Jinv_10*Jinv_20 + Jinv_11*Jinv_21) + Jinv_12*Jinv_22*(Jinv_00*Jinv_10 + Jinv_01*Jinv_11) + Jinv_01*Jinv_10*Jinv_20*Jinv_11 + Jinv_00*Jinv_11*Jinv_21*Jinv_10);
    const double G4_0_2_0_1 = det*(Jinv_02*Jinv_12*(Jinv_20*Jinv_00 + Jinv_21*Jinv_01) + Jinv_22*Jinv_02*(Jinv_00*Jinv_10 + Jinv_01*Jinv_11) + Jinv_01*Jinv_20*Jinv_00*Jinv_11 + Jinv_00*Jinv_21*Jinv_01*Jinv_10);
    const double G4_0_2_0_2 = det*(Jinv_02*Jinv_22*(Jinv_20*Jinv_00 + Jinv_21*Jinv_01) + Jinv_22*Jinv_02*(Jinv_00*Jinv_20 + Jinv_01*Jinv_21) + Jinv_01*Jinv_20*Jinv_00*Jinv_21 + Jinv_00*Jinv_21*Jinv_01*Jinv_20);
    const double G4_0_2_1_0 = det*(Jinv_02*Jinv_02*(Jinv_20*Jinv_10 + Jinv_21*Jinv_11) + Jinv_22*Jinv_12*(Jinv_00*Jinv_00 + Jinv_01*Jinv_01) + Jinv_01*Jinv_20*Jinv_10*Jinv_01 + Jinv_00*Jinv_21*Jinv_11*Jinv_00);
    const double G4_0_2_1_2 = det*(Jinv_02*Jinv_22*(Jinv_20*Jinv_10 + Jinv_21*Jinv_11) + Jinv_22*Jinv_12*(Jinv_00*Jinv_20 + Jinv_01*Jinv_21) + Jinv_01*Jinv_20*Jinv_10*Jinv_21 + Jinv_00*Jinv_21*Jinv_11*Jinv_20);
    const double G4_0_2_2_0 = det*(Jinv_02*Jinv_02*(Jinv_20*Jinv_20 + Jinv_21*Jinv_21) + Jinv_22*Jinv_22*(Jinv_00*Jinv_00 + Jinv_01*Jinv_01) + Jinv_01*Jinv_20*Jinv_20*Jinv_01 + Jinv_00*Jinv_21*Jinv_21*Jinv_00);
    const double G4_0_2_2_1 = det*(Jinv_02*Jinv_12*(Jinv_20*Jinv_20 + Jinv_21*Jinv_21) + Jinv_22*Jinv_22*(Jinv_00*Jinv_10 + Jinv_01*Jinv_11) + Jinv_01*Jinv_20*Jinv_20*Jinv_11 + Jinv_00*Jinv_21*Jinv_21*Jinv_10);
    const double G4_1_0_0_1 = det*(Jinv_12*Jinv_12*(Jinv_00*Jinv_00 + Jinv_01*Jinv_01) + Jinv_02*Jinv_02*(Jinv_10*Jinv_10 + Jinv_11*Jinv_11) + Jinv_11*Jinv_00*Jinv_00*Jinv_11 + Jinv_10*Jinv_01*Jinv_01*Jinv_10);
    const double G4_1_0_0_2 = det*(Jinv_12*Jinv_22*(Jinv_00*Jinv_00 + Jinv_01*Jinv_01) + Jinv_02*Jinv_02*(Jinv_10*Jinv_20 + Jinv_11*Jinv_21) + Jinv_11*Jinv_00*Jinv_00*Jinv_21 + Jinv_10*Jinv_01*Jinv_01*Jinv_20);
    const double G4_1_0_1_0 = det*(Jinv_12*Jinv_02*(Jinv_00*Jinv_10 + Jinv_01*Jinv_11) + Jinv_02*Jinv_12*(Jinv_10*Jinv_00 + Jinv_11*Jinv_01) + Jinv_11*Jinv_00*Jinv_10*Jinv_01 + Jinv_10*Jinv_01*Jinv_11*Jinv_00);
    const double G4_1_0_1_2 = det*(Jinv_12*Jinv_22*(Jinv_00*Jinv_10 + Jinv_01*Jinv_11) + Jinv_02*Jinv_12*(Jinv_10*Jinv_20 + Jinv_11*Jinv_21) + Jinv_11*Jinv_00*Jinv_10*Jinv_21 + Jinv_10*Jinv_01*Jinv_11*Jinv_20);
    const double G4_1_0_2_0 = det*(Jinv_12*Jinv_02*(Jinv_00*Jinv_20 + Jinv_01*Jinv_21) + Jinv_02*Jinv_22*(Jinv_10*Jinv_00 + Jinv_11*Jinv_01) + Jinv_11*Jinv_00*Jinv_20*Jinv_01 + Jinv_10*Jinv_01*Jinv_21*Jinv_00);
    const double G4_1_0_2_1 = det*(Jinv_12*Jinv_12*(Jinv_00*Jinv_20 + Jinv_01*Jinv_21) + Jinv_02*Jinv_22*(Jinv_10*Jinv_10 + Jinv_11*Jinv_11) + Jinv_11*Jinv_00*Jinv_20*Jinv_11 + Jinv_10*Jinv_01*Jinv_21*Jinv_10);
    const double G4_1_2_0_1 = det*(Jinv_12*Jinv_12*(Jinv_20*Jinv_00 + Jinv_21*Jinv_01) + Jinv_22*Jinv_02*(Jinv_10*Jinv_10 + Jinv_11*Jinv_11) + Jinv_11*Jinv_20*Jinv_00*Jinv_11 + Jinv_10*Jinv_21*Jinv_01*Jinv_10);
    const double G4_1_2_0_2 = det*(Jinv_12*Jinv_22*(Jinv_20*Jinv_00 + Jinv_21*Jinv_01) + Jinv_22*Jinv_02*(Jinv_10*Jinv_20 + Jinv_11*Jinv_21) + Jinv_11*Jinv_20*Jinv_00*Jinv_21 + Jinv_10*Jinv_21*Jinv_01*Jinv_20);
    const double G4_1_2_1_0 = det*(Jinv_12*Jinv_02*(Jinv_20*Jinv_10 + Jinv_21*Jinv_11) + Jinv_22*Jinv_12*(Jinv_10*Jinv_00 + Jinv_11*Jinv_01) + Jinv_11*Jinv_20*Jinv_10*Jinv_01 + Jinv_10*Jinv_21*Jinv_11*Jinv_00);
    const double G4_1_2_1_2 = det*(Jinv_12*Jinv_22*(Jinv_20*Jinv_10 + Jinv_21*Jinv_11) + Jinv_22*Jinv_12*(Jinv_10*Jinv_20 + Jinv_11*Jinv_21) + Jinv_11*Jinv_20*Jinv_10*Jinv_21 + Jinv_10*Jinv_21*Jinv_11*Jinv_20);
    const double G4_1_2_2_0 = det*(Jinv_12*Jinv_02*(Jinv_20*Jinv_20 + Jinv_21*Jinv_21) + Jinv_22*Jinv_22*(Jinv_10*Jinv_00 + Jinv_11*Jinv_01) + Jinv_11*Jinv_20*Jinv_20*Jinv_01 + Jinv_10*Jinv_21*Jinv_21*Jinv_00);
    const double G4_1_2_2_1 = det*(Jinv_12*Jinv_12*(Jinv_20*Jinv_20 + Jinv_21*Jinv_21) + Jinv_22*Jinv_22*(Jinv_10*Jinv_10 + Jinv_11*Jinv_11) + Jinv_11*Jinv_20*Jinv_20*Jinv_11 + Jinv_10*Jinv_21*Jinv_21*Jinv_10);
    const double G4_2_0_0_1 = det*(Jinv_22*Jinv_12*(Jinv_00*Jinv_00 + Jinv_01*Jinv_01) + Jinv_02*Jinv_02*(Jinv_20*Jinv_10 + Jinv_21*Jinv_11) + Jinv_21*Jinv_00*Jinv_00*Jinv_11 + Jinv_20*Jinv_01*Jinv_01*Jinv_10);
    const double G4_2_0_0_2 = det*(Jinv_22*Jinv_22*(Jinv_00*Jinv_00 + Jinv_01*Jinv_01) + Jinv_02*Jinv_02*(Jinv_20*Jinv_20 + Jinv_21*Jinv_21) + Jinv_21*Jinv_00*Jinv_00*Jinv_21 + Jinv_20*Jinv_01*Jinv_01*Jinv_20);
    const double G4_2_0_1_0 = det*(Jinv_22*Jinv_02*(Jinv_00*Jinv_10 + Jinv_01*Jinv_11) + Jinv_02*Jinv_12*(Jinv_20*Jinv_00 + Jinv_21*Jinv_01) + Jinv_21*Jinv_00*Jinv_10*Jinv_01 + Jinv_20*Jinv_01*Jinv_11*Jinv_00);
    const double G4_2_0_1_2 = det*(Jinv_22*Jinv_22*(Jinv_00*Jinv_10 + Jinv_01*Jinv_11) + Jinv_02*Jinv_12*(Jinv_20*Jinv_20 + Jinv_21*Jinv_21) + Jinv_21*Jinv_00*Jinv_10*Jinv_21 + Jinv_20*Jinv_01*Jinv_11*Jinv_20);
    const double G4_2_0_2_0 = det*(Jinv_22*Jinv_02*(Jinv_00*Jinv_20 + Jinv_01*Jinv_21) + Jinv_02*Jinv_22*(Jinv_20*Jinv_00 + Jinv_21*Jinv_01) + Jinv_21*Jinv_00*Jinv_20*Jinv_01 + Jinv_20*Jinv_01*Jinv_21*Jinv_00);
    const double G4_2_0_2_1 = det*(Jinv_22*Jinv_12*(Jinv_00*Jinv_20 + Jinv_01*Jinv_21) + Jinv_02*Jinv_22*(Jinv_20*Jinv_10 + Jinv_21*Jinv_11) + Jinv_21*Jinv_00*Jinv_20*Jinv_11 + Jinv_20*Jinv_01*Jinv_21*Jinv_10);
    const double G4_2_1_0_1 = det*(Jinv_22*Jinv_12*(Jinv_10*Jinv_00 + Jinv_11*Jinv_01) + Jinv_12*Jinv_02*(Jinv_20*Jinv_10 + Jinv_21*Jinv_11) + Jinv_21*Jinv_10*Jinv_00*Jinv_11 + Jinv_20*Jinv_11*Jinv_01*Jinv_10);
    const double G4_2_1_0_2 = det*(Jinv_22*Jinv_22*(Jinv_10*Jinv_00 + Jinv_11*Jinv_01) + Jinv_12*Jinv_02*(Jinv_20*Jinv_20 + Jinv_21*Jinv_21) + Jinv_21*Jinv_10*Jinv_00*Jinv_21 + Jinv_20*Jinv_11*Jinv_01*Jinv_20);
    const double G4_2_1_1_0 = det*(Jinv_22*Jinv_02*(Jinv_10*Jinv_10 + Jinv_11*Jinv_11) + Jinv_12*Jinv_12*(Jinv_20*Jinv_00 + Jinv_21*Jinv_01) + Jinv_21*Jinv_10*Jinv_10*Jinv_01 + Jinv_20*Jinv_11*Jinv_11*Jinv_00);
    const double G4_2_1_1_2 = det*(Jinv_22*Jinv_22*(Jinv_10*Jinv_10 + Jinv_11*Jinv_11) + Jinv_12*Jinv_12*(Jinv_20*Jinv_20 + Jinv_21*Jinv_21) + Jinv_21*Jinv_10*Jinv_10*Jinv_21 + Jinv_20*Jinv_11*Jinv_11*Jinv_20);
    const double G4_2_1_2_0 = det*(Jinv_22*Jinv_02*(Jinv_10*Jinv_20 + Jinv_11*Jinv_21) + Jinv_12*Jinv_22*(Jinv_20*Jinv_00 + Jinv_21*Jinv_01) + Jinv_21*Jinv_10*Jinv_20*Jinv_01 + Jinv_20*Jinv_11*Jinv_21*Jinv_00);
    const double G4_2_1_2_1 = det*(Jinv_22*Jinv_12*(Jinv_10*Jinv_20 + Jinv_11*Jinv_21) + Jinv_12*Jinv_22*(Jinv_20*Jinv_10 + Jinv_21*Jinv_11) + Jinv_21*Jinv_10*Jinv_20*Jinv_11 + Jinv_20*Jinv_11*Jinv_21*Jinv_10);
    
    // Compute element tensor
    A[0] = 0.01666666667*G0_;
    A[1] = 0.008333333333*G0_;
    A[2] = 0.008333333333*G0_;
    A[3] = 0.008333333333*G0_;
    A[4] = -0.04166666667*G1_0_1 + 0.04166666667*G1_0_2 - 0.04166666667*G1_1_1 + 0.04166666667*G1_1_2 - 0.04166666667*G1_2_1 + 0.04166666667*G1_2_2;
    A[5] = -0.04166666667*G1_0_0 + 0.04166666667*G1_0_2 - 0.04166666667*G1_1_0 + 0.04166666667*G1_1_2 - 0.04166666667*G1_2_0 + 0.04166666667*G1_2_2;
    A[6] = -0.04166666667*G1_0_0 + 0.04166666667*G1_0_1 - 0.04166666667*G1_1_0 + 0.04166666667*G1_1_1 - 0.04166666667*G1_2_0 + 0.04166666667*G1_2_1;
    A[7] = 0.04166666667*G1_0_0 + 0.04166666667*G1_0_1 + 0.08333333333*G1_0_2 + 0.04166666667*G1_1_0 + 0.04166666667*G1_1_1 + 0.08333333333*G1_1_2 + 0.04166666667*G1_2_0 + 0.04166666667*G1_2_1 + 0.08333333333*G1_2_2;
    A[8] = 0.04166666667*G1_0_0 + 0.08333333333*G1_0_1 + 0.04166666667*G1_0_2 + 0.04166666667*G1_1_0 + 0.08333333333*G1_1_1 + 0.04166666667*G1_1_2 + 0.04166666667*G1_2_0 + 0.08333333333*G1_2_1 + 0.04166666667*G1_2_2;
    A[9] = 0.08333333333*G1_0_0 + 0.04166666667*G1_0_1 + 0.04166666667*G1_0_2 + 0.08333333333*G1_1_0 + 0.04166666667*G1_1_1 + 0.04166666667*G1_1_2 + 0.08333333333*G1_2_0 + 0.04166666667*G1_2_1 + 0.04166666667*G1_2_2;
    A[10] = 0.008333333333*G0_;
    A[11] = 0.01666666667*G0_;
    A[12] = 0.008333333333*G0_;
    A[13] = 0.008333333333*G0_;
    A[14] = 0.04166666667*G1_0_1 - 0.04166666667*G1_0_2;
    A[15] = 0.04166666667*G1_0_0 - 0.04166666667*G1_0_2;
    A[16] = 0.04166666667*G1_0_0 - 0.04166666667*G1_0_1;
    A[17] = -0.04166666667*G1_0_0 - 0.04166666667*G1_0_1 - 0.08333333333*G1_0_2;
    A[18] = -0.04166666667*G1_0_0 - 0.08333333333*G1_0_1 - 0.04166666667*G1_0_2;
    A[19] = -0.08333333333*G1_0_0 - 0.04166666667*G1_0_1 - 0.04166666667*G1_0_2;
    A[20] = 0.008333333333*G0_;
    A[21] = 0.008333333333*G0_;
    A[22] = 0.01666666667*G0_;
    A[23] = 0.008333333333*G0_;
    A[24] = 0.04166666667*G1_1_1 - 0.04166666667*G1_1_2;
    A[25] = 0.04166666667*G1_1_0 - 0.04166666667*G1_1_2;
    A[26] = 0.04166666667*G1_1_0 - 0.04166666667*G1_1_1;
    A[27] = -0.04166666667*G1_1_0 - 0.04166666667*G1_1_1 - 0.08333333333*G1_1_2;
    A[28] = -0.04166666667*G1_1_0 - 0.08333333333*G1_1_1 - 0.04166666667*G1_1_2;
    A[29] = -0.08333333333*G1_1_0 - 0.04166666667*G1_1_1 - 0.04166666667*G1_1_2;
    A[30] = 0.008333333333*G0_;
    A[31] = 0.008333333333*G0_;
    A[32] = 0.008333333333*G0_;
    A[33] = 0.01666666667*G0_;
    A[34] = 0.04166666667*G1_2_1 - 0.04166666667*G1_2_2;
    A[35] = 0.04166666667*G1_2_0 - 0.04166666667*G1_2_2;
    A[36] = 0.04166666667*G1_2_0 - 0.04166666667*G1_2_1;
    A[37] = -0.04166666667*G1_2_0 - 0.04166666667*G1_2_1 - 0.08333333333*G1_2_2;
    A[38] = -0.04166666667*G1_2_0 - 0.08333333333*G1_2_1 - 0.04166666667*G1_2_2;
    A[39] = -0.08333333333*G1_2_0 - 0.04166666667*G1_2_1 - 0.04166666667*G1_2_2;
    A[40] = 0.04166666667*G2_1_0 + 0.04166666667*G2_1_1 + 0.04166666667*G2_1_2 - 0.04166666667*G2_2_0 - 0.04166666667*G2_2_1 - 0.04166666667*G2_2_2;
    A[41] = -0.04166666667*G2_1_0 + 0.04166666667*G2_2_0;
    A[42] = -0.04166666667*G2_1_1 + 0.04166666667*G2_2_1;
    A[43] = -0.04166666667*G2_1_2 + 0.04166666667*G2_2_2;
    A[44] = 0.1666666667*G3_1_2_1_2 - 0.1666666667*G3_1_2_2_1 - 0.1666666667*G3_2_1_1_2 + 0.1666666667*G3_2_1_2_1 - 0.1666666667*G4_1_2_1_2 + 0.1666666667*G4_1_2_2_1 + 0.1666666667*G4_2_1_1_2 - 0.1666666667*G4_2_1_2_1;
    A[45] = 0.1666666667*G3_1_2_0_2 - 0.1666666667*G3_1_2_2_0 - 0.1666666667*G3_2_1_0_2 + 0.1666666667*G3_2_1_2_0 - 0.1666666667*G4_1_2_0_2 + 0.1666666667*G4_1_2_2_0 + 0.1666666667*G4_2_1_0_2 - 0.1666666667*G4_2_1_2_0;
    A[46] = 0.1666666667*G3_1_2_0_1 - 0.1666666667*G3_1_2_1_0 - 0.1666666667*G3_2_1_0_1 + 0.1666666667*G3_2_1_1_0 - 0.1666666667*G4_1_2_0_1 + 0.1666666667*G4_1_2_1_0 + 0.1666666667*G4_2_1_0_1 - 0.1666666667*G4_2_1_1_0;
    A[47] = -0.1666666667*G3_1_2_0_2 - 0.1666666667*G3_1_2_1_2 + 0.1666666667*G3_1_2_2_0 + 0.1666666667*G3_1_2_2_1 + 0.1666666667*G3_2_1_0_2 + 0.1666666667*G3_2_1_1_2 - 0.1666666667*G3_2_1_2_0 - 0.1666666667*G3_2_1_2_1 + 0.1666666667*G4_1_2_0_2 + 0.1666666667*G4_1_2_1_2 - 0.1666666667*G4_1_2_2_0 - 0.1666666667*G4_1_2_2_1 - 0.1666666667*G4_2_1_0_2 - 0.1666666667*G4_2_1_1_2 + 0.1666666667*G4_2_1_2_0 + 0.1666666667*G4_2_1_2_1;
    A[48] = -0.1666666667*G3_1_2_0_1 + 0.1666666667*G3_1_2_1_0 + 0.1666666667*G3_1_2_1_2 - 0.1666666667*G3_1_2_2_1 + 0.1666666667*G3_2_1_0_1 - 0.1666666667*G3_2_1_1_0 - 0.1666666667*G3_2_1_1_2 + 0.1666666667*G3_2_1_2_1 + 0.1666666667*G4_1_2_0_1 - 0.1666666667*G4_1_2_1_0 - 0.1666666667*G4_1_2_1_2 + 0.1666666667*G4_1_2_2_1 - 0.1666666667*G4_2_1_0_1 + 0.1666666667*G4_2_1_1_0 + 0.1666666667*G4_2_1_1_2 - 0.1666666667*G4_2_1_2_1;
    A[49] = 0.1666666667*G3_1_2_0_1 + 0.1666666667*G3_1_2_0_2 - 0.1666666667*G3_1_2_1_0 - 0.1666666667*G3_1_2_2_0 - 0.1666666667*G3_2_1_0_1 - 0.1666666667*G3_2_1_0_2 + 0.1666666667*G3_2_1_1_0 + 0.1666666667*G3_2_1_2_0 - 0.1666666667*G4_1_2_0_1 - 0.1666666667*G4_1_2_0_2 + 0.1666666667*G4_1_2_1_0 + 0.1666666667*G4_1_2_2_0 + 0.1666666667*G4_2_1_0_1 + 0.1666666667*G4_2_1_0_2 - 0.1666666667*G4_2_1_1_0 - 0.1666666667*G4_2_1_2_0;
    A[50] = 0.04166666667*G2_0_0 + 0.04166666667*G2_0_1 + 0.04166666667*G2_0_2 - 0.04166666667*G2_2_0 - 0.04166666667*G2_2_1 - 0.04166666667*G2_2_2;
    A[51] = -0.04166666667*G2_0_0 + 0.04166666667*G2_2_0;
    A[52] = -0.04166666667*G2_0_1 + 0.04166666667*G2_2_1;
    A[53] = -0.04166666667*G2_0_2 + 0.04166666667*G2_2_2;
    A[54] = 0.1666666667*G3_0_2_1_2 - 0.1666666667*G3_0_2_2_1 - 0.1666666667*G3_2_0_1_2 + 0.1666666667*G3_2_0_2_1 - 0.1666666667*G4_0_2_1_2 + 0.1666666667*G4_0_2_2_1 + 0.1666666667*G4_2_0_1_2 - 0.1666666667*G4_2_0_2_1;
    A[55] = 0.1666666667*G3_0_2_0_2 - 0.1666666667*G3_0_2_2_0 - 0.1666666667*G3_2_0_0_2 + 0.1666666667*G3_2_0_2_0 - 0.1666666667*G4_0_2_0_2 + 0.1666666667*G4_0_2_2_0 + 0.1666666667*G4_2_0_0_2 - 0.1666666667*G4_2_0_2_0;
    A[56] = 0.1666666667*G3_0_2_0_1 - 0.1666666667*G3_0_2_1_0 - 0.1666666667*G3_2_0_0_1 + 0.1666666667*G3_2_0_1_0 - 0.1666666667*G4_0_2_0_1 + 0.1666666667*G4_0_2_1_0 + 0.1666666667*G4_2_0_0_1 - 0.1666666667*G4_2_0_1_0;
    A[57] = -0.1666666667*G3_0_2_0_2 - 0.1666666667*G3_0_2_1_2 + 0.1666666667*G3_0_2_2_0 + 0.1666666667*G3_0_2_2_1 + 0.1666666667*G3_2_0_0_2 + 0.1666666667*G3_2_0_1_2 - 0.1666666667*G3_2_0_2_0 - 0.1666666667*G3_2_0_2_1 + 0.1666666667*G4_0_2_0_2 + 0.1666666667*G4_0_2_1_2 - 0.1666666667*G4_0_2_2_0 - 0.1666666667*G4_0_2_2_1 - 0.1666666667*G4_2_0_0_2 - 0.1666666667*G4_2_0_1_2 + 0.1666666667*G4_2_0_2_0 + 0.1666666667*G4_2_0_2_1;
    A[58] = -0.1666666667*G3_0_2_0_1 + 0.1666666667*G3_0_2_1_0 + 0.1666666667*G3_0_2_1_2 - 0.1666666667*G3_0_2_2_1 + 0.1666666667*G3_2_0_0_1 - 0.1666666667*G3_2_0_1_0 - 0.1666666667*G3_2_0_1_2 + 0.1666666667*G3_2_0_2_1 + 0.1666666667*G4_0_2_0_1 - 0.1666666667*G4_0_2_1_0 - 0.1666666667*G4_0_2_1_2 + 0.1666666667*G4_0_2_2_1 - 0.1666666667*G4_2_0_0_1 + 0.1666666667*G4_2_0_1_0 + 0.1666666667*G4_2_0_1_2 - 0.1666666667*G4_2_0_2_1;
    A[59] = 0.1666666667*G3_0_2_0_1 + 0.1666666667*G3_0_2_0_2 - 0.1666666667*G3_0_2_1_0 - 0.1666666667*G3_0_2_2_0 - 0.1666666667*G3_2_0_0_1 - 0.1666666667*G3_2_0_0_2 + 0.1666666667*G3_2_0_1_0 + 0.1666666667*G3_2_0_2_0 - 0.1666666667*G4_0_2_0_1 - 0.1666666667*G4_0_2_0_2 + 0.1666666667*G4_0_2_1_0 + 0.1666666667*G4_0_2_2_0 + 0.1666666667*G4_2_0_0_1 + 0.1666666667*G4_2_0_0_2 - 0.1666666667*G4_2_0_1_0 - 0.1666666667*G4_2_0_2_0;
    A[60] = 0.04166666667*G2_0_0 + 0.04166666667*G2_0_1 + 0.04166666667*G2_0_2 - 0.04166666667*G2_1_0 - 0.04166666667*G2_1_1 - 0.04166666667*G2_1_2;
    A[61] = -0.04166666667*G2_0_0 + 0.04166666667*G2_1_0;
    A[62] = -0.04166666667*G2_0_1 + 0.04166666667*G2_1_1;
    A[63] = -0.04166666667*G2_0_2 + 0.04166666667*G2_1_2;
    A[64] = 0.1666666667*G3_0_1_1_2 - 0.1666666667*G3_0_1_2_1 - 0.1666666667*G3_1_0_1_2 + 0.1666666667*G3_1_0_2_1 - 0.1666666667*G4_0_1_1_2 + 0.1666666667*G4_0_1_2_1 + 0.1666666667*G4_1_0_1_2 - 0.1666666667*G4_1_0_2_1;
    A[65] = 0.1666666667*G3_0_1_0_2 - 0.1666666667*G3_0_1_2_0 - 0.1666666667*G3_1_0_0_2 + 0.1666666667*G3_1_0_2_0 - 0.1666666667*G4_0_1_0_2 + 0.1666666667*G4_0_1_2_0 + 0.1666666667*G4_1_0_0_2 - 0.1666666667*G4_1_0_2_0;
    A[66] = 0.1666666667*G3_0_1_0_1 - 0.1666666667*G3_0_1_1_0 - 0.1666666667*G3_1_0_0_1 + 0.1666666667*G3_1_0_1_0 - 0.1666666667*G4_0_1_0_1 + 0.1666666667*G4_0_1_1_0 + 0.1666666667*G4_1_0_0_1 - 0.1666666667*G4_1_0_1_0;
    A[67] = -0.1666666667*G3_0_1_0_2 - 0.1666666667*G3_0_1_1_2 + 0.1666666667*G3_0_1_2_0 + 0.1666666667*G3_0_1_2_1 + 0.1666666667*G3_1_0_0_2 + 0.1666666667*G3_1_0_1_2 - 0.1666666667*G3_1_0_2_0 - 0.1666666667*G3_1_0_2_1 + 0.1666666667*G4_0_1_0_2 + 0.1666666667*G4_0_1_1_2 - 0.1666666667*G4_0_1_2_0 - 0.1666666667*G4_0_1_2_1 - 0.1666666667*G4_1_0_0_2 - 0.1666666667*G4_1_0_1_2 + 0.1666666667*G4_1_0_2_0 + 0.1666666667*G4_1_0_2_1;
    A[68] = -0.1666666667*G3_0_1_0_1 + 0.1666666667*G3_0_1_1_0 + 0.1666666667*G3_0_1_1_2 - 0.1666666667*G3_0_1_2_1 + 0.1666666667*G3_1_0_0_1 - 0.1666666667*G3_1_0_1_0 - 0.1666666667*G3_1_0_1_2 + 0.1666666667*G3_1_0_2_1 + 0.1666666667*G4_0_1_0_1 - 0.1666666667*G4_0_1_1_0 - 0.1666666667*G4_0_1_1_2 + 0.1666666667*G4_0_1_2_1 - 0.1666666667*G4_1_0_0_1 + 0.1666666667*G4_1_0_1_0 + 0.1666666667*G4_1_0_1_2 - 0.1666666667*G4_1_0_2_1;
    A[69] = 0.1666666667*G3_0_1_0_1 + 0.1666666667*G3_0_1_0_2 - 0.1666666667*G3_0_1_1_0 - 0.1666666667*G3_0_1_2_0 - 0.1666666667*G3_1_0_0_1 - 0.1666666667*G3_1_0_0_2 + 0.1666666667*G3_1_0_1_0 + 0.1666666667*G3_1_0_2_0 - 0.1666666667*G4_0_1_0_1 - 0.1666666667*G4_0_1_0_2 + 0.1666666667*G4_0_1_1_0 + 0.1666666667*G4_0_1_2_0 + 0.1666666667*G4_1_0_0_1 + 0.1666666667*G4_1_0_0_2 - 0.1666666667*G4_1_0_1_0 - 0.1666666667*G4_1_0_2_0;
    A[70] = -0.04166666667*G2_0_0 - 0.04166666667*G2_0_1 - 0.04166666667*G2_0_2 - 0.04166666667*G2_1_0 - 0.04166666667*G2_1_1 - 0.04166666667*G2_1_2 - 0.08333333333*G2_2_0 - 0.08333333333*G2_2_1 - 0.08333333333*G2_2_2;
    A[71] = 0.04166666667*G2_0_0 + 0.04166666667*G2_1_0 + 0.08333333333*G2_2_0;
    A[72] = 0.04166666667*G2_0_1 + 0.04166666667*G2_1_1 + 0.08333333333*G2_2_1;
    A[73] = 0.04166666667*G2_0_2 + 0.04166666667*G2_1_2 + 0.08333333333*G2_2_2;
    A[74] = -0.1666666667*G3_0_2_1_2 + 0.1666666667*G3_0_2_2_1 - 0.1666666667*G3_1_2_1_2 + 0.1666666667*G3_1_2_2_1 + 0.1666666667*G3_2_0_1_2 - 0.1666666667*G3_2_0_2_1 + 0.1666666667*G3_2_1_1_2 - 0.1666666667*G3_2_1_2_1 + 0.1666666667*G4_0_2_1_2 - 0.1666666667*G4_0_2_2_1 + 0.1666666667*G4_1_2_1_2 - 0.1666666667*G4_1_2_2_1 - 0.1666666667*G4_2_0_1_2 + 0.1666666667*G4_2_0_2_1 - 0.1666666667*G4_2_1_1_2 + 0.1666666667*G4_2_1_2_1;
    A[75] = -0.1666666667*G3_0_2_0_2 + 0.1666666667*G3_0_2_2_0 - 0.1666666667*G3_1_2_0_2 + 0.1666666667*G3_1_2_2_0 + 0.1666666667*G3_2_0_0_2 - 0.1666666667*G3_2_0_2_0 + 0.1666666667*G3_2_1_0_2 - 0.1666666667*G3_2_1_2_0 + 0.1666666667*G4_0_2_0_2 - 0.1666666667*G4_0_2_2_0 + 0.1666666667*G4_1_2_0_2 - 0.1666666667*G4_1_2_2_0 - 0.1666666667*G4_2_0_0_2 + 0.1666666667*G4_2_0_2_0 - 0.1666666667*G4_2_1_0_2 + 0.1666666667*G4_2_1_2_0;
    A[76] = -0.1666666667*G3_0_2_0_1 + 0.1666666667*G3_0_2_1_0 - 0.1666666667*G3_1_2_0_1 + 0.1666666667*G3_1_2_1_0 + 0.1666666667*G3_2_0_0_1 - 0.1666666667*G3_2_0_1_0 + 0.1666666667*G3_2_1_0_1 - 0.1666666667*G3_2_1_1_0 + 0.1666666667*G4_0_2_0_1 - 0.1666666667*G4_0_2_1_0 + 0.1666666667*G4_1_2_0_1 - 0.1666666667*G4_1_2_1_0 - 0.1666666667*G4_2_0_0_1 + 0.1666666667*G4_2_0_1_0 - 0.1666666667*G4_2_1_0_1 + 0.1666666667*G4_2_1_1_0;
    A[77] = 0.1666666667*G3_0_2_0_2 + 0.1666666667*G3_0_2_1_2 - 0.1666666667*G3_0_2_2_0 - 0.1666666667*G3_0_2_2_1 + 0.1666666667*G3_1_2_0_2 + 0.1666666667*G3_1_2_1_2 - 0.1666666667*G3_1_2_2_0 - 0.1666666667*G3_1_2_2_1 - 0.1666666667*G3_2_0_0_2 - 0.1666666667*G3_2_0_1_2 + 0.1666666667*G3_2_0_2_0 + 0.1666666667*G3_2_0_2_1 - 0.1666666667*G3_2_1_0_2 - 0.1666666667*G3_2_1_1_2 + 0.1666666667*G3_2_1_2_0 + 0.1666666667*G3_2_1_2_1 - 0.1666666667*G4_0_2_0_2 - 0.1666666667*G4_0_2_1_2 + 0.1666666667*G4_0_2_2_0 + 0.1666666667*G4_0_2_2_1 - 0.1666666667*G4_1_2_0_2 - 0.1666666667*G4_1_2_1_2 + 0.1666666667*G4_1_2_2_0 + 0.1666666667*G4_1_2_2_1 + 0.1666666667*G4_2_0_0_2 + 0.1666666667*G4_2_0_1_2 - 0.1666666667*G4_2_0_2_0 - 0.1666666667*G4_2_0_2_1 + 0.1666666667*G4_2_1_0_2 + 0.1666666667*G4_2_1_1_2 - 0.1666666667*G4_2_1_2_0 - 0.1666666667*G4_2_1_2_1;
    A[78] = 0.1666666667*G3_0_2_0_1 - 0.1666666667*G3_0_2_1_0 - 0.1666666667*G3_0_2_1_2 + 0.1666666667*G3_0_2_2_1 + 0.1666666667*G3_1_2_0_1 - 0.1666666667*G3_1_2_1_0 - 0.1666666667*G3_1_2_1_2 + 0.1666666667*G3_1_2_2_1 - 0.1666666667*G3_2_0_0_1 + 0.1666666667*G3_2_0_1_0 + 0.1666666667*G3_2_0_1_2 - 0.1666666667*G3_2_0_2_1 - 0.1666666667*G3_2_1_0_1 + 0.1666666667*G3_2_1_1_0 + 0.1666666667*G3_2_1_1_2 - 0.1666666667*G3_2_1_2_1 - 0.1666666667*G4_0_2_0_1 + 0.1666666667*G4_0_2_1_0 + 0.1666666667*G4_0_2_1_2 - 0.1666666667*G4_0_2_2_1 - 0.1666666667*G4_1_2_0_1 + 0.1666666667*G4_1_2_1_0 + 0.1666666667*G4_1_2_1_2 - 0.1666666667*G4_1_2_2_1 + 0.1666666667*G4_2_0_0_1 - 0.1666666667*G4_2_0_1_0 - 0.1666666667*G4_2_0_1_2 + 0.1666666667*G4_2_0_2_1 + 0.1666666667*G4_2_1_0_1 - 0.1666666667*G4_2_1_1_0 - 0.1666666667*G4_2_1_1_2 + 0.1666666667*G4_2_1_2_1;
    A[79] = -0.1666666667*G3_0_2_0_1 - 0.1666666667*G3_0_2_0_2 + 0.1666666667*G3_0_2_1_0 + 0.1666666667*G3_0_2_2_0 - 0.1666666667*G3_1_2_0_1 - 0.1666666667*G3_1_2_0_2 + 0.1666666667*G3_1_2_1_0 + 0.1666666667*G3_1_2_2_0 + 0.1666666667*G3_2_0_0_1 + 0.1666666667*G3_2_0_0_2 - 0.1666666667*G3_2_0_1_0 - 0.1666666667*G3_2_0_2_0 + 0.1666666667*G3_2_1_0_1 + 0.1666666667*G3_2_1_0_2 - 0.1666666667*G3_2_1_1_0 - 0.1666666667*G3_2_1_2_0 + 0.1666666667*G4_0_2_0_1 + 0.1666666667*G4_0_2_0_2 - 0.1666666667*G4_0_2_1_0 - 0.1666666667*G4_0_2_2_0 + 0.1666666667*G4_1_2_0_1 + 0.1666666667*G4_1_2_0_2 - 0.1666666667*G4_1_2_1_0 - 0.1666666667*G4_1_2_2_0 - 0.1666666667*G4_2_0_0_1 - 0.1666666667*G4_2_0_0_2 + 0.1666666667*G4_2_0_1_0 + 0.1666666667*G4_2_0_2_0 - 0.1666666667*G4_2_1_0_1 - 0.1666666667*G4_2_1_0_2 + 0.1666666667*G4_2_1_1_0 + 0.1666666667*G4_2_1_2_0;
    A[80] = -0.04166666667*G2_0_0 - 0.04166666667*G2_0_1 - 0.04166666667*G2_0_2 - 0.08333333333*G2_1_0 - 0.08333333333*G2_1_1 - 0.08333333333*G2_1_2 - 0.04166666667*G2_2_0 - 0.04166666667*G2_2_1 - 0.04166666667*G2_2_2;
    A[81] = 0.04166666667*G2_0_0 + 0.08333333333*G2_1_0 + 0.04166666667*G2_2_0;
    A[82] = 0.04166666667*G2_0_1 + 0.08333333333*G2_1_1 + 0.04166666667*G2_2_1;
    A[83] = 0.04166666667*G2_0_2 + 0.08333333333*G2_1_2 + 0.04166666667*G2_2_2;
    A[84] = -0.1666666667*G3_0_1_1_2 + 0.1666666667*G3_0_1_2_1 + 0.1666666667*G3_1_0_1_2 - 0.1666666667*G3_1_0_2_1 + 0.1666666667*G3_1_2_1_2 - 0.1666666667*G3_1_2_2_1 - 0.1666666667*G3_2_1_1_2 + 0.1666666667*G3_2_1_2_1 + 0.1666666667*G4_0_1_1_2 - 0.1666666667*G4_0_1_2_1 - 0.1666666667*G4_1_0_1_2 + 0.1666666667*G4_1_0_2_1 - 0.1666666667*G4_1_2_1_2 + 0.1666666667*G4_1_2_2_1 + 0.1666666667*G4_2_1_1_2 - 0.1666666667*G4_2_1_2_1;
    A[85] = -0.1666666667*G3_0_1_0_2 + 0.1666666667*G3_0_1_2_0 + 0.1666666667*G3_1_0_0_2 - 0.1666666667*G3_1_0_2_0 + 0.1666666667*G3_1_2_0_2 - 0.1666666667*G3_1_2_2_0 - 0.1666666667*G3_2_1_0_2 + 0.1666666667*G3_2_1_2_0 + 0.1666666667*G4_0_1_0_2 - 0.1666666667*G4_0_1_2_0 - 0.1666666667*G4_1_0_0_2 + 0.1666666667*G4_1_0_2_0 - 0.1666666667*G4_1_2_0_2 + 0.1666666667*G4_1_2_2_0 + 0.1666666667*G4_2_1_0_2 - 0.1666666667*G4_2_1_2_0;
    A[86] = -0.1666666667*G3_0_1_0_1 + 0.1666666667*G3_0_1_1_0 + 0.1666666667*G3_1_0_0_1 - 0.1666666667*G3_1_0_1_0 + 0.1666666667*G3_1_2_0_1 - 0.1666666667*G3_1_2_1_0 - 0.1666666667*G3_2_1_0_1 + 0.1666666667*G3_2_1_1_0 + 0.1666666667*G4_0_1_0_1 - 0.1666666667*G4_0_1_1_0 - 0.1666666667*G4_1_0_0_1 + 0.1666666667*G4_1_0_1_0 - 0.1666666667*G4_1_2_0_1 + 0.1666666667*G4_1_2_1_0 + 0.1666666667*G4_2_1_0_1 - 0.1666666667*G4_2_1_1_0;
    A[87] = 0.1666666667*G3_0_1_0_2 + 0.1666666667*G3_0_1_1_2 - 0.1666666667*G3_0_1_2_0 - 0.1666666667*G3_0_1_2_1 - 0.1666666667*G3_1_0_0_2 - 0.1666666667*G3_1_0_1_2 + 0.1666666667*G3_1_0_2_0 + 0.1666666667*G3_1_0_2_1 - 0.1666666667*G3_1_2_0_2 - 0.1666666667*G3_1_2_1_2 + 0.1666666667*G3_1_2_2_0 + 0.1666666667*G3_1_2_2_1 + 0.1666666667*G3_2_1_0_2 + 0.1666666667*G3_2_1_1_2 - 0.1666666667*G3_2_1_2_0 - 0.1666666667*G3_2_1_2_1 - 0.1666666667*G4_0_1_0_2 - 0.1666666667*G4_0_1_1_2 + 0.1666666667*G4_0_1_2_0 + 0.1666666667*G4_0_1_2_1 + 0.1666666667*G4_1_0_0_2 + 0.1666666667*G4_1_0_1_2 - 0.1666666667*G4_1_0_2_0 - 0.1666666667*G4_1_0_2_1 + 0.1666666667*G4_1_2_0_2 + 0.1666666667*G4_1_2_1_2 - 0.1666666667*G4_1_2_2_0 - 0.1666666667*G4_1_2_2_1 - 0.1666666667*G4_2_1_0_2 - 0.1666666667*G4_2_1_1_2 + 0.1666666667*G4_2_1_2_0 + 0.1666666667*G4_2_1_2_1;
    A[88] = 0.1666666667*G3_0_1_0_1 - 0.1666666667*G3_0_1_1_0 - 0.1666666667*G3_0_1_1_2 + 0.1666666667*G3_0_1_2_1 - 0.1666666667*G3_1_0_0_1 + 0.1666666667*G3_1_0_1_0 + 0.1666666667*G3_1_0_1_2 - 0.1666666667*G3_1_0_2_1 - 0.1666666667*G3_1_2_0_1 + 0.1666666667*G3_1_2_1_0 + 0.1666666667*G3_1_2_1_2 - 0.1666666667*G3_1_2_2_1 + 0.1666666667*G3_2_1_0_1 - 0.1666666667*G3_2_1_1_0 - 0.1666666667*G3_2_1_1_2 + 0.1666666667*G3_2_1_2_1 - 0.1666666667*G4_0_1_0_1 + 0.1666666667*G4_0_1_1_0 + 0.1666666667*G4_0_1_1_2 - 0.1666666667*G4_0_1_2_1 + 0.1666666667*G4_1_0_0_1 - 0.1666666667*G4_1_0_1_0 - 0.1666666667*G4_1_0_1_2 + 0.1666666667*G4_1_0_2_1 + 0.1666666667*G4_1_2_0_1 - 0.1666666667*G4_1_2_1_0 - 0.1666666667*G4_1_2_1_2 + 0.1666666667*G4_1_2_2_1 - 0.1666666667*G4_2_1_0_1 + 0.1666666667*G4_2_1_1_0 + 0.1666666667*G4_2_1_1_2 - 0.1666666667*G4_2_1_2_1;
    A[89] = -0.1666666667*G3_0_1_0_1 - 0.1666666667*G3_0_1_0_2 + 0.1666666667*G3_0_1_1_0 + 0.1666666667*G3_0_1_2_0 + 0.1666666667*G3_1_0_0_1 + 0.1666666667*G3_1_0_0_2 - 0.1666666667*G3_1_0_1_0 - 0.1666666667*G3_1_0_2_0 + 0.1666666667*G3_1_2_0_1 + 0.1666666667*G3_1_2_0_2 - 0.1666666667*G3_1_2_1_0 - 0.1666666667*G3_1_2_2_0 - 0.1666666667*G3_2_1_0_1 - 0.1666666667*G3_2_1_0_2 + 0.1666666667*G3_2_1_1_0 + 0.1666666667*G3_2_1_2_0 + 0.1666666667*G4_0_1_0_1 + 0.1666666667*G4_0_1_0_2 - 0.1666666667*G4_0_1_1_0 - 0.1666666667*G4_0_1_2_0 - 0.1666666667*G4_1_0_0_1 - 0.1666666667*G4_1_0_0_2 + 0.1666666667*G4_1_0_1_0 + 0.1666666667*G4_1_0_2_0 - 0.1666666667*G4_1_2_0_1 - 0.1666666667*G4_1_2_0_2 + 0.1666666667*G4_1_2_1_0 + 0.1666666667*G4_1_2_2_0 + 0.1666666667*G4_2_1_0_1 + 0.1666666667*G4_2_1_0_2 - 0.1666666667*G4_2_1_1_0 - 0.1666666667*G4_2_1_2_0;
    A[90] = -0.08333333333*G2_0_0 - 0.08333333333*G2_0_1 - 0.08333333333*G2_0_2 - 0.04166666667*G2_1_0 - 0.04166666667*G2_1_1 - 0.04166666667*G2_1_2 - 0.04166666667*G2_2_0 - 0.04166666667*G2_2_1 - 0.04166666667*G2_2_2;
    A[91] = 0.08333333333*G2_0_0 + 0.04166666667*G2_1_0 + 0.04166666667*G2_2_0;
    A[92] = 0.08333333333*G2_0_1 + 0.04166666667*G2_1_1 + 0.04166666667*G2_2_1;
    A[93] = 0.08333333333*G2_0_2 + 0.04166666667*G2_1_2 + 0.04166666667*G2_2_2;
    A[94] = 0.1666666667*G3_0_1_1_2 - 0.1666666667*G3_0_1_2_1 + 0.1666666667*G3_0_2_1_2 - 0.1666666667*G3_0_2_2_1 - 0.1666666667*G3_1_0_1_2 + 0.1666666667*G3_1_0_2_1 - 0.1666666667*G3_2_0_1_2 + 0.1666666667*G3_2_0_2_1 - 0.1666666667*G4_0_1_1_2 + 0.1666666667*G4_0_1_2_1 - 0.1666666667*G4_0_2_1_2 + 0.1666666667*G4_0_2_2_1 + 0.1666666667*G4_1_0_1_2 - 0.1666666667*G4_1_0_2_1 + 0.1666666667*G4_2_0_1_2 - 0.1666666667*G4_2_0_2_1;
    A[95] = 0.1666666667*G3_0_1_0_2 - 0.1666666667*G3_0_1_2_0 + 0.1666666667*G3_0_2_0_2 - 0.1666666667*G3_0_2_2_0 - 0.1666666667*G3_1_0_0_2 + 0.1666666667*G3_1_0_2_0 - 0.1666666667*G3_2_0_0_2 + 0.1666666667*G3_2_0_2_0 - 0.1666666667*G4_0_1_0_2 + 0.1666666667*G4_0_1_2_0 - 0.1666666667*G4_0_2_0_2 + 0.1666666667*G4_0_2_2_0 + 0.1666666667*G4_1_0_0_2 - 0.1666666667*G4_1_0_2_0 + 0.1666666667*G4_2_0_0_2 - 0.1666666667*G4_2_0_2_0;
    A[96] = 0.1666666667*G3_0_1_0_1 - 0.1666666667*G3_0_1_1_0 + 0.1666666667*G3_0_2_0_1 - 0.1666666667*G3_0_2_1_0 - 0.1666666667*G3_1_0_0_1 + 0.1666666667*G3_1_0_1_0 - 0.1666666667*G3_2_0_0_1 + 0.1666666667*G3_2_0_1_0 - 0.1666666667*G4_0_1_0_1 + 0.1666666667*G4_0_1_1_0 - 0.1666666667*G4_0_2_0_1 + 0.1666666667*G4_0_2_1_0 + 0.1666666667*G4_1_0_0_1 - 0.1666666667*G4_1_0_1_0 + 0.1666666667*G4_2_0_0_1 - 0.1666666667*G4_2_0_1_0;
    A[97] = -0.1666666667*G3_0_1_0_2 - 0.1666666667*G3_0_1_1_2 + 0.1666666667*G3_0_1_2_0 + 0.1666666667*G3_0_1_2_1 - 0.1666666667*G3_0_2_0_2 - 0.1666666667*G3_0_2_1_2 + 0.1666666667*G3_0_2_2_0 + 0.1666666667*G3_0_2_2_1 + 0.1666666667*G3_1_0_0_2 + 0.1666666667*G3_1_0_1_2 - 0.1666666667*G3_1_0_2_0 - 0.1666666667*G3_1_0_2_1 + 0.1666666667*G3_2_0_0_2 + 0.1666666667*G3_2_0_1_2 - 0.1666666667*G3_2_0_2_0 - 0.1666666667*G3_2_0_2_1 + 0.1666666667*G4_0_1_0_2 + 0.1666666667*G4_0_1_1_2 - 0.1666666667*G4_0_1_2_0 - 0.1666666667*G4_0_1_2_1 + 0.1666666667*G4_0_2_0_2 + 0.1666666667*G4_0_2_1_2 - 0.1666666667*G4_0_2_2_0 - 0.1666666667*G4_0_2_2_1 - 0.1666666667*G4_1_0_0_2 - 0.1666666667*G4_1_0_1_2 + 0.1666666667*G4_1_0_2_0 + 0.1666666667*G4_1_0_2_1 - 0.1666666667*G4_2_0_0_2 - 0.1666666667*G4_2_0_1_2 + 0.1666666667*G4_2_0_2_0 + 0.1666666667*G4_2_0_2_1;
    A[98] = -0.1666666667*G3_0_1_0_1 + 0.1666666667*G3_0_1_1_0 + 0.1666666667*G3_0_1_1_2 - 0.1666666667*G3_0_1_2_1 - 0.1666666667*G3_0_2_0_1 + 0.1666666667*G3_0_2_1_0 + 0.1666666667*G3_0_2_1_2 - 0.1666666667*G3_0_2_2_1 + 0.1666666667*G3_1_0_0_1 - 0.1666666667*G3_1_0_1_0 - 0.1666666667*G3_1_0_1_2 + 0.1666666667*G3_1_0_2_1 + 0.1666666667*G3_2_0_0_1 - 0.1666666667*G3_2_0_1_0 - 0.1666666667*G3_2_0_1_2 + 0.1666666667*G3_2_0_2_1 + 0.1666666667*G4_0_1_0_1 - 0.1666666667*G4_0_1_1_0 - 0.1666666667*G4_0_1_1_2 + 0.1666666667*G4_0_1_2_1 + 0.1666666667*G4_0_2_0_1 - 0.1666666667*G4_0_2_1_0 - 0.1666666667*G4_0_2_1_2 + 0.1666666667*G4_0_2_2_1 - 0.1666666667*G4_1_0_0_1 + 0.1666666667*G4_1_0_1_0 + 0.1666666667*G4_1_0_1_2 - 0.1666666667*G4_1_0_2_1 - 0.1666666667*G4_2_0_0_1 + 0.1666666667*G4_2_0_1_0 + 0.1666666667*G4_2_0_1_2 - 0.1666666667*G4_2_0_2_1;
    A[99] = 0.1666666667*G3_0_1_0_1 + 0.1666666667*G3_0_1_0_2 - 0.1666666667*G3_0_1_1_0 - 0.1666666667*G3_0_1_2_0 + 0.1666666667*G3_0_2_0_1 + 0.1666666667*G3_0_2_0_2 - 0.1666666667*G3_0_2_1_0 - 0.1666666667*G3_0_2_2_0 - 0.1666666667*G3_1_0_0_1 - 0.1666666667*G3_1_0_0_2 + 0.1666666667*G3_1_0_1_0 + 0.1666666667*G3_1_0_2_0 - 0.1666666667*G3_2_0_0_1 - 0.1666666667*G3_2_0_0_2 + 0.1666666667*G3_2_0_1_0 + 0.1666666667*G3_2_0_2_0 - 0.1666666667*G4_0_1_0_1 - 0.1666666667*G4_0_1_0_2 + 0.1666666667*G4_0_1_1_0 + 0.1666666667*G4_0_1_2_0 - 0.1666666667*G4_0_2_0_1 - 0.1666666667*G4_0_2_0_2 + 0.1666666667*G4_0_2_1_0 + 0.1666666667*G4_0_2_2_0 + 0.1666666667*G4_1_0_0_1 + 0.1666666667*G4_1_0_0_2 - 0.1666666667*G4_1_0_1_0 - 0.1666666667*G4_1_0_2_0 + 0.1666666667*G4_2_0_0_1 + 0.1666666667*G4_2_0_0_2 - 0.1666666667*G4_2_0_1_0 - 0.1666666667*G4_2_0_2_0;
  }

};

/// This class defines the interface for the assembly of the global
/// tensor corresponding to a form with r + n arguments, that is, a
/// mapping
///
///     a : V1 x V2 x ... Vr x W1 x W2 x ... x Wn -> R
///
/// with arguments v1, v2, ..., vr, w1, w2, ..., wn. The rank r
/// global tensor A is defined by
///
///     A = a(V1, V2, ..., Vr, w1, w2, ..., wn),
///
/// where each argument Vj represents the application to the
/// sequence of basis functions of Vj and w1, w2, ..., wn are given
/// fixed functions (coefficients).

class VectorLaplaceGradCurlBilinearForm: public ufc::form
{
public:

  /// Constructor
  VectorLaplaceGradCurlBilinearForm() : ufc::form()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~VectorLaplaceGradCurlBilinearForm()
  {
    // Do nothing
  }

  /// Return a string identifying the form
  virtual const char* signature() const
  {
    return " | vi0[0]*vi1[0]*dX(0) + -(dXa0/dxb0)(dXa1/dxb0) | ((d/dXa0)vi0[0])*vi1[a1 + 1]*dX(0) + (dXa0/dxb0)(dXa1/dxb0) | vi0[a0 + 1]*((d/dXa1)vi1[0])*dX(0) + (dXa0/dx2)(dXa1/dxb0)(dXa2/dx2)(dXa3/dxb0) | ((d/dXa1)vi0[a0 + 1])*((d/dXa3)vi1[a2 + 1])*dX(0) + -(dXa0/dx2)(dXa1/dxb0)(dXa2/dxb0)(dXa3/dx2) | ((d/dXa1)vi0[a0 + 1])*((d/dXa3)vi1[a2 + 1])*dX(0) + -(dXa0/dxb0)(dXa1/dx2)(dXa2/dx2)(dXa3/dxb0) | ((d/dXa1)vi0[a0 + 1])*((d/dXa3)vi1[a2 + 1])*dX(0) + (dXa0/dxb0)(dXa1/dx2)(dXa2/dxb0)(dXa3/dx2) | ((d/dXa1)vi0[a0 + 1])*((d/dXa3)vi1[a2 + 1])*dX(0) + (dXa0/dx1)(dXa1/dx0)(dXa2/dx1)(dXa3/dx0) | ((d/dXa1)vi0[a0 + 1])*((d/dXa3)vi1[a2 + 1])*dX(0) + -(dXa0/dx1)(dXa1/dx0)(dXa2/dx0)(dXa3/dx1) | ((d/dXa1)vi0[a0 + 1])*((d/dXa3)vi1[a2 + 1])*dX(0) + -(dXa0/dx0)(dXa1/dx1)(dXa2/dx1)(dXa3/dx0) | ((d/dXa1)vi0[a0 + 1])*((d/dXa3)vi1[a2 + 1])*dX(0) + (dXa0/dx0)(dXa1/dx1)(dXa2/dx0)(dXa3/dx1) | ((d/dXa1)vi0[a0 + 1])*((d/dXa3)vi1[a2 + 1])*dX(0)";
  }

  /// Return the rank of the global tensor (r)
  virtual unsigned int rank() const
  {
    return 2;
  }

  /// Return the number of coefficients (n)
  virtual unsigned int num_coefficients() const
  {
    return 0;
  }

  /// Return the number of cell integrals
  virtual unsigned int num_cell_integrals() const
  {
    return 1;
  }
  
  /// Return the number of exterior facet integrals
  virtual unsigned int num_exterior_facet_integrals() const
  {
    return 0;
  }
  
  /// Return the number of interior facet integrals
  virtual unsigned int num_interior_facet_integrals() const
  {
    return 0;
  }
    
  /// Create a new finite element for argument function i
  virtual ufc::finite_element* create_finite_element(unsigned int i) const
  {
    switch ( i )
    {
    case 0:
      return new VectorLaplaceGradCurlBilinearForm_finite_element_0();
      break;
    case 1:
      return new VectorLaplaceGradCurlBilinearForm_finite_element_1();
      break;
    }
    return 0;
  }
  
  /// Create a new dof map for argument function i
  virtual ufc::dof_map* create_dof_map(unsigned int i) const
  {
    switch ( i )
    {
    case 0:
      return new VectorLaplaceGradCurlBilinearForm_dof_map_0();
      break;
    case 1:
      return new VectorLaplaceGradCurlBilinearForm_dof_map_1();
      break;
    }
    return 0;
  }

  /// Create a new cell integral on sub domain i
  virtual ufc::cell_integral* create_cell_integral(unsigned int i) const
  {
    return new VectorLaplaceGradCurlBilinearForm_cell_integral_0();
  }

  /// Create a new exterior facet integral on sub domain i
  virtual ufc::exterior_facet_integral* create_exterior_facet_integral(unsigned int i) const
  {
    return 0;
  }

  /// Create a new interior facet integral on sub domain i
  virtual ufc::interior_facet_integral* create_interior_facet_integral(unsigned int i) const
  {
    return 0;
  }

};

/// This class defines the interface for a finite element.

class VectorLaplaceGradCurlLinearForm_finite_element_0_0: public ufc::finite_element
{
public:

  /// Constructor
  VectorLaplaceGradCurlLinearForm_finite_element_0_0() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~VectorLaplaceGradCurlLinearForm_finite_element_0_0()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "Lagrange finite element of degree 1 on a tetrahedron";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::tetrahedron;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 4;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 0;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 1;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-09)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-09)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Reset values
    *values = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
    const double scalings_z_0 = 1;
    const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    const double psitilde_a_1 = x;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    const double psitilde_bs_0_1 = 1.5*y + 0.5;
    const double psitilde_bs_1_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    const double psitilde_cs_00_1 = 2*z + 1;
    const double psitilde_cs_01_0 = 1;
    const double psitilde_cs_10_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.8660254038*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    const double basisvalue1 = 2.738612788*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
    const double basisvalue2 = 1.58113883*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
    const double basisvalue3 = 1.118033989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    
    // Table(s) of coefficients
    const static double coefficients0[4][4] = \
    {{0.2886751346, -0.1825741858, -0.1054092553, -0.07453559925},
    {0.2886751346, 0.1825741858, -0.1054092553, -0.07453559925},
    {0.2886751346, 0, 0.2108185107, -0.07453559925},
    {0.2886751346, 0, 0, 0.2236067977}};
    
    // Extract relevant coefficients
    const double coeff0_0 = coefficients0[dof][0];
    const double coeff0_1 = coefficients0[dof][1];
    const double coeff0_2 = coefficients0[dof][2];
    const double coeff0_3 = coefficients0[dof][3];
    
    // Compute value(s)
    *values = coeff0_0*basisvalue0 + coeff0_1*basisvalue1 + coeff0_2*basisvalue2 + coeff0_3*basisvalue3;
  }

  /// Evaluate all basis functions at given point in cell
  virtual void evaluate_basis_all(double* values,
                                  const double* coordinates,
                                  const ufc::cell& c) const
  {
    throw std::runtime_error("The vectorised version of evaluate_basis() is not yet implemented.");
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-09)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-09)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Compute number of derivatives
    unsigned int num_derivatives = 1;
    
    for (unsigned int j = 0; j < n; j++)
      num_derivatives *= 3;
    
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      combinations[j] = new unsigned int [n];
      for (unsigned int k = 0; k < n; k++)
        combinations[j][k] = 0;
    }
        
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 2)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian
    const double Jinv[3][3] ={{d00 / detJ, d10 / detJ, d20 / detJ}, {d01 / detJ, d11 / detJ, d21 / detJ}, {d02 / detJ, d12 / detJ, d22 / detJ}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values
    for (unsigned int j = 0; j < 1*num_derivatives; j++)
      values[j] = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
    const double scalings_z_0 = 1;
    const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    const double psitilde_a_1 = x;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    const double psitilde_bs_0_1 = 1.5*y + 0.5;
    const double psitilde_bs_1_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    const double psitilde_cs_00_1 = 2*z + 1;
    const double psitilde_cs_01_0 = 1;
    const double psitilde_cs_10_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.8660254038*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    const double basisvalue1 = 2.738612788*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
    const double basisvalue2 = 1.58113883*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
    const double basisvalue3 = 1.118033989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    
    // Table(s) of coefficients
    const static double coefficients0[4][4] = \
    {{0.2886751346, -0.1825741858, -0.1054092553, -0.07453559925},
    {0.2886751346, 0.1825741858, -0.1054092553, -0.07453559925},
    {0.2886751346, 0, 0.2108185107, -0.07453559925},
    {0.2886751346, 0, 0, 0.2236067977}};
    
    // Interesting (new) part
    // Tables of derivatives of the polynomial base (transpose)
    const static double dmats0[4][4] = \
    {{0, 0, 0, 0},
    {6.32455532, 0, 0, 0},
    {0, 0, 0, 0},
    {0, 0, 0, 0}};
    
    const static double dmats1[4][4] = \
    {{0, 0, 0, 0},
    {3.16227766, 0, 0, 0},
    {5.477225575, 0, 0, 0},
    {0, 0, 0, 0}};
    
    const static double dmats2[4][4] = \
    {{0, 0, 0, 0},
    {3.16227766, 0, 0, 0},
    {1.825741858, 0, 0, 0},
    {5.163977795, 0, 0, 0}};
    
    // Compute reference derivatives
    // Declare pointer to array of derivatives on FIAT element
    double *derivatives = new double [num_derivatives];
    
    // Declare coefficients
    double coeff0_0 = 0;
    double coeff0_1 = 0;
    double coeff0_2 = 0;
    double coeff0_3 = 0;
    
    // Declare new coefficients
    double new_coeff0_0 = 0;
    double new_coeff0_1 = 0;
    double new_coeff0_2 = 0;
    double new_coeff0_3 = 0;
    
    // Loop possible derivatives
    for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
    {
      // Get values from coefficients array
      new_coeff0_0 = coefficients0[dof][0];
      new_coeff0_1 = coefficients0[dof][1];
      new_coeff0_2 = coefficients0[dof][2];
      new_coeff0_3 = coefficients0[dof][3];
    
      // Loop derivative order
      for (unsigned int j = 0; j < n; j++)
      {
        // Update old coefficients
        coeff0_0 = new_coeff0_0;
        coeff0_1 = new_coeff0_1;
        coeff0_2 = new_coeff0_2;
        coeff0_3 = new_coeff0_3;
    
        if(combinations[deriv_num][j] == 0)
        {
          new_coeff0_0 = coeff0_0*dmats0[0][0] + coeff0_1*dmats0[1][0] + coeff0_2*dmats0[2][0] + coeff0_3*dmats0[3][0];
          new_coeff0_1 = coeff0_0*dmats0[0][1] + coeff0_1*dmats0[1][1] + coeff0_2*dmats0[2][1] + coeff0_3*dmats0[3][1];
          new_coeff0_2 = coeff0_0*dmats0[0][2] + coeff0_1*dmats0[1][2] + coeff0_2*dmats0[2][2] + coeff0_3*dmats0[3][2];
          new_coeff0_3 = coeff0_0*dmats0[0][3] + coeff0_1*dmats0[1][3] + coeff0_2*dmats0[2][3] + coeff0_3*dmats0[3][3];
        }
        if(combinations[deriv_num][j] == 1)
        {
          new_coeff0_0 = coeff0_0*dmats1[0][0] + coeff0_1*dmats1[1][0] + coeff0_2*dmats1[2][0] + coeff0_3*dmats1[3][0];
          new_coeff0_1 = coeff0_0*dmats1[0][1] + coeff0_1*dmats1[1][1] + coeff0_2*dmats1[2][1] + coeff0_3*dmats1[3][1];
          new_coeff0_2 = coeff0_0*dmats1[0][2] + coeff0_1*dmats1[1][2] + coeff0_2*dmats1[2][2] + coeff0_3*dmats1[3][2];
          new_coeff0_3 = coeff0_0*dmats1[0][3] + coeff0_1*dmats1[1][3] + coeff0_2*dmats1[2][3] + coeff0_3*dmats1[3][3];
        }
        if(combinations[deriv_num][j] == 2)
        {
          new_coeff0_0 = coeff0_0*dmats2[0][0] + coeff0_1*dmats2[1][0] + coeff0_2*dmats2[2][0] + coeff0_3*dmats2[3][0];
          new_coeff0_1 = coeff0_0*dmats2[0][1] + coeff0_1*dmats2[1][1] + coeff0_2*dmats2[2][1] + coeff0_3*dmats2[3][1];
          new_coeff0_2 = coeff0_0*dmats2[0][2] + coeff0_1*dmats2[1][2] + coeff0_2*dmats2[2][2] + coeff0_3*dmats2[3][2];
          new_coeff0_3 = coeff0_0*dmats2[0][3] + coeff0_1*dmats2[1][3] + coeff0_2*dmats2[2][3] + coeff0_3*dmats2[3][3];
        }
    
      }
      // Compute derivatives on reference element as dot product of coefficients and basisvalues
      derivatives[deriv_num] = new_coeff0_0*basisvalue0 + new_coeff0_1*basisvalue1 + new_coeff0_2*basisvalue2 + new_coeff0_3*basisvalue3;
    }
    
    // Transform derivatives back to physical element
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        values[row] += transform[row][col]*derivatives[col];
      }
    }
    // Delete pointer to array of derivatives on FIAT element
    delete [] derivatives;
    
    // Delete pointer to array of combinations of derivatives and transform
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      delete [] combinations[row];
      delete [] transform[row];
    }
    
    delete [] combinations;
    delete [] transform;
  }

  /// Evaluate order n derivatives of all basis functions at given point in cell
  virtual void evaluate_basis_derivatives_all(unsigned int n,
                                              double* values,
                                              const double* coordinates,
                                              const ufc::cell& c) const
  {
    throw std::runtime_error("The vectorised version of evaluate_basis_derivatives() is not yet implemented.");
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    // The reference points, direction and weights:
    const static double X[4][1][3] = {{{0, 0, 0}}, {{1, 0, 0}}, {{0, 1, 0}}, {{0, 0, 1}}};
    const static double W[4][1] = {{1}, {1}, {1}, {1}};
    const static double D[4][1][1] = {{{1}}, {{1}}, {{1}}, {{1}}};
    
    const double * const * x = c.coordinates;
    double result = 0.0;
    // Iterate over the points:
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0][0] - X[i][0][1] - X[i][0][2];
    const double w1 = X[i][0][0];
    const double w2 = X[i][0][1];
    const double w3 = X[i][0][2];
    
    // Compute affine mapping y = F(X)
    double y[3];
    y[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0] + w3*x[3][0];
    y[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1] + w3*x[3][1];
    y[2] = w0*x[0][2] + w1*x[1][2] + w2*x[2][2] + w3*x[3][2];
    
    // Evaluate function at physical points
    double values[1];
    f.evaluate(values, y, c);
    
    // Map function values using appropriate mapping
    // Affine map: Do nothing
    
    // Note that we do not map the weights (yet).
    
    // Take directional components
    for(int k = 0; k < 1; k++)
      result += values[k]*D[i][0][k];
    // Multiply by weights 
    result *= W[i][0];
    
    return result;
  }

  /// Evaluate linear functionals for all dofs on the function f
  virtual void evaluate_dofs(double* values,
                             const ufc::function& f,
                             const ufc::cell& c) const
  {
    throw std::runtime_error("Not implemented (introduced in UFC v1.1).");
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    // Evaluate at vertices and use affine mapping
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[1];
    vertex_values[2] = dof_values[2];
    vertex_values[3] = dof_values[3];
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 1;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    return new VectorLaplaceGradCurlLinearForm_finite_element_0_0();
  }

};

/// This class defines the interface for a finite element.

class VectorLaplaceGradCurlLinearForm_finite_element_0_1: public ufc::finite_element
{
public:

  /// Constructor
  VectorLaplaceGradCurlLinearForm_finite_element_0_1() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~VectorLaplaceGradCurlLinearForm_finite_element_0_1()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "Nedelec finite element of degree 1 on a tetrahedron";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::tetrahedron;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 6;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 1;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 3;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    const double Jinv_00 = d00 / detJ;
    const double Jinv_01 = d10 / detJ;
    const double Jinv_02 = d20 / detJ;
    const double Jinv_10 = d01 / detJ;
    const double Jinv_11 = d11 / detJ;
    const double Jinv_12 = d21 / detJ;
    const double Jinv_20 = d02 / detJ;
    const double Jinv_21 = d12 / detJ;
    const double Jinv_22 = d22 / detJ;
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-09)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-09)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Reset values
    values[0] = 0;
    values[1] = 0;
    values[2] = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
    const double scalings_z_0 = 1;
    const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    const double psitilde_a_1 = x;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    const double psitilde_bs_0_1 = 1.5*y + 0.5;
    const double psitilde_bs_1_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    const double psitilde_cs_00_1 = 2*z + 1;
    const double psitilde_cs_01_0 = 1;
    const double psitilde_cs_10_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.8660254038*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    const double basisvalue1 = 2.738612788*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
    const double basisvalue2 = 1.58113883*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
    const double basisvalue3 = 1.118033989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    
    // Table(s) of coefficients
    const static double coefficients0[6][4] = \
    {{0, 0, 0, 0},
    {-0.2886751346, 0, 0, -0.2236067977},
    {-0.2886751346, 0, -0.2108185107, 0.07453559925},
    {0.2886751346, 0, 0, 0.2236067977},
    {0.2886751346, 0, 0.2108185107, -0.07453559925},
    {0.5773502692, 0, -0.2108185107, -0.1490711985}};
    
    const static double coefficients1[6][4] = \
    {{-0.2886751346, 0, 0, -0.2236067977},
    {0, 0, 0, 0},
    {0.2886751346, 0.1825741858, -0.1054092553, -0.07453559925},
    {0.2886751346, 0, 0, 0.2236067977},
    {0.5773502692, -0.1825741858, 0.1054092553, -0.1490711985},
    {0.2886751346, 0.1825741858, -0.1054092553, -0.07453559925}};
    
    const static double coefficients2[6][4] = \
    {{0.2886751346, 0, 0.2108185107, -0.07453559925},
    {0.2886751346, 0.1825741858, -0.1054092553, -0.07453559925},
    {0, 0, 0, 0},
    {0.5773502692, -0.1825741858, -0.1054092553, 0.1490711985},
    {0.2886751346, 0, 0.2108185107, -0.07453559925},
    {0.2886751346, 0.1825741858, -0.1054092553, -0.07453559925}};
    
    // Extract relevant coefficients
    const double coeff0_0 = coefficients0[dof][0];
    const double coeff0_1 = coefficients0[dof][1];
    const double coeff0_2 = coefficients0[dof][2];
    const double coeff0_3 = coefficients0[dof][3];
    const double coeff1_0 = coefficients1[dof][0];
    const double coeff1_1 = coefficients1[dof][1];
    const double coeff1_2 = coefficients1[dof][2];
    const double coeff1_3 = coefficients1[dof][3];
    const double coeff2_0 = coefficients2[dof][0];
    const double coeff2_1 = coefficients2[dof][1];
    const double coeff2_2 = coefficients2[dof][2];
    const double coeff2_3 = coefficients2[dof][3];
    
    // Compute value(s)
    const double tmp0_0 = coeff0_0*basisvalue0 + coeff0_1*basisvalue1 + coeff0_2*basisvalue2 + coeff0_3*basisvalue3;
    const double tmp0_1 = coeff1_0*basisvalue0 + coeff1_1*basisvalue1 + coeff1_2*basisvalue2 + coeff1_3*basisvalue3;
    const double tmp0_2 = coeff2_0*basisvalue0 + coeff2_1*basisvalue1 + coeff2_2*basisvalue2 + coeff2_3*basisvalue3;
    // Using covariant Piola transform to map values back to the physical element
    values[0] = (Jinv_00*tmp0_0 + Jinv_10*tmp0_1 + Jinv_20*tmp0_2);
    values[1] = (Jinv_01*tmp0_0 + Jinv_11*tmp0_1 + Jinv_21*tmp0_2);
    values[2] = (Jinv_02*tmp0_0 + Jinv_12*tmp0_1 + Jinv_22*tmp0_2);
  }

  /// Evaluate all basis functions at given point in cell
  virtual void evaluate_basis_all(double* values,
                                  const double* coordinates,
                                  const ufc::cell& c) const
  {
    throw std::runtime_error("The vectorised version of evaluate_basis() is not yet implemented.");
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    const double Jinv_00 = d00 / detJ;
    const double Jinv_01 = d10 / detJ;
    const double Jinv_02 = d20 / detJ;
    const double Jinv_10 = d01 / detJ;
    const double Jinv_11 = d11 / detJ;
    const double Jinv_12 = d21 / detJ;
    const double Jinv_20 = d02 / detJ;
    const double Jinv_21 = d12 / detJ;
    const double Jinv_22 = d22 / detJ;
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-09)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-09)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Compute number of derivatives
    unsigned int num_derivatives = 1;
    
    for (unsigned int j = 0; j < n; j++)
      num_derivatives *= 3;
    
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      combinations[j] = new unsigned int [n];
      for (unsigned int k = 0; k < n; k++)
        combinations[j][k] = 0;
    }
        
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 2)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian
    const double Jinv[3][3] ={{d00 / detJ, d10 / detJ, d20 / detJ}, {d01 / detJ, d11 / detJ, d21 / detJ}, {d02 / detJ, d12 / detJ, d22 / detJ}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values
    for (unsigned int j = 0; j < 3*num_derivatives; j++)
      values[j] = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
    const double scalings_z_0 = 1;
    const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    const double psitilde_a_1 = x;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    const double psitilde_bs_0_1 = 1.5*y + 0.5;
    const double psitilde_bs_1_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    const double psitilde_cs_00_1 = 2*z + 1;
    const double psitilde_cs_01_0 = 1;
    const double psitilde_cs_10_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.8660254038*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    const double basisvalue1 = 2.738612788*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
    const double basisvalue2 = 1.58113883*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
    const double basisvalue3 = 1.118033989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    
    // Table(s) of coefficients
    const static double coefficients0[6][4] = \
    {{0, 0, 0, 0},
    {-0.2886751346, 0, 0, -0.2236067977},
    {-0.2886751346, 0, -0.2108185107, 0.07453559925},
    {0.2886751346, 0, 0, 0.2236067977},
    {0.2886751346, 0, 0.2108185107, -0.07453559925},
    {0.5773502692, 0, -0.2108185107, -0.1490711985}};
    
    const static double coefficients1[6][4] = \
    {{-0.2886751346, 0, 0, -0.2236067977},
    {0, 0, 0, 0},
    {0.2886751346, 0.1825741858, -0.1054092553, -0.07453559925},
    {0.2886751346, 0, 0, 0.2236067977},
    {0.5773502692, -0.1825741858, 0.1054092553, -0.1490711985},
    {0.2886751346, 0.1825741858, -0.1054092553, -0.07453559925}};
    
    const static double coefficients2[6][4] = \
    {{0.2886751346, 0, 0.2108185107, -0.07453559925},
    {0.2886751346, 0.1825741858, -0.1054092553, -0.07453559925},
    {0, 0, 0, 0},
    {0.5773502692, -0.1825741858, -0.1054092553, 0.1490711985},
    {0.2886751346, 0, 0.2108185107, -0.07453559925},
    {0.2886751346, 0.1825741858, -0.1054092553, -0.07453559925}};
    
    // Interesting (new) part
    // Tables of derivatives of the polynomial base (transpose)
    const static double dmats0[4][4] = \
    {{0, 0, 0, 0},
    {6.32455532, 0, 0, 0},
    {0, 0, 0, 0},
    {0, 0, 0, 0}};
    
    const static double dmats1[4][4] = \
    {{0, 0, 0, 0},
    {3.16227766, 0, 0, 0},
    {5.477225575, 0, 0, 0},
    {0, 0, 0, 0}};
    
    const static double dmats2[4][4] = \
    {{0, 0, 0, 0},
    {3.16227766, 0, 0, 0},
    {1.825741858, 0, 0, 0},
    {5.163977795, 0, 0, 0}};
    
    // Compute reference derivatives
    // Declare pointer to array of derivatives on FIAT element
    double *derivatives = new double [3*num_derivatives];
    
    // Declare coefficients
    double coeff0_0 = 0;
    double coeff0_1 = 0;
    double coeff0_2 = 0;
    double coeff0_3 = 0;
    double coeff1_0 = 0;
    double coeff1_1 = 0;
    double coeff1_2 = 0;
    double coeff1_3 = 0;
    double coeff2_0 = 0;
    double coeff2_1 = 0;
    double coeff2_2 = 0;
    double coeff2_3 = 0;
    
    // Declare new coefficients
    double new_coeff0_0 = 0;
    double new_coeff0_1 = 0;
    double new_coeff0_2 = 0;
    double new_coeff0_3 = 0;
    double new_coeff1_0 = 0;
    double new_coeff1_1 = 0;
    double new_coeff1_2 = 0;
    double new_coeff1_3 = 0;
    double new_coeff2_0 = 0;
    double new_coeff2_1 = 0;
    double new_coeff2_2 = 0;
    double new_coeff2_3 = 0;
    
    // Loop possible derivatives
    for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
    {
      // Get values from coefficients array
      new_coeff0_0 = coefficients0[dof][0];
      new_coeff0_1 = coefficients0[dof][1];
      new_coeff0_2 = coefficients0[dof][2];
      new_coeff0_3 = coefficients0[dof][3];
      new_coeff1_0 = coefficients1[dof][0];
      new_coeff1_1 = coefficients1[dof][1];
      new_coeff1_2 = coefficients1[dof][2];
      new_coeff1_3 = coefficients1[dof][3];
      new_coeff2_0 = coefficients2[dof][0];
      new_coeff2_1 = coefficients2[dof][1];
      new_coeff2_2 = coefficients2[dof][2];
      new_coeff2_3 = coefficients2[dof][3];
    
      // Loop derivative order
      for (unsigned int j = 0; j < n; j++)
      {
        // Update old coefficients
        coeff0_0 = new_coeff0_0;
        coeff0_1 = new_coeff0_1;
        coeff0_2 = new_coeff0_2;
        coeff0_3 = new_coeff0_3;
        coeff1_0 = new_coeff1_0;
        coeff1_1 = new_coeff1_1;
        coeff1_2 = new_coeff1_2;
        coeff1_3 = new_coeff1_3;
        coeff2_0 = new_coeff2_0;
        coeff2_1 = new_coeff2_1;
        coeff2_2 = new_coeff2_2;
        coeff2_3 = new_coeff2_3;
    
        if(combinations[deriv_num][j] == 0)
        {
          new_coeff0_0 = coeff0_0*dmats0[0][0] + coeff0_1*dmats0[1][0] + coeff0_2*dmats0[2][0] + coeff0_3*dmats0[3][0];
          new_coeff0_1 = coeff0_0*dmats0[0][1] + coeff0_1*dmats0[1][1] + coeff0_2*dmats0[2][1] + coeff0_3*dmats0[3][1];
          new_coeff0_2 = coeff0_0*dmats0[0][2] + coeff0_1*dmats0[1][2] + coeff0_2*dmats0[2][2] + coeff0_3*dmats0[3][2];
          new_coeff0_3 = coeff0_0*dmats0[0][3] + coeff0_1*dmats0[1][3] + coeff0_2*dmats0[2][3] + coeff0_3*dmats0[3][3];
          new_coeff1_0 = coeff1_0*dmats0[0][0] + coeff1_1*dmats0[1][0] + coeff1_2*dmats0[2][0] + coeff1_3*dmats0[3][0];
          new_coeff1_1 = coeff1_0*dmats0[0][1] + coeff1_1*dmats0[1][1] + coeff1_2*dmats0[2][1] + coeff1_3*dmats0[3][1];
          new_coeff1_2 = coeff1_0*dmats0[0][2] + coeff1_1*dmats0[1][2] + coeff1_2*dmats0[2][2] + coeff1_3*dmats0[3][2];
          new_coeff1_3 = coeff1_0*dmats0[0][3] + coeff1_1*dmats0[1][3] + coeff1_2*dmats0[2][3] + coeff1_3*dmats0[3][3];
          new_coeff2_0 = coeff2_0*dmats0[0][0] + coeff2_1*dmats0[1][0] + coeff2_2*dmats0[2][0] + coeff2_3*dmats0[3][0];
          new_coeff2_1 = coeff2_0*dmats0[0][1] + coeff2_1*dmats0[1][1] + coeff2_2*dmats0[2][1] + coeff2_3*dmats0[3][1];
          new_coeff2_2 = coeff2_0*dmats0[0][2] + coeff2_1*dmats0[1][2] + coeff2_2*dmats0[2][2] + coeff2_3*dmats0[3][2];
          new_coeff2_3 = coeff2_0*dmats0[0][3] + coeff2_1*dmats0[1][3] + coeff2_2*dmats0[2][3] + coeff2_3*dmats0[3][3];
        }
        if(combinations[deriv_num][j] == 1)
        {
          new_coeff0_0 = coeff0_0*dmats1[0][0] + coeff0_1*dmats1[1][0] + coeff0_2*dmats1[2][0] + coeff0_3*dmats1[3][0];
          new_coeff0_1 = coeff0_0*dmats1[0][1] + coeff0_1*dmats1[1][1] + coeff0_2*dmats1[2][1] + coeff0_3*dmats1[3][1];
          new_coeff0_2 = coeff0_0*dmats1[0][2] + coeff0_1*dmats1[1][2] + coeff0_2*dmats1[2][2] + coeff0_3*dmats1[3][2];
          new_coeff0_3 = coeff0_0*dmats1[0][3] + coeff0_1*dmats1[1][3] + coeff0_2*dmats1[2][3] + coeff0_3*dmats1[3][3];
          new_coeff1_0 = coeff1_0*dmats1[0][0] + coeff1_1*dmats1[1][0] + coeff1_2*dmats1[2][0] + coeff1_3*dmats1[3][0];
          new_coeff1_1 = coeff1_0*dmats1[0][1] + coeff1_1*dmats1[1][1] + coeff1_2*dmats1[2][1] + coeff1_3*dmats1[3][1];
          new_coeff1_2 = coeff1_0*dmats1[0][2] + coeff1_1*dmats1[1][2] + coeff1_2*dmats1[2][2] + coeff1_3*dmats1[3][2];
          new_coeff1_3 = coeff1_0*dmats1[0][3] + coeff1_1*dmats1[1][3] + coeff1_2*dmats1[2][3] + coeff1_3*dmats1[3][3];
          new_coeff2_0 = coeff2_0*dmats1[0][0] + coeff2_1*dmats1[1][0] + coeff2_2*dmats1[2][0] + coeff2_3*dmats1[3][0];
          new_coeff2_1 = coeff2_0*dmats1[0][1] + coeff2_1*dmats1[1][1] + coeff2_2*dmats1[2][1] + coeff2_3*dmats1[3][1];
          new_coeff2_2 = coeff2_0*dmats1[0][2] + coeff2_1*dmats1[1][2] + coeff2_2*dmats1[2][2] + coeff2_3*dmats1[3][2];
          new_coeff2_3 = coeff2_0*dmats1[0][3] + coeff2_1*dmats1[1][3] + coeff2_2*dmats1[2][3] + coeff2_3*dmats1[3][3];
        }
        if(combinations[deriv_num][j] == 2)
        {
          new_coeff0_0 = coeff0_0*dmats2[0][0] + coeff0_1*dmats2[1][0] + coeff0_2*dmats2[2][0] + coeff0_3*dmats2[3][0];
          new_coeff0_1 = coeff0_0*dmats2[0][1] + coeff0_1*dmats2[1][1] + coeff0_2*dmats2[2][1] + coeff0_3*dmats2[3][1];
          new_coeff0_2 = coeff0_0*dmats2[0][2] + coeff0_1*dmats2[1][2] + coeff0_2*dmats2[2][2] + coeff0_3*dmats2[3][2];
          new_coeff0_3 = coeff0_0*dmats2[0][3] + coeff0_1*dmats2[1][3] + coeff0_2*dmats2[2][3] + coeff0_3*dmats2[3][3];
          new_coeff1_0 = coeff1_0*dmats2[0][0] + coeff1_1*dmats2[1][0] + coeff1_2*dmats2[2][0] + coeff1_3*dmats2[3][0];
          new_coeff1_1 = coeff1_0*dmats2[0][1] + coeff1_1*dmats2[1][1] + coeff1_2*dmats2[2][1] + coeff1_3*dmats2[3][1];
          new_coeff1_2 = coeff1_0*dmats2[0][2] + coeff1_1*dmats2[1][2] + coeff1_2*dmats2[2][2] + coeff1_3*dmats2[3][2];
          new_coeff1_3 = coeff1_0*dmats2[0][3] + coeff1_1*dmats2[1][3] + coeff1_2*dmats2[2][3] + coeff1_3*dmats2[3][3];
          new_coeff2_0 = coeff2_0*dmats2[0][0] + coeff2_1*dmats2[1][0] + coeff2_2*dmats2[2][0] + coeff2_3*dmats2[3][0];
          new_coeff2_1 = coeff2_0*dmats2[0][1] + coeff2_1*dmats2[1][1] + coeff2_2*dmats2[2][1] + coeff2_3*dmats2[3][1];
          new_coeff2_2 = coeff2_0*dmats2[0][2] + coeff2_1*dmats2[1][2] + coeff2_2*dmats2[2][2] + coeff2_3*dmats2[3][2];
          new_coeff2_3 = coeff2_0*dmats2[0][3] + coeff2_1*dmats2[1][3] + coeff2_2*dmats2[2][3] + coeff2_3*dmats2[3][3];
        }
    
      }
      // Compute derivatives on reference element as dot product of coefficients and basisvalues
      // Correct values by the covariant Piola transform
      const double tmp0_0 = new_coeff0_0*basisvalue0 + new_coeff0_1*basisvalue1 + new_coeff0_2*basisvalue2 + new_coeff0_3*basisvalue3;
      const double tmp0_1 = new_coeff1_0*basisvalue0 + new_coeff1_1*basisvalue1 + new_coeff1_2*basisvalue2 + new_coeff1_3*basisvalue3;
      const double tmp0_2 = new_coeff2_0*basisvalue0 + new_coeff2_1*basisvalue1 + new_coeff2_2*basisvalue2 + new_coeff2_3*basisvalue3;
      derivatives[deriv_num] = (Jinv_00*tmp0_0 + Jinv_10*tmp0_1 + Jinv_20*tmp0_2);
      derivatives[num_derivatives + deriv_num] = (Jinv_01*tmp0_0 + Jinv_11*tmp0_1 + Jinv_21*tmp0_2);
      derivatives[2*num_derivatives + deriv_num] = (Jinv_02*tmp0_0 + Jinv_12*tmp0_1 + Jinv_22*tmp0_2);
    }
    
    // Transform derivatives back to physical element
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        values[row] += transform[row][col]*derivatives[col];
        values[num_derivatives + row] += transform[row][col]*derivatives[num_derivatives + col];
        values[2*num_derivatives + row] += transform[row][col]*derivatives[2*num_derivatives + col];
      }
    }
    // Delete pointer to array of derivatives on FIAT element
    delete [] derivatives;
    
    // Delete pointer to array of combinations of derivatives and transform
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      delete [] combinations[row];
      delete [] transform[row];
    }
    
    delete [] combinations;
    delete [] transform;
  }

  /// Evaluate order n derivatives of all basis functions at given point in cell
  virtual void evaluate_basis_derivatives_all(unsigned int n,
                                              double* values,
                                              const double* coordinates,
                                              const ufc::cell& c) const
  {
    throw std::runtime_error("The vectorised version of evaluate_basis_derivatives() is not yet implemented.");
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    // The reference points, direction and weights:
    const static double X[6][1][3] = {{{0, 0.5, 0.5}}, {{0.5, 0, 0.5}}, {{0.5, 0.5, 0}}, {{0, 0, 0.5}}, {{0, 0.5, 0}}, {{0.5, 0, 0}}};
    const static double W[6][1] = {{1}, {1}, {1}, {1}, {1}, {1}};
    const static double D[6][1][3] = {{{0, -1, 1}}, {{-1, 0, 1}}, {{-1, 1, 0}}, {{0, 0, 1}}, {{0, 1, 0}}, {{1, 0, 0}}};
    
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = x[1][0] - x[0][0];
    const double J_01 = x[2][0] - x[0][0];
    const double J_02 = x[3][0] - x[0][0];
    const double J_10 = x[1][1] - x[0][1];
    const double J_11 = x[2][1] - x[0][1];
    const double J_12 = x[3][1] - x[0][1];
    const double J_20 = x[1][2] - x[0][2];
    const double J_21 = x[2][2] - x[0][2];
    const double J_22 = x[3][2] - x[0][2];
      
    // Compute sub determinants
    const double d_00 = J_11*J_22 - J_12*J_21;
    const double d_01 = J_12*J_20 - J_10*J_22;
    const double d_02 = J_10*J_21 - J_11*J_20;
    
    const double d_10 = J_02*J_21 - J_01*J_22;
    const double d_11 = J_00*J_22 - J_02*J_20;
    const double d_12 = J_01*J_20 - J_00*J_21;
    
    const double d_20 = J_01*J_12 - J_02*J_11;
    const double d_21 = J_02*J_10 - J_00*J_12;
    const double d_22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d_00 + J_10*d_10 + J_20*d_20;
      
    // Compute inverse of Jacobian
    const double Jinv_00 = d_00 / detJ;
    const double Jinv_01 = d_10 / detJ;
    const double Jinv_02 = d_20 / detJ;
    const double Jinv_10 = d_01 / detJ;
    const double Jinv_11 = d_11 / detJ;
    const double Jinv_12 = d_21 / detJ;
    const double Jinv_20 = d_02 / detJ;
    const double Jinv_21 = d_12 / detJ;
    const double Jinv_22 = d_22 / detJ;
    
    double copyofvalues[3];
    double result = 0.0;
    // Iterate over the points:
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0][0] - X[i][0][1] - X[i][0][2];
    const double w1 = X[i][0][0];
    const double w2 = X[i][0][1];
    const double w3 = X[i][0][2];
    
    // Compute affine mapping y = F(X)
    double y[3];
    y[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0] + w3*x[3][0];
    y[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1] + w3*x[3][1];
    y[2] = w0*x[0][2] + w1*x[1][2] + w2*x[2][2] + w3*x[3][2];
    
    // Evaluate function at physical points
    double values[3];
    f.evaluate(values, y, c);
    
    // Map function values using appropriate mapping
    // Copy old values:
    copyofvalues[0] = values[0];
    copyofvalues[1] = values[1];
    copyofvalues[2] = values[2];
    // Do the inverse of curl piola 
    values[0] = J_00*copyofvalues[0]+J_10*copyofvalues[1]+J_20*copyofvalues[2];
    values[1] = J_01*copyofvalues[0]+J_11*copyofvalues[1]+J_21*copyofvalues[2];
    values[2] = J_02*copyofvalues[0]+J_12*copyofvalues[1]+J_22*copyofvalues[2];
    
    // Note that we do not map the weights (yet).
    
    // Take directional components
    for(int k = 0; k < 3; k++)
      result += values[k]*D[i][0][k];
    // Multiply by weights 
    result *= W[i][0];
    
    return result;
  }

  /// Evaluate linear functionals for all dofs on the function f
  virtual void evaluate_dofs(double* values,
                             const ufc::function& f,
                             const ufc::cell& c) const
  {
    throw std::runtime_error("Not implemented (introduced in UFC v1.1).");
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = x[1][0] - x[0][0];
    const double J_01 = x[2][0] - x[0][0];
    const double J_02 = x[3][0] - x[0][0];
    const double J_10 = x[1][1] - x[0][1];
    const double J_11 = x[2][1] - x[0][1];
    const double J_12 = x[3][1] - x[0][1];
    const double J_20 = x[1][2] - x[0][2];
    const double J_21 = x[2][2] - x[0][2];
    const double J_22 = x[3][2] - x[0][2];
      
    // Compute sub determinants
    const double d_00 = J_11*J_22 - J_12*J_21;
    const double d_01 = J_12*J_20 - J_10*J_22;
    const double d_02 = J_10*J_21 - J_11*J_20;
    
    const double d_10 = J_02*J_21 - J_01*J_22;
    const double d_11 = J_00*J_22 - J_02*J_20;
    const double d_12 = J_01*J_20 - J_00*J_21;
    
    const double d_20 = J_01*J_12 - J_02*J_11;
    const double d_21 = J_02*J_10 - J_00*J_12;
    const double d_22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d_00 + J_10*d_10 + J_20*d_20;
      
    // Compute inverse of Jacobian
    const double Jinv_00 = d_00 / detJ;
    const double Jinv_01 = d_10 / detJ;
    const double Jinv_02 = d_20 / detJ;
    const double Jinv_10 = d_01 / detJ;
    const double Jinv_11 = d_11 / detJ;
    const double Jinv_12 = d_21 / detJ;
    const double Jinv_20 = d_02 / detJ;
    const double Jinv_21 = d_12 / detJ;
    const double Jinv_22 = d_22 / detJ;
    // Evaluate at vertices and use Piola mapping
    vertex_values[0] = (dof_values[3]*Jinv_20 + dof_values[4]*Jinv_10 + dof_values[5]*Jinv_00);
    vertex_values[1] = (dof_values[1]*Jinv_20 + dof_values[2]*Jinv_10 + dof_values[5]*(Jinv_00 + Jinv_10 + Jinv_20));
    vertex_values[2] = (dof_values[0]*Jinv_20 + dof_values[2]*Jinv_00 + dof_values[4]*(Jinv_00 + Jinv_10 + Jinv_20));
    vertex_values[3] = (dof_values[0]*Jinv_10 + dof_values[1]*Jinv_00 + dof_values[3]*(Jinv_00 + Jinv_10 + Jinv_20));
    vertex_values[4] = (dof_values[3]*Jinv_21 + dof_values[4]*Jinv_11 + dof_values[5]*Jinv_01);
    vertex_values[5] = (dof_values[1]*Jinv_21 + dof_values[2]*Jinv_11 + dof_values[5]*(Jinv_01 + Jinv_11 + Jinv_21));
    vertex_values[6] = (dof_values[0]*Jinv_21 + dof_values[2]*Jinv_01 + dof_values[4]*(Jinv_01 + Jinv_11 + Jinv_21));
    vertex_values[7] = (dof_values[0]*Jinv_11 + dof_values[1]*Jinv_01 + dof_values[3]*(Jinv_01 + Jinv_11 + Jinv_21));
    vertex_values[8] = (dof_values[3]*Jinv_22 + dof_values[4]*Jinv_12 + dof_values[5]*Jinv_02);
    vertex_values[9] = (dof_values[1]*Jinv_22 + dof_values[2]*Jinv_12 + dof_values[5]*(Jinv_02 + Jinv_12 + Jinv_22));
    vertex_values[10] = (dof_values[0]*Jinv_22 + dof_values[2]*Jinv_02 + dof_values[4]*(Jinv_02 + Jinv_12 + Jinv_22));
    vertex_values[11] = (dof_values[0]*Jinv_12 + dof_values[1]*Jinv_02 + dof_values[3]*(Jinv_02 + Jinv_12 + Jinv_22));
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 1;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    return new VectorLaplaceGradCurlLinearForm_finite_element_0_1();
  }

};

/// This class defines the interface for a finite element.

class VectorLaplaceGradCurlLinearForm_finite_element_0: public ufc::finite_element
{
public:

  /// Constructor
  VectorLaplaceGradCurlLinearForm_finite_element_0() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~VectorLaplaceGradCurlLinearForm_finite_element_0()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "Mixed finite element: [Lagrange finite element of degree 1 on a tetrahedron, Nedelec finite element of degree 1 on a tetrahedron]";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::tetrahedron;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 10;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 1;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 4;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    const double Jinv_00 = d00 / detJ;
    const double Jinv_01 = d10 / detJ;
    const double Jinv_02 = d20 / detJ;
    const double Jinv_10 = d01 / detJ;
    const double Jinv_11 = d11 / detJ;
    const double Jinv_12 = d21 / detJ;
    const double Jinv_20 = d02 / detJ;
    const double Jinv_21 = d12 / detJ;
    const double Jinv_22 = d22 / detJ;
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-09)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-09)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Reset values
    values[0] = 0;
    values[1] = 0;
    values[2] = 0;
    values[3] = 0;
    
    if (0 <= i && i <= 3)
    {
      // Map degree of freedom to element degree of freedom
      const unsigned int dof = i;
    
      // Generate scalings
      const double scalings_y_0 = 1;
      const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
      const double scalings_z_0 = 1;
      const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    
      // Compute psitilde_a
      const double psitilde_a_0 = 1;
      const double psitilde_a_1 = x;
    
      // Compute psitilde_bs
      const double psitilde_bs_0_0 = 1;
      const double psitilde_bs_0_1 = 1.5*y + 0.5;
      const double psitilde_bs_1_0 = 1;
    
      // Compute psitilde_cs
      const double psitilde_cs_00_0 = 1;
      const double psitilde_cs_00_1 = 2*z + 1;
      const double psitilde_cs_01_0 = 1;
      const double psitilde_cs_10_0 = 1;
    
      // Compute basisvalues
      const double basisvalue0 = 0.8660254038*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
      const double basisvalue1 = 2.738612788*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
      const double basisvalue2 = 1.58113883*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
      const double basisvalue3 = 1.118033989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    
      // Table(s) of coefficients
      const static double coefficients0[4][4] =   \
      {{0.2886751346, -0.1825741858, -0.1054092553, -0.07453559925},
      {0.2886751346, 0.1825741858, -0.1054092553, -0.07453559925},
      {0.2886751346, 0, 0.2108185107, -0.07453559925},
      {0.2886751346, 0, 0, 0.2236067977}};
    
      // Extract relevant coefficients
      const double coeff0_0 =   coefficients0[dof][0];
      const double coeff0_1 =   coefficients0[dof][1];
      const double coeff0_2 =   coefficients0[dof][2];
      const double coeff0_3 =   coefficients0[dof][3];
    
      // Compute value(s)
      values[0] = coeff0_0*basisvalue0 + coeff0_1*basisvalue1 + coeff0_2*basisvalue2 + coeff0_3*basisvalue3;
    }
    
    if (4 <= i && i <= 9)
    {
      // Map degree of freedom to element degree of freedom
      const unsigned int dof = i - 4;
    
      // Generate scalings
      const double scalings_y_0 = 1;
      const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
      const double scalings_z_0 = 1;
      const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    
      // Compute psitilde_a
      const double psitilde_a_0 = 1;
      const double psitilde_a_1 = x;
    
      // Compute psitilde_bs
      const double psitilde_bs_0_0 = 1;
      const double psitilde_bs_0_1 = 1.5*y + 0.5;
      const double psitilde_bs_1_0 = 1;
    
      // Compute psitilde_cs
      const double psitilde_cs_00_0 = 1;
      const double psitilde_cs_00_1 = 2*z + 1;
      const double psitilde_cs_01_0 = 1;
      const double psitilde_cs_10_0 = 1;
    
      // Compute basisvalues
      const double basisvalue0 = 0.8660254038*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
      const double basisvalue1 = 2.738612788*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
      const double basisvalue2 = 1.58113883*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
      const double basisvalue3 = 1.118033989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    
      // Table(s) of coefficients
      const static double coefficients0[6][4] =   \
      {{0, 0, 0, 0},
      {-0.2886751346, 0, 0, -0.2236067977},
      {-0.2886751346, 0, -0.2108185107, 0.07453559925},
      {0.2886751346, 0, 0, 0.2236067977},
      {0.2886751346, 0, 0.2108185107, -0.07453559925},
      {0.5773502692, 0, -0.2108185107, -0.1490711985}};
    
      const static double coefficients1[6][4] =   \
      {{-0.2886751346, 0, 0, -0.2236067977},
      {0, 0, 0, 0},
      {0.2886751346, 0.1825741858, -0.1054092553, -0.07453559925},
      {0.2886751346, 0, 0, 0.2236067977},
      {0.5773502692, -0.1825741858, 0.1054092553, -0.1490711985},
      {0.2886751346, 0.1825741858, -0.1054092553, -0.07453559925}};
    
      const static double coefficients2[6][4] =   \
      {{0.2886751346, 0, 0.2108185107, -0.07453559925},
      {0.2886751346, 0.1825741858, -0.1054092553, -0.07453559925},
      {0, 0, 0, 0},
      {0.5773502692, -0.1825741858, -0.1054092553, 0.1490711985},
      {0.2886751346, 0, 0.2108185107, -0.07453559925},
      {0.2886751346, 0.1825741858, -0.1054092553, -0.07453559925}};
    
      // Extract relevant coefficients
      const double coeff0_0 =   coefficients0[dof][0];
      const double coeff0_1 =   coefficients0[dof][1];
      const double coeff0_2 =   coefficients0[dof][2];
      const double coeff0_3 =   coefficients0[dof][3];
      const double coeff1_0 =   coefficients1[dof][0];
      const double coeff1_1 =   coefficients1[dof][1];
      const double coeff1_2 =   coefficients1[dof][2];
      const double coeff1_3 =   coefficients1[dof][3];
      const double coeff2_0 =   coefficients2[dof][0];
      const double coeff2_1 =   coefficients2[dof][1];
      const double coeff2_2 =   coefficients2[dof][2];
      const double coeff2_3 =   coefficients2[dof][3];
    
      // Compute value(s)
      const double tmp0_0 = coeff0_0*basisvalue0 + coeff0_1*basisvalue1 + coeff0_2*basisvalue2 + coeff0_3*basisvalue3;
      const double tmp0_1 = coeff1_0*basisvalue0 + coeff1_1*basisvalue1 + coeff1_2*basisvalue2 + coeff1_3*basisvalue3;
      const double tmp0_2 = coeff2_0*basisvalue0 + coeff2_1*basisvalue1 + coeff2_2*basisvalue2 + coeff2_3*basisvalue3;
      // Using covariant Piola transform to map values back to the physical element
      values[1] = (Jinv_00*tmp0_0 + Jinv_10*tmp0_1 + Jinv_20*tmp0_2);
      values[2] = (Jinv_01*tmp0_0 + Jinv_11*tmp0_1 + Jinv_21*tmp0_2);
      values[3] = (Jinv_02*tmp0_0 + Jinv_12*tmp0_1 + Jinv_22*tmp0_2);
    }
    
  }

  /// Evaluate all basis functions at given point in cell
  virtual void evaluate_basis_all(double* values,
                                  const double* coordinates,
                                  const ufc::cell& c) const
  {
    throw std::runtime_error("The vectorised version of evaluate_basis() is not yet implemented.");
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    const double Jinv_00 = d00 / detJ;
    const double Jinv_01 = d10 / detJ;
    const double Jinv_02 = d20 / detJ;
    const double Jinv_10 = d01 / detJ;
    const double Jinv_11 = d11 / detJ;
    const double Jinv_12 = d21 / detJ;
    const double Jinv_20 = d02 / detJ;
    const double Jinv_21 = d12 / detJ;
    const double Jinv_22 = d22 / detJ;
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-09)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-09)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Compute number of derivatives
    unsigned int num_derivatives = 1;
    
    for (unsigned int j = 0; j < n; j++)
      num_derivatives *= 3;
    
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      combinations[j] = new unsigned int [n];
      for (unsigned int k = 0; k < n; k++)
        combinations[j][k] = 0;
    }
        
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 2)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian
    const double Jinv[3][3] ={{d00 / detJ, d10 / detJ, d20 / detJ}, {d01 / detJ, d11 / detJ, d21 / detJ}, {d02 / detJ, d12 / detJ, d22 / detJ}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values
    for (unsigned int j = 0; j < 4*num_derivatives; j++)
      values[j] = 0;
    
    if (0 <= i && i <= 3)
    {
      // Map degree of freedom to element degree of freedom
      const unsigned int dof = i;
    
      // Generate scalings
      const double scalings_y_0 = 1;
      const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
      const double scalings_z_0 = 1;
      const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    
      // Compute psitilde_a
      const double psitilde_a_0 = 1;
      const double psitilde_a_1 = x;
    
      // Compute psitilde_bs
      const double psitilde_bs_0_0 = 1;
      const double psitilde_bs_0_1 = 1.5*y + 0.5;
      const double psitilde_bs_1_0 = 1;
    
      // Compute psitilde_cs
      const double psitilde_cs_00_0 = 1;
      const double psitilde_cs_00_1 = 2*z + 1;
      const double psitilde_cs_01_0 = 1;
      const double psitilde_cs_10_0 = 1;
    
      // Compute basisvalues
      const double basisvalue0 = 0.8660254038*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
      const double basisvalue1 = 2.738612788*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
      const double basisvalue2 = 1.58113883*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
      const double basisvalue3 = 1.118033989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    
      // Table(s) of coefficients
      const static double coefficients0[4][4] =   \
      {{0.2886751346, -0.1825741858, -0.1054092553, -0.07453559925},
      {0.2886751346, 0.1825741858, -0.1054092553, -0.07453559925},
      {0.2886751346, 0, 0.2108185107, -0.07453559925},
      {0.2886751346, 0, 0, 0.2236067977}};
    
      // Interesting (new) part
      // Tables of derivatives of the polynomial base (transpose)
      const static double dmats0[4][4] =   \
      {{0, 0, 0, 0},
      {6.32455532, 0, 0, 0},
      {0, 0, 0, 0},
      {0, 0, 0, 0}};
    
      const static double dmats1[4][4] =   \
      {{0, 0, 0, 0},
      {3.16227766, 0, 0, 0},
      {5.477225575, 0, 0, 0},
      {0, 0, 0, 0}};
    
      const static double dmats2[4][4] =   \
      {{0, 0, 0, 0},
      {3.16227766, 0, 0, 0},
      {1.825741858, 0, 0, 0},
      {5.163977795, 0, 0, 0}};
    
      // Compute reference derivatives
      // Declare pointer to array of derivatives on FIAT element
      double *derivatives = new double [num_derivatives];
    
      // Declare coefficients
      double coeff0_0 = 0;
      double coeff0_1 = 0;
      double coeff0_2 = 0;
      double coeff0_3 = 0;
    
      // Declare new coefficients
      double new_coeff0_0 = 0;
      double new_coeff0_1 = 0;
      double new_coeff0_2 = 0;
      double new_coeff0_3 = 0;
    
      // Loop possible derivatives
      for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
      {
        // Get values from coefficients array
        new_coeff0_0 = coefficients0[dof][0];
        new_coeff0_1 = coefficients0[dof][1];
        new_coeff0_2 = coefficients0[dof][2];
        new_coeff0_3 = coefficients0[dof][3];
    
        // Loop derivative order
        for (unsigned int j = 0; j < n; j++)
        {
          // Update old coefficients
          coeff0_0 = new_coeff0_0;
          coeff0_1 = new_coeff0_1;
          coeff0_2 = new_coeff0_2;
          coeff0_3 = new_coeff0_3;
    
          if(combinations[deriv_num][j] == 0)
          {
            new_coeff0_0 = coeff0_0*dmats0[0][0] + coeff0_1*dmats0[1][0] + coeff0_2*dmats0[2][0] + coeff0_3*dmats0[3][0];
            new_coeff0_1 = coeff0_0*dmats0[0][1] + coeff0_1*dmats0[1][1] + coeff0_2*dmats0[2][1] + coeff0_3*dmats0[3][1];
            new_coeff0_2 = coeff0_0*dmats0[0][2] + coeff0_1*dmats0[1][2] + coeff0_2*dmats0[2][2] + coeff0_3*dmats0[3][2];
            new_coeff0_3 = coeff0_0*dmats0[0][3] + coeff0_1*dmats0[1][3] + coeff0_2*dmats0[2][3] + coeff0_3*dmats0[3][3];
          }
          if(combinations[deriv_num][j] == 1)
          {
            new_coeff0_0 = coeff0_0*dmats1[0][0] + coeff0_1*dmats1[1][0] + coeff0_2*dmats1[2][0] + coeff0_3*dmats1[3][0];
            new_coeff0_1 = coeff0_0*dmats1[0][1] + coeff0_1*dmats1[1][1] + coeff0_2*dmats1[2][1] + coeff0_3*dmats1[3][1];
            new_coeff0_2 = coeff0_0*dmats1[0][2] + coeff0_1*dmats1[1][2] + coeff0_2*dmats1[2][2] + coeff0_3*dmats1[3][2];
            new_coeff0_3 = coeff0_0*dmats1[0][3] + coeff0_1*dmats1[1][3] + coeff0_2*dmats1[2][3] + coeff0_3*dmats1[3][3];
          }
          if(combinations[deriv_num][j] == 2)
          {
            new_coeff0_0 = coeff0_0*dmats2[0][0] + coeff0_1*dmats2[1][0] + coeff0_2*dmats2[2][0] + coeff0_3*dmats2[3][0];
            new_coeff0_1 = coeff0_0*dmats2[0][1] + coeff0_1*dmats2[1][1] + coeff0_2*dmats2[2][1] + coeff0_3*dmats2[3][1];
            new_coeff0_2 = coeff0_0*dmats2[0][2] + coeff0_1*dmats2[1][2] + coeff0_2*dmats2[2][2] + coeff0_3*dmats2[3][2];
            new_coeff0_3 = coeff0_0*dmats2[0][3] + coeff0_1*dmats2[1][3] + coeff0_2*dmats2[2][3] + coeff0_3*dmats2[3][3];
          }
    
        }
        // Compute derivatives on reference element as dot product of coefficients and basisvalues
        derivatives[deriv_num] = new_coeff0_0*basisvalue0 + new_coeff0_1*basisvalue1 + new_coeff0_2*basisvalue2 + new_coeff0_3*basisvalue3;
      }
    
      // Transform derivatives back to physical element
      for (unsigned int row = 0; row < num_derivatives; row++)
      {
        for (unsigned int col = 0; col < num_derivatives; col++)
        {
          values[row] += transform[row][col]*derivatives[col];
        }
      }
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
    
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int row = 0; row < num_derivatives; row++)
      {
        delete [] combinations[row];
        delete [] transform[row];
      }
    
      delete [] combinations;
      delete [] transform;
    }
    
    if (4 <= i && i <= 9)
    {
      // Map degree of freedom to element degree of freedom
      const unsigned int dof = i - 4;
    
      // Generate scalings
      const double scalings_y_0 = 1;
      const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
      const double scalings_z_0 = 1;
      const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    
      // Compute psitilde_a
      const double psitilde_a_0 = 1;
      const double psitilde_a_1 = x;
    
      // Compute psitilde_bs
      const double psitilde_bs_0_0 = 1;
      const double psitilde_bs_0_1 = 1.5*y + 0.5;
      const double psitilde_bs_1_0 = 1;
    
      // Compute psitilde_cs
      const double psitilde_cs_00_0 = 1;
      const double psitilde_cs_00_1 = 2*z + 1;
      const double psitilde_cs_01_0 = 1;
      const double psitilde_cs_10_0 = 1;
    
      // Compute basisvalues
      const double basisvalue0 = 0.8660254038*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
      const double basisvalue1 = 2.738612788*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
      const double basisvalue2 = 1.58113883*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
      const double basisvalue3 = 1.118033989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    
      // Table(s) of coefficients
      const static double coefficients0[6][4] =   \
      {{0, 0, 0, 0},
      {-0.2886751346, 0, 0, -0.2236067977},
      {-0.2886751346, 0, -0.2108185107, 0.07453559925},
      {0.2886751346, 0, 0, 0.2236067977},
      {0.2886751346, 0, 0.2108185107, -0.07453559925},
      {0.5773502692, 0, -0.2108185107, -0.1490711985}};
    
      const static double coefficients1[6][4] =   \
      {{-0.2886751346, 0, 0, -0.2236067977},
      {0, 0, 0, 0},
      {0.2886751346, 0.1825741858, -0.1054092553, -0.07453559925},
      {0.2886751346, 0, 0, 0.2236067977},
      {0.5773502692, -0.1825741858, 0.1054092553, -0.1490711985},
      {0.2886751346, 0.1825741858, -0.1054092553, -0.07453559925}};
    
      const static double coefficients2[6][4] =   \
      {{0.2886751346, 0, 0.2108185107, -0.07453559925},
      {0.2886751346, 0.1825741858, -0.1054092553, -0.07453559925},
      {0, 0, 0, 0},
      {0.5773502692, -0.1825741858, -0.1054092553, 0.1490711985},
      {0.2886751346, 0, 0.2108185107, -0.07453559925},
      {0.2886751346, 0.1825741858, -0.1054092553, -0.07453559925}};
    
      // Interesting (new) part
      // Tables of derivatives of the polynomial base (transpose)
      const static double dmats0[4][4] =   \
      {{0, 0, 0, 0},
      {6.32455532, 0, 0, 0},
      {0, 0, 0, 0},
      {0, 0, 0, 0}};
    
      const static double dmats1[4][4] =   \
      {{0, 0, 0, 0},
      {3.16227766, 0, 0, 0},
      {5.477225575, 0, 0, 0},
      {0, 0, 0, 0}};
    
      const static double dmats2[4][4] =   \
      {{0, 0, 0, 0},
      {3.16227766, 0, 0, 0},
      {1.825741858, 0, 0, 0},
      {5.163977795, 0, 0, 0}};
    
      // Compute reference derivatives
      // Declare pointer to array of derivatives on FIAT element
      double *derivatives = new double [3*num_derivatives];
    
      // Declare coefficients
      double coeff0_0 = 0;
      double coeff0_1 = 0;
      double coeff0_2 = 0;
      double coeff0_3 = 0;
      double coeff1_0 = 0;
      double coeff1_1 = 0;
      double coeff1_2 = 0;
      double coeff1_3 = 0;
      double coeff2_0 = 0;
      double coeff2_1 = 0;
      double coeff2_2 = 0;
      double coeff2_3 = 0;
    
      // Declare new coefficients
      double new_coeff0_0 = 0;
      double new_coeff0_1 = 0;
      double new_coeff0_2 = 0;
      double new_coeff0_3 = 0;
      double new_coeff1_0 = 0;
      double new_coeff1_1 = 0;
      double new_coeff1_2 = 0;
      double new_coeff1_3 = 0;
      double new_coeff2_0 = 0;
      double new_coeff2_1 = 0;
      double new_coeff2_2 = 0;
      double new_coeff2_3 = 0;
    
      // Loop possible derivatives
      for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
      {
        // Get values from coefficients array
        new_coeff0_0 = coefficients0[dof][0];
        new_coeff0_1 = coefficients0[dof][1];
        new_coeff0_2 = coefficients0[dof][2];
        new_coeff0_3 = coefficients0[dof][3];
        new_coeff1_0 = coefficients1[dof][0];
        new_coeff1_1 = coefficients1[dof][1];
        new_coeff1_2 = coefficients1[dof][2];
        new_coeff1_3 = coefficients1[dof][3];
        new_coeff2_0 = coefficients2[dof][0];
        new_coeff2_1 = coefficients2[dof][1];
        new_coeff2_2 = coefficients2[dof][2];
        new_coeff2_3 = coefficients2[dof][3];
    
        // Loop derivative order
        for (unsigned int j = 0; j < n; j++)
        {
          // Update old coefficients
          coeff0_0 = new_coeff0_0;
          coeff0_1 = new_coeff0_1;
          coeff0_2 = new_coeff0_2;
          coeff0_3 = new_coeff0_3;
          coeff1_0 = new_coeff1_0;
          coeff1_1 = new_coeff1_1;
          coeff1_2 = new_coeff1_2;
          coeff1_3 = new_coeff1_3;
          coeff2_0 = new_coeff2_0;
          coeff2_1 = new_coeff2_1;
          coeff2_2 = new_coeff2_2;
          coeff2_3 = new_coeff2_3;
    
          if(combinations[deriv_num][j] == 0)
          {
            new_coeff0_0 = coeff0_0*dmats0[0][0] + coeff0_1*dmats0[1][0] + coeff0_2*dmats0[2][0] + coeff0_3*dmats0[3][0];
            new_coeff0_1 = coeff0_0*dmats0[0][1] + coeff0_1*dmats0[1][1] + coeff0_2*dmats0[2][1] + coeff0_3*dmats0[3][1];
            new_coeff0_2 = coeff0_0*dmats0[0][2] + coeff0_1*dmats0[1][2] + coeff0_2*dmats0[2][2] + coeff0_3*dmats0[3][2];
            new_coeff0_3 = coeff0_0*dmats0[0][3] + coeff0_1*dmats0[1][3] + coeff0_2*dmats0[2][3] + coeff0_3*dmats0[3][3];
            new_coeff1_0 = coeff1_0*dmats0[0][0] + coeff1_1*dmats0[1][0] + coeff1_2*dmats0[2][0] + coeff1_3*dmats0[3][0];
            new_coeff1_1 = coeff1_0*dmats0[0][1] + coeff1_1*dmats0[1][1] + coeff1_2*dmats0[2][1] + coeff1_3*dmats0[3][1];
            new_coeff1_2 = coeff1_0*dmats0[0][2] + coeff1_1*dmats0[1][2] + coeff1_2*dmats0[2][2] + coeff1_3*dmats0[3][2];
            new_coeff1_3 = coeff1_0*dmats0[0][3] + coeff1_1*dmats0[1][3] + coeff1_2*dmats0[2][3] + coeff1_3*dmats0[3][3];
            new_coeff2_0 = coeff2_0*dmats0[0][0] + coeff2_1*dmats0[1][0] + coeff2_2*dmats0[2][0] + coeff2_3*dmats0[3][0];
            new_coeff2_1 = coeff2_0*dmats0[0][1] + coeff2_1*dmats0[1][1] + coeff2_2*dmats0[2][1] + coeff2_3*dmats0[3][1];
            new_coeff2_2 = coeff2_0*dmats0[0][2] + coeff2_1*dmats0[1][2] + coeff2_2*dmats0[2][2] + coeff2_3*dmats0[3][2];
            new_coeff2_3 = coeff2_0*dmats0[0][3] + coeff2_1*dmats0[1][3] + coeff2_2*dmats0[2][3] + coeff2_3*dmats0[3][3];
          }
          if(combinations[deriv_num][j] == 1)
          {
            new_coeff0_0 = coeff0_0*dmats1[0][0] + coeff0_1*dmats1[1][0] + coeff0_2*dmats1[2][0] + coeff0_3*dmats1[3][0];
            new_coeff0_1 = coeff0_0*dmats1[0][1] + coeff0_1*dmats1[1][1] + coeff0_2*dmats1[2][1] + coeff0_3*dmats1[3][1];
            new_coeff0_2 = coeff0_0*dmats1[0][2] + coeff0_1*dmats1[1][2] + coeff0_2*dmats1[2][2] + coeff0_3*dmats1[3][2];
            new_coeff0_3 = coeff0_0*dmats1[0][3] + coeff0_1*dmats1[1][3] + coeff0_2*dmats1[2][3] + coeff0_3*dmats1[3][3];
            new_coeff1_0 = coeff1_0*dmats1[0][0] + coeff1_1*dmats1[1][0] + coeff1_2*dmats1[2][0] + coeff1_3*dmats1[3][0];
            new_coeff1_1 = coeff1_0*dmats1[0][1] + coeff1_1*dmats1[1][1] + coeff1_2*dmats1[2][1] + coeff1_3*dmats1[3][1];
            new_coeff1_2 = coeff1_0*dmats1[0][2] + coeff1_1*dmats1[1][2] + coeff1_2*dmats1[2][2] + coeff1_3*dmats1[3][2];
            new_coeff1_3 = coeff1_0*dmats1[0][3] + coeff1_1*dmats1[1][3] + coeff1_2*dmats1[2][3] + coeff1_3*dmats1[3][3];
            new_coeff2_0 = coeff2_0*dmats1[0][0] + coeff2_1*dmats1[1][0] + coeff2_2*dmats1[2][0] + coeff2_3*dmats1[3][0];
            new_coeff2_1 = coeff2_0*dmats1[0][1] + coeff2_1*dmats1[1][1] + coeff2_2*dmats1[2][1] + coeff2_3*dmats1[3][1];
            new_coeff2_2 = coeff2_0*dmats1[0][2] + coeff2_1*dmats1[1][2] + coeff2_2*dmats1[2][2] + coeff2_3*dmats1[3][2];
            new_coeff2_3 = coeff2_0*dmats1[0][3] + coeff2_1*dmats1[1][3] + coeff2_2*dmats1[2][3] + coeff2_3*dmats1[3][3];
          }
          if(combinations[deriv_num][j] == 2)
          {
            new_coeff0_0 = coeff0_0*dmats2[0][0] + coeff0_1*dmats2[1][0] + coeff0_2*dmats2[2][0] + coeff0_3*dmats2[3][0];
            new_coeff0_1 = coeff0_0*dmats2[0][1] + coeff0_1*dmats2[1][1] + coeff0_2*dmats2[2][1] + coeff0_3*dmats2[3][1];
            new_coeff0_2 = coeff0_0*dmats2[0][2] + coeff0_1*dmats2[1][2] + coeff0_2*dmats2[2][2] + coeff0_3*dmats2[3][2];
            new_coeff0_3 = coeff0_0*dmats2[0][3] + coeff0_1*dmats2[1][3] + coeff0_2*dmats2[2][3] + coeff0_3*dmats2[3][3];
            new_coeff1_0 = coeff1_0*dmats2[0][0] + coeff1_1*dmats2[1][0] + coeff1_2*dmats2[2][0] + coeff1_3*dmats2[3][0];
            new_coeff1_1 = coeff1_0*dmats2[0][1] + coeff1_1*dmats2[1][1] + coeff1_2*dmats2[2][1] + coeff1_3*dmats2[3][1];
            new_coeff1_2 = coeff1_0*dmats2[0][2] + coeff1_1*dmats2[1][2] + coeff1_2*dmats2[2][2] + coeff1_3*dmats2[3][2];
            new_coeff1_3 = coeff1_0*dmats2[0][3] + coeff1_1*dmats2[1][3] + coeff1_2*dmats2[2][3] + coeff1_3*dmats2[3][3];
            new_coeff2_0 = coeff2_0*dmats2[0][0] + coeff2_1*dmats2[1][0] + coeff2_2*dmats2[2][0] + coeff2_3*dmats2[3][0];
            new_coeff2_1 = coeff2_0*dmats2[0][1] + coeff2_1*dmats2[1][1] + coeff2_2*dmats2[2][1] + coeff2_3*dmats2[3][1];
            new_coeff2_2 = coeff2_0*dmats2[0][2] + coeff2_1*dmats2[1][2] + coeff2_2*dmats2[2][2] + coeff2_3*dmats2[3][2];
            new_coeff2_3 = coeff2_0*dmats2[0][3] + coeff2_1*dmats2[1][3] + coeff2_2*dmats2[2][3] + coeff2_3*dmats2[3][3];
          }
    
        }
        // Compute derivatives on reference element as dot product of coefficients and basisvalues
        // Correct values by the covariant Piola transform
        const double tmp0_0 = new_coeff0_0*basisvalue0 + new_coeff0_1*basisvalue1 + new_coeff0_2*basisvalue2 + new_coeff0_3*basisvalue3;
        const double tmp0_1 = new_coeff1_0*basisvalue0 + new_coeff1_1*basisvalue1 + new_coeff1_2*basisvalue2 + new_coeff1_3*basisvalue3;
        const double tmp0_2 = new_coeff2_0*basisvalue0 + new_coeff2_1*basisvalue1 + new_coeff2_2*basisvalue2 + new_coeff2_3*basisvalue3;
        derivatives[deriv_num] = (Jinv_00*tmp0_0 + Jinv_10*tmp0_1 + Jinv_20*tmp0_2);
        derivatives[num_derivatives + deriv_num] = (Jinv_01*tmp0_0 + Jinv_11*tmp0_1 + Jinv_21*tmp0_2);
        derivatives[2*num_derivatives + deriv_num] = (Jinv_02*tmp0_0 + Jinv_12*tmp0_1 + Jinv_22*tmp0_2);
      }
    
      // Transform derivatives back to physical element
      for (unsigned int row = 0; row < num_derivatives; row++)
      {
        for (unsigned int col = 0; col < num_derivatives; col++)
        {
          values[num_derivatives + row] += transform[row][col]*derivatives[col];
          values[2*num_derivatives + row] += transform[row][col]*derivatives[num_derivatives + col];
          values[3*num_derivatives + row] += transform[row][col]*derivatives[2*num_derivatives + col];
        }
      }
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
    
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int row = 0; row < num_derivatives; row++)
      {
        delete [] combinations[row];
        delete [] transform[row];
      }
    
      delete [] combinations;
      delete [] transform;
    }
    
  }

  /// Evaluate order n derivatives of all basis functions at given point in cell
  virtual void evaluate_basis_derivatives_all(unsigned int n,
                                              double* values,
                                              const double* coordinates,
                                              const ufc::cell& c) const
  {
    throw std::runtime_error("The vectorised version of evaluate_basis_derivatives() is not yet implemented.");
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    // The reference points, direction and weights:
    const static double X[10][1][3] = {{{0, 0, 0}}, {{1, 0, 0}}, {{0, 1, 0}}, {{0, 0, 1}}, {{0, 0.5, 0.5}}, {{0.5, 0, 0.5}}, {{0.5, 0.5, 0}}, {{0, 0, 0.5}}, {{0, 0.5, 0}}, {{0.5, 0, 0}}};
    const static double W[10][1] = {{1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}};
    const static double D[10][1][4] = {{{1, 0, 0, 0}}, {{1, 0, 0, 0}}, {{1, 0, 0, 0}}, {{1, 0, 0, 0}}, {{0, 0, -1, 1}}, {{0, -1, 0, 1}}, {{0, -1, 1, 0}}, {{0, 0, 0, 1}}, {{0, 0, 1, 0}}, {{0, 1, 0, 0}}};
    
    static const unsigned int mappings[10] = {0, 0, 0, 0, 2, 2, 2, 2, 2, 2};
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = x[1][0] - x[0][0];
    const double J_01 = x[2][0] - x[0][0];
    const double J_02 = x[3][0] - x[0][0];
    const double J_10 = x[1][1] - x[0][1];
    const double J_11 = x[2][1] - x[0][1];
    const double J_12 = x[3][1] - x[0][1];
    const double J_20 = x[1][2] - x[0][2];
    const double J_21 = x[2][2] - x[0][2];
    const double J_22 = x[3][2] - x[0][2];
      
    // Compute sub determinants
    const double d_00 = J_11*J_22 - J_12*J_21;
    const double d_01 = J_12*J_20 - J_10*J_22;
    const double d_02 = J_10*J_21 - J_11*J_20;
    
    const double d_10 = J_02*J_21 - J_01*J_22;
    const double d_11 = J_00*J_22 - J_02*J_20;
    const double d_12 = J_01*J_20 - J_00*J_21;
    
    const double d_20 = J_01*J_12 - J_02*J_11;
    const double d_21 = J_02*J_10 - J_00*J_12;
    const double d_22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d_00 + J_10*d_10 + J_20*d_20;
      
    // Compute inverse of Jacobian
    const double Jinv_00 = d_00 / detJ;
    const double Jinv_01 = d_10 / detJ;
    const double Jinv_02 = d_20 / detJ;
    const double Jinv_10 = d_01 / detJ;
    const double Jinv_11 = d_11 / detJ;
    const double Jinv_12 = d_21 / detJ;
    const double Jinv_20 = d_02 / detJ;
    const double Jinv_21 = d_12 / detJ;
    const double Jinv_22 = d_22 / detJ;
    
    double copyofvalues[4];
    const int offsets[10] = {0, 0, 0, 0, 1, 1, 1, 1, 1, 1};
    double result = 0.0;
    // Iterate over the points:
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0][0] - X[i][0][1] - X[i][0][2];
    const double w1 = X[i][0][0];
    const double w2 = X[i][0][1];
    const double w3 = X[i][0][2];
    
    // Compute affine mapping y = F(X)
    double y[3];
    y[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0] + w3*x[3][0];
    y[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1] + w3*x[3][1];
    y[2] = w0*x[0][2] + w1*x[1][2] + w2*x[2][2] + w3*x[3][2];
    
    // Evaluate function at physical points
    double values[4];
    f.evaluate(values, y, c);
    
    // Map function values using appropriate mapping
    if (mappings[i] == 0) { 
      // Affine map: Do nothing
    } else if (mappings[i] == 2) {
       // Copy old values:
      copyofvalues[offsets[i] + 0] = values[offsets[i] + 0];
      copyofvalues[offsets[i] + 1] = values[offsets[i] + 1];
      copyofvalues[offsets[i] + 2] = values[offsets[i] + 2];
      // Do the inverse of curl piola 
      values[offsets[i] + 0] = J_00*copyofvalues[offsets[i] + 0]+J_10*copyofvalues[offsets[i] + 1]+J_20*copyofvalues[offsets[i] + 2];
      values[offsets[i] + 1] = J_01*copyofvalues[offsets[i] + 0]+J_11*copyofvalues[offsets[i] + 1]+J_21*copyofvalues[offsets[i] + 2];
      values[offsets[i] + 2] = J_02*copyofvalues[offsets[i] + 0]+J_12*copyofvalues[offsets[i] + 1]+J_22*copyofvalues[offsets[i] + 2]; 
    } else { 
       // Other mappings not applicable. 
    }
    
    // Note that we do not map the weights (yet).
    
    // Take directional components
    for(int k = 0; k < 4; k++)
      result += values[k]*D[i][0][k];
    // Multiply by weights 
    result *= W[i][0];
    
    return result;
  }

  /// Evaluate linear functionals for all dofs on the function f
  virtual void evaluate_dofs(double* values,
                             const ufc::function& f,
                             const ufc::cell& c) const
  {
    throw std::runtime_error("Not implemented (introduced in UFC v1.1).");
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = x[1][0] - x[0][0];
    const double J_01 = x[2][0] - x[0][0];
    const double J_02 = x[3][0] - x[0][0];
    const double J_10 = x[1][1] - x[0][1];
    const double J_11 = x[2][1] - x[0][1];
    const double J_12 = x[3][1] - x[0][1];
    const double J_20 = x[1][2] - x[0][2];
    const double J_21 = x[2][2] - x[0][2];
    const double J_22 = x[3][2] - x[0][2];
      
    // Compute sub determinants
    const double d_00 = J_11*J_22 - J_12*J_21;
    const double d_01 = J_12*J_20 - J_10*J_22;
    const double d_02 = J_10*J_21 - J_11*J_20;
    
    const double d_10 = J_02*J_21 - J_01*J_22;
    const double d_11 = J_00*J_22 - J_02*J_20;
    const double d_12 = J_01*J_20 - J_00*J_21;
    
    const double d_20 = J_01*J_12 - J_02*J_11;
    const double d_21 = J_02*J_10 - J_00*J_12;
    const double d_22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d_00 + J_10*d_10 + J_20*d_20;
      
    // Compute inverse of Jacobian
    const double Jinv_00 = d_00 / detJ;
    const double Jinv_01 = d_10 / detJ;
    const double Jinv_02 = d_20 / detJ;
    const double Jinv_10 = d_01 / detJ;
    const double Jinv_11 = d_11 / detJ;
    const double Jinv_12 = d_21 / detJ;
    const double Jinv_20 = d_02 / detJ;
    const double Jinv_21 = d_12 / detJ;
    const double Jinv_22 = d_22 / detJ;
    // Evaluate at vertices and use affine mapping
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[1];
    vertex_values[2] = dof_values[2];
    vertex_values[3] = dof_values[3];
    // Evaluate at vertices and use Piola mapping
    vertex_values[4] = (dof_values[7]*Jinv_20 + dof_values[8]*Jinv_10 + dof_values[9]*Jinv_00);
    vertex_values[5] = (dof_values[5]*Jinv_20 + dof_values[6]*Jinv_10 + dof_values[9]*(Jinv_00 + Jinv_10 + Jinv_20));
    vertex_values[6] = (dof_values[4]*Jinv_20 + dof_values[6]*Jinv_00 + dof_values[8]*(Jinv_00 + Jinv_10 + Jinv_20));
    vertex_values[7] = (dof_values[4]*Jinv_10 + dof_values[5]*Jinv_00 + dof_values[7]*(Jinv_00 + Jinv_10 + Jinv_20));
    vertex_values[8] = (dof_values[7]*Jinv_21 + dof_values[8]*Jinv_11 + dof_values[9]*Jinv_01);
    vertex_values[9] = (dof_values[5]*Jinv_21 + dof_values[6]*Jinv_11 + dof_values[9]*(Jinv_01 + Jinv_11 + Jinv_21));
    vertex_values[10] = (dof_values[4]*Jinv_21 + dof_values[6]*Jinv_01 + dof_values[8]*(Jinv_01 + Jinv_11 + Jinv_21));
    vertex_values[11] = (dof_values[4]*Jinv_11 + dof_values[5]*Jinv_01 + dof_values[7]*(Jinv_01 + Jinv_11 + Jinv_21));
    vertex_values[12] = (dof_values[7]*Jinv_22 + dof_values[8]*Jinv_12 + dof_values[9]*Jinv_02);
    vertex_values[13] = (dof_values[5]*Jinv_22 + dof_values[6]*Jinv_12 + dof_values[9]*(Jinv_02 + Jinv_12 + Jinv_22));
    vertex_values[14] = (dof_values[4]*Jinv_22 + dof_values[6]*Jinv_02 + dof_values[8]*(Jinv_02 + Jinv_12 + Jinv_22));
    vertex_values[15] = (dof_values[4]*Jinv_12 + dof_values[5]*Jinv_02 + dof_values[7]*(Jinv_02 + Jinv_12 + Jinv_22));
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 2;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    switch ( i )
    {
    case 0:
      return new VectorLaplaceGradCurlLinearForm_finite_element_0_0();
      break;
    case 1:
      return new VectorLaplaceGradCurlLinearForm_finite_element_0_1();
      break;
    }
    return 0;
  }

};

/// This class defines the interface for a finite element.

class VectorLaplaceGradCurlLinearForm_finite_element_1_0: public ufc::finite_element
{
public:

  /// Constructor
  VectorLaplaceGradCurlLinearForm_finite_element_1_0() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~VectorLaplaceGradCurlLinearForm_finite_element_1_0()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "Lagrange finite element of degree 2 on a tetrahedron";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::tetrahedron;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 10;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 0;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 1;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-09)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-09)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Reset values
    *values = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
    const double scalings_y_2 = scalings_y_1*(0.5 - 0.5*y);
    const double scalings_z_0 = 1;
    const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    const double scalings_z_2 = scalings_z_1*(0.5 - 0.5*z);
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    const double psitilde_a_1 = x;
    const double psitilde_a_2 = 1.5*x*psitilde_a_1 - 0.5*psitilde_a_0;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    const double psitilde_bs_0_1 = 1.5*y + 0.5;
    const double psitilde_bs_0_2 = 0.1111111111*psitilde_bs_0_1 + 1.666666667*y*psitilde_bs_0_1 - 0.5555555556*psitilde_bs_0_0;
    const double psitilde_bs_1_0 = 1;
    const double psitilde_bs_1_1 = 2.5*y + 1.5;
    const double psitilde_bs_2_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    const double psitilde_cs_00_1 = 2*z + 1;
    const double psitilde_cs_00_2 = 0.3125*psitilde_cs_00_1 + 1.875*z*psitilde_cs_00_1 - 0.5625*psitilde_cs_00_0;
    const double psitilde_cs_01_0 = 1;
    const double psitilde_cs_01_1 = 3*z + 2;
    const double psitilde_cs_02_0 = 1;
    const double psitilde_cs_10_0 = 1;
    const double psitilde_cs_10_1 = 3*z + 2;
    const double psitilde_cs_11_0 = 1;
    const double psitilde_cs_20_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.8660254038*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    const double basisvalue1 = 2.738612788*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
    const double basisvalue2 = 1.58113883*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
    const double basisvalue3 = 1.118033989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    const double basisvalue4 = 5.123475383*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_0;
    const double basisvalue5 = 3.968626967*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_0;
    const double basisvalue6 = 2.291287847*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_0;
    const double basisvalue7 = 3.240370349*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_1;
    const double basisvalue8 = 1.870828693*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_1;
    const double basisvalue9 = 1.322875656*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_2;
    
    // Table(s) of coefficients
    const static double coefficients0[10][10] = \
    {{-0.05773502692, -0.06085806195, -0.03513641845, -0.02484519975, 0.06506000486, 0.05039526307, 0.0290957187, 0.04114755999, 0.02375655484, 0.01679842102},
    {-0.05773502692, 0.06085806195, -0.03513641845, -0.02484519975, 0.06506000486, -0.05039526307, 0.0290957187, -0.04114755999, 0.02375655484, 0.01679842102},
    {-0.05773502692, 0, 0.07027283689, -0.02484519975, 0, 0, 0.08728715609, 0, -0.04751310967, 0.01679842102},
    {-0.05773502692, 0, 0, 0.07453559925, 0, 0, 0, 0, 0, 0.1007905261},
    {0.2309401077, 0, 0.1405456738, 0.099380799, 0, 0, 0, 0, 0.1187827742, -0.06719368409},
    {0.2309401077, 0.1217161239, -0.07027283689, 0.099380799, 0, 0, 0, 0.1028689, -0.05939138709, -0.06719368409},
    {0.2309401077, 0.1217161239, 0.07027283689, -0.099380799, 0, 0.1007905261, -0.08728715609, -0.02057377999, -0.01187827742, 0.01679842102},
    {0.2309401077, -0.1217161239, -0.07027283689, 0.099380799, 0, 0, 0, -0.1028689, -0.05939138709, -0.06719368409},
    {0.2309401077, -0.1217161239, 0.07027283689, -0.099380799, 0, -0.1007905261, -0.08728715609, 0.02057377999, -0.01187827742, 0.01679842102},
    {0.2309401077, 0, -0.1405456738, -0.099380799, -0.1301200097, 0, 0.0290957187, 0, 0.02375655484, 0.01679842102}};
    
    // Extract relevant coefficients
    const double coeff0_0 = coefficients0[dof][0];
    const double coeff0_1 = coefficients0[dof][1];
    const double coeff0_2 = coefficients0[dof][2];
    const double coeff0_3 = coefficients0[dof][3];
    const double coeff0_4 = coefficients0[dof][4];
    const double coeff0_5 = coefficients0[dof][5];
    const double coeff0_6 = coefficients0[dof][6];
    const double coeff0_7 = coefficients0[dof][7];
    const double coeff0_8 = coefficients0[dof][8];
    const double coeff0_9 = coefficients0[dof][9];
    
    // Compute value(s)
    *values = coeff0_0*basisvalue0 + coeff0_1*basisvalue1 + coeff0_2*basisvalue2 + coeff0_3*basisvalue3 + coeff0_4*basisvalue4 + coeff0_5*basisvalue5 + coeff0_6*basisvalue6 + coeff0_7*basisvalue7 + coeff0_8*basisvalue8 + coeff0_9*basisvalue9;
  }

  /// Evaluate all basis functions at given point in cell
  virtual void evaluate_basis_all(double* values,
                                  const double* coordinates,
                                  const ufc::cell& c) const
  {
    throw std::runtime_error("The vectorised version of evaluate_basis() is not yet implemented.");
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-09)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-09)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Compute number of derivatives
    unsigned int num_derivatives = 1;
    
    for (unsigned int j = 0; j < n; j++)
      num_derivatives *= 3;
    
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      combinations[j] = new unsigned int [n];
      for (unsigned int k = 0; k < n; k++)
        combinations[j][k] = 0;
    }
        
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 2)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian
    const double Jinv[3][3] ={{d00 / detJ, d10 / detJ, d20 / detJ}, {d01 / detJ, d11 / detJ, d21 / detJ}, {d02 / detJ, d12 / detJ, d22 / detJ}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values
    for (unsigned int j = 0; j < 1*num_derivatives; j++)
      values[j] = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
    const double scalings_y_2 = scalings_y_1*(0.5 - 0.5*y);
    const double scalings_z_0 = 1;
    const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    const double scalings_z_2 = scalings_z_1*(0.5 - 0.5*z);
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    const double psitilde_a_1 = x;
    const double psitilde_a_2 = 1.5*x*psitilde_a_1 - 0.5*psitilde_a_0;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    const double psitilde_bs_0_1 = 1.5*y + 0.5;
    const double psitilde_bs_0_2 = 0.1111111111*psitilde_bs_0_1 + 1.666666667*y*psitilde_bs_0_1 - 0.5555555556*psitilde_bs_0_0;
    const double psitilde_bs_1_0 = 1;
    const double psitilde_bs_1_1 = 2.5*y + 1.5;
    const double psitilde_bs_2_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    const double psitilde_cs_00_1 = 2*z + 1;
    const double psitilde_cs_00_2 = 0.3125*psitilde_cs_00_1 + 1.875*z*psitilde_cs_00_1 - 0.5625*psitilde_cs_00_0;
    const double psitilde_cs_01_0 = 1;
    const double psitilde_cs_01_1 = 3*z + 2;
    const double psitilde_cs_02_0 = 1;
    const double psitilde_cs_10_0 = 1;
    const double psitilde_cs_10_1 = 3*z + 2;
    const double psitilde_cs_11_0 = 1;
    const double psitilde_cs_20_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.8660254038*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    const double basisvalue1 = 2.738612788*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
    const double basisvalue2 = 1.58113883*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
    const double basisvalue3 = 1.118033989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    const double basisvalue4 = 5.123475383*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_0;
    const double basisvalue5 = 3.968626967*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_0;
    const double basisvalue6 = 2.291287847*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_0;
    const double basisvalue7 = 3.240370349*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_1;
    const double basisvalue8 = 1.870828693*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_1;
    const double basisvalue9 = 1.322875656*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_2;
    
    // Table(s) of coefficients
    const static double coefficients0[10][10] = \
    {{-0.05773502692, -0.06085806195, -0.03513641845, -0.02484519975, 0.06506000486, 0.05039526307, 0.0290957187, 0.04114755999, 0.02375655484, 0.01679842102},
    {-0.05773502692, 0.06085806195, -0.03513641845, -0.02484519975, 0.06506000486, -0.05039526307, 0.0290957187, -0.04114755999, 0.02375655484, 0.01679842102},
    {-0.05773502692, 0, 0.07027283689, -0.02484519975, 0, 0, 0.08728715609, 0, -0.04751310967, 0.01679842102},
    {-0.05773502692, 0, 0, 0.07453559925, 0, 0, 0, 0, 0, 0.1007905261},
    {0.2309401077, 0, 0.1405456738, 0.099380799, 0, 0, 0, 0, 0.1187827742, -0.06719368409},
    {0.2309401077, 0.1217161239, -0.07027283689, 0.099380799, 0, 0, 0, 0.1028689, -0.05939138709, -0.06719368409},
    {0.2309401077, 0.1217161239, 0.07027283689, -0.099380799, 0, 0.1007905261, -0.08728715609, -0.02057377999, -0.01187827742, 0.01679842102},
    {0.2309401077, -0.1217161239, -0.07027283689, 0.099380799, 0, 0, 0, -0.1028689, -0.05939138709, -0.06719368409},
    {0.2309401077, -0.1217161239, 0.07027283689, -0.099380799, 0, -0.1007905261, -0.08728715609, 0.02057377999, -0.01187827742, 0.01679842102},
    {0.2309401077, 0, -0.1405456738, -0.099380799, -0.1301200097, 0, 0.0290957187, 0, 0.02375655484, 0.01679842102}};
    
    // Interesting (new) part
    // Tables of derivatives of the polynomial base (transpose)
    const static double dmats0[10][10] = \
    {{0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {6.32455532, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 11.22497216, 0, 0, 0, 0, 0, 0, 0, 0},
    {4.582575695, 0, 8.366600265, -1.183215957, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {3.741657387, 0, 0, 8.694826048, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0}};
    
    const static double dmats1[10][10] = \
    {{0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {3.16227766, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {5.477225575, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.958039892, 5.61248608, -1.08012345, -0.7637626158, 0, 0, 0, 0, 0, 0},
    {2.291287847, 7.245688373, 4.183300133, -0.5916079783, 0, 0, 0, 0, 0, 0},
    {-2.645751311, 0, 9.660917831, 0.6831300511, 0, 0, 0, 0, 0, 0},
    {1.870828693, 0, 0, 4.347413024, 0, 0, 0, 0, 0, 0},
    {3.240370349, 0, 0, 7.529940239, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0}};
    
    const static double dmats2[10][10] = \
    {{0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {3.16227766, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.825741858, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {5.163977795, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.958039892, 5.61248608, -1.08012345, -0.7637626158, 0, 0, 0, 0, 0, 0},
    {2.291287847, 1.449137675, 4.183300133, -0.5916079783, 0, 0, 0, 0, 0, 0},
    {1.322875656, 0, 3.864367132, -0.3415650255, 0, 0, 0, 0, 0, 0},
    {1.870828693, 7.09929574, 0, 4.347413024, 0, 0, 0, 0, 0, 0},
    {1.08012345, 0, 7.09929574, 2.50998008, 0, 0, 0, 0, 0, 0},
    {-3.818813079, 0, 0, 8.874119675, 0, 0, 0, 0, 0, 0}};
    
    // Compute reference derivatives
    // Declare pointer to array of derivatives on FIAT element
    double *derivatives = new double [num_derivatives];
    
    // Declare coefficients
    double coeff0_0 = 0;
    double coeff0_1 = 0;
    double coeff0_2 = 0;
    double coeff0_3 = 0;
    double coeff0_4 = 0;
    double coeff0_5 = 0;
    double coeff0_6 = 0;
    double coeff0_7 = 0;
    double coeff0_8 = 0;
    double coeff0_9 = 0;
    
    // Declare new coefficients
    double new_coeff0_0 = 0;
    double new_coeff0_1 = 0;
    double new_coeff0_2 = 0;
    double new_coeff0_3 = 0;
    double new_coeff0_4 = 0;
    double new_coeff0_5 = 0;
    double new_coeff0_6 = 0;
    double new_coeff0_7 = 0;
    double new_coeff0_8 = 0;
    double new_coeff0_9 = 0;
    
    // Loop possible derivatives
    for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
    {
      // Get values from coefficients array
      new_coeff0_0 = coefficients0[dof][0];
      new_coeff0_1 = coefficients0[dof][1];
      new_coeff0_2 = coefficients0[dof][2];
      new_coeff0_3 = coefficients0[dof][3];
      new_coeff0_4 = coefficients0[dof][4];
      new_coeff0_5 = coefficients0[dof][5];
      new_coeff0_6 = coefficients0[dof][6];
      new_coeff0_7 = coefficients0[dof][7];
      new_coeff0_8 = coefficients0[dof][8];
      new_coeff0_9 = coefficients0[dof][9];
    
      // Loop derivative order
      for (unsigned int j = 0; j < n; j++)
      {
        // Update old coefficients
        coeff0_0 = new_coeff0_0;
        coeff0_1 = new_coeff0_1;
        coeff0_2 = new_coeff0_2;
        coeff0_3 = new_coeff0_3;
        coeff0_4 = new_coeff0_4;
        coeff0_5 = new_coeff0_5;
        coeff0_6 = new_coeff0_6;
        coeff0_7 = new_coeff0_7;
        coeff0_8 = new_coeff0_8;
        coeff0_9 = new_coeff0_9;
    
        if(combinations[deriv_num][j] == 0)
        {
          new_coeff0_0 = coeff0_0*dmats0[0][0] + coeff0_1*dmats0[1][0] + coeff0_2*dmats0[2][0] + coeff0_3*dmats0[3][0] + coeff0_4*dmats0[4][0] + coeff0_5*dmats0[5][0] + coeff0_6*dmats0[6][0] + coeff0_7*dmats0[7][0] + coeff0_8*dmats0[8][0] + coeff0_9*dmats0[9][0];
          new_coeff0_1 = coeff0_0*dmats0[0][1] + coeff0_1*dmats0[1][1] + coeff0_2*dmats0[2][1] + coeff0_3*dmats0[3][1] + coeff0_4*dmats0[4][1] + coeff0_5*dmats0[5][1] + coeff0_6*dmats0[6][1] + coeff0_7*dmats0[7][1] + coeff0_8*dmats0[8][1] + coeff0_9*dmats0[9][1];
          new_coeff0_2 = coeff0_0*dmats0[0][2] + coeff0_1*dmats0[1][2] + coeff0_2*dmats0[2][2] + coeff0_3*dmats0[3][2] + coeff0_4*dmats0[4][2] + coeff0_5*dmats0[5][2] + coeff0_6*dmats0[6][2] + coeff0_7*dmats0[7][2] + coeff0_8*dmats0[8][2] + coeff0_9*dmats0[9][2];
          new_coeff0_3 = coeff0_0*dmats0[0][3] + coeff0_1*dmats0[1][3] + coeff0_2*dmats0[2][3] + coeff0_3*dmats0[3][3] + coeff0_4*dmats0[4][3] + coeff0_5*dmats0[5][3] + coeff0_6*dmats0[6][3] + coeff0_7*dmats0[7][3] + coeff0_8*dmats0[8][3] + coeff0_9*dmats0[9][3];
          new_coeff0_4 = coeff0_0*dmats0[0][4] + coeff0_1*dmats0[1][4] + coeff0_2*dmats0[2][4] + coeff0_3*dmats0[3][4] + coeff0_4*dmats0[4][4] + coeff0_5*dmats0[5][4] + coeff0_6*dmats0[6][4] + coeff0_7*dmats0[7][4] + coeff0_8*dmats0[8][4] + coeff0_9*dmats0[9][4];
          new_coeff0_5 = coeff0_0*dmats0[0][5] + coeff0_1*dmats0[1][5] + coeff0_2*dmats0[2][5] + coeff0_3*dmats0[3][5] + coeff0_4*dmats0[4][5] + coeff0_5*dmats0[5][5] + coeff0_6*dmats0[6][5] + coeff0_7*dmats0[7][5] + coeff0_8*dmats0[8][5] + coeff0_9*dmats0[9][5];
          new_coeff0_6 = coeff0_0*dmats0[0][6] + coeff0_1*dmats0[1][6] + coeff0_2*dmats0[2][6] + coeff0_3*dmats0[3][6] + coeff0_4*dmats0[4][6] + coeff0_5*dmats0[5][6] + coeff0_6*dmats0[6][6] + coeff0_7*dmats0[7][6] + coeff0_8*dmats0[8][6] + coeff0_9*dmats0[9][6];
          new_coeff0_7 = coeff0_0*dmats0[0][7] + coeff0_1*dmats0[1][7] + coeff0_2*dmats0[2][7] + coeff0_3*dmats0[3][7] + coeff0_4*dmats0[4][7] + coeff0_5*dmats0[5][7] + coeff0_6*dmats0[6][7] + coeff0_7*dmats0[7][7] + coeff0_8*dmats0[8][7] + coeff0_9*dmats0[9][7];
          new_coeff0_8 = coeff0_0*dmats0[0][8] + coeff0_1*dmats0[1][8] + coeff0_2*dmats0[2][8] + coeff0_3*dmats0[3][8] + coeff0_4*dmats0[4][8] + coeff0_5*dmats0[5][8] + coeff0_6*dmats0[6][8] + coeff0_7*dmats0[7][8] + coeff0_8*dmats0[8][8] + coeff0_9*dmats0[9][8];
          new_coeff0_9 = coeff0_0*dmats0[0][9] + coeff0_1*dmats0[1][9] + coeff0_2*dmats0[2][9] + coeff0_3*dmats0[3][9] + coeff0_4*dmats0[4][9] + coeff0_5*dmats0[5][9] + coeff0_6*dmats0[6][9] + coeff0_7*dmats0[7][9] + coeff0_8*dmats0[8][9] + coeff0_9*dmats0[9][9];
        }
        if(combinations[deriv_num][j] == 1)
        {
          new_coeff0_0 = coeff0_0*dmats1[0][0] + coeff0_1*dmats1[1][0] + coeff0_2*dmats1[2][0] + coeff0_3*dmats1[3][0] + coeff0_4*dmats1[4][0] + coeff0_5*dmats1[5][0] + coeff0_6*dmats1[6][0] + coeff0_7*dmats1[7][0] + coeff0_8*dmats1[8][0] + coeff0_9*dmats1[9][0];
          new_coeff0_1 = coeff0_0*dmats1[0][1] + coeff0_1*dmats1[1][1] + coeff0_2*dmats1[2][1] + coeff0_3*dmats1[3][1] + coeff0_4*dmats1[4][1] + coeff0_5*dmats1[5][1] + coeff0_6*dmats1[6][1] + coeff0_7*dmats1[7][1] + coeff0_8*dmats1[8][1] + coeff0_9*dmats1[9][1];
          new_coeff0_2 = coeff0_0*dmats1[0][2] + coeff0_1*dmats1[1][2] + coeff0_2*dmats1[2][2] + coeff0_3*dmats1[3][2] + coeff0_4*dmats1[4][2] + coeff0_5*dmats1[5][2] + coeff0_6*dmats1[6][2] + coeff0_7*dmats1[7][2] + coeff0_8*dmats1[8][2] + coeff0_9*dmats1[9][2];
          new_coeff0_3 = coeff0_0*dmats1[0][3] + coeff0_1*dmats1[1][3] + coeff0_2*dmats1[2][3] + coeff0_3*dmats1[3][3] + coeff0_4*dmats1[4][3] + coeff0_5*dmats1[5][3] + coeff0_6*dmats1[6][3] + coeff0_7*dmats1[7][3] + coeff0_8*dmats1[8][3] + coeff0_9*dmats1[9][3];
          new_coeff0_4 = coeff0_0*dmats1[0][4] + coeff0_1*dmats1[1][4] + coeff0_2*dmats1[2][4] + coeff0_3*dmats1[3][4] + coeff0_4*dmats1[4][4] + coeff0_5*dmats1[5][4] + coeff0_6*dmats1[6][4] + coeff0_7*dmats1[7][4] + coeff0_8*dmats1[8][4] + coeff0_9*dmats1[9][4];
          new_coeff0_5 = coeff0_0*dmats1[0][5] + coeff0_1*dmats1[1][5] + coeff0_2*dmats1[2][5] + coeff0_3*dmats1[3][5] + coeff0_4*dmats1[4][5] + coeff0_5*dmats1[5][5] + coeff0_6*dmats1[6][5] + coeff0_7*dmats1[7][5] + coeff0_8*dmats1[8][5] + coeff0_9*dmats1[9][5];
          new_coeff0_6 = coeff0_0*dmats1[0][6] + coeff0_1*dmats1[1][6] + coeff0_2*dmats1[2][6] + coeff0_3*dmats1[3][6] + coeff0_4*dmats1[4][6] + coeff0_5*dmats1[5][6] + coeff0_6*dmats1[6][6] + coeff0_7*dmats1[7][6] + coeff0_8*dmats1[8][6] + coeff0_9*dmats1[9][6];
          new_coeff0_7 = coeff0_0*dmats1[0][7] + coeff0_1*dmats1[1][7] + coeff0_2*dmats1[2][7] + coeff0_3*dmats1[3][7] + coeff0_4*dmats1[4][7] + coeff0_5*dmats1[5][7] + coeff0_6*dmats1[6][7] + coeff0_7*dmats1[7][7] + coeff0_8*dmats1[8][7] + coeff0_9*dmats1[9][7];
          new_coeff0_8 = coeff0_0*dmats1[0][8] + coeff0_1*dmats1[1][8] + coeff0_2*dmats1[2][8] + coeff0_3*dmats1[3][8] + coeff0_4*dmats1[4][8] + coeff0_5*dmats1[5][8] + coeff0_6*dmats1[6][8] + coeff0_7*dmats1[7][8] + coeff0_8*dmats1[8][8] + coeff0_9*dmats1[9][8];
          new_coeff0_9 = coeff0_0*dmats1[0][9] + coeff0_1*dmats1[1][9] + coeff0_2*dmats1[2][9] + coeff0_3*dmats1[3][9] + coeff0_4*dmats1[4][9] + coeff0_5*dmats1[5][9] + coeff0_6*dmats1[6][9] + coeff0_7*dmats1[7][9] + coeff0_8*dmats1[8][9] + coeff0_9*dmats1[9][9];
        }
        if(combinations[deriv_num][j] == 2)
        {
          new_coeff0_0 = coeff0_0*dmats2[0][0] + coeff0_1*dmats2[1][0] + coeff0_2*dmats2[2][0] + coeff0_3*dmats2[3][0] + coeff0_4*dmats2[4][0] + coeff0_5*dmats2[5][0] + coeff0_6*dmats2[6][0] + coeff0_7*dmats2[7][0] + coeff0_8*dmats2[8][0] + coeff0_9*dmats2[9][0];
          new_coeff0_1 = coeff0_0*dmats2[0][1] + coeff0_1*dmats2[1][1] + coeff0_2*dmats2[2][1] + coeff0_3*dmats2[3][1] + coeff0_4*dmats2[4][1] + coeff0_5*dmats2[5][1] + coeff0_6*dmats2[6][1] + coeff0_7*dmats2[7][1] + coeff0_8*dmats2[8][1] + coeff0_9*dmats2[9][1];
          new_coeff0_2 = coeff0_0*dmats2[0][2] + coeff0_1*dmats2[1][2] + coeff0_2*dmats2[2][2] + coeff0_3*dmats2[3][2] + coeff0_4*dmats2[4][2] + coeff0_5*dmats2[5][2] + coeff0_6*dmats2[6][2] + coeff0_7*dmats2[7][2] + coeff0_8*dmats2[8][2] + coeff0_9*dmats2[9][2];
          new_coeff0_3 = coeff0_0*dmats2[0][3] + coeff0_1*dmats2[1][3] + coeff0_2*dmats2[2][3] + coeff0_3*dmats2[3][3] + coeff0_4*dmats2[4][3] + coeff0_5*dmats2[5][3] + coeff0_6*dmats2[6][3] + coeff0_7*dmats2[7][3] + coeff0_8*dmats2[8][3] + coeff0_9*dmats2[9][3];
          new_coeff0_4 = coeff0_0*dmats2[0][4] + coeff0_1*dmats2[1][4] + coeff0_2*dmats2[2][4] + coeff0_3*dmats2[3][4] + coeff0_4*dmats2[4][4] + coeff0_5*dmats2[5][4] + coeff0_6*dmats2[6][4] + coeff0_7*dmats2[7][4] + coeff0_8*dmats2[8][4] + coeff0_9*dmats2[9][4];
          new_coeff0_5 = coeff0_0*dmats2[0][5] + coeff0_1*dmats2[1][5] + coeff0_2*dmats2[2][5] + coeff0_3*dmats2[3][5] + coeff0_4*dmats2[4][5] + coeff0_5*dmats2[5][5] + coeff0_6*dmats2[6][5] + coeff0_7*dmats2[7][5] + coeff0_8*dmats2[8][5] + coeff0_9*dmats2[9][5];
          new_coeff0_6 = coeff0_0*dmats2[0][6] + coeff0_1*dmats2[1][6] + coeff0_2*dmats2[2][6] + coeff0_3*dmats2[3][6] + coeff0_4*dmats2[4][6] + coeff0_5*dmats2[5][6] + coeff0_6*dmats2[6][6] + coeff0_7*dmats2[7][6] + coeff0_8*dmats2[8][6] + coeff0_9*dmats2[9][6];
          new_coeff0_7 = coeff0_0*dmats2[0][7] + coeff0_1*dmats2[1][7] + coeff0_2*dmats2[2][7] + coeff0_3*dmats2[3][7] + coeff0_4*dmats2[4][7] + coeff0_5*dmats2[5][7] + coeff0_6*dmats2[6][7] + coeff0_7*dmats2[7][7] + coeff0_8*dmats2[8][7] + coeff0_9*dmats2[9][7];
          new_coeff0_8 = coeff0_0*dmats2[0][8] + coeff0_1*dmats2[1][8] + coeff0_2*dmats2[2][8] + coeff0_3*dmats2[3][8] + coeff0_4*dmats2[4][8] + coeff0_5*dmats2[5][8] + coeff0_6*dmats2[6][8] + coeff0_7*dmats2[7][8] + coeff0_8*dmats2[8][8] + coeff0_9*dmats2[9][8];
          new_coeff0_9 = coeff0_0*dmats2[0][9] + coeff0_1*dmats2[1][9] + coeff0_2*dmats2[2][9] + coeff0_3*dmats2[3][9] + coeff0_4*dmats2[4][9] + coeff0_5*dmats2[5][9] + coeff0_6*dmats2[6][9] + coeff0_7*dmats2[7][9] + coeff0_8*dmats2[8][9] + coeff0_9*dmats2[9][9];
        }
    
      }
      // Compute derivatives on reference element as dot product of coefficients and basisvalues
      derivatives[deriv_num] = new_coeff0_0*basisvalue0 + new_coeff0_1*basisvalue1 + new_coeff0_2*basisvalue2 + new_coeff0_3*basisvalue3 + new_coeff0_4*basisvalue4 + new_coeff0_5*basisvalue5 + new_coeff0_6*basisvalue6 + new_coeff0_7*basisvalue7 + new_coeff0_8*basisvalue8 + new_coeff0_9*basisvalue9;
    }
    
    // Transform derivatives back to physical element
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        values[row] += transform[row][col]*derivatives[col];
      }
    }
    // Delete pointer to array of derivatives on FIAT element
    delete [] derivatives;
    
    // Delete pointer to array of combinations of derivatives and transform
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      delete [] combinations[row];
      delete [] transform[row];
    }
    
    delete [] combinations;
    delete [] transform;
  }

  /// Evaluate order n derivatives of all basis functions at given point in cell
  virtual void evaluate_basis_derivatives_all(unsigned int n,
                                              double* values,
                                              const double* coordinates,
                                              const ufc::cell& c) const
  {
    throw std::runtime_error("The vectorised version of evaluate_basis_derivatives() is not yet implemented.");
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    // The reference points, direction and weights:
    const static double X[10][1][3] = {{{0, 0, 0}}, {{1, 0, 0}}, {{0, 1, 0}}, {{0, 0, 1}}, {{0, 0.5, 0.5}}, {{0.5, 0, 0.5}}, {{0.5, 0.5, 0}}, {{0, 0, 0.5}}, {{0, 0.5, 0}}, {{0.5, 0, 0}}};
    const static double W[10][1] = {{1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}};
    const static double D[10][1][1] = {{{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}};
    
    const double * const * x = c.coordinates;
    double result = 0.0;
    // Iterate over the points:
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0][0] - X[i][0][1] - X[i][0][2];
    const double w1 = X[i][0][0];
    const double w2 = X[i][0][1];
    const double w3 = X[i][0][2];
    
    // Compute affine mapping y = F(X)
    double y[3];
    y[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0] + w3*x[3][0];
    y[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1] + w3*x[3][1];
    y[2] = w0*x[0][2] + w1*x[1][2] + w2*x[2][2] + w3*x[3][2];
    
    // Evaluate function at physical points
    double values[1];
    f.evaluate(values, y, c);
    
    // Map function values using appropriate mapping
    // Affine map: Do nothing
    
    // Note that we do not map the weights (yet).
    
    // Take directional components
    for(int k = 0; k < 1; k++)
      result += values[k]*D[i][0][k];
    // Multiply by weights 
    result *= W[i][0];
    
    return result;
  }

  /// Evaluate linear functionals for all dofs on the function f
  virtual void evaluate_dofs(double* values,
                             const ufc::function& f,
                             const ufc::cell& c) const
  {
    throw std::runtime_error("Not implemented (introduced in UFC v1.1).");
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    // Evaluate at vertices and use affine mapping
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[1];
    vertex_values[2] = dof_values[2];
    vertex_values[3] = dof_values[3];
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 1;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    return new VectorLaplaceGradCurlLinearForm_finite_element_1_0();
  }

};

/// This class defines the interface for a finite element.

class VectorLaplaceGradCurlLinearForm_finite_element_1_1: public ufc::finite_element
{
public:

  /// Constructor
  VectorLaplaceGradCurlLinearForm_finite_element_1_1() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~VectorLaplaceGradCurlLinearForm_finite_element_1_1()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "Lagrange finite element of degree 2 on a tetrahedron";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::tetrahedron;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 10;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 0;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 1;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-09)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-09)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Reset values
    *values = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
    const double scalings_y_2 = scalings_y_1*(0.5 - 0.5*y);
    const double scalings_z_0 = 1;
    const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    const double scalings_z_2 = scalings_z_1*(0.5 - 0.5*z);
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    const double psitilde_a_1 = x;
    const double psitilde_a_2 = 1.5*x*psitilde_a_1 - 0.5*psitilde_a_0;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    const double psitilde_bs_0_1 = 1.5*y + 0.5;
    const double psitilde_bs_0_2 = 0.1111111111*psitilde_bs_0_1 + 1.666666667*y*psitilde_bs_0_1 - 0.5555555556*psitilde_bs_0_0;
    const double psitilde_bs_1_0 = 1;
    const double psitilde_bs_1_1 = 2.5*y + 1.5;
    const double psitilde_bs_2_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    const double psitilde_cs_00_1 = 2*z + 1;
    const double psitilde_cs_00_2 = 0.3125*psitilde_cs_00_1 + 1.875*z*psitilde_cs_00_1 - 0.5625*psitilde_cs_00_0;
    const double psitilde_cs_01_0 = 1;
    const double psitilde_cs_01_1 = 3*z + 2;
    const double psitilde_cs_02_0 = 1;
    const double psitilde_cs_10_0 = 1;
    const double psitilde_cs_10_1 = 3*z + 2;
    const double psitilde_cs_11_0 = 1;
    const double psitilde_cs_20_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.8660254038*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    const double basisvalue1 = 2.738612788*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
    const double basisvalue2 = 1.58113883*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
    const double basisvalue3 = 1.118033989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    const double basisvalue4 = 5.123475383*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_0;
    const double basisvalue5 = 3.968626967*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_0;
    const double basisvalue6 = 2.291287847*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_0;
    const double basisvalue7 = 3.240370349*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_1;
    const double basisvalue8 = 1.870828693*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_1;
    const double basisvalue9 = 1.322875656*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_2;
    
    // Table(s) of coefficients
    const static double coefficients0[10][10] = \
    {{-0.05773502692, -0.06085806195, -0.03513641845, -0.02484519975, 0.06506000486, 0.05039526307, 0.0290957187, 0.04114755999, 0.02375655484, 0.01679842102},
    {-0.05773502692, 0.06085806195, -0.03513641845, -0.02484519975, 0.06506000486, -0.05039526307, 0.0290957187, -0.04114755999, 0.02375655484, 0.01679842102},
    {-0.05773502692, 0, 0.07027283689, -0.02484519975, 0, 0, 0.08728715609, 0, -0.04751310967, 0.01679842102},
    {-0.05773502692, 0, 0, 0.07453559925, 0, 0, 0, 0, 0, 0.1007905261},
    {0.2309401077, 0, 0.1405456738, 0.099380799, 0, 0, 0, 0, 0.1187827742, -0.06719368409},
    {0.2309401077, 0.1217161239, -0.07027283689, 0.099380799, 0, 0, 0, 0.1028689, -0.05939138709, -0.06719368409},
    {0.2309401077, 0.1217161239, 0.07027283689, -0.099380799, 0, 0.1007905261, -0.08728715609, -0.02057377999, -0.01187827742, 0.01679842102},
    {0.2309401077, -0.1217161239, -0.07027283689, 0.099380799, 0, 0, 0, -0.1028689, -0.05939138709, -0.06719368409},
    {0.2309401077, -0.1217161239, 0.07027283689, -0.099380799, 0, -0.1007905261, -0.08728715609, 0.02057377999, -0.01187827742, 0.01679842102},
    {0.2309401077, 0, -0.1405456738, -0.099380799, -0.1301200097, 0, 0.0290957187, 0, 0.02375655484, 0.01679842102}};
    
    // Extract relevant coefficients
    const double coeff0_0 = coefficients0[dof][0];
    const double coeff0_1 = coefficients0[dof][1];
    const double coeff0_2 = coefficients0[dof][2];
    const double coeff0_3 = coefficients0[dof][3];
    const double coeff0_4 = coefficients0[dof][4];
    const double coeff0_5 = coefficients0[dof][5];
    const double coeff0_6 = coefficients0[dof][6];
    const double coeff0_7 = coefficients0[dof][7];
    const double coeff0_8 = coefficients0[dof][8];
    const double coeff0_9 = coefficients0[dof][9];
    
    // Compute value(s)
    *values = coeff0_0*basisvalue0 + coeff0_1*basisvalue1 + coeff0_2*basisvalue2 + coeff0_3*basisvalue3 + coeff0_4*basisvalue4 + coeff0_5*basisvalue5 + coeff0_6*basisvalue6 + coeff0_7*basisvalue7 + coeff0_8*basisvalue8 + coeff0_9*basisvalue9;
  }

  /// Evaluate all basis functions at given point in cell
  virtual void evaluate_basis_all(double* values,
                                  const double* coordinates,
                                  const ufc::cell& c) const
  {
    throw std::runtime_error("The vectorised version of evaluate_basis() is not yet implemented.");
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-09)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-09)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Compute number of derivatives
    unsigned int num_derivatives = 1;
    
    for (unsigned int j = 0; j < n; j++)
      num_derivatives *= 3;
    
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      combinations[j] = new unsigned int [n];
      for (unsigned int k = 0; k < n; k++)
        combinations[j][k] = 0;
    }
        
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 2)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian
    const double Jinv[3][3] ={{d00 / detJ, d10 / detJ, d20 / detJ}, {d01 / detJ, d11 / detJ, d21 / detJ}, {d02 / detJ, d12 / detJ, d22 / detJ}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values
    for (unsigned int j = 0; j < 1*num_derivatives; j++)
      values[j] = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
    const double scalings_y_2 = scalings_y_1*(0.5 - 0.5*y);
    const double scalings_z_0 = 1;
    const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    const double scalings_z_2 = scalings_z_1*(0.5 - 0.5*z);
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    const double psitilde_a_1 = x;
    const double psitilde_a_2 = 1.5*x*psitilde_a_1 - 0.5*psitilde_a_0;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    const double psitilde_bs_0_1 = 1.5*y + 0.5;
    const double psitilde_bs_0_2 = 0.1111111111*psitilde_bs_0_1 + 1.666666667*y*psitilde_bs_0_1 - 0.5555555556*psitilde_bs_0_0;
    const double psitilde_bs_1_0 = 1;
    const double psitilde_bs_1_1 = 2.5*y + 1.5;
    const double psitilde_bs_2_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    const double psitilde_cs_00_1 = 2*z + 1;
    const double psitilde_cs_00_2 = 0.3125*psitilde_cs_00_1 + 1.875*z*psitilde_cs_00_1 - 0.5625*psitilde_cs_00_0;
    const double psitilde_cs_01_0 = 1;
    const double psitilde_cs_01_1 = 3*z + 2;
    const double psitilde_cs_02_0 = 1;
    const double psitilde_cs_10_0 = 1;
    const double psitilde_cs_10_1 = 3*z + 2;
    const double psitilde_cs_11_0 = 1;
    const double psitilde_cs_20_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.8660254038*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    const double basisvalue1 = 2.738612788*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
    const double basisvalue2 = 1.58113883*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
    const double basisvalue3 = 1.118033989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    const double basisvalue4 = 5.123475383*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_0;
    const double basisvalue5 = 3.968626967*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_0;
    const double basisvalue6 = 2.291287847*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_0;
    const double basisvalue7 = 3.240370349*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_1;
    const double basisvalue8 = 1.870828693*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_1;
    const double basisvalue9 = 1.322875656*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_2;
    
    // Table(s) of coefficients
    const static double coefficients0[10][10] = \
    {{-0.05773502692, -0.06085806195, -0.03513641845, -0.02484519975, 0.06506000486, 0.05039526307, 0.0290957187, 0.04114755999, 0.02375655484, 0.01679842102},
    {-0.05773502692, 0.06085806195, -0.03513641845, -0.02484519975, 0.06506000486, -0.05039526307, 0.0290957187, -0.04114755999, 0.02375655484, 0.01679842102},
    {-0.05773502692, 0, 0.07027283689, -0.02484519975, 0, 0, 0.08728715609, 0, -0.04751310967, 0.01679842102},
    {-0.05773502692, 0, 0, 0.07453559925, 0, 0, 0, 0, 0, 0.1007905261},
    {0.2309401077, 0, 0.1405456738, 0.099380799, 0, 0, 0, 0, 0.1187827742, -0.06719368409},
    {0.2309401077, 0.1217161239, -0.07027283689, 0.099380799, 0, 0, 0, 0.1028689, -0.05939138709, -0.06719368409},
    {0.2309401077, 0.1217161239, 0.07027283689, -0.099380799, 0, 0.1007905261, -0.08728715609, -0.02057377999, -0.01187827742, 0.01679842102},
    {0.2309401077, -0.1217161239, -0.07027283689, 0.099380799, 0, 0, 0, -0.1028689, -0.05939138709, -0.06719368409},
    {0.2309401077, -0.1217161239, 0.07027283689, -0.099380799, 0, -0.1007905261, -0.08728715609, 0.02057377999, -0.01187827742, 0.01679842102},
    {0.2309401077, 0, -0.1405456738, -0.099380799, -0.1301200097, 0, 0.0290957187, 0, 0.02375655484, 0.01679842102}};
    
    // Interesting (new) part
    // Tables of derivatives of the polynomial base (transpose)
    const static double dmats0[10][10] = \
    {{0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {6.32455532, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 11.22497216, 0, 0, 0, 0, 0, 0, 0, 0},
    {4.582575695, 0, 8.366600265, -1.183215957, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {3.741657387, 0, 0, 8.694826048, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0}};
    
    const static double dmats1[10][10] = \
    {{0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {3.16227766, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {5.477225575, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.958039892, 5.61248608, -1.08012345, -0.7637626158, 0, 0, 0, 0, 0, 0},
    {2.291287847, 7.245688373, 4.183300133, -0.5916079783, 0, 0, 0, 0, 0, 0},
    {-2.645751311, 0, 9.660917831, 0.6831300511, 0, 0, 0, 0, 0, 0},
    {1.870828693, 0, 0, 4.347413024, 0, 0, 0, 0, 0, 0},
    {3.240370349, 0, 0, 7.529940239, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0}};
    
    const static double dmats2[10][10] = \
    {{0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {3.16227766, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.825741858, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {5.163977795, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.958039892, 5.61248608, -1.08012345, -0.7637626158, 0, 0, 0, 0, 0, 0},
    {2.291287847, 1.449137675, 4.183300133, -0.5916079783, 0, 0, 0, 0, 0, 0},
    {1.322875656, 0, 3.864367132, -0.3415650255, 0, 0, 0, 0, 0, 0},
    {1.870828693, 7.09929574, 0, 4.347413024, 0, 0, 0, 0, 0, 0},
    {1.08012345, 0, 7.09929574, 2.50998008, 0, 0, 0, 0, 0, 0},
    {-3.818813079, 0, 0, 8.874119675, 0, 0, 0, 0, 0, 0}};
    
    // Compute reference derivatives
    // Declare pointer to array of derivatives on FIAT element
    double *derivatives = new double [num_derivatives];
    
    // Declare coefficients
    double coeff0_0 = 0;
    double coeff0_1 = 0;
    double coeff0_2 = 0;
    double coeff0_3 = 0;
    double coeff0_4 = 0;
    double coeff0_5 = 0;
    double coeff0_6 = 0;
    double coeff0_7 = 0;
    double coeff0_8 = 0;
    double coeff0_9 = 0;
    
    // Declare new coefficients
    double new_coeff0_0 = 0;
    double new_coeff0_1 = 0;
    double new_coeff0_2 = 0;
    double new_coeff0_3 = 0;
    double new_coeff0_4 = 0;
    double new_coeff0_5 = 0;
    double new_coeff0_6 = 0;
    double new_coeff0_7 = 0;
    double new_coeff0_8 = 0;
    double new_coeff0_9 = 0;
    
    // Loop possible derivatives
    for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
    {
      // Get values from coefficients array
      new_coeff0_0 = coefficients0[dof][0];
      new_coeff0_1 = coefficients0[dof][1];
      new_coeff0_2 = coefficients0[dof][2];
      new_coeff0_3 = coefficients0[dof][3];
      new_coeff0_4 = coefficients0[dof][4];
      new_coeff0_5 = coefficients0[dof][5];
      new_coeff0_6 = coefficients0[dof][6];
      new_coeff0_7 = coefficients0[dof][7];
      new_coeff0_8 = coefficients0[dof][8];
      new_coeff0_9 = coefficients0[dof][9];
    
      // Loop derivative order
      for (unsigned int j = 0; j < n; j++)
      {
        // Update old coefficients
        coeff0_0 = new_coeff0_0;
        coeff0_1 = new_coeff0_1;
        coeff0_2 = new_coeff0_2;
        coeff0_3 = new_coeff0_3;
        coeff0_4 = new_coeff0_4;
        coeff0_5 = new_coeff0_5;
        coeff0_6 = new_coeff0_6;
        coeff0_7 = new_coeff0_7;
        coeff0_8 = new_coeff0_8;
        coeff0_9 = new_coeff0_9;
    
        if(combinations[deriv_num][j] == 0)
        {
          new_coeff0_0 = coeff0_0*dmats0[0][0] + coeff0_1*dmats0[1][0] + coeff0_2*dmats0[2][0] + coeff0_3*dmats0[3][0] + coeff0_4*dmats0[4][0] + coeff0_5*dmats0[5][0] + coeff0_6*dmats0[6][0] + coeff0_7*dmats0[7][0] + coeff0_8*dmats0[8][0] + coeff0_9*dmats0[9][0];
          new_coeff0_1 = coeff0_0*dmats0[0][1] + coeff0_1*dmats0[1][1] + coeff0_2*dmats0[2][1] + coeff0_3*dmats0[3][1] + coeff0_4*dmats0[4][1] + coeff0_5*dmats0[5][1] + coeff0_6*dmats0[6][1] + coeff0_7*dmats0[7][1] + coeff0_8*dmats0[8][1] + coeff0_9*dmats0[9][1];
          new_coeff0_2 = coeff0_0*dmats0[0][2] + coeff0_1*dmats0[1][2] + coeff0_2*dmats0[2][2] + coeff0_3*dmats0[3][2] + coeff0_4*dmats0[4][2] + coeff0_5*dmats0[5][2] + coeff0_6*dmats0[6][2] + coeff0_7*dmats0[7][2] + coeff0_8*dmats0[8][2] + coeff0_9*dmats0[9][2];
          new_coeff0_3 = coeff0_0*dmats0[0][3] + coeff0_1*dmats0[1][3] + coeff0_2*dmats0[2][3] + coeff0_3*dmats0[3][3] + coeff0_4*dmats0[4][3] + coeff0_5*dmats0[5][3] + coeff0_6*dmats0[6][3] + coeff0_7*dmats0[7][3] + coeff0_8*dmats0[8][3] + coeff0_9*dmats0[9][3];
          new_coeff0_4 = coeff0_0*dmats0[0][4] + coeff0_1*dmats0[1][4] + coeff0_2*dmats0[2][4] + coeff0_3*dmats0[3][4] + coeff0_4*dmats0[4][4] + coeff0_5*dmats0[5][4] + coeff0_6*dmats0[6][4] + coeff0_7*dmats0[7][4] + coeff0_8*dmats0[8][4] + coeff0_9*dmats0[9][4];
          new_coeff0_5 = coeff0_0*dmats0[0][5] + coeff0_1*dmats0[1][5] + coeff0_2*dmats0[2][5] + coeff0_3*dmats0[3][5] + coeff0_4*dmats0[4][5] + coeff0_5*dmats0[5][5] + coeff0_6*dmats0[6][5] + coeff0_7*dmats0[7][5] + coeff0_8*dmats0[8][5] + coeff0_9*dmats0[9][5];
          new_coeff0_6 = coeff0_0*dmats0[0][6] + coeff0_1*dmats0[1][6] + coeff0_2*dmats0[2][6] + coeff0_3*dmats0[3][6] + coeff0_4*dmats0[4][6] + coeff0_5*dmats0[5][6] + coeff0_6*dmats0[6][6] + coeff0_7*dmats0[7][6] + coeff0_8*dmats0[8][6] + coeff0_9*dmats0[9][6];
          new_coeff0_7 = coeff0_0*dmats0[0][7] + coeff0_1*dmats0[1][7] + coeff0_2*dmats0[2][7] + coeff0_3*dmats0[3][7] + coeff0_4*dmats0[4][7] + coeff0_5*dmats0[5][7] + coeff0_6*dmats0[6][7] + coeff0_7*dmats0[7][7] + coeff0_8*dmats0[8][7] + coeff0_9*dmats0[9][7];
          new_coeff0_8 = coeff0_0*dmats0[0][8] + coeff0_1*dmats0[1][8] + coeff0_2*dmats0[2][8] + coeff0_3*dmats0[3][8] + coeff0_4*dmats0[4][8] + coeff0_5*dmats0[5][8] + coeff0_6*dmats0[6][8] + coeff0_7*dmats0[7][8] + coeff0_8*dmats0[8][8] + coeff0_9*dmats0[9][8];
          new_coeff0_9 = coeff0_0*dmats0[0][9] + coeff0_1*dmats0[1][9] + coeff0_2*dmats0[2][9] + coeff0_3*dmats0[3][9] + coeff0_4*dmats0[4][9] + coeff0_5*dmats0[5][9] + coeff0_6*dmats0[6][9] + coeff0_7*dmats0[7][9] + coeff0_8*dmats0[8][9] + coeff0_9*dmats0[9][9];
        }
        if(combinations[deriv_num][j] == 1)
        {
          new_coeff0_0 = coeff0_0*dmats1[0][0] + coeff0_1*dmats1[1][0] + coeff0_2*dmats1[2][0] + coeff0_3*dmats1[3][0] + coeff0_4*dmats1[4][0] + coeff0_5*dmats1[5][0] + coeff0_6*dmats1[6][0] + coeff0_7*dmats1[7][0] + coeff0_8*dmats1[8][0] + coeff0_9*dmats1[9][0];
          new_coeff0_1 = coeff0_0*dmats1[0][1] + coeff0_1*dmats1[1][1] + coeff0_2*dmats1[2][1] + coeff0_3*dmats1[3][1] + coeff0_4*dmats1[4][1] + coeff0_5*dmats1[5][1] + coeff0_6*dmats1[6][1] + coeff0_7*dmats1[7][1] + coeff0_8*dmats1[8][1] + coeff0_9*dmats1[9][1];
          new_coeff0_2 = coeff0_0*dmats1[0][2] + coeff0_1*dmats1[1][2] + coeff0_2*dmats1[2][2] + coeff0_3*dmats1[3][2] + coeff0_4*dmats1[4][2] + coeff0_5*dmats1[5][2] + coeff0_6*dmats1[6][2] + coeff0_7*dmats1[7][2] + coeff0_8*dmats1[8][2] + coeff0_9*dmats1[9][2];
          new_coeff0_3 = coeff0_0*dmats1[0][3] + coeff0_1*dmats1[1][3] + coeff0_2*dmats1[2][3] + coeff0_3*dmats1[3][3] + coeff0_4*dmats1[4][3] + coeff0_5*dmats1[5][3] + coeff0_6*dmats1[6][3] + coeff0_7*dmats1[7][3] + coeff0_8*dmats1[8][3] + coeff0_9*dmats1[9][3];
          new_coeff0_4 = coeff0_0*dmats1[0][4] + coeff0_1*dmats1[1][4] + coeff0_2*dmats1[2][4] + coeff0_3*dmats1[3][4] + coeff0_4*dmats1[4][4] + coeff0_5*dmats1[5][4] + coeff0_6*dmats1[6][4] + coeff0_7*dmats1[7][4] + coeff0_8*dmats1[8][4] + coeff0_9*dmats1[9][4];
          new_coeff0_5 = coeff0_0*dmats1[0][5] + coeff0_1*dmats1[1][5] + coeff0_2*dmats1[2][5] + coeff0_3*dmats1[3][5] + coeff0_4*dmats1[4][5] + coeff0_5*dmats1[5][5] + coeff0_6*dmats1[6][5] + coeff0_7*dmats1[7][5] + coeff0_8*dmats1[8][5] + coeff0_9*dmats1[9][5];
          new_coeff0_6 = coeff0_0*dmats1[0][6] + coeff0_1*dmats1[1][6] + coeff0_2*dmats1[2][6] + coeff0_3*dmats1[3][6] + coeff0_4*dmats1[4][6] + coeff0_5*dmats1[5][6] + coeff0_6*dmats1[6][6] + coeff0_7*dmats1[7][6] + coeff0_8*dmats1[8][6] + coeff0_9*dmats1[9][6];
          new_coeff0_7 = coeff0_0*dmats1[0][7] + coeff0_1*dmats1[1][7] + coeff0_2*dmats1[2][7] + coeff0_3*dmats1[3][7] + coeff0_4*dmats1[4][7] + coeff0_5*dmats1[5][7] + coeff0_6*dmats1[6][7] + coeff0_7*dmats1[7][7] + coeff0_8*dmats1[8][7] + coeff0_9*dmats1[9][7];
          new_coeff0_8 = coeff0_0*dmats1[0][8] + coeff0_1*dmats1[1][8] + coeff0_2*dmats1[2][8] + coeff0_3*dmats1[3][8] + coeff0_4*dmats1[4][8] + coeff0_5*dmats1[5][8] + coeff0_6*dmats1[6][8] + coeff0_7*dmats1[7][8] + coeff0_8*dmats1[8][8] + coeff0_9*dmats1[9][8];
          new_coeff0_9 = coeff0_0*dmats1[0][9] + coeff0_1*dmats1[1][9] + coeff0_2*dmats1[2][9] + coeff0_3*dmats1[3][9] + coeff0_4*dmats1[4][9] + coeff0_5*dmats1[5][9] + coeff0_6*dmats1[6][9] + coeff0_7*dmats1[7][9] + coeff0_8*dmats1[8][9] + coeff0_9*dmats1[9][9];
        }
        if(combinations[deriv_num][j] == 2)
        {
          new_coeff0_0 = coeff0_0*dmats2[0][0] + coeff0_1*dmats2[1][0] + coeff0_2*dmats2[2][0] + coeff0_3*dmats2[3][0] + coeff0_4*dmats2[4][0] + coeff0_5*dmats2[5][0] + coeff0_6*dmats2[6][0] + coeff0_7*dmats2[7][0] + coeff0_8*dmats2[8][0] + coeff0_9*dmats2[9][0];
          new_coeff0_1 = coeff0_0*dmats2[0][1] + coeff0_1*dmats2[1][1] + coeff0_2*dmats2[2][1] + coeff0_3*dmats2[3][1] + coeff0_4*dmats2[4][1] + coeff0_5*dmats2[5][1] + coeff0_6*dmats2[6][1] + coeff0_7*dmats2[7][1] + coeff0_8*dmats2[8][1] + coeff0_9*dmats2[9][1];
          new_coeff0_2 = coeff0_0*dmats2[0][2] + coeff0_1*dmats2[1][2] + coeff0_2*dmats2[2][2] + coeff0_3*dmats2[3][2] + coeff0_4*dmats2[4][2] + coeff0_5*dmats2[5][2] + coeff0_6*dmats2[6][2] + coeff0_7*dmats2[7][2] + coeff0_8*dmats2[8][2] + coeff0_9*dmats2[9][2];
          new_coeff0_3 = coeff0_0*dmats2[0][3] + coeff0_1*dmats2[1][3] + coeff0_2*dmats2[2][3] + coeff0_3*dmats2[3][3] + coeff0_4*dmats2[4][3] + coeff0_5*dmats2[5][3] + coeff0_6*dmats2[6][3] + coeff0_7*dmats2[7][3] + coeff0_8*dmats2[8][3] + coeff0_9*dmats2[9][3];
          new_coeff0_4 = coeff0_0*dmats2[0][4] + coeff0_1*dmats2[1][4] + coeff0_2*dmats2[2][4] + coeff0_3*dmats2[3][4] + coeff0_4*dmats2[4][4] + coeff0_5*dmats2[5][4] + coeff0_6*dmats2[6][4] + coeff0_7*dmats2[7][4] + coeff0_8*dmats2[8][4] + coeff0_9*dmats2[9][4];
          new_coeff0_5 = coeff0_0*dmats2[0][5] + coeff0_1*dmats2[1][5] + coeff0_2*dmats2[2][5] + coeff0_3*dmats2[3][5] + coeff0_4*dmats2[4][5] + coeff0_5*dmats2[5][5] + coeff0_6*dmats2[6][5] + coeff0_7*dmats2[7][5] + coeff0_8*dmats2[8][5] + coeff0_9*dmats2[9][5];
          new_coeff0_6 = coeff0_0*dmats2[0][6] + coeff0_1*dmats2[1][6] + coeff0_2*dmats2[2][6] + coeff0_3*dmats2[3][6] + coeff0_4*dmats2[4][6] + coeff0_5*dmats2[5][6] + coeff0_6*dmats2[6][6] + coeff0_7*dmats2[7][6] + coeff0_8*dmats2[8][6] + coeff0_9*dmats2[9][6];
          new_coeff0_7 = coeff0_0*dmats2[0][7] + coeff0_1*dmats2[1][7] + coeff0_2*dmats2[2][7] + coeff0_3*dmats2[3][7] + coeff0_4*dmats2[4][7] + coeff0_5*dmats2[5][7] + coeff0_6*dmats2[6][7] + coeff0_7*dmats2[7][7] + coeff0_8*dmats2[8][7] + coeff0_9*dmats2[9][7];
          new_coeff0_8 = coeff0_0*dmats2[0][8] + coeff0_1*dmats2[1][8] + coeff0_2*dmats2[2][8] + coeff0_3*dmats2[3][8] + coeff0_4*dmats2[4][8] + coeff0_5*dmats2[5][8] + coeff0_6*dmats2[6][8] + coeff0_7*dmats2[7][8] + coeff0_8*dmats2[8][8] + coeff0_9*dmats2[9][8];
          new_coeff0_9 = coeff0_0*dmats2[0][9] + coeff0_1*dmats2[1][9] + coeff0_2*dmats2[2][9] + coeff0_3*dmats2[3][9] + coeff0_4*dmats2[4][9] + coeff0_5*dmats2[5][9] + coeff0_6*dmats2[6][9] + coeff0_7*dmats2[7][9] + coeff0_8*dmats2[8][9] + coeff0_9*dmats2[9][9];
        }
    
      }
      // Compute derivatives on reference element as dot product of coefficients and basisvalues
      derivatives[deriv_num] = new_coeff0_0*basisvalue0 + new_coeff0_1*basisvalue1 + new_coeff0_2*basisvalue2 + new_coeff0_3*basisvalue3 + new_coeff0_4*basisvalue4 + new_coeff0_5*basisvalue5 + new_coeff0_6*basisvalue6 + new_coeff0_7*basisvalue7 + new_coeff0_8*basisvalue8 + new_coeff0_9*basisvalue9;
    }
    
    // Transform derivatives back to physical element
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        values[row] += transform[row][col]*derivatives[col];
      }
    }
    // Delete pointer to array of derivatives on FIAT element
    delete [] derivatives;
    
    // Delete pointer to array of combinations of derivatives and transform
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      delete [] combinations[row];
      delete [] transform[row];
    }
    
    delete [] combinations;
    delete [] transform;
  }

  /// Evaluate order n derivatives of all basis functions at given point in cell
  virtual void evaluate_basis_derivatives_all(unsigned int n,
                                              double* values,
                                              const double* coordinates,
                                              const ufc::cell& c) const
  {
    throw std::runtime_error("The vectorised version of evaluate_basis_derivatives() is not yet implemented.");
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    // The reference points, direction and weights:
    const static double X[10][1][3] = {{{0, 0, 0}}, {{1, 0, 0}}, {{0, 1, 0}}, {{0, 0, 1}}, {{0, 0.5, 0.5}}, {{0.5, 0, 0.5}}, {{0.5, 0.5, 0}}, {{0, 0, 0.5}}, {{0, 0.5, 0}}, {{0.5, 0, 0}}};
    const static double W[10][1] = {{1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}};
    const static double D[10][1][1] = {{{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}};
    
    const double * const * x = c.coordinates;
    double result = 0.0;
    // Iterate over the points:
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0][0] - X[i][0][1] - X[i][0][2];
    const double w1 = X[i][0][0];
    const double w2 = X[i][0][1];
    const double w3 = X[i][0][2];
    
    // Compute affine mapping y = F(X)
    double y[3];
    y[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0] + w3*x[3][0];
    y[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1] + w3*x[3][1];
    y[2] = w0*x[0][2] + w1*x[1][2] + w2*x[2][2] + w3*x[3][2];
    
    // Evaluate function at physical points
    double values[1];
    f.evaluate(values, y, c);
    
    // Map function values using appropriate mapping
    // Affine map: Do nothing
    
    // Note that we do not map the weights (yet).
    
    // Take directional components
    for(int k = 0; k < 1; k++)
      result += values[k]*D[i][0][k];
    // Multiply by weights 
    result *= W[i][0];
    
    return result;
  }

  /// Evaluate linear functionals for all dofs on the function f
  virtual void evaluate_dofs(double* values,
                             const ufc::function& f,
                             const ufc::cell& c) const
  {
    throw std::runtime_error("Not implemented (introduced in UFC v1.1).");
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    // Evaluate at vertices and use affine mapping
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[1];
    vertex_values[2] = dof_values[2];
    vertex_values[3] = dof_values[3];
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 1;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    return new VectorLaplaceGradCurlLinearForm_finite_element_1_1();
  }

};

/// This class defines the interface for a finite element.

class VectorLaplaceGradCurlLinearForm_finite_element_1_2: public ufc::finite_element
{
public:

  /// Constructor
  VectorLaplaceGradCurlLinearForm_finite_element_1_2() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~VectorLaplaceGradCurlLinearForm_finite_element_1_2()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "Lagrange finite element of degree 2 on a tetrahedron";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::tetrahedron;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 10;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 0;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 1;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-09)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-09)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Reset values
    *values = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
    const double scalings_y_2 = scalings_y_1*(0.5 - 0.5*y);
    const double scalings_z_0 = 1;
    const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    const double scalings_z_2 = scalings_z_1*(0.5 - 0.5*z);
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    const double psitilde_a_1 = x;
    const double psitilde_a_2 = 1.5*x*psitilde_a_1 - 0.5*psitilde_a_0;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    const double psitilde_bs_0_1 = 1.5*y + 0.5;
    const double psitilde_bs_0_2 = 0.1111111111*psitilde_bs_0_1 + 1.666666667*y*psitilde_bs_0_1 - 0.5555555556*psitilde_bs_0_0;
    const double psitilde_bs_1_0 = 1;
    const double psitilde_bs_1_1 = 2.5*y + 1.5;
    const double psitilde_bs_2_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    const double psitilde_cs_00_1 = 2*z + 1;
    const double psitilde_cs_00_2 = 0.3125*psitilde_cs_00_1 + 1.875*z*psitilde_cs_00_1 - 0.5625*psitilde_cs_00_0;
    const double psitilde_cs_01_0 = 1;
    const double psitilde_cs_01_1 = 3*z + 2;
    const double psitilde_cs_02_0 = 1;
    const double psitilde_cs_10_0 = 1;
    const double psitilde_cs_10_1 = 3*z + 2;
    const double psitilde_cs_11_0 = 1;
    const double psitilde_cs_20_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.8660254038*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    const double basisvalue1 = 2.738612788*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
    const double basisvalue2 = 1.58113883*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
    const double basisvalue3 = 1.118033989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    const double basisvalue4 = 5.123475383*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_0;
    const double basisvalue5 = 3.968626967*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_0;
    const double basisvalue6 = 2.291287847*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_0;
    const double basisvalue7 = 3.240370349*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_1;
    const double basisvalue8 = 1.870828693*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_1;
    const double basisvalue9 = 1.322875656*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_2;
    
    // Table(s) of coefficients
    const static double coefficients0[10][10] = \
    {{-0.05773502692, -0.06085806195, -0.03513641845, -0.02484519975, 0.06506000486, 0.05039526307, 0.0290957187, 0.04114755999, 0.02375655484, 0.01679842102},
    {-0.05773502692, 0.06085806195, -0.03513641845, -0.02484519975, 0.06506000486, -0.05039526307, 0.0290957187, -0.04114755999, 0.02375655484, 0.01679842102},
    {-0.05773502692, 0, 0.07027283689, -0.02484519975, 0, 0, 0.08728715609, 0, -0.04751310967, 0.01679842102},
    {-0.05773502692, 0, 0, 0.07453559925, 0, 0, 0, 0, 0, 0.1007905261},
    {0.2309401077, 0, 0.1405456738, 0.099380799, 0, 0, 0, 0, 0.1187827742, -0.06719368409},
    {0.2309401077, 0.1217161239, -0.07027283689, 0.099380799, 0, 0, 0, 0.1028689, -0.05939138709, -0.06719368409},
    {0.2309401077, 0.1217161239, 0.07027283689, -0.099380799, 0, 0.1007905261, -0.08728715609, -0.02057377999, -0.01187827742, 0.01679842102},
    {0.2309401077, -0.1217161239, -0.07027283689, 0.099380799, 0, 0, 0, -0.1028689, -0.05939138709, -0.06719368409},
    {0.2309401077, -0.1217161239, 0.07027283689, -0.099380799, 0, -0.1007905261, -0.08728715609, 0.02057377999, -0.01187827742, 0.01679842102},
    {0.2309401077, 0, -0.1405456738, -0.099380799, -0.1301200097, 0, 0.0290957187, 0, 0.02375655484, 0.01679842102}};
    
    // Extract relevant coefficients
    const double coeff0_0 = coefficients0[dof][0];
    const double coeff0_1 = coefficients0[dof][1];
    const double coeff0_2 = coefficients0[dof][2];
    const double coeff0_3 = coefficients0[dof][3];
    const double coeff0_4 = coefficients0[dof][4];
    const double coeff0_5 = coefficients0[dof][5];
    const double coeff0_6 = coefficients0[dof][6];
    const double coeff0_7 = coefficients0[dof][7];
    const double coeff0_8 = coefficients0[dof][8];
    const double coeff0_9 = coefficients0[dof][9];
    
    // Compute value(s)
    *values = coeff0_0*basisvalue0 + coeff0_1*basisvalue1 + coeff0_2*basisvalue2 + coeff0_3*basisvalue3 + coeff0_4*basisvalue4 + coeff0_5*basisvalue5 + coeff0_6*basisvalue6 + coeff0_7*basisvalue7 + coeff0_8*basisvalue8 + coeff0_9*basisvalue9;
  }

  /// Evaluate all basis functions at given point in cell
  virtual void evaluate_basis_all(double* values,
                                  const double* coordinates,
                                  const ufc::cell& c) const
  {
    throw std::runtime_error("The vectorised version of evaluate_basis() is not yet implemented.");
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-09)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-09)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Compute number of derivatives
    unsigned int num_derivatives = 1;
    
    for (unsigned int j = 0; j < n; j++)
      num_derivatives *= 3;
    
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      combinations[j] = new unsigned int [n];
      for (unsigned int k = 0; k < n; k++)
        combinations[j][k] = 0;
    }
        
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 2)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian
    const double Jinv[3][3] ={{d00 / detJ, d10 / detJ, d20 / detJ}, {d01 / detJ, d11 / detJ, d21 / detJ}, {d02 / detJ, d12 / detJ, d22 / detJ}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values
    for (unsigned int j = 0; j < 1*num_derivatives; j++)
      values[j] = 0;
    
    // Map degree of freedom to element degree of freedom
    const unsigned int dof = i;
    
    // Generate scalings
    const double scalings_y_0 = 1;
    const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
    const double scalings_y_2 = scalings_y_1*(0.5 - 0.5*y);
    const double scalings_z_0 = 1;
    const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
    const double scalings_z_2 = scalings_z_1*(0.5 - 0.5*z);
    
    // Compute psitilde_a
    const double psitilde_a_0 = 1;
    const double psitilde_a_1 = x;
    const double psitilde_a_2 = 1.5*x*psitilde_a_1 - 0.5*psitilde_a_0;
    
    // Compute psitilde_bs
    const double psitilde_bs_0_0 = 1;
    const double psitilde_bs_0_1 = 1.5*y + 0.5;
    const double psitilde_bs_0_2 = 0.1111111111*psitilde_bs_0_1 + 1.666666667*y*psitilde_bs_0_1 - 0.5555555556*psitilde_bs_0_0;
    const double psitilde_bs_1_0 = 1;
    const double psitilde_bs_1_1 = 2.5*y + 1.5;
    const double psitilde_bs_2_0 = 1;
    
    // Compute psitilde_cs
    const double psitilde_cs_00_0 = 1;
    const double psitilde_cs_00_1 = 2*z + 1;
    const double psitilde_cs_00_2 = 0.3125*psitilde_cs_00_1 + 1.875*z*psitilde_cs_00_1 - 0.5625*psitilde_cs_00_0;
    const double psitilde_cs_01_0 = 1;
    const double psitilde_cs_01_1 = 3*z + 2;
    const double psitilde_cs_02_0 = 1;
    const double psitilde_cs_10_0 = 1;
    const double psitilde_cs_10_1 = 3*z + 2;
    const double psitilde_cs_11_0 = 1;
    const double psitilde_cs_20_0 = 1;
    
    // Compute basisvalues
    const double basisvalue0 = 0.8660254038*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
    const double basisvalue1 = 2.738612788*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
    const double basisvalue2 = 1.58113883*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
    const double basisvalue3 = 1.118033989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
    const double basisvalue4 = 5.123475383*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_0;
    const double basisvalue5 = 3.968626967*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_0;
    const double basisvalue6 = 2.291287847*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_0;
    const double basisvalue7 = 3.240370349*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_1;
    const double basisvalue8 = 1.870828693*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_1;
    const double basisvalue9 = 1.322875656*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_2;
    
    // Table(s) of coefficients
    const static double coefficients0[10][10] = \
    {{-0.05773502692, -0.06085806195, -0.03513641845, -0.02484519975, 0.06506000486, 0.05039526307, 0.0290957187, 0.04114755999, 0.02375655484, 0.01679842102},
    {-0.05773502692, 0.06085806195, -0.03513641845, -0.02484519975, 0.06506000486, -0.05039526307, 0.0290957187, -0.04114755999, 0.02375655484, 0.01679842102},
    {-0.05773502692, 0, 0.07027283689, -0.02484519975, 0, 0, 0.08728715609, 0, -0.04751310967, 0.01679842102},
    {-0.05773502692, 0, 0, 0.07453559925, 0, 0, 0, 0, 0, 0.1007905261},
    {0.2309401077, 0, 0.1405456738, 0.099380799, 0, 0, 0, 0, 0.1187827742, -0.06719368409},
    {0.2309401077, 0.1217161239, -0.07027283689, 0.099380799, 0, 0, 0, 0.1028689, -0.05939138709, -0.06719368409},
    {0.2309401077, 0.1217161239, 0.07027283689, -0.099380799, 0, 0.1007905261, -0.08728715609, -0.02057377999, -0.01187827742, 0.01679842102},
    {0.2309401077, -0.1217161239, -0.07027283689, 0.099380799, 0, 0, 0, -0.1028689, -0.05939138709, -0.06719368409},
    {0.2309401077, -0.1217161239, 0.07027283689, -0.099380799, 0, -0.1007905261, -0.08728715609, 0.02057377999, -0.01187827742, 0.01679842102},
    {0.2309401077, 0, -0.1405456738, -0.099380799, -0.1301200097, 0, 0.0290957187, 0, 0.02375655484, 0.01679842102}};
    
    // Interesting (new) part
    // Tables of derivatives of the polynomial base (transpose)
    const static double dmats0[10][10] = \
    {{0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {6.32455532, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 11.22497216, 0, 0, 0, 0, 0, 0, 0, 0},
    {4.582575695, 0, 8.366600265, -1.183215957, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {3.741657387, 0, 0, 8.694826048, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0}};
    
    const static double dmats1[10][10] = \
    {{0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {3.16227766, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {5.477225575, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.958039892, 5.61248608, -1.08012345, -0.7637626158, 0, 0, 0, 0, 0, 0},
    {2.291287847, 7.245688373, 4.183300133, -0.5916079783, 0, 0, 0, 0, 0, 0},
    {-2.645751311, 0, 9.660917831, 0.6831300511, 0, 0, 0, 0, 0, 0},
    {1.870828693, 0, 0, 4.347413024, 0, 0, 0, 0, 0, 0},
    {3.240370349, 0, 0, 7.529940239, 0, 0, 0, 0, 0, 0},
    {0, 0, 0, 0, 0, 0, 0, 0, 0, 0}};
    
    const static double dmats2[10][10] = \
    {{0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {3.16227766, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {1.825741858, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {5.163977795, 0, 0, 0, 0, 0, 0, 0, 0, 0},
    {2.958039892, 5.61248608, -1.08012345, -0.7637626158, 0, 0, 0, 0, 0, 0},
    {2.291287847, 1.449137675, 4.183300133, -0.5916079783, 0, 0, 0, 0, 0, 0},
    {1.322875656, 0, 3.864367132, -0.3415650255, 0, 0, 0, 0, 0, 0},
    {1.870828693, 7.09929574, 0, 4.347413024, 0, 0, 0, 0, 0, 0},
    {1.08012345, 0, 7.09929574, 2.50998008, 0, 0, 0, 0, 0, 0},
    {-3.818813079, 0, 0, 8.874119675, 0, 0, 0, 0, 0, 0}};
    
    // Compute reference derivatives
    // Declare pointer to array of derivatives on FIAT element
    double *derivatives = new double [num_derivatives];
    
    // Declare coefficients
    double coeff0_0 = 0;
    double coeff0_1 = 0;
    double coeff0_2 = 0;
    double coeff0_3 = 0;
    double coeff0_4 = 0;
    double coeff0_5 = 0;
    double coeff0_6 = 0;
    double coeff0_7 = 0;
    double coeff0_8 = 0;
    double coeff0_9 = 0;
    
    // Declare new coefficients
    double new_coeff0_0 = 0;
    double new_coeff0_1 = 0;
    double new_coeff0_2 = 0;
    double new_coeff0_3 = 0;
    double new_coeff0_4 = 0;
    double new_coeff0_5 = 0;
    double new_coeff0_6 = 0;
    double new_coeff0_7 = 0;
    double new_coeff0_8 = 0;
    double new_coeff0_9 = 0;
    
    // Loop possible derivatives
    for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
    {
      // Get values from coefficients array
      new_coeff0_0 = coefficients0[dof][0];
      new_coeff0_1 = coefficients0[dof][1];
      new_coeff0_2 = coefficients0[dof][2];
      new_coeff0_3 = coefficients0[dof][3];
      new_coeff0_4 = coefficients0[dof][4];
      new_coeff0_5 = coefficients0[dof][5];
      new_coeff0_6 = coefficients0[dof][6];
      new_coeff0_7 = coefficients0[dof][7];
      new_coeff0_8 = coefficients0[dof][8];
      new_coeff0_9 = coefficients0[dof][9];
    
      // Loop derivative order
      for (unsigned int j = 0; j < n; j++)
      {
        // Update old coefficients
        coeff0_0 = new_coeff0_0;
        coeff0_1 = new_coeff0_1;
        coeff0_2 = new_coeff0_2;
        coeff0_3 = new_coeff0_3;
        coeff0_4 = new_coeff0_4;
        coeff0_5 = new_coeff0_5;
        coeff0_6 = new_coeff0_6;
        coeff0_7 = new_coeff0_7;
        coeff0_8 = new_coeff0_8;
        coeff0_9 = new_coeff0_9;
    
        if(combinations[deriv_num][j] == 0)
        {
          new_coeff0_0 = coeff0_0*dmats0[0][0] + coeff0_1*dmats0[1][0] + coeff0_2*dmats0[2][0] + coeff0_3*dmats0[3][0] + coeff0_4*dmats0[4][0] + coeff0_5*dmats0[5][0] + coeff0_6*dmats0[6][0] + coeff0_7*dmats0[7][0] + coeff0_8*dmats0[8][0] + coeff0_9*dmats0[9][0];
          new_coeff0_1 = coeff0_0*dmats0[0][1] + coeff0_1*dmats0[1][1] + coeff0_2*dmats0[2][1] + coeff0_3*dmats0[3][1] + coeff0_4*dmats0[4][1] + coeff0_5*dmats0[5][1] + coeff0_6*dmats0[6][1] + coeff0_7*dmats0[7][1] + coeff0_8*dmats0[8][1] + coeff0_9*dmats0[9][1];
          new_coeff0_2 = coeff0_0*dmats0[0][2] + coeff0_1*dmats0[1][2] + coeff0_2*dmats0[2][2] + coeff0_3*dmats0[3][2] + coeff0_4*dmats0[4][2] + coeff0_5*dmats0[5][2] + coeff0_6*dmats0[6][2] + coeff0_7*dmats0[7][2] + coeff0_8*dmats0[8][2] + coeff0_9*dmats0[9][2];
          new_coeff0_3 = coeff0_0*dmats0[0][3] + coeff0_1*dmats0[1][3] + coeff0_2*dmats0[2][3] + coeff0_3*dmats0[3][3] + coeff0_4*dmats0[4][3] + coeff0_5*dmats0[5][3] + coeff0_6*dmats0[6][3] + coeff0_7*dmats0[7][3] + coeff0_8*dmats0[8][3] + coeff0_9*dmats0[9][3];
          new_coeff0_4 = coeff0_0*dmats0[0][4] + coeff0_1*dmats0[1][4] + coeff0_2*dmats0[2][4] + coeff0_3*dmats0[3][4] + coeff0_4*dmats0[4][4] + coeff0_5*dmats0[5][4] + coeff0_6*dmats0[6][4] + coeff0_7*dmats0[7][4] + coeff0_8*dmats0[8][4] + coeff0_9*dmats0[9][4];
          new_coeff0_5 = coeff0_0*dmats0[0][5] + coeff0_1*dmats0[1][5] + coeff0_2*dmats0[2][5] + coeff0_3*dmats0[3][5] + coeff0_4*dmats0[4][5] + coeff0_5*dmats0[5][5] + coeff0_6*dmats0[6][5] + coeff0_7*dmats0[7][5] + coeff0_8*dmats0[8][5] + coeff0_9*dmats0[9][5];
          new_coeff0_6 = coeff0_0*dmats0[0][6] + coeff0_1*dmats0[1][6] + coeff0_2*dmats0[2][6] + coeff0_3*dmats0[3][6] + coeff0_4*dmats0[4][6] + coeff0_5*dmats0[5][6] + coeff0_6*dmats0[6][6] + coeff0_7*dmats0[7][6] + coeff0_8*dmats0[8][6] + coeff0_9*dmats0[9][6];
          new_coeff0_7 = coeff0_0*dmats0[0][7] + coeff0_1*dmats0[1][7] + coeff0_2*dmats0[2][7] + coeff0_3*dmats0[3][7] + coeff0_4*dmats0[4][7] + coeff0_5*dmats0[5][7] + coeff0_6*dmats0[6][7] + coeff0_7*dmats0[7][7] + coeff0_8*dmats0[8][7] + coeff0_9*dmats0[9][7];
          new_coeff0_8 = coeff0_0*dmats0[0][8] + coeff0_1*dmats0[1][8] + coeff0_2*dmats0[2][8] + coeff0_3*dmats0[3][8] + coeff0_4*dmats0[4][8] + coeff0_5*dmats0[5][8] + coeff0_6*dmats0[6][8] + coeff0_7*dmats0[7][8] + coeff0_8*dmats0[8][8] + coeff0_9*dmats0[9][8];
          new_coeff0_9 = coeff0_0*dmats0[0][9] + coeff0_1*dmats0[1][9] + coeff0_2*dmats0[2][9] + coeff0_3*dmats0[3][9] + coeff0_4*dmats0[4][9] + coeff0_5*dmats0[5][9] + coeff0_6*dmats0[6][9] + coeff0_7*dmats0[7][9] + coeff0_8*dmats0[8][9] + coeff0_9*dmats0[9][9];
        }
        if(combinations[deriv_num][j] == 1)
        {
          new_coeff0_0 = coeff0_0*dmats1[0][0] + coeff0_1*dmats1[1][0] + coeff0_2*dmats1[2][0] + coeff0_3*dmats1[3][0] + coeff0_4*dmats1[4][0] + coeff0_5*dmats1[5][0] + coeff0_6*dmats1[6][0] + coeff0_7*dmats1[7][0] + coeff0_8*dmats1[8][0] + coeff0_9*dmats1[9][0];
          new_coeff0_1 = coeff0_0*dmats1[0][1] + coeff0_1*dmats1[1][1] + coeff0_2*dmats1[2][1] + coeff0_3*dmats1[3][1] + coeff0_4*dmats1[4][1] + coeff0_5*dmats1[5][1] + coeff0_6*dmats1[6][1] + coeff0_7*dmats1[7][1] + coeff0_8*dmats1[8][1] + coeff0_9*dmats1[9][1];
          new_coeff0_2 = coeff0_0*dmats1[0][2] + coeff0_1*dmats1[1][2] + coeff0_2*dmats1[2][2] + coeff0_3*dmats1[3][2] + coeff0_4*dmats1[4][2] + coeff0_5*dmats1[5][2] + coeff0_6*dmats1[6][2] + coeff0_7*dmats1[7][2] + coeff0_8*dmats1[8][2] + coeff0_9*dmats1[9][2];
          new_coeff0_3 = coeff0_0*dmats1[0][3] + coeff0_1*dmats1[1][3] + coeff0_2*dmats1[2][3] + coeff0_3*dmats1[3][3] + coeff0_4*dmats1[4][3] + coeff0_5*dmats1[5][3] + coeff0_6*dmats1[6][3] + coeff0_7*dmats1[7][3] + coeff0_8*dmats1[8][3] + coeff0_9*dmats1[9][3];
          new_coeff0_4 = coeff0_0*dmats1[0][4] + coeff0_1*dmats1[1][4] + coeff0_2*dmats1[2][4] + coeff0_3*dmats1[3][4] + coeff0_4*dmats1[4][4] + coeff0_5*dmats1[5][4] + coeff0_6*dmats1[6][4] + coeff0_7*dmats1[7][4] + coeff0_8*dmats1[8][4] + coeff0_9*dmats1[9][4];
          new_coeff0_5 = coeff0_0*dmats1[0][5] + coeff0_1*dmats1[1][5] + coeff0_2*dmats1[2][5] + coeff0_3*dmats1[3][5] + coeff0_4*dmats1[4][5] + coeff0_5*dmats1[5][5] + coeff0_6*dmats1[6][5] + coeff0_7*dmats1[7][5] + coeff0_8*dmats1[8][5] + coeff0_9*dmats1[9][5];
          new_coeff0_6 = coeff0_0*dmats1[0][6] + coeff0_1*dmats1[1][6] + coeff0_2*dmats1[2][6] + coeff0_3*dmats1[3][6] + coeff0_4*dmats1[4][6] + coeff0_5*dmats1[5][6] + coeff0_6*dmats1[6][6] + coeff0_7*dmats1[7][6] + coeff0_8*dmats1[8][6] + coeff0_9*dmats1[9][6];
          new_coeff0_7 = coeff0_0*dmats1[0][7] + coeff0_1*dmats1[1][7] + coeff0_2*dmats1[2][7] + coeff0_3*dmats1[3][7] + coeff0_4*dmats1[4][7] + coeff0_5*dmats1[5][7] + coeff0_6*dmats1[6][7] + coeff0_7*dmats1[7][7] + coeff0_8*dmats1[8][7] + coeff0_9*dmats1[9][7];
          new_coeff0_8 = coeff0_0*dmats1[0][8] + coeff0_1*dmats1[1][8] + coeff0_2*dmats1[2][8] + coeff0_3*dmats1[3][8] + coeff0_4*dmats1[4][8] + coeff0_5*dmats1[5][8] + coeff0_6*dmats1[6][8] + coeff0_7*dmats1[7][8] + coeff0_8*dmats1[8][8] + coeff0_9*dmats1[9][8];
          new_coeff0_9 = coeff0_0*dmats1[0][9] + coeff0_1*dmats1[1][9] + coeff0_2*dmats1[2][9] + coeff0_3*dmats1[3][9] + coeff0_4*dmats1[4][9] + coeff0_5*dmats1[5][9] + coeff0_6*dmats1[6][9] + coeff0_7*dmats1[7][9] + coeff0_8*dmats1[8][9] + coeff0_9*dmats1[9][9];
        }
        if(combinations[deriv_num][j] == 2)
        {
          new_coeff0_0 = coeff0_0*dmats2[0][0] + coeff0_1*dmats2[1][0] + coeff0_2*dmats2[2][0] + coeff0_3*dmats2[3][0] + coeff0_4*dmats2[4][0] + coeff0_5*dmats2[5][0] + coeff0_6*dmats2[6][0] + coeff0_7*dmats2[7][0] + coeff0_8*dmats2[8][0] + coeff0_9*dmats2[9][0];
          new_coeff0_1 = coeff0_0*dmats2[0][1] + coeff0_1*dmats2[1][1] + coeff0_2*dmats2[2][1] + coeff0_3*dmats2[3][1] + coeff0_4*dmats2[4][1] + coeff0_5*dmats2[5][1] + coeff0_6*dmats2[6][1] + coeff0_7*dmats2[7][1] + coeff0_8*dmats2[8][1] + coeff0_9*dmats2[9][1];
          new_coeff0_2 = coeff0_0*dmats2[0][2] + coeff0_1*dmats2[1][2] + coeff0_2*dmats2[2][2] + coeff0_3*dmats2[3][2] + coeff0_4*dmats2[4][2] + coeff0_5*dmats2[5][2] + coeff0_6*dmats2[6][2] + coeff0_7*dmats2[7][2] + coeff0_8*dmats2[8][2] + coeff0_9*dmats2[9][2];
          new_coeff0_3 = coeff0_0*dmats2[0][3] + coeff0_1*dmats2[1][3] + coeff0_2*dmats2[2][3] + coeff0_3*dmats2[3][3] + coeff0_4*dmats2[4][3] + coeff0_5*dmats2[5][3] + coeff0_6*dmats2[6][3] + coeff0_7*dmats2[7][3] + coeff0_8*dmats2[8][3] + coeff0_9*dmats2[9][3];
          new_coeff0_4 = coeff0_0*dmats2[0][4] + coeff0_1*dmats2[1][4] + coeff0_2*dmats2[2][4] + coeff0_3*dmats2[3][4] + coeff0_4*dmats2[4][4] + coeff0_5*dmats2[5][4] + coeff0_6*dmats2[6][4] + coeff0_7*dmats2[7][4] + coeff0_8*dmats2[8][4] + coeff0_9*dmats2[9][4];
          new_coeff0_5 = coeff0_0*dmats2[0][5] + coeff0_1*dmats2[1][5] + coeff0_2*dmats2[2][5] + coeff0_3*dmats2[3][5] + coeff0_4*dmats2[4][5] + coeff0_5*dmats2[5][5] + coeff0_6*dmats2[6][5] + coeff0_7*dmats2[7][5] + coeff0_8*dmats2[8][5] + coeff0_9*dmats2[9][5];
          new_coeff0_6 = coeff0_0*dmats2[0][6] + coeff0_1*dmats2[1][6] + coeff0_2*dmats2[2][6] + coeff0_3*dmats2[3][6] + coeff0_4*dmats2[4][6] + coeff0_5*dmats2[5][6] + coeff0_6*dmats2[6][6] + coeff0_7*dmats2[7][6] + coeff0_8*dmats2[8][6] + coeff0_9*dmats2[9][6];
          new_coeff0_7 = coeff0_0*dmats2[0][7] + coeff0_1*dmats2[1][7] + coeff0_2*dmats2[2][7] + coeff0_3*dmats2[3][7] + coeff0_4*dmats2[4][7] + coeff0_5*dmats2[5][7] + coeff0_6*dmats2[6][7] + coeff0_7*dmats2[7][7] + coeff0_8*dmats2[8][7] + coeff0_9*dmats2[9][7];
          new_coeff0_8 = coeff0_0*dmats2[0][8] + coeff0_1*dmats2[1][8] + coeff0_2*dmats2[2][8] + coeff0_3*dmats2[3][8] + coeff0_4*dmats2[4][8] + coeff0_5*dmats2[5][8] + coeff0_6*dmats2[6][8] + coeff0_7*dmats2[7][8] + coeff0_8*dmats2[8][8] + coeff0_9*dmats2[9][8];
          new_coeff0_9 = coeff0_0*dmats2[0][9] + coeff0_1*dmats2[1][9] + coeff0_2*dmats2[2][9] + coeff0_3*dmats2[3][9] + coeff0_4*dmats2[4][9] + coeff0_5*dmats2[5][9] + coeff0_6*dmats2[6][9] + coeff0_7*dmats2[7][9] + coeff0_8*dmats2[8][9] + coeff0_9*dmats2[9][9];
        }
    
      }
      // Compute derivatives on reference element as dot product of coefficients and basisvalues
      derivatives[deriv_num] = new_coeff0_0*basisvalue0 + new_coeff0_1*basisvalue1 + new_coeff0_2*basisvalue2 + new_coeff0_3*basisvalue3 + new_coeff0_4*basisvalue4 + new_coeff0_5*basisvalue5 + new_coeff0_6*basisvalue6 + new_coeff0_7*basisvalue7 + new_coeff0_8*basisvalue8 + new_coeff0_9*basisvalue9;
    }
    
    // Transform derivatives back to physical element
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        values[row] += transform[row][col]*derivatives[col];
      }
    }
    // Delete pointer to array of derivatives on FIAT element
    delete [] derivatives;
    
    // Delete pointer to array of combinations of derivatives and transform
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      delete [] combinations[row];
      delete [] transform[row];
    }
    
    delete [] combinations;
    delete [] transform;
  }

  /// Evaluate order n derivatives of all basis functions at given point in cell
  virtual void evaluate_basis_derivatives_all(unsigned int n,
                                              double* values,
                                              const double* coordinates,
                                              const ufc::cell& c) const
  {
    throw std::runtime_error("The vectorised version of evaluate_basis_derivatives() is not yet implemented.");
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    // The reference points, direction and weights:
    const static double X[10][1][3] = {{{0, 0, 0}}, {{1, 0, 0}}, {{0, 1, 0}}, {{0, 0, 1}}, {{0, 0.5, 0.5}}, {{0.5, 0, 0.5}}, {{0.5, 0.5, 0}}, {{0, 0, 0.5}}, {{0, 0.5, 0}}, {{0.5, 0, 0}}};
    const static double W[10][1] = {{1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}};
    const static double D[10][1][1] = {{{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}, {{1}}};
    
    const double * const * x = c.coordinates;
    double result = 0.0;
    // Iterate over the points:
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0][0] - X[i][0][1] - X[i][0][2];
    const double w1 = X[i][0][0];
    const double w2 = X[i][0][1];
    const double w3 = X[i][0][2];
    
    // Compute affine mapping y = F(X)
    double y[3];
    y[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0] + w3*x[3][0];
    y[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1] + w3*x[3][1];
    y[2] = w0*x[0][2] + w1*x[1][2] + w2*x[2][2] + w3*x[3][2];
    
    // Evaluate function at physical points
    double values[1];
    f.evaluate(values, y, c);
    
    // Map function values using appropriate mapping
    // Affine map: Do nothing
    
    // Note that we do not map the weights (yet).
    
    // Take directional components
    for(int k = 0; k < 1; k++)
      result += values[k]*D[i][0][k];
    // Multiply by weights 
    result *= W[i][0];
    
    return result;
  }

  /// Evaluate linear functionals for all dofs on the function f
  virtual void evaluate_dofs(double* values,
                             const ufc::function& f,
                             const ufc::cell& c) const
  {
    throw std::runtime_error("Not implemented (introduced in UFC v1.1).");
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    // Evaluate at vertices and use affine mapping
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[1];
    vertex_values[2] = dof_values[2];
    vertex_values[3] = dof_values[3];
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 1;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    return new VectorLaplaceGradCurlLinearForm_finite_element_1_2();
  }

};

/// This class defines the interface for a finite element.

class VectorLaplaceGradCurlLinearForm_finite_element_1: public ufc::finite_element
{
public:

  /// Constructor
  VectorLaplaceGradCurlLinearForm_finite_element_1() : ufc::finite_element()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~VectorLaplaceGradCurlLinearForm_finite_element_1()
  {
    // Do nothing
  }

  /// Return a string identifying the finite element
  virtual const char* signature() const
  {
    return "Mixed finite element: [Lagrange finite element of degree 2 on a tetrahedron, Lagrange finite element of degree 2 on a tetrahedron, Lagrange finite element of degree 2 on a tetrahedron]";
  }

  /// Return the cell shape
  virtual ufc::shape cell_shape() const
  {
    return ufc::tetrahedron;
  }

  /// Return the dimension of the finite element function space
  virtual unsigned int space_dimension() const
  {
    return 30;
  }

  /// Return the rank of the value space
  virtual unsigned int value_rank() const
  {
    return 1;
  }

  /// Return the dimension of the value space for axis i
  virtual unsigned int value_dimension(unsigned int i) const
  {
    return 3;
  }

  /// Evaluate basis function i at given point in cell
  virtual void evaluate_basis(unsigned int i,
                              double* values,
                              const double* coordinates,
                              const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-09)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-09)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Reset values
    values[0] = 0;
    values[1] = 0;
    values[2] = 0;
    
    if (0 <= i && i <= 9)
    {
      // Map degree of freedom to element degree of freedom
      const unsigned int dof = i;
    
      // Generate scalings
      const double scalings_y_0 = 1;
      const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
      const double scalings_y_2 = scalings_y_1*(0.5 - 0.5*y);
      const double scalings_z_0 = 1;
      const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
      const double scalings_z_2 = scalings_z_1*(0.5 - 0.5*z);
    
      // Compute psitilde_a
      const double psitilde_a_0 = 1;
      const double psitilde_a_1 = x;
      const double psitilde_a_2 = 1.5*x*psitilde_a_1 - 0.5*psitilde_a_0;
    
      // Compute psitilde_bs
      const double psitilde_bs_0_0 = 1;
      const double psitilde_bs_0_1 = 1.5*y + 0.5;
      const double psitilde_bs_0_2 = 0.1111111111*psitilde_bs_0_1 + 1.666666667*y*psitilde_bs_0_1 - 0.5555555556*psitilde_bs_0_0;
      const double psitilde_bs_1_0 = 1;
      const double psitilde_bs_1_1 = 2.5*y + 1.5;
      const double psitilde_bs_2_0 = 1;
    
      // Compute psitilde_cs
      const double psitilde_cs_00_0 = 1;
      const double psitilde_cs_00_1 = 2*z + 1;
      const double psitilde_cs_00_2 = 0.3125*psitilde_cs_00_1 + 1.875*z*psitilde_cs_00_1 - 0.5625*psitilde_cs_00_0;
      const double psitilde_cs_01_0 = 1;
      const double psitilde_cs_01_1 = 3*z + 2;
      const double psitilde_cs_02_0 = 1;
      const double psitilde_cs_10_0 = 1;
      const double psitilde_cs_10_1 = 3*z + 2;
      const double psitilde_cs_11_0 = 1;
      const double psitilde_cs_20_0 = 1;
    
      // Compute basisvalues
      const double basisvalue0 = 0.8660254038*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
      const double basisvalue1 = 2.738612788*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
      const double basisvalue2 = 1.58113883*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
      const double basisvalue3 = 1.118033989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
      const double basisvalue4 = 5.123475383*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_0;
      const double basisvalue5 = 3.968626967*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_0;
      const double basisvalue6 = 2.291287847*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_0;
      const double basisvalue7 = 3.240370349*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_1;
      const double basisvalue8 = 1.870828693*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_1;
      const double basisvalue9 = 1.322875656*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_2;
    
      // Table(s) of coefficients
      const static double coefficients0[10][10] =   \
      {{-0.05773502692, -0.06085806195, -0.03513641845, -0.02484519975, 0.06506000486, 0.05039526307, 0.0290957187, 0.04114755999, 0.02375655484, 0.01679842102},
      {-0.05773502692, 0.06085806195, -0.03513641845, -0.02484519975, 0.06506000486, -0.05039526307, 0.0290957187, -0.04114755999, 0.02375655484, 0.01679842102},
      {-0.05773502692, 0, 0.07027283689, -0.02484519975, 0, 0, 0.08728715609, 0, -0.04751310967, 0.01679842102},
      {-0.05773502692, 0, 0, 0.07453559925, 0, 0, 0, 0, 0, 0.1007905261},
      {0.2309401077, 0, 0.1405456738, 0.099380799, 0, 0, 0, 0, 0.1187827742, -0.06719368409},
      {0.2309401077, 0.1217161239, -0.07027283689, 0.099380799, 0, 0, 0, 0.1028689, -0.05939138709, -0.06719368409},
      {0.2309401077, 0.1217161239, 0.07027283689, -0.099380799, 0, 0.1007905261, -0.08728715609, -0.02057377999, -0.01187827742, 0.01679842102},
      {0.2309401077, -0.1217161239, -0.07027283689, 0.099380799, 0, 0, 0, -0.1028689, -0.05939138709, -0.06719368409},
      {0.2309401077, -0.1217161239, 0.07027283689, -0.099380799, 0, -0.1007905261, -0.08728715609, 0.02057377999, -0.01187827742, 0.01679842102},
      {0.2309401077, 0, -0.1405456738, -0.099380799, -0.1301200097, 0, 0.0290957187, 0, 0.02375655484, 0.01679842102}};
    
      // Extract relevant coefficients
      const double coeff0_0 =   coefficients0[dof][0];
      const double coeff0_1 =   coefficients0[dof][1];
      const double coeff0_2 =   coefficients0[dof][2];
      const double coeff0_3 =   coefficients0[dof][3];
      const double coeff0_4 =   coefficients0[dof][4];
      const double coeff0_5 =   coefficients0[dof][5];
      const double coeff0_6 =   coefficients0[dof][6];
      const double coeff0_7 =   coefficients0[dof][7];
      const double coeff0_8 =   coefficients0[dof][8];
      const double coeff0_9 =   coefficients0[dof][9];
    
      // Compute value(s)
      values[0] = coeff0_0*basisvalue0 + coeff0_1*basisvalue1 + coeff0_2*basisvalue2 + coeff0_3*basisvalue3 + coeff0_4*basisvalue4 + coeff0_5*basisvalue5 + coeff0_6*basisvalue6 + coeff0_7*basisvalue7 + coeff0_8*basisvalue8 + coeff0_9*basisvalue9;
    }
    
    if (10 <= i && i <= 19)
    {
      // Map degree of freedom to element degree of freedom
      const unsigned int dof = i - 10;
    
      // Generate scalings
      const double scalings_y_0 = 1;
      const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
      const double scalings_y_2 = scalings_y_1*(0.5 - 0.5*y);
      const double scalings_z_0 = 1;
      const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
      const double scalings_z_2 = scalings_z_1*(0.5 - 0.5*z);
    
      // Compute psitilde_a
      const double psitilde_a_0 = 1;
      const double psitilde_a_1 = x;
      const double psitilde_a_2 = 1.5*x*psitilde_a_1 - 0.5*psitilde_a_0;
    
      // Compute psitilde_bs
      const double psitilde_bs_0_0 = 1;
      const double psitilde_bs_0_1 = 1.5*y + 0.5;
      const double psitilde_bs_0_2 = 0.1111111111*psitilde_bs_0_1 + 1.666666667*y*psitilde_bs_0_1 - 0.5555555556*psitilde_bs_0_0;
      const double psitilde_bs_1_0 = 1;
      const double psitilde_bs_1_1 = 2.5*y + 1.5;
      const double psitilde_bs_2_0 = 1;
    
      // Compute psitilde_cs
      const double psitilde_cs_00_0 = 1;
      const double psitilde_cs_00_1 = 2*z + 1;
      const double psitilde_cs_00_2 = 0.3125*psitilde_cs_00_1 + 1.875*z*psitilde_cs_00_1 - 0.5625*psitilde_cs_00_0;
      const double psitilde_cs_01_0 = 1;
      const double psitilde_cs_01_1 = 3*z + 2;
      const double psitilde_cs_02_0 = 1;
      const double psitilde_cs_10_0 = 1;
      const double psitilde_cs_10_1 = 3*z + 2;
      const double psitilde_cs_11_0 = 1;
      const double psitilde_cs_20_0 = 1;
    
      // Compute basisvalues
      const double basisvalue0 = 0.8660254038*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
      const double basisvalue1 = 2.738612788*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
      const double basisvalue2 = 1.58113883*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
      const double basisvalue3 = 1.118033989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
      const double basisvalue4 = 5.123475383*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_0;
      const double basisvalue5 = 3.968626967*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_0;
      const double basisvalue6 = 2.291287847*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_0;
      const double basisvalue7 = 3.240370349*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_1;
      const double basisvalue8 = 1.870828693*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_1;
      const double basisvalue9 = 1.322875656*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_2;
    
      // Table(s) of coefficients
      const static double coefficients0[10][10] =   \
      {{-0.05773502692, -0.06085806195, -0.03513641845, -0.02484519975, 0.06506000486, 0.05039526307, 0.0290957187, 0.04114755999, 0.02375655484, 0.01679842102},
      {-0.05773502692, 0.06085806195, -0.03513641845, -0.02484519975, 0.06506000486, -0.05039526307, 0.0290957187, -0.04114755999, 0.02375655484, 0.01679842102},
      {-0.05773502692, 0, 0.07027283689, -0.02484519975, 0, 0, 0.08728715609, 0, -0.04751310967, 0.01679842102},
      {-0.05773502692, 0, 0, 0.07453559925, 0, 0, 0, 0, 0, 0.1007905261},
      {0.2309401077, 0, 0.1405456738, 0.099380799, 0, 0, 0, 0, 0.1187827742, -0.06719368409},
      {0.2309401077, 0.1217161239, -0.07027283689, 0.099380799, 0, 0, 0, 0.1028689, -0.05939138709, -0.06719368409},
      {0.2309401077, 0.1217161239, 0.07027283689, -0.099380799, 0, 0.1007905261, -0.08728715609, -0.02057377999, -0.01187827742, 0.01679842102},
      {0.2309401077, -0.1217161239, -0.07027283689, 0.099380799, 0, 0, 0, -0.1028689, -0.05939138709, -0.06719368409},
      {0.2309401077, -0.1217161239, 0.07027283689, -0.099380799, 0, -0.1007905261, -0.08728715609, 0.02057377999, -0.01187827742, 0.01679842102},
      {0.2309401077, 0, -0.1405456738, -0.099380799, -0.1301200097, 0, 0.0290957187, 0, 0.02375655484, 0.01679842102}};
    
      // Extract relevant coefficients
      const double coeff0_0 =   coefficients0[dof][0];
      const double coeff0_1 =   coefficients0[dof][1];
      const double coeff0_2 =   coefficients0[dof][2];
      const double coeff0_3 =   coefficients0[dof][3];
      const double coeff0_4 =   coefficients0[dof][4];
      const double coeff0_5 =   coefficients0[dof][5];
      const double coeff0_6 =   coefficients0[dof][6];
      const double coeff0_7 =   coefficients0[dof][7];
      const double coeff0_8 =   coefficients0[dof][8];
      const double coeff0_9 =   coefficients0[dof][9];
    
      // Compute value(s)
      values[1] = coeff0_0*basisvalue0 + coeff0_1*basisvalue1 + coeff0_2*basisvalue2 + coeff0_3*basisvalue3 + coeff0_4*basisvalue4 + coeff0_5*basisvalue5 + coeff0_6*basisvalue6 + coeff0_7*basisvalue7 + coeff0_8*basisvalue8 + coeff0_9*basisvalue9;
    }
    
    if (20 <= i && i <= 29)
    {
      // Map degree of freedom to element degree of freedom
      const unsigned int dof = i - 20;
    
      // Generate scalings
      const double scalings_y_0 = 1;
      const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
      const double scalings_y_2 = scalings_y_1*(0.5 - 0.5*y);
      const double scalings_z_0 = 1;
      const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
      const double scalings_z_2 = scalings_z_1*(0.5 - 0.5*z);
    
      // Compute psitilde_a
      const double psitilde_a_0 = 1;
      const double psitilde_a_1 = x;
      const double psitilde_a_2 = 1.5*x*psitilde_a_1 - 0.5*psitilde_a_0;
    
      // Compute psitilde_bs
      const double psitilde_bs_0_0 = 1;
      const double psitilde_bs_0_1 = 1.5*y + 0.5;
      const double psitilde_bs_0_2 = 0.1111111111*psitilde_bs_0_1 + 1.666666667*y*psitilde_bs_0_1 - 0.5555555556*psitilde_bs_0_0;
      const double psitilde_bs_1_0 = 1;
      const double psitilde_bs_1_1 = 2.5*y + 1.5;
      const double psitilde_bs_2_0 = 1;
    
      // Compute psitilde_cs
      const double psitilde_cs_00_0 = 1;
      const double psitilde_cs_00_1 = 2*z + 1;
      const double psitilde_cs_00_2 = 0.3125*psitilde_cs_00_1 + 1.875*z*psitilde_cs_00_1 - 0.5625*psitilde_cs_00_0;
      const double psitilde_cs_01_0 = 1;
      const double psitilde_cs_01_1 = 3*z + 2;
      const double psitilde_cs_02_0 = 1;
      const double psitilde_cs_10_0 = 1;
      const double psitilde_cs_10_1 = 3*z + 2;
      const double psitilde_cs_11_0 = 1;
      const double psitilde_cs_20_0 = 1;
    
      // Compute basisvalues
      const double basisvalue0 = 0.8660254038*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
      const double basisvalue1 = 2.738612788*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
      const double basisvalue2 = 1.58113883*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
      const double basisvalue3 = 1.118033989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
      const double basisvalue4 = 5.123475383*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_0;
      const double basisvalue5 = 3.968626967*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_0;
      const double basisvalue6 = 2.291287847*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_0;
      const double basisvalue7 = 3.240370349*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_1;
      const double basisvalue8 = 1.870828693*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_1;
      const double basisvalue9 = 1.322875656*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_2;
    
      // Table(s) of coefficients
      const static double coefficients0[10][10] =   \
      {{-0.05773502692, -0.06085806195, -0.03513641845, -0.02484519975, 0.06506000486, 0.05039526307, 0.0290957187, 0.04114755999, 0.02375655484, 0.01679842102},
      {-0.05773502692, 0.06085806195, -0.03513641845, -0.02484519975, 0.06506000486, -0.05039526307, 0.0290957187, -0.04114755999, 0.02375655484, 0.01679842102},
      {-0.05773502692, 0, 0.07027283689, -0.02484519975, 0, 0, 0.08728715609, 0, -0.04751310967, 0.01679842102},
      {-0.05773502692, 0, 0, 0.07453559925, 0, 0, 0, 0, 0, 0.1007905261},
      {0.2309401077, 0, 0.1405456738, 0.099380799, 0, 0, 0, 0, 0.1187827742, -0.06719368409},
      {0.2309401077, 0.1217161239, -0.07027283689, 0.099380799, 0, 0, 0, 0.1028689, -0.05939138709, -0.06719368409},
      {0.2309401077, 0.1217161239, 0.07027283689, -0.099380799, 0, 0.1007905261, -0.08728715609, -0.02057377999, -0.01187827742, 0.01679842102},
      {0.2309401077, -0.1217161239, -0.07027283689, 0.099380799, 0, 0, 0, -0.1028689, -0.05939138709, -0.06719368409},
      {0.2309401077, -0.1217161239, 0.07027283689, -0.099380799, 0, -0.1007905261, -0.08728715609, 0.02057377999, -0.01187827742, 0.01679842102},
      {0.2309401077, 0, -0.1405456738, -0.099380799, -0.1301200097, 0, 0.0290957187, 0, 0.02375655484, 0.01679842102}};
    
      // Extract relevant coefficients
      const double coeff0_0 =   coefficients0[dof][0];
      const double coeff0_1 =   coefficients0[dof][1];
      const double coeff0_2 =   coefficients0[dof][2];
      const double coeff0_3 =   coefficients0[dof][3];
      const double coeff0_4 =   coefficients0[dof][4];
      const double coeff0_5 =   coefficients0[dof][5];
      const double coeff0_6 =   coefficients0[dof][6];
      const double coeff0_7 =   coefficients0[dof][7];
      const double coeff0_8 =   coefficients0[dof][8];
      const double coeff0_9 =   coefficients0[dof][9];
    
      // Compute value(s)
      values[2] = coeff0_0*basisvalue0 + coeff0_1*basisvalue1 + coeff0_2*basisvalue2 + coeff0_3*basisvalue3 + coeff0_4*basisvalue4 + coeff0_5*basisvalue5 + coeff0_6*basisvalue6 + coeff0_7*basisvalue7 + coeff0_8*basisvalue8 + coeff0_9*basisvalue9;
    }
    
  }

  /// Evaluate all basis functions at given point in cell
  virtual void evaluate_basis_all(double* values,
                                  const double* coordinates,
                                  const ufc::cell& c) const
  {
    throw std::runtime_error("The vectorised version of evaluate_basis() is not yet implemented.");
  }

  /// Evaluate order n derivatives of basis function i at given point in cell
  virtual void evaluate_basis_derivatives(unsigned int i,
                                          unsigned int n,
                                          double* values,
                                          const double* coordinates,
                                          const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * element_coordinates = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = element_coordinates[1][0] - element_coordinates[0][0];
    const double J_01 = element_coordinates[2][0] - element_coordinates[0][0];
    const double J_02 = element_coordinates[3][0] - element_coordinates[0][0];
    const double J_10 = element_coordinates[1][1] - element_coordinates[0][1];
    const double J_11 = element_coordinates[2][1] - element_coordinates[0][1];
    const double J_12 = element_coordinates[3][1] - element_coordinates[0][1];
    const double J_20 = element_coordinates[1][2] - element_coordinates[0][2];
    const double J_21 = element_coordinates[2][2] - element_coordinates[0][2];
    const double J_22 = element_coordinates[3][2] - element_coordinates[0][2];
      
    // Compute sub determinants
    const double d00 = J_11*J_22 - J_12*J_21;
    const double d01 = J_12*J_20 - J_10*J_22;
    const double d02 = J_10*J_21 - J_11*J_20;
    
    const double d10 = J_02*J_21 - J_01*J_22;
    const double d11 = J_00*J_22 - J_02*J_20;
    const double d12 = J_01*J_20 - J_00*J_21;
    
    const double d20 = J_01*J_12 - J_02*J_11;
    const double d21 = J_02*J_10 - J_00*J_12;
    const double d22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d00 + J_10*d10 + J_20*d20;
    
    // Compute inverse of Jacobian
    
    // Compute constants
    const double C0 = d00*(element_coordinates[0][0] - element_coordinates[2][0] - element_coordinates[3][0]) \
                    + d10*(element_coordinates[0][1] - element_coordinates[2][1] - element_coordinates[3][1]) \
                    + d20*(element_coordinates[0][2] - element_coordinates[2][2] - element_coordinates[3][2]);
    
    const double C1 = d01*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[3][0]) \
                    + d11*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[3][1]) \
                    + d21*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[3][2]);
    
    const double C2 = d02*(element_coordinates[0][0] - element_coordinates[1][0] - element_coordinates[2][0]) \
                    + d12*(element_coordinates[0][1] - element_coordinates[1][1] - element_coordinates[2][1]) \
                    + d22*(element_coordinates[0][2] - element_coordinates[1][2] - element_coordinates[2][2]);
    
    // Get coordinates and map to the UFC reference element
    double x = (C0 + d00*coordinates[0] + d10*coordinates[1] + d20*coordinates[2]) / detJ;
    double y = (C1 + d01*coordinates[0] + d11*coordinates[1] + d21*coordinates[2]) / detJ;
    double z = (C2 + d02*coordinates[0] + d12*coordinates[1] + d22*coordinates[2]) / detJ;
    
    // Map coordinates to the reference cube
    if (std::abs(y + z - 1.0) < 1e-09)
      x = 1.0;
    else
      x = -2.0 * x/(y + z - 1.0) - 1.0;
    if (std::abs(z - 1.0) < 1e-09)
      y = -1.0;
    else
      y = 2.0 * y/(1.0 - z) - 1.0;
    z = 2.0 * z - 1.0;
    
    // Compute number of derivatives
    unsigned int num_derivatives = 1;
    
    for (unsigned int j = 0; j < n; j++)
      num_derivatives *= 3;
    
    
    // Declare pointer to two dimensional array that holds combinations of derivatives and initialise
    unsigned int **combinations = new unsigned int *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      combinations[j] = new unsigned int [n];
      for (unsigned int k = 0; k < n; k++)
        combinations[j][k] = 0;
    }
        
    // Generate combinations of derivatives
    for (unsigned int row = 1; row < num_derivatives; row++)
    {
      for (unsigned int num = 0; num < row; num++)
      {
        for (unsigned int col = n-1; col+1 > 0; col--)
        {
          if (combinations[row][col] + 1 > 2)
            combinations[row][col] = 0;
          else
          {
            combinations[row][col] += 1;
            break;
          }
        }
      }
    }
    
    // Compute inverse of Jacobian
    const double Jinv[3][3] ={{d00 / detJ, d10 / detJ, d20 / detJ}, {d01 / detJ, d11 / detJ, d21 / detJ}, {d02 / detJ, d12 / detJ, d22 / detJ}};
    
    // Declare transformation matrix
    // Declare pointer to two dimensional array and initialise
    double **transform = new double *[num_derivatives];
        
    for (unsigned int j = 0; j < num_derivatives; j++)
    {
      transform[j] = new double [num_derivatives];
      for (unsigned int k = 0; k < num_derivatives; k++)
        transform[j][k] = 1;
    }
    
    // Construct transformation matrix
    for (unsigned int row = 0; row < num_derivatives; row++)
    {
      for (unsigned int col = 0; col < num_derivatives; col++)
      {
        for (unsigned int k = 0; k < n; k++)
          transform[row][col] *= Jinv[combinations[col][k]][combinations[row][k]];
      }
    }
    
    // Reset values
    for (unsigned int j = 0; j < 3*num_derivatives; j++)
      values[j] = 0;
    
    if (0 <= i && i <= 9)
    {
      // Map degree of freedom to element degree of freedom
      const unsigned int dof = i;
    
      // Generate scalings
      const double scalings_y_0 = 1;
      const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
      const double scalings_y_2 = scalings_y_1*(0.5 - 0.5*y);
      const double scalings_z_0 = 1;
      const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
      const double scalings_z_2 = scalings_z_1*(0.5 - 0.5*z);
    
      // Compute psitilde_a
      const double psitilde_a_0 = 1;
      const double psitilde_a_1 = x;
      const double psitilde_a_2 = 1.5*x*psitilde_a_1 - 0.5*psitilde_a_0;
    
      // Compute psitilde_bs
      const double psitilde_bs_0_0 = 1;
      const double psitilde_bs_0_1 = 1.5*y + 0.5;
      const double psitilde_bs_0_2 = 0.1111111111*psitilde_bs_0_1 + 1.666666667*y*psitilde_bs_0_1 - 0.5555555556*psitilde_bs_0_0;
      const double psitilde_bs_1_0 = 1;
      const double psitilde_bs_1_1 = 2.5*y + 1.5;
      const double psitilde_bs_2_0 = 1;
    
      // Compute psitilde_cs
      const double psitilde_cs_00_0 = 1;
      const double psitilde_cs_00_1 = 2*z + 1;
      const double psitilde_cs_00_2 = 0.3125*psitilde_cs_00_1 + 1.875*z*psitilde_cs_00_1 - 0.5625*psitilde_cs_00_0;
      const double psitilde_cs_01_0 = 1;
      const double psitilde_cs_01_1 = 3*z + 2;
      const double psitilde_cs_02_0 = 1;
      const double psitilde_cs_10_0 = 1;
      const double psitilde_cs_10_1 = 3*z + 2;
      const double psitilde_cs_11_0 = 1;
      const double psitilde_cs_20_0 = 1;
    
      // Compute basisvalues
      const double basisvalue0 = 0.8660254038*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
      const double basisvalue1 = 2.738612788*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
      const double basisvalue2 = 1.58113883*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
      const double basisvalue3 = 1.118033989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
      const double basisvalue4 = 5.123475383*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_0;
      const double basisvalue5 = 3.968626967*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_0;
      const double basisvalue6 = 2.291287847*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_0;
      const double basisvalue7 = 3.240370349*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_1;
      const double basisvalue8 = 1.870828693*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_1;
      const double basisvalue9 = 1.322875656*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_2;
    
      // Table(s) of coefficients
      const static double coefficients0[10][10] =   \
      {{-0.05773502692, -0.06085806195, -0.03513641845, -0.02484519975, 0.06506000486, 0.05039526307, 0.0290957187, 0.04114755999, 0.02375655484, 0.01679842102},
      {-0.05773502692, 0.06085806195, -0.03513641845, -0.02484519975, 0.06506000486, -0.05039526307, 0.0290957187, -0.04114755999, 0.02375655484, 0.01679842102},
      {-0.05773502692, 0, 0.07027283689, -0.02484519975, 0, 0, 0.08728715609, 0, -0.04751310967, 0.01679842102},
      {-0.05773502692, 0, 0, 0.07453559925, 0, 0, 0, 0, 0, 0.1007905261},
      {0.2309401077, 0, 0.1405456738, 0.099380799, 0, 0, 0, 0, 0.1187827742, -0.06719368409},
      {0.2309401077, 0.1217161239, -0.07027283689, 0.099380799, 0, 0, 0, 0.1028689, -0.05939138709, -0.06719368409},
      {0.2309401077, 0.1217161239, 0.07027283689, -0.099380799, 0, 0.1007905261, -0.08728715609, -0.02057377999, -0.01187827742, 0.01679842102},
      {0.2309401077, -0.1217161239, -0.07027283689, 0.099380799, 0, 0, 0, -0.1028689, -0.05939138709, -0.06719368409},
      {0.2309401077, -0.1217161239, 0.07027283689, -0.099380799, 0, -0.1007905261, -0.08728715609, 0.02057377999, -0.01187827742, 0.01679842102},
      {0.2309401077, 0, -0.1405456738, -0.099380799, -0.1301200097, 0, 0.0290957187, 0, 0.02375655484, 0.01679842102}};
    
      // Interesting (new) part
      // Tables of derivatives of the polynomial base (transpose)
      const static double dmats0[10][10] =   \
      {{0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
      {6.32455532, 0, 0, 0, 0, 0, 0, 0, 0, 0},
      {0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
      {0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
      {0, 11.22497216, 0, 0, 0, 0, 0, 0, 0, 0},
      {4.582575695, 0, 8.366600265, -1.183215957, 0, 0, 0, 0, 0, 0},
      {0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
      {3.741657387, 0, 0, 8.694826048, 0, 0, 0, 0, 0, 0},
      {0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
      {0, 0, 0, 0, 0, 0, 0, 0, 0, 0}};
    
      const static double dmats1[10][10] =   \
      {{0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
      {3.16227766, 0, 0, 0, 0, 0, 0, 0, 0, 0},
      {5.477225575, 0, 0, 0, 0, 0, 0, 0, 0, 0},
      {0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
      {2.958039892, 5.61248608, -1.08012345, -0.7637626158, 0, 0, 0, 0, 0, 0},
      {2.291287847, 7.245688373, 4.183300133, -0.5916079783, 0, 0, 0, 0, 0, 0},
      {-2.645751311, 0, 9.660917831, 0.6831300511, 0, 0, 0, 0, 0, 0},
      {1.870828693, 0, 0, 4.347413024, 0, 0, 0, 0, 0, 0},
      {3.240370349, 0, 0, 7.529940239, 0, 0, 0, 0, 0, 0},
      {0, 0, 0, 0, 0, 0, 0, 0, 0, 0}};
    
      const static double dmats2[10][10] =   \
      {{0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
      {3.16227766, 0, 0, 0, 0, 0, 0, 0, 0, 0},
      {1.825741858, 0, 0, 0, 0, 0, 0, 0, 0, 0},
      {5.163977795, 0, 0, 0, 0, 0, 0, 0, 0, 0},
      {2.958039892, 5.61248608, -1.08012345, -0.7637626158, 0, 0, 0, 0, 0, 0},
      {2.291287847, 1.449137675, 4.183300133, -0.5916079783, 0, 0, 0, 0, 0, 0},
      {1.322875656, 0, 3.864367132, -0.3415650255, 0, 0, 0, 0, 0, 0},
      {1.870828693, 7.09929574, 0, 4.347413024, 0, 0, 0, 0, 0, 0},
      {1.08012345, 0, 7.09929574, 2.50998008, 0, 0, 0, 0, 0, 0},
      {-3.818813079, 0, 0, 8.874119675, 0, 0, 0, 0, 0, 0}};
    
      // Compute reference derivatives
      // Declare pointer to array of derivatives on FIAT element
      double *derivatives = new double [num_derivatives];
    
      // Declare coefficients
      double coeff0_0 = 0;
      double coeff0_1 = 0;
      double coeff0_2 = 0;
      double coeff0_3 = 0;
      double coeff0_4 = 0;
      double coeff0_5 = 0;
      double coeff0_6 = 0;
      double coeff0_7 = 0;
      double coeff0_8 = 0;
      double coeff0_9 = 0;
    
      // Declare new coefficients
      double new_coeff0_0 = 0;
      double new_coeff0_1 = 0;
      double new_coeff0_2 = 0;
      double new_coeff0_3 = 0;
      double new_coeff0_4 = 0;
      double new_coeff0_5 = 0;
      double new_coeff0_6 = 0;
      double new_coeff0_7 = 0;
      double new_coeff0_8 = 0;
      double new_coeff0_9 = 0;
    
      // Loop possible derivatives
      for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
      {
        // Get values from coefficients array
        new_coeff0_0 = coefficients0[dof][0];
        new_coeff0_1 = coefficients0[dof][1];
        new_coeff0_2 = coefficients0[dof][2];
        new_coeff0_3 = coefficients0[dof][3];
        new_coeff0_4 = coefficients0[dof][4];
        new_coeff0_5 = coefficients0[dof][5];
        new_coeff0_6 = coefficients0[dof][6];
        new_coeff0_7 = coefficients0[dof][7];
        new_coeff0_8 = coefficients0[dof][8];
        new_coeff0_9 = coefficients0[dof][9];
    
        // Loop derivative order
        for (unsigned int j = 0; j < n; j++)
        {
          // Update old coefficients
          coeff0_0 = new_coeff0_0;
          coeff0_1 = new_coeff0_1;
          coeff0_2 = new_coeff0_2;
          coeff0_3 = new_coeff0_3;
          coeff0_4 = new_coeff0_4;
          coeff0_5 = new_coeff0_5;
          coeff0_6 = new_coeff0_6;
          coeff0_7 = new_coeff0_7;
          coeff0_8 = new_coeff0_8;
          coeff0_9 = new_coeff0_9;
    
          if(combinations[deriv_num][j] == 0)
          {
            new_coeff0_0 = coeff0_0*dmats0[0][0] + coeff0_1*dmats0[1][0] + coeff0_2*dmats0[2][0] + coeff0_3*dmats0[3][0] + coeff0_4*dmats0[4][0] + coeff0_5*dmats0[5][0] + coeff0_6*dmats0[6][0] + coeff0_7*dmats0[7][0] + coeff0_8*dmats0[8][0] + coeff0_9*dmats0[9][0];
            new_coeff0_1 = coeff0_0*dmats0[0][1] + coeff0_1*dmats0[1][1] + coeff0_2*dmats0[2][1] + coeff0_3*dmats0[3][1] + coeff0_4*dmats0[4][1] + coeff0_5*dmats0[5][1] + coeff0_6*dmats0[6][1] + coeff0_7*dmats0[7][1] + coeff0_8*dmats0[8][1] + coeff0_9*dmats0[9][1];
            new_coeff0_2 = coeff0_0*dmats0[0][2] + coeff0_1*dmats0[1][2] + coeff0_2*dmats0[2][2] + coeff0_3*dmats0[3][2] + coeff0_4*dmats0[4][2] + coeff0_5*dmats0[5][2] + coeff0_6*dmats0[6][2] + coeff0_7*dmats0[7][2] + coeff0_8*dmats0[8][2] + coeff0_9*dmats0[9][2];
            new_coeff0_3 = coeff0_0*dmats0[0][3] + coeff0_1*dmats0[1][3] + coeff0_2*dmats0[2][3] + coeff0_3*dmats0[3][3] + coeff0_4*dmats0[4][3] + coeff0_5*dmats0[5][3] + coeff0_6*dmats0[6][3] + coeff0_7*dmats0[7][3] + coeff0_8*dmats0[8][3] + coeff0_9*dmats0[9][3];
            new_coeff0_4 = coeff0_0*dmats0[0][4] + coeff0_1*dmats0[1][4] + coeff0_2*dmats0[2][4] + coeff0_3*dmats0[3][4] + coeff0_4*dmats0[4][4] + coeff0_5*dmats0[5][4] + coeff0_6*dmats0[6][4] + coeff0_7*dmats0[7][4] + coeff0_8*dmats0[8][4] + coeff0_9*dmats0[9][4];
            new_coeff0_5 = coeff0_0*dmats0[0][5] + coeff0_1*dmats0[1][5] + coeff0_2*dmats0[2][5] + coeff0_3*dmats0[3][5] + coeff0_4*dmats0[4][5] + coeff0_5*dmats0[5][5] + coeff0_6*dmats0[6][5] + coeff0_7*dmats0[7][5] + coeff0_8*dmats0[8][5] + coeff0_9*dmats0[9][5];
            new_coeff0_6 = coeff0_0*dmats0[0][6] + coeff0_1*dmats0[1][6] + coeff0_2*dmats0[2][6] + coeff0_3*dmats0[3][6] + coeff0_4*dmats0[4][6] + coeff0_5*dmats0[5][6] + coeff0_6*dmats0[6][6] + coeff0_7*dmats0[7][6] + coeff0_8*dmats0[8][6] + coeff0_9*dmats0[9][6];
            new_coeff0_7 = coeff0_0*dmats0[0][7] + coeff0_1*dmats0[1][7] + coeff0_2*dmats0[2][7] + coeff0_3*dmats0[3][7] + coeff0_4*dmats0[4][7] + coeff0_5*dmats0[5][7] + coeff0_6*dmats0[6][7] + coeff0_7*dmats0[7][7] + coeff0_8*dmats0[8][7] + coeff0_9*dmats0[9][7];
            new_coeff0_8 = coeff0_0*dmats0[0][8] + coeff0_1*dmats0[1][8] + coeff0_2*dmats0[2][8] + coeff0_3*dmats0[3][8] + coeff0_4*dmats0[4][8] + coeff0_5*dmats0[5][8] + coeff0_6*dmats0[6][8] + coeff0_7*dmats0[7][8] + coeff0_8*dmats0[8][8] + coeff0_9*dmats0[9][8];
            new_coeff0_9 = coeff0_0*dmats0[0][9] + coeff0_1*dmats0[1][9] + coeff0_2*dmats0[2][9] + coeff0_3*dmats0[3][9] + coeff0_4*dmats0[4][9] + coeff0_5*dmats0[5][9] + coeff0_6*dmats0[6][9] + coeff0_7*dmats0[7][9] + coeff0_8*dmats0[8][9] + coeff0_9*dmats0[9][9];
          }
          if(combinations[deriv_num][j] == 1)
          {
            new_coeff0_0 = coeff0_0*dmats1[0][0] + coeff0_1*dmats1[1][0] + coeff0_2*dmats1[2][0] + coeff0_3*dmats1[3][0] + coeff0_4*dmats1[4][0] + coeff0_5*dmats1[5][0] + coeff0_6*dmats1[6][0] + coeff0_7*dmats1[7][0] + coeff0_8*dmats1[8][0] + coeff0_9*dmats1[9][0];
            new_coeff0_1 = coeff0_0*dmats1[0][1] + coeff0_1*dmats1[1][1] + coeff0_2*dmats1[2][1] + coeff0_3*dmats1[3][1] + coeff0_4*dmats1[4][1] + coeff0_5*dmats1[5][1] + coeff0_6*dmats1[6][1] + coeff0_7*dmats1[7][1] + coeff0_8*dmats1[8][1] + coeff0_9*dmats1[9][1];
            new_coeff0_2 = coeff0_0*dmats1[0][2] + coeff0_1*dmats1[1][2] + coeff0_2*dmats1[2][2] + coeff0_3*dmats1[3][2] + coeff0_4*dmats1[4][2] + coeff0_5*dmats1[5][2] + coeff0_6*dmats1[6][2] + coeff0_7*dmats1[7][2] + coeff0_8*dmats1[8][2] + coeff0_9*dmats1[9][2];
            new_coeff0_3 = coeff0_0*dmats1[0][3] + coeff0_1*dmats1[1][3] + coeff0_2*dmats1[2][3] + coeff0_3*dmats1[3][3] + coeff0_4*dmats1[4][3] + coeff0_5*dmats1[5][3] + coeff0_6*dmats1[6][3] + coeff0_7*dmats1[7][3] + coeff0_8*dmats1[8][3] + coeff0_9*dmats1[9][3];
            new_coeff0_4 = coeff0_0*dmats1[0][4] + coeff0_1*dmats1[1][4] + coeff0_2*dmats1[2][4] + coeff0_3*dmats1[3][4] + coeff0_4*dmats1[4][4] + coeff0_5*dmats1[5][4] + coeff0_6*dmats1[6][4] + coeff0_7*dmats1[7][4] + coeff0_8*dmats1[8][4] + coeff0_9*dmats1[9][4];
            new_coeff0_5 = coeff0_0*dmats1[0][5] + coeff0_1*dmats1[1][5] + coeff0_2*dmats1[2][5] + coeff0_3*dmats1[3][5] + coeff0_4*dmats1[4][5] + coeff0_5*dmats1[5][5] + coeff0_6*dmats1[6][5] + coeff0_7*dmats1[7][5] + coeff0_8*dmats1[8][5] + coeff0_9*dmats1[9][5];
            new_coeff0_6 = coeff0_0*dmats1[0][6] + coeff0_1*dmats1[1][6] + coeff0_2*dmats1[2][6] + coeff0_3*dmats1[3][6] + coeff0_4*dmats1[4][6] + coeff0_5*dmats1[5][6] + coeff0_6*dmats1[6][6] + coeff0_7*dmats1[7][6] + coeff0_8*dmats1[8][6] + coeff0_9*dmats1[9][6];
            new_coeff0_7 = coeff0_0*dmats1[0][7] + coeff0_1*dmats1[1][7] + coeff0_2*dmats1[2][7] + coeff0_3*dmats1[3][7] + coeff0_4*dmats1[4][7] + coeff0_5*dmats1[5][7] + coeff0_6*dmats1[6][7] + coeff0_7*dmats1[7][7] + coeff0_8*dmats1[8][7] + coeff0_9*dmats1[9][7];
            new_coeff0_8 = coeff0_0*dmats1[0][8] + coeff0_1*dmats1[1][8] + coeff0_2*dmats1[2][8] + coeff0_3*dmats1[3][8] + coeff0_4*dmats1[4][8] + coeff0_5*dmats1[5][8] + coeff0_6*dmats1[6][8] + coeff0_7*dmats1[7][8] + coeff0_8*dmats1[8][8] + coeff0_9*dmats1[9][8];
            new_coeff0_9 = coeff0_0*dmats1[0][9] + coeff0_1*dmats1[1][9] + coeff0_2*dmats1[2][9] + coeff0_3*dmats1[3][9] + coeff0_4*dmats1[4][9] + coeff0_5*dmats1[5][9] + coeff0_6*dmats1[6][9] + coeff0_7*dmats1[7][9] + coeff0_8*dmats1[8][9] + coeff0_9*dmats1[9][9];
          }
          if(combinations[deriv_num][j] == 2)
          {
            new_coeff0_0 = coeff0_0*dmats2[0][0] + coeff0_1*dmats2[1][0] + coeff0_2*dmats2[2][0] + coeff0_3*dmats2[3][0] + coeff0_4*dmats2[4][0] + coeff0_5*dmats2[5][0] + coeff0_6*dmats2[6][0] + coeff0_7*dmats2[7][0] + coeff0_8*dmats2[8][0] + coeff0_9*dmats2[9][0];
            new_coeff0_1 = coeff0_0*dmats2[0][1] + coeff0_1*dmats2[1][1] + coeff0_2*dmats2[2][1] + coeff0_3*dmats2[3][1] + coeff0_4*dmats2[4][1] + coeff0_5*dmats2[5][1] + coeff0_6*dmats2[6][1] + coeff0_7*dmats2[7][1] + coeff0_8*dmats2[8][1] + coeff0_9*dmats2[9][1];
            new_coeff0_2 = coeff0_0*dmats2[0][2] + coeff0_1*dmats2[1][2] + coeff0_2*dmats2[2][2] + coeff0_3*dmats2[3][2] + coeff0_4*dmats2[4][2] + coeff0_5*dmats2[5][2] + coeff0_6*dmats2[6][2] + coeff0_7*dmats2[7][2] + coeff0_8*dmats2[8][2] + coeff0_9*dmats2[9][2];
            new_coeff0_3 = coeff0_0*dmats2[0][3] + coeff0_1*dmats2[1][3] + coeff0_2*dmats2[2][3] + coeff0_3*dmats2[3][3] + coeff0_4*dmats2[4][3] + coeff0_5*dmats2[5][3] + coeff0_6*dmats2[6][3] + coeff0_7*dmats2[7][3] + coeff0_8*dmats2[8][3] + coeff0_9*dmats2[9][3];
            new_coeff0_4 = coeff0_0*dmats2[0][4] + coeff0_1*dmats2[1][4] + coeff0_2*dmats2[2][4] + coeff0_3*dmats2[3][4] + coeff0_4*dmats2[4][4] + coeff0_5*dmats2[5][4] + coeff0_6*dmats2[6][4] + coeff0_7*dmats2[7][4] + coeff0_8*dmats2[8][4] + coeff0_9*dmats2[9][4];
            new_coeff0_5 = coeff0_0*dmats2[0][5] + coeff0_1*dmats2[1][5] + coeff0_2*dmats2[2][5] + coeff0_3*dmats2[3][5] + coeff0_4*dmats2[4][5] + coeff0_5*dmats2[5][5] + coeff0_6*dmats2[6][5] + coeff0_7*dmats2[7][5] + coeff0_8*dmats2[8][5] + coeff0_9*dmats2[9][5];
            new_coeff0_6 = coeff0_0*dmats2[0][6] + coeff0_1*dmats2[1][6] + coeff0_2*dmats2[2][6] + coeff0_3*dmats2[3][6] + coeff0_4*dmats2[4][6] + coeff0_5*dmats2[5][6] + coeff0_6*dmats2[6][6] + coeff0_7*dmats2[7][6] + coeff0_8*dmats2[8][6] + coeff0_9*dmats2[9][6];
            new_coeff0_7 = coeff0_0*dmats2[0][7] + coeff0_1*dmats2[1][7] + coeff0_2*dmats2[2][7] + coeff0_3*dmats2[3][7] + coeff0_4*dmats2[4][7] + coeff0_5*dmats2[5][7] + coeff0_6*dmats2[6][7] + coeff0_7*dmats2[7][7] + coeff0_8*dmats2[8][7] + coeff0_9*dmats2[9][7];
            new_coeff0_8 = coeff0_0*dmats2[0][8] + coeff0_1*dmats2[1][8] + coeff0_2*dmats2[2][8] + coeff0_3*dmats2[3][8] + coeff0_4*dmats2[4][8] + coeff0_5*dmats2[5][8] + coeff0_6*dmats2[6][8] + coeff0_7*dmats2[7][8] + coeff0_8*dmats2[8][8] + coeff0_9*dmats2[9][8];
            new_coeff0_9 = coeff0_0*dmats2[0][9] + coeff0_1*dmats2[1][9] + coeff0_2*dmats2[2][9] + coeff0_3*dmats2[3][9] + coeff0_4*dmats2[4][9] + coeff0_5*dmats2[5][9] + coeff0_6*dmats2[6][9] + coeff0_7*dmats2[7][9] + coeff0_8*dmats2[8][9] + coeff0_9*dmats2[9][9];
          }
    
        }
        // Compute derivatives on reference element as dot product of coefficients and basisvalues
        derivatives[deriv_num] = new_coeff0_0*basisvalue0 + new_coeff0_1*basisvalue1 + new_coeff0_2*basisvalue2 + new_coeff0_3*basisvalue3 + new_coeff0_4*basisvalue4 + new_coeff0_5*basisvalue5 + new_coeff0_6*basisvalue6 + new_coeff0_7*basisvalue7 + new_coeff0_8*basisvalue8 + new_coeff0_9*basisvalue9;
      }
    
      // Transform derivatives back to physical element
      for (unsigned int row = 0; row < num_derivatives; row++)
      {
        for (unsigned int col = 0; col < num_derivatives; col++)
        {
          values[row] += transform[row][col]*derivatives[col];
        }
      }
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
    
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int row = 0; row < num_derivatives; row++)
      {
        delete [] combinations[row];
        delete [] transform[row];
      }
    
      delete [] combinations;
      delete [] transform;
    }
    
    if (10 <= i && i <= 19)
    {
      // Map degree of freedom to element degree of freedom
      const unsigned int dof = i - 10;
    
      // Generate scalings
      const double scalings_y_0 = 1;
      const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
      const double scalings_y_2 = scalings_y_1*(0.5 - 0.5*y);
      const double scalings_z_0 = 1;
      const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
      const double scalings_z_2 = scalings_z_1*(0.5 - 0.5*z);
    
      // Compute psitilde_a
      const double psitilde_a_0 = 1;
      const double psitilde_a_1 = x;
      const double psitilde_a_2 = 1.5*x*psitilde_a_1 - 0.5*psitilde_a_0;
    
      // Compute psitilde_bs
      const double psitilde_bs_0_0 = 1;
      const double psitilde_bs_0_1 = 1.5*y + 0.5;
      const double psitilde_bs_0_2 = 0.1111111111*psitilde_bs_0_1 + 1.666666667*y*psitilde_bs_0_1 - 0.5555555556*psitilde_bs_0_0;
      const double psitilde_bs_1_0 = 1;
      const double psitilde_bs_1_1 = 2.5*y + 1.5;
      const double psitilde_bs_2_0 = 1;
    
      // Compute psitilde_cs
      const double psitilde_cs_00_0 = 1;
      const double psitilde_cs_00_1 = 2*z + 1;
      const double psitilde_cs_00_2 = 0.3125*psitilde_cs_00_1 + 1.875*z*psitilde_cs_00_1 - 0.5625*psitilde_cs_00_0;
      const double psitilde_cs_01_0 = 1;
      const double psitilde_cs_01_1 = 3*z + 2;
      const double psitilde_cs_02_0 = 1;
      const double psitilde_cs_10_0 = 1;
      const double psitilde_cs_10_1 = 3*z + 2;
      const double psitilde_cs_11_0 = 1;
      const double psitilde_cs_20_0 = 1;
    
      // Compute basisvalues
      const double basisvalue0 = 0.8660254038*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
      const double basisvalue1 = 2.738612788*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
      const double basisvalue2 = 1.58113883*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
      const double basisvalue3 = 1.118033989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
      const double basisvalue4 = 5.123475383*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_0;
      const double basisvalue5 = 3.968626967*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_0;
      const double basisvalue6 = 2.291287847*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_0;
      const double basisvalue7 = 3.240370349*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_1;
      const double basisvalue8 = 1.870828693*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_1;
      const double basisvalue9 = 1.322875656*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_2;
    
      // Table(s) of coefficients
      const static double coefficients0[10][10] =   \
      {{-0.05773502692, -0.06085806195, -0.03513641845, -0.02484519975, 0.06506000486, 0.05039526307, 0.0290957187, 0.04114755999, 0.02375655484, 0.01679842102},
      {-0.05773502692, 0.06085806195, -0.03513641845, -0.02484519975, 0.06506000486, -0.05039526307, 0.0290957187, -0.04114755999, 0.02375655484, 0.01679842102},
      {-0.05773502692, 0, 0.07027283689, -0.02484519975, 0, 0, 0.08728715609, 0, -0.04751310967, 0.01679842102},
      {-0.05773502692, 0, 0, 0.07453559925, 0, 0, 0, 0, 0, 0.1007905261},
      {0.2309401077, 0, 0.1405456738, 0.099380799, 0, 0, 0, 0, 0.1187827742, -0.06719368409},
      {0.2309401077, 0.1217161239, -0.07027283689, 0.099380799, 0, 0, 0, 0.1028689, -0.05939138709, -0.06719368409},
      {0.2309401077, 0.1217161239, 0.07027283689, -0.099380799, 0, 0.1007905261, -0.08728715609, -0.02057377999, -0.01187827742, 0.01679842102},
      {0.2309401077, -0.1217161239, -0.07027283689, 0.099380799, 0, 0, 0, -0.1028689, -0.05939138709, -0.06719368409},
      {0.2309401077, -0.1217161239, 0.07027283689, -0.099380799, 0, -0.1007905261, -0.08728715609, 0.02057377999, -0.01187827742, 0.01679842102},
      {0.2309401077, 0, -0.1405456738, -0.099380799, -0.1301200097, 0, 0.0290957187, 0, 0.02375655484, 0.01679842102}};
    
      // Interesting (new) part
      // Tables of derivatives of the polynomial base (transpose)
      const static double dmats0[10][10] =   \
      {{0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
      {6.32455532, 0, 0, 0, 0, 0, 0, 0, 0, 0},
      {0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
      {0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
      {0, 11.22497216, 0, 0, 0, 0, 0, 0, 0, 0},
      {4.582575695, 0, 8.366600265, -1.183215957, 0, 0, 0, 0, 0, 0},
      {0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
      {3.741657387, 0, 0, 8.694826048, 0, 0, 0, 0, 0, 0},
      {0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
      {0, 0, 0, 0, 0, 0, 0, 0, 0, 0}};
    
      const static double dmats1[10][10] =   \
      {{0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
      {3.16227766, 0, 0, 0, 0, 0, 0, 0, 0, 0},
      {5.477225575, 0, 0, 0, 0, 0, 0, 0, 0, 0},
      {0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
      {2.958039892, 5.61248608, -1.08012345, -0.7637626158, 0, 0, 0, 0, 0, 0},
      {2.291287847, 7.245688373, 4.183300133, -0.5916079783, 0, 0, 0, 0, 0, 0},
      {-2.645751311, 0, 9.660917831, 0.6831300511, 0, 0, 0, 0, 0, 0},
      {1.870828693, 0, 0, 4.347413024, 0, 0, 0, 0, 0, 0},
      {3.240370349, 0, 0, 7.529940239, 0, 0, 0, 0, 0, 0},
      {0, 0, 0, 0, 0, 0, 0, 0, 0, 0}};
    
      const static double dmats2[10][10] =   \
      {{0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
      {3.16227766, 0, 0, 0, 0, 0, 0, 0, 0, 0},
      {1.825741858, 0, 0, 0, 0, 0, 0, 0, 0, 0},
      {5.163977795, 0, 0, 0, 0, 0, 0, 0, 0, 0},
      {2.958039892, 5.61248608, -1.08012345, -0.7637626158, 0, 0, 0, 0, 0, 0},
      {2.291287847, 1.449137675, 4.183300133, -0.5916079783, 0, 0, 0, 0, 0, 0},
      {1.322875656, 0, 3.864367132, -0.3415650255, 0, 0, 0, 0, 0, 0},
      {1.870828693, 7.09929574, 0, 4.347413024, 0, 0, 0, 0, 0, 0},
      {1.08012345, 0, 7.09929574, 2.50998008, 0, 0, 0, 0, 0, 0},
      {-3.818813079, 0, 0, 8.874119675, 0, 0, 0, 0, 0, 0}};
    
      // Compute reference derivatives
      // Declare pointer to array of derivatives on FIAT element
      double *derivatives = new double [num_derivatives];
    
      // Declare coefficients
      double coeff0_0 = 0;
      double coeff0_1 = 0;
      double coeff0_2 = 0;
      double coeff0_3 = 0;
      double coeff0_4 = 0;
      double coeff0_5 = 0;
      double coeff0_6 = 0;
      double coeff0_7 = 0;
      double coeff0_8 = 0;
      double coeff0_9 = 0;
    
      // Declare new coefficients
      double new_coeff0_0 = 0;
      double new_coeff0_1 = 0;
      double new_coeff0_2 = 0;
      double new_coeff0_3 = 0;
      double new_coeff0_4 = 0;
      double new_coeff0_5 = 0;
      double new_coeff0_6 = 0;
      double new_coeff0_7 = 0;
      double new_coeff0_8 = 0;
      double new_coeff0_9 = 0;
    
      // Loop possible derivatives
      for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
      {
        // Get values from coefficients array
        new_coeff0_0 = coefficients0[dof][0];
        new_coeff0_1 = coefficients0[dof][1];
        new_coeff0_2 = coefficients0[dof][2];
        new_coeff0_3 = coefficients0[dof][3];
        new_coeff0_4 = coefficients0[dof][4];
        new_coeff0_5 = coefficients0[dof][5];
        new_coeff0_6 = coefficients0[dof][6];
        new_coeff0_7 = coefficients0[dof][7];
        new_coeff0_8 = coefficients0[dof][8];
        new_coeff0_9 = coefficients0[dof][9];
    
        // Loop derivative order
        for (unsigned int j = 0; j < n; j++)
        {
          // Update old coefficients
          coeff0_0 = new_coeff0_0;
          coeff0_1 = new_coeff0_1;
          coeff0_2 = new_coeff0_2;
          coeff0_3 = new_coeff0_3;
          coeff0_4 = new_coeff0_4;
          coeff0_5 = new_coeff0_5;
          coeff0_6 = new_coeff0_6;
          coeff0_7 = new_coeff0_7;
          coeff0_8 = new_coeff0_8;
          coeff0_9 = new_coeff0_9;
    
          if(combinations[deriv_num][j] == 0)
          {
            new_coeff0_0 = coeff0_0*dmats0[0][0] + coeff0_1*dmats0[1][0] + coeff0_2*dmats0[2][0] + coeff0_3*dmats0[3][0] + coeff0_4*dmats0[4][0] + coeff0_5*dmats0[5][0] + coeff0_6*dmats0[6][0] + coeff0_7*dmats0[7][0] + coeff0_8*dmats0[8][0] + coeff0_9*dmats0[9][0];
            new_coeff0_1 = coeff0_0*dmats0[0][1] + coeff0_1*dmats0[1][1] + coeff0_2*dmats0[2][1] + coeff0_3*dmats0[3][1] + coeff0_4*dmats0[4][1] + coeff0_5*dmats0[5][1] + coeff0_6*dmats0[6][1] + coeff0_7*dmats0[7][1] + coeff0_8*dmats0[8][1] + coeff0_9*dmats0[9][1];
            new_coeff0_2 = coeff0_0*dmats0[0][2] + coeff0_1*dmats0[1][2] + coeff0_2*dmats0[2][2] + coeff0_3*dmats0[3][2] + coeff0_4*dmats0[4][2] + coeff0_5*dmats0[5][2] + coeff0_6*dmats0[6][2] + coeff0_7*dmats0[7][2] + coeff0_8*dmats0[8][2] + coeff0_9*dmats0[9][2];
            new_coeff0_3 = coeff0_0*dmats0[0][3] + coeff0_1*dmats0[1][3] + coeff0_2*dmats0[2][3] + coeff0_3*dmats0[3][3] + coeff0_4*dmats0[4][3] + coeff0_5*dmats0[5][3] + coeff0_6*dmats0[6][3] + coeff0_7*dmats0[7][3] + coeff0_8*dmats0[8][3] + coeff0_9*dmats0[9][3];
            new_coeff0_4 = coeff0_0*dmats0[0][4] + coeff0_1*dmats0[1][4] + coeff0_2*dmats0[2][4] + coeff0_3*dmats0[3][4] + coeff0_4*dmats0[4][4] + coeff0_5*dmats0[5][4] + coeff0_6*dmats0[6][4] + coeff0_7*dmats0[7][4] + coeff0_8*dmats0[8][4] + coeff0_9*dmats0[9][4];
            new_coeff0_5 = coeff0_0*dmats0[0][5] + coeff0_1*dmats0[1][5] + coeff0_2*dmats0[2][5] + coeff0_3*dmats0[3][5] + coeff0_4*dmats0[4][5] + coeff0_5*dmats0[5][5] + coeff0_6*dmats0[6][5] + coeff0_7*dmats0[7][5] + coeff0_8*dmats0[8][5] + coeff0_9*dmats0[9][5];
            new_coeff0_6 = coeff0_0*dmats0[0][6] + coeff0_1*dmats0[1][6] + coeff0_2*dmats0[2][6] + coeff0_3*dmats0[3][6] + coeff0_4*dmats0[4][6] + coeff0_5*dmats0[5][6] + coeff0_6*dmats0[6][6] + coeff0_7*dmats0[7][6] + coeff0_8*dmats0[8][6] + coeff0_9*dmats0[9][6];
            new_coeff0_7 = coeff0_0*dmats0[0][7] + coeff0_1*dmats0[1][7] + coeff0_2*dmats0[2][7] + coeff0_3*dmats0[3][7] + coeff0_4*dmats0[4][7] + coeff0_5*dmats0[5][7] + coeff0_6*dmats0[6][7] + coeff0_7*dmats0[7][7] + coeff0_8*dmats0[8][7] + coeff0_9*dmats0[9][7];
            new_coeff0_8 = coeff0_0*dmats0[0][8] + coeff0_1*dmats0[1][8] + coeff0_2*dmats0[2][8] + coeff0_3*dmats0[3][8] + coeff0_4*dmats0[4][8] + coeff0_5*dmats0[5][8] + coeff0_6*dmats0[6][8] + coeff0_7*dmats0[7][8] + coeff0_8*dmats0[8][8] + coeff0_9*dmats0[9][8];
            new_coeff0_9 = coeff0_0*dmats0[0][9] + coeff0_1*dmats0[1][9] + coeff0_2*dmats0[2][9] + coeff0_3*dmats0[3][9] + coeff0_4*dmats0[4][9] + coeff0_5*dmats0[5][9] + coeff0_6*dmats0[6][9] + coeff0_7*dmats0[7][9] + coeff0_8*dmats0[8][9] + coeff0_9*dmats0[9][9];
          }
          if(combinations[deriv_num][j] == 1)
          {
            new_coeff0_0 = coeff0_0*dmats1[0][0] + coeff0_1*dmats1[1][0] + coeff0_2*dmats1[2][0] + coeff0_3*dmats1[3][0] + coeff0_4*dmats1[4][0] + coeff0_5*dmats1[5][0] + coeff0_6*dmats1[6][0] + coeff0_7*dmats1[7][0] + coeff0_8*dmats1[8][0] + coeff0_9*dmats1[9][0];
            new_coeff0_1 = coeff0_0*dmats1[0][1] + coeff0_1*dmats1[1][1] + coeff0_2*dmats1[2][1] + coeff0_3*dmats1[3][1] + coeff0_4*dmats1[4][1] + coeff0_5*dmats1[5][1] + coeff0_6*dmats1[6][1] + coeff0_7*dmats1[7][1] + coeff0_8*dmats1[8][1] + coeff0_9*dmats1[9][1];
            new_coeff0_2 = coeff0_0*dmats1[0][2] + coeff0_1*dmats1[1][2] + coeff0_2*dmats1[2][2] + coeff0_3*dmats1[3][2] + coeff0_4*dmats1[4][2] + coeff0_5*dmats1[5][2] + coeff0_6*dmats1[6][2] + coeff0_7*dmats1[7][2] + coeff0_8*dmats1[8][2] + coeff0_9*dmats1[9][2];
            new_coeff0_3 = coeff0_0*dmats1[0][3] + coeff0_1*dmats1[1][3] + coeff0_2*dmats1[2][3] + coeff0_3*dmats1[3][3] + coeff0_4*dmats1[4][3] + coeff0_5*dmats1[5][3] + coeff0_6*dmats1[6][3] + coeff0_7*dmats1[7][3] + coeff0_8*dmats1[8][3] + coeff0_9*dmats1[9][3];
            new_coeff0_4 = coeff0_0*dmats1[0][4] + coeff0_1*dmats1[1][4] + coeff0_2*dmats1[2][4] + coeff0_3*dmats1[3][4] + coeff0_4*dmats1[4][4] + coeff0_5*dmats1[5][4] + coeff0_6*dmats1[6][4] + coeff0_7*dmats1[7][4] + coeff0_8*dmats1[8][4] + coeff0_9*dmats1[9][4];
            new_coeff0_5 = coeff0_0*dmats1[0][5] + coeff0_1*dmats1[1][5] + coeff0_2*dmats1[2][5] + coeff0_3*dmats1[3][5] + coeff0_4*dmats1[4][5] + coeff0_5*dmats1[5][5] + coeff0_6*dmats1[6][5] + coeff0_7*dmats1[7][5] + coeff0_8*dmats1[8][5] + coeff0_9*dmats1[9][5];
            new_coeff0_6 = coeff0_0*dmats1[0][6] + coeff0_1*dmats1[1][6] + coeff0_2*dmats1[2][6] + coeff0_3*dmats1[3][6] + coeff0_4*dmats1[4][6] + coeff0_5*dmats1[5][6] + coeff0_6*dmats1[6][6] + coeff0_7*dmats1[7][6] + coeff0_8*dmats1[8][6] + coeff0_9*dmats1[9][6];
            new_coeff0_7 = coeff0_0*dmats1[0][7] + coeff0_1*dmats1[1][7] + coeff0_2*dmats1[2][7] + coeff0_3*dmats1[3][7] + coeff0_4*dmats1[4][7] + coeff0_5*dmats1[5][7] + coeff0_6*dmats1[6][7] + coeff0_7*dmats1[7][7] + coeff0_8*dmats1[8][7] + coeff0_9*dmats1[9][7];
            new_coeff0_8 = coeff0_0*dmats1[0][8] + coeff0_1*dmats1[1][8] + coeff0_2*dmats1[2][8] + coeff0_3*dmats1[3][8] + coeff0_4*dmats1[4][8] + coeff0_5*dmats1[5][8] + coeff0_6*dmats1[6][8] + coeff0_7*dmats1[7][8] + coeff0_8*dmats1[8][8] + coeff0_9*dmats1[9][8];
            new_coeff0_9 = coeff0_0*dmats1[0][9] + coeff0_1*dmats1[1][9] + coeff0_2*dmats1[2][9] + coeff0_3*dmats1[3][9] + coeff0_4*dmats1[4][9] + coeff0_5*dmats1[5][9] + coeff0_6*dmats1[6][9] + coeff0_7*dmats1[7][9] + coeff0_8*dmats1[8][9] + coeff0_9*dmats1[9][9];
          }
          if(combinations[deriv_num][j] == 2)
          {
            new_coeff0_0 = coeff0_0*dmats2[0][0] + coeff0_1*dmats2[1][0] + coeff0_2*dmats2[2][0] + coeff0_3*dmats2[3][0] + coeff0_4*dmats2[4][0] + coeff0_5*dmats2[5][0] + coeff0_6*dmats2[6][0] + coeff0_7*dmats2[7][0] + coeff0_8*dmats2[8][0] + coeff0_9*dmats2[9][0];
            new_coeff0_1 = coeff0_0*dmats2[0][1] + coeff0_1*dmats2[1][1] + coeff0_2*dmats2[2][1] + coeff0_3*dmats2[3][1] + coeff0_4*dmats2[4][1] + coeff0_5*dmats2[5][1] + coeff0_6*dmats2[6][1] + coeff0_7*dmats2[7][1] + coeff0_8*dmats2[8][1] + coeff0_9*dmats2[9][1];
            new_coeff0_2 = coeff0_0*dmats2[0][2] + coeff0_1*dmats2[1][2] + coeff0_2*dmats2[2][2] + coeff0_3*dmats2[3][2] + coeff0_4*dmats2[4][2] + coeff0_5*dmats2[5][2] + coeff0_6*dmats2[6][2] + coeff0_7*dmats2[7][2] + coeff0_8*dmats2[8][2] + coeff0_9*dmats2[9][2];
            new_coeff0_3 = coeff0_0*dmats2[0][3] + coeff0_1*dmats2[1][3] + coeff0_2*dmats2[2][3] + coeff0_3*dmats2[3][3] + coeff0_4*dmats2[4][3] + coeff0_5*dmats2[5][3] + coeff0_6*dmats2[6][3] + coeff0_7*dmats2[7][3] + coeff0_8*dmats2[8][3] + coeff0_9*dmats2[9][3];
            new_coeff0_4 = coeff0_0*dmats2[0][4] + coeff0_1*dmats2[1][4] + coeff0_2*dmats2[2][4] + coeff0_3*dmats2[3][4] + coeff0_4*dmats2[4][4] + coeff0_5*dmats2[5][4] + coeff0_6*dmats2[6][4] + coeff0_7*dmats2[7][4] + coeff0_8*dmats2[8][4] + coeff0_9*dmats2[9][4];
            new_coeff0_5 = coeff0_0*dmats2[0][5] + coeff0_1*dmats2[1][5] + coeff0_2*dmats2[2][5] + coeff0_3*dmats2[3][5] + coeff0_4*dmats2[4][5] + coeff0_5*dmats2[5][5] + coeff0_6*dmats2[6][5] + coeff0_7*dmats2[7][5] + coeff0_8*dmats2[8][5] + coeff0_9*dmats2[9][5];
            new_coeff0_6 = coeff0_0*dmats2[0][6] + coeff0_1*dmats2[1][6] + coeff0_2*dmats2[2][6] + coeff0_3*dmats2[3][6] + coeff0_4*dmats2[4][6] + coeff0_5*dmats2[5][6] + coeff0_6*dmats2[6][6] + coeff0_7*dmats2[7][6] + coeff0_8*dmats2[8][6] + coeff0_9*dmats2[9][6];
            new_coeff0_7 = coeff0_0*dmats2[0][7] + coeff0_1*dmats2[1][7] + coeff0_2*dmats2[2][7] + coeff0_3*dmats2[3][7] + coeff0_4*dmats2[4][7] + coeff0_5*dmats2[5][7] + coeff0_6*dmats2[6][7] + coeff0_7*dmats2[7][7] + coeff0_8*dmats2[8][7] + coeff0_9*dmats2[9][7];
            new_coeff0_8 = coeff0_0*dmats2[0][8] + coeff0_1*dmats2[1][8] + coeff0_2*dmats2[2][8] + coeff0_3*dmats2[3][8] + coeff0_4*dmats2[4][8] + coeff0_5*dmats2[5][8] + coeff0_6*dmats2[6][8] + coeff0_7*dmats2[7][8] + coeff0_8*dmats2[8][8] + coeff0_9*dmats2[9][8];
            new_coeff0_9 = coeff0_0*dmats2[0][9] + coeff0_1*dmats2[1][9] + coeff0_2*dmats2[2][9] + coeff0_3*dmats2[3][9] + coeff0_4*dmats2[4][9] + coeff0_5*dmats2[5][9] + coeff0_6*dmats2[6][9] + coeff0_7*dmats2[7][9] + coeff0_8*dmats2[8][9] + coeff0_9*dmats2[9][9];
          }
    
        }
        // Compute derivatives on reference element as dot product of coefficients and basisvalues
        derivatives[deriv_num] = new_coeff0_0*basisvalue0 + new_coeff0_1*basisvalue1 + new_coeff0_2*basisvalue2 + new_coeff0_3*basisvalue3 + new_coeff0_4*basisvalue4 + new_coeff0_5*basisvalue5 + new_coeff0_6*basisvalue6 + new_coeff0_7*basisvalue7 + new_coeff0_8*basisvalue8 + new_coeff0_9*basisvalue9;
      }
    
      // Transform derivatives back to physical element
      for (unsigned int row = 0; row < num_derivatives; row++)
      {
        for (unsigned int col = 0; col < num_derivatives; col++)
        {
          values[num_derivatives + row] += transform[row][col]*derivatives[col];
        }
      }
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
    
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int row = 0; row < num_derivatives; row++)
      {
        delete [] combinations[row];
        delete [] transform[row];
      }
    
      delete [] combinations;
      delete [] transform;
    }
    
    if (20 <= i && i <= 29)
    {
      // Map degree of freedom to element degree of freedom
      const unsigned int dof = i - 20;
    
      // Generate scalings
      const double scalings_y_0 = 1;
      const double scalings_y_1 = scalings_y_0*(0.5 - 0.5*y);
      const double scalings_y_2 = scalings_y_1*(0.5 - 0.5*y);
      const double scalings_z_0 = 1;
      const double scalings_z_1 = scalings_z_0*(0.5 - 0.5*z);
      const double scalings_z_2 = scalings_z_1*(0.5 - 0.5*z);
    
      // Compute psitilde_a
      const double psitilde_a_0 = 1;
      const double psitilde_a_1 = x;
      const double psitilde_a_2 = 1.5*x*psitilde_a_1 - 0.5*psitilde_a_0;
    
      // Compute psitilde_bs
      const double psitilde_bs_0_0 = 1;
      const double psitilde_bs_0_1 = 1.5*y + 0.5;
      const double psitilde_bs_0_2 = 0.1111111111*psitilde_bs_0_1 + 1.666666667*y*psitilde_bs_0_1 - 0.5555555556*psitilde_bs_0_0;
      const double psitilde_bs_1_0 = 1;
      const double psitilde_bs_1_1 = 2.5*y + 1.5;
      const double psitilde_bs_2_0 = 1;
    
      // Compute psitilde_cs
      const double psitilde_cs_00_0 = 1;
      const double psitilde_cs_00_1 = 2*z + 1;
      const double psitilde_cs_00_2 = 0.3125*psitilde_cs_00_1 + 1.875*z*psitilde_cs_00_1 - 0.5625*psitilde_cs_00_0;
      const double psitilde_cs_01_0 = 1;
      const double psitilde_cs_01_1 = 3*z + 2;
      const double psitilde_cs_02_0 = 1;
      const double psitilde_cs_10_0 = 1;
      const double psitilde_cs_10_1 = 3*z + 2;
      const double psitilde_cs_11_0 = 1;
      const double psitilde_cs_20_0 = 1;
    
      // Compute basisvalues
      const double basisvalue0 = 0.8660254038*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_0;
      const double basisvalue1 = 2.738612788*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_0;
      const double basisvalue2 = 1.58113883*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_0;
      const double basisvalue3 = 1.118033989*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_1;
      const double basisvalue4 = 5.123475383*psitilde_a_2*scalings_y_2*psitilde_bs_2_0*scalings_z_2*psitilde_cs_20_0;
      const double basisvalue5 = 3.968626967*psitilde_a_1*scalings_y_1*psitilde_bs_1_1*scalings_z_2*psitilde_cs_11_0;
      const double basisvalue6 = 2.291287847*psitilde_a_0*scalings_y_0*psitilde_bs_0_2*scalings_z_2*psitilde_cs_02_0;
      const double basisvalue7 = 3.240370349*psitilde_a_1*scalings_y_1*psitilde_bs_1_0*scalings_z_1*psitilde_cs_10_1;
      const double basisvalue8 = 1.870828693*psitilde_a_0*scalings_y_0*psitilde_bs_0_1*scalings_z_1*psitilde_cs_01_1;
      const double basisvalue9 = 1.322875656*psitilde_a_0*scalings_y_0*psitilde_bs_0_0*scalings_z_0*psitilde_cs_00_2;
    
      // Table(s) of coefficients
      const static double coefficients0[10][10] =   \
      {{-0.05773502692, -0.06085806195, -0.03513641845, -0.02484519975, 0.06506000486, 0.05039526307, 0.0290957187, 0.04114755999, 0.02375655484, 0.01679842102},
      {-0.05773502692, 0.06085806195, -0.03513641845, -0.02484519975, 0.06506000486, -0.05039526307, 0.0290957187, -0.04114755999, 0.02375655484, 0.01679842102},
      {-0.05773502692, 0, 0.07027283689, -0.02484519975, 0, 0, 0.08728715609, 0, -0.04751310967, 0.01679842102},
      {-0.05773502692, 0, 0, 0.07453559925, 0, 0, 0, 0, 0, 0.1007905261},
      {0.2309401077, 0, 0.1405456738, 0.099380799, 0, 0, 0, 0, 0.1187827742, -0.06719368409},
      {0.2309401077, 0.1217161239, -0.07027283689, 0.099380799, 0, 0, 0, 0.1028689, -0.05939138709, -0.06719368409},
      {0.2309401077, 0.1217161239, 0.07027283689, -0.099380799, 0, 0.1007905261, -0.08728715609, -0.02057377999, -0.01187827742, 0.01679842102},
      {0.2309401077, -0.1217161239, -0.07027283689, 0.099380799, 0, 0, 0, -0.1028689, -0.05939138709, -0.06719368409},
      {0.2309401077, -0.1217161239, 0.07027283689, -0.099380799, 0, -0.1007905261, -0.08728715609, 0.02057377999, -0.01187827742, 0.01679842102},
      {0.2309401077, 0, -0.1405456738, -0.099380799, -0.1301200097, 0, 0.0290957187, 0, 0.02375655484, 0.01679842102}};
    
      // Interesting (new) part
      // Tables of derivatives of the polynomial base (transpose)
      const static double dmats0[10][10] =   \
      {{0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
      {6.32455532, 0, 0, 0, 0, 0, 0, 0, 0, 0},
      {0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
      {0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
      {0, 11.22497216, 0, 0, 0, 0, 0, 0, 0, 0},
      {4.582575695, 0, 8.366600265, -1.183215957, 0, 0, 0, 0, 0, 0},
      {0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
      {3.741657387, 0, 0, 8.694826048, 0, 0, 0, 0, 0, 0},
      {0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
      {0, 0, 0, 0, 0, 0, 0, 0, 0, 0}};
    
      const static double dmats1[10][10] =   \
      {{0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
      {3.16227766, 0, 0, 0, 0, 0, 0, 0, 0, 0},
      {5.477225575, 0, 0, 0, 0, 0, 0, 0, 0, 0},
      {0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
      {2.958039892, 5.61248608, -1.08012345, -0.7637626158, 0, 0, 0, 0, 0, 0},
      {2.291287847, 7.245688373, 4.183300133, -0.5916079783, 0, 0, 0, 0, 0, 0},
      {-2.645751311, 0, 9.660917831, 0.6831300511, 0, 0, 0, 0, 0, 0},
      {1.870828693, 0, 0, 4.347413024, 0, 0, 0, 0, 0, 0},
      {3.240370349, 0, 0, 7.529940239, 0, 0, 0, 0, 0, 0},
      {0, 0, 0, 0, 0, 0, 0, 0, 0, 0}};
    
      const static double dmats2[10][10] =   \
      {{0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
      {3.16227766, 0, 0, 0, 0, 0, 0, 0, 0, 0},
      {1.825741858, 0, 0, 0, 0, 0, 0, 0, 0, 0},
      {5.163977795, 0, 0, 0, 0, 0, 0, 0, 0, 0},
      {2.958039892, 5.61248608, -1.08012345, -0.7637626158, 0, 0, 0, 0, 0, 0},
      {2.291287847, 1.449137675, 4.183300133, -0.5916079783, 0, 0, 0, 0, 0, 0},
      {1.322875656, 0, 3.864367132, -0.3415650255, 0, 0, 0, 0, 0, 0},
      {1.870828693, 7.09929574, 0, 4.347413024, 0, 0, 0, 0, 0, 0},
      {1.08012345, 0, 7.09929574, 2.50998008, 0, 0, 0, 0, 0, 0},
      {-3.818813079, 0, 0, 8.874119675, 0, 0, 0, 0, 0, 0}};
    
      // Compute reference derivatives
      // Declare pointer to array of derivatives on FIAT element
      double *derivatives = new double [num_derivatives];
    
      // Declare coefficients
      double coeff0_0 = 0;
      double coeff0_1 = 0;
      double coeff0_2 = 0;
      double coeff0_3 = 0;
      double coeff0_4 = 0;
      double coeff0_5 = 0;
      double coeff0_6 = 0;
      double coeff0_7 = 0;
      double coeff0_8 = 0;
      double coeff0_9 = 0;
    
      // Declare new coefficients
      double new_coeff0_0 = 0;
      double new_coeff0_1 = 0;
      double new_coeff0_2 = 0;
      double new_coeff0_3 = 0;
      double new_coeff0_4 = 0;
      double new_coeff0_5 = 0;
      double new_coeff0_6 = 0;
      double new_coeff0_7 = 0;
      double new_coeff0_8 = 0;
      double new_coeff0_9 = 0;
    
      // Loop possible derivatives
      for (unsigned int deriv_num = 0; deriv_num < num_derivatives; deriv_num++)
      {
        // Get values from coefficients array
        new_coeff0_0 = coefficients0[dof][0];
        new_coeff0_1 = coefficients0[dof][1];
        new_coeff0_2 = coefficients0[dof][2];
        new_coeff0_3 = coefficients0[dof][3];
        new_coeff0_4 = coefficients0[dof][4];
        new_coeff0_5 = coefficients0[dof][5];
        new_coeff0_6 = coefficients0[dof][6];
        new_coeff0_7 = coefficients0[dof][7];
        new_coeff0_8 = coefficients0[dof][8];
        new_coeff0_9 = coefficients0[dof][9];
    
        // Loop derivative order
        for (unsigned int j = 0; j < n; j++)
        {
          // Update old coefficients
          coeff0_0 = new_coeff0_0;
          coeff0_1 = new_coeff0_1;
          coeff0_2 = new_coeff0_2;
          coeff0_3 = new_coeff0_3;
          coeff0_4 = new_coeff0_4;
          coeff0_5 = new_coeff0_5;
          coeff0_6 = new_coeff0_6;
          coeff0_7 = new_coeff0_7;
          coeff0_8 = new_coeff0_8;
          coeff0_9 = new_coeff0_9;
    
          if(combinations[deriv_num][j] == 0)
          {
            new_coeff0_0 = coeff0_0*dmats0[0][0] + coeff0_1*dmats0[1][0] + coeff0_2*dmats0[2][0] + coeff0_3*dmats0[3][0] + coeff0_4*dmats0[4][0] + coeff0_5*dmats0[5][0] + coeff0_6*dmats0[6][0] + coeff0_7*dmats0[7][0] + coeff0_8*dmats0[8][0] + coeff0_9*dmats0[9][0];
            new_coeff0_1 = coeff0_0*dmats0[0][1] + coeff0_1*dmats0[1][1] + coeff0_2*dmats0[2][1] + coeff0_3*dmats0[3][1] + coeff0_4*dmats0[4][1] + coeff0_5*dmats0[5][1] + coeff0_6*dmats0[6][1] + coeff0_7*dmats0[7][1] + coeff0_8*dmats0[8][1] + coeff0_9*dmats0[9][1];
            new_coeff0_2 = coeff0_0*dmats0[0][2] + coeff0_1*dmats0[1][2] + coeff0_2*dmats0[2][2] + coeff0_3*dmats0[3][2] + coeff0_4*dmats0[4][2] + coeff0_5*dmats0[5][2] + coeff0_6*dmats0[6][2] + coeff0_7*dmats0[7][2] + coeff0_8*dmats0[8][2] + coeff0_9*dmats0[9][2];
            new_coeff0_3 = coeff0_0*dmats0[0][3] + coeff0_1*dmats0[1][3] + coeff0_2*dmats0[2][3] + coeff0_3*dmats0[3][3] + coeff0_4*dmats0[4][3] + coeff0_5*dmats0[5][3] + coeff0_6*dmats0[6][3] + coeff0_7*dmats0[7][3] + coeff0_8*dmats0[8][3] + coeff0_9*dmats0[9][3];
            new_coeff0_4 = coeff0_0*dmats0[0][4] + coeff0_1*dmats0[1][4] + coeff0_2*dmats0[2][4] + coeff0_3*dmats0[3][4] + coeff0_4*dmats0[4][4] + coeff0_5*dmats0[5][4] + coeff0_6*dmats0[6][4] + coeff0_7*dmats0[7][4] + coeff0_8*dmats0[8][4] + coeff0_9*dmats0[9][4];
            new_coeff0_5 = coeff0_0*dmats0[0][5] + coeff0_1*dmats0[1][5] + coeff0_2*dmats0[2][5] + coeff0_3*dmats0[3][5] + coeff0_4*dmats0[4][5] + coeff0_5*dmats0[5][5] + coeff0_6*dmats0[6][5] + coeff0_7*dmats0[7][5] + coeff0_8*dmats0[8][5] + coeff0_9*dmats0[9][5];
            new_coeff0_6 = coeff0_0*dmats0[0][6] + coeff0_1*dmats0[1][6] + coeff0_2*dmats0[2][6] + coeff0_3*dmats0[3][6] + coeff0_4*dmats0[4][6] + coeff0_5*dmats0[5][6] + coeff0_6*dmats0[6][6] + coeff0_7*dmats0[7][6] + coeff0_8*dmats0[8][6] + coeff0_9*dmats0[9][6];
            new_coeff0_7 = coeff0_0*dmats0[0][7] + coeff0_1*dmats0[1][7] + coeff0_2*dmats0[2][7] + coeff0_3*dmats0[3][7] + coeff0_4*dmats0[4][7] + coeff0_5*dmats0[5][7] + coeff0_6*dmats0[6][7] + coeff0_7*dmats0[7][7] + coeff0_8*dmats0[8][7] + coeff0_9*dmats0[9][7];
            new_coeff0_8 = coeff0_0*dmats0[0][8] + coeff0_1*dmats0[1][8] + coeff0_2*dmats0[2][8] + coeff0_3*dmats0[3][8] + coeff0_4*dmats0[4][8] + coeff0_5*dmats0[5][8] + coeff0_6*dmats0[6][8] + coeff0_7*dmats0[7][8] + coeff0_8*dmats0[8][8] + coeff0_9*dmats0[9][8];
            new_coeff0_9 = coeff0_0*dmats0[0][9] + coeff0_1*dmats0[1][9] + coeff0_2*dmats0[2][9] + coeff0_3*dmats0[3][9] + coeff0_4*dmats0[4][9] + coeff0_5*dmats0[5][9] + coeff0_6*dmats0[6][9] + coeff0_7*dmats0[7][9] + coeff0_8*dmats0[8][9] + coeff0_9*dmats0[9][9];
          }
          if(combinations[deriv_num][j] == 1)
          {
            new_coeff0_0 = coeff0_0*dmats1[0][0] + coeff0_1*dmats1[1][0] + coeff0_2*dmats1[2][0] + coeff0_3*dmats1[3][0] + coeff0_4*dmats1[4][0] + coeff0_5*dmats1[5][0] + coeff0_6*dmats1[6][0] + coeff0_7*dmats1[7][0] + coeff0_8*dmats1[8][0] + coeff0_9*dmats1[9][0];
            new_coeff0_1 = coeff0_0*dmats1[0][1] + coeff0_1*dmats1[1][1] + coeff0_2*dmats1[2][1] + coeff0_3*dmats1[3][1] + coeff0_4*dmats1[4][1] + coeff0_5*dmats1[5][1] + coeff0_6*dmats1[6][1] + coeff0_7*dmats1[7][1] + coeff0_8*dmats1[8][1] + coeff0_9*dmats1[9][1];
            new_coeff0_2 = coeff0_0*dmats1[0][2] + coeff0_1*dmats1[1][2] + coeff0_2*dmats1[2][2] + coeff0_3*dmats1[3][2] + coeff0_4*dmats1[4][2] + coeff0_5*dmats1[5][2] + coeff0_6*dmats1[6][2] + coeff0_7*dmats1[7][2] + coeff0_8*dmats1[8][2] + coeff0_9*dmats1[9][2];
            new_coeff0_3 = coeff0_0*dmats1[0][3] + coeff0_1*dmats1[1][3] + coeff0_2*dmats1[2][3] + coeff0_3*dmats1[3][3] + coeff0_4*dmats1[4][3] + coeff0_5*dmats1[5][3] + coeff0_6*dmats1[6][3] + coeff0_7*dmats1[7][3] + coeff0_8*dmats1[8][3] + coeff0_9*dmats1[9][3];
            new_coeff0_4 = coeff0_0*dmats1[0][4] + coeff0_1*dmats1[1][4] + coeff0_2*dmats1[2][4] + coeff0_3*dmats1[3][4] + coeff0_4*dmats1[4][4] + coeff0_5*dmats1[5][4] + coeff0_6*dmats1[6][4] + coeff0_7*dmats1[7][4] + coeff0_8*dmats1[8][4] + coeff0_9*dmats1[9][4];
            new_coeff0_5 = coeff0_0*dmats1[0][5] + coeff0_1*dmats1[1][5] + coeff0_2*dmats1[2][5] + coeff0_3*dmats1[3][5] + coeff0_4*dmats1[4][5] + coeff0_5*dmats1[5][5] + coeff0_6*dmats1[6][5] + coeff0_7*dmats1[7][5] + coeff0_8*dmats1[8][5] + coeff0_9*dmats1[9][5];
            new_coeff0_6 = coeff0_0*dmats1[0][6] + coeff0_1*dmats1[1][6] + coeff0_2*dmats1[2][6] + coeff0_3*dmats1[3][6] + coeff0_4*dmats1[4][6] + coeff0_5*dmats1[5][6] + coeff0_6*dmats1[6][6] + coeff0_7*dmats1[7][6] + coeff0_8*dmats1[8][6] + coeff0_9*dmats1[9][6];
            new_coeff0_7 = coeff0_0*dmats1[0][7] + coeff0_1*dmats1[1][7] + coeff0_2*dmats1[2][7] + coeff0_3*dmats1[3][7] + coeff0_4*dmats1[4][7] + coeff0_5*dmats1[5][7] + coeff0_6*dmats1[6][7] + coeff0_7*dmats1[7][7] + coeff0_8*dmats1[8][7] + coeff0_9*dmats1[9][7];
            new_coeff0_8 = coeff0_0*dmats1[0][8] + coeff0_1*dmats1[1][8] + coeff0_2*dmats1[2][8] + coeff0_3*dmats1[3][8] + coeff0_4*dmats1[4][8] + coeff0_5*dmats1[5][8] + coeff0_6*dmats1[6][8] + coeff0_7*dmats1[7][8] + coeff0_8*dmats1[8][8] + coeff0_9*dmats1[9][8];
            new_coeff0_9 = coeff0_0*dmats1[0][9] + coeff0_1*dmats1[1][9] + coeff0_2*dmats1[2][9] + coeff0_3*dmats1[3][9] + coeff0_4*dmats1[4][9] + coeff0_5*dmats1[5][9] + coeff0_6*dmats1[6][9] + coeff0_7*dmats1[7][9] + coeff0_8*dmats1[8][9] + coeff0_9*dmats1[9][9];
          }
          if(combinations[deriv_num][j] == 2)
          {
            new_coeff0_0 = coeff0_0*dmats2[0][0] + coeff0_1*dmats2[1][0] + coeff0_2*dmats2[2][0] + coeff0_3*dmats2[3][0] + coeff0_4*dmats2[4][0] + coeff0_5*dmats2[5][0] + coeff0_6*dmats2[6][0] + coeff0_7*dmats2[7][0] + coeff0_8*dmats2[8][0] + coeff0_9*dmats2[9][0];
            new_coeff0_1 = coeff0_0*dmats2[0][1] + coeff0_1*dmats2[1][1] + coeff0_2*dmats2[2][1] + coeff0_3*dmats2[3][1] + coeff0_4*dmats2[4][1] + coeff0_5*dmats2[5][1] + coeff0_6*dmats2[6][1] + coeff0_7*dmats2[7][1] + coeff0_8*dmats2[8][1] + coeff0_9*dmats2[9][1];
            new_coeff0_2 = coeff0_0*dmats2[0][2] + coeff0_1*dmats2[1][2] + coeff0_2*dmats2[2][2] + coeff0_3*dmats2[3][2] + coeff0_4*dmats2[4][2] + coeff0_5*dmats2[5][2] + coeff0_6*dmats2[6][2] + coeff0_7*dmats2[7][2] + coeff0_8*dmats2[8][2] + coeff0_9*dmats2[9][2];
            new_coeff0_3 = coeff0_0*dmats2[0][3] + coeff0_1*dmats2[1][3] + coeff0_2*dmats2[2][3] + coeff0_3*dmats2[3][3] + coeff0_4*dmats2[4][3] + coeff0_5*dmats2[5][3] + coeff0_6*dmats2[6][3] + coeff0_7*dmats2[7][3] + coeff0_8*dmats2[8][3] + coeff0_9*dmats2[9][3];
            new_coeff0_4 = coeff0_0*dmats2[0][4] + coeff0_1*dmats2[1][4] + coeff0_2*dmats2[2][4] + coeff0_3*dmats2[3][4] + coeff0_4*dmats2[4][4] + coeff0_5*dmats2[5][4] + coeff0_6*dmats2[6][4] + coeff0_7*dmats2[7][4] + coeff0_8*dmats2[8][4] + coeff0_9*dmats2[9][4];
            new_coeff0_5 = coeff0_0*dmats2[0][5] + coeff0_1*dmats2[1][5] + coeff0_2*dmats2[2][5] + coeff0_3*dmats2[3][5] + coeff0_4*dmats2[4][5] + coeff0_5*dmats2[5][5] + coeff0_6*dmats2[6][5] + coeff0_7*dmats2[7][5] + coeff0_8*dmats2[8][5] + coeff0_9*dmats2[9][5];
            new_coeff0_6 = coeff0_0*dmats2[0][6] + coeff0_1*dmats2[1][6] + coeff0_2*dmats2[2][6] + coeff0_3*dmats2[3][6] + coeff0_4*dmats2[4][6] + coeff0_5*dmats2[5][6] + coeff0_6*dmats2[6][6] + coeff0_7*dmats2[7][6] + coeff0_8*dmats2[8][6] + coeff0_9*dmats2[9][6];
            new_coeff0_7 = coeff0_0*dmats2[0][7] + coeff0_1*dmats2[1][7] + coeff0_2*dmats2[2][7] + coeff0_3*dmats2[3][7] + coeff0_4*dmats2[4][7] + coeff0_5*dmats2[5][7] + coeff0_6*dmats2[6][7] + coeff0_7*dmats2[7][7] + coeff0_8*dmats2[8][7] + coeff0_9*dmats2[9][7];
            new_coeff0_8 = coeff0_0*dmats2[0][8] + coeff0_1*dmats2[1][8] + coeff0_2*dmats2[2][8] + coeff0_3*dmats2[3][8] + coeff0_4*dmats2[4][8] + coeff0_5*dmats2[5][8] + coeff0_6*dmats2[6][8] + coeff0_7*dmats2[7][8] + coeff0_8*dmats2[8][8] + coeff0_9*dmats2[9][8];
            new_coeff0_9 = coeff0_0*dmats2[0][9] + coeff0_1*dmats2[1][9] + coeff0_2*dmats2[2][9] + coeff0_3*dmats2[3][9] + coeff0_4*dmats2[4][9] + coeff0_5*dmats2[5][9] + coeff0_6*dmats2[6][9] + coeff0_7*dmats2[7][9] + coeff0_8*dmats2[8][9] + coeff0_9*dmats2[9][9];
          }
    
        }
        // Compute derivatives on reference element as dot product of coefficients and basisvalues
        derivatives[deriv_num] = new_coeff0_0*basisvalue0 + new_coeff0_1*basisvalue1 + new_coeff0_2*basisvalue2 + new_coeff0_3*basisvalue3 + new_coeff0_4*basisvalue4 + new_coeff0_5*basisvalue5 + new_coeff0_6*basisvalue6 + new_coeff0_7*basisvalue7 + new_coeff0_8*basisvalue8 + new_coeff0_9*basisvalue9;
      }
    
      // Transform derivatives back to physical element
      for (unsigned int row = 0; row < num_derivatives; row++)
      {
        for (unsigned int col = 0; col < num_derivatives; col++)
        {
          values[2*num_derivatives + row] += transform[row][col]*derivatives[col];
        }
      }
      // Delete pointer to array of derivatives on FIAT element
      delete [] derivatives;
    
      // Delete pointer to array of combinations of derivatives and transform
      for (unsigned int row = 0; row < num_derivatives; row++)
      {
        delete [] combinations[row];
        delete [] transform[row];
      }
    
      delete [] combinations;
      delete [] transform;
    }
    
  }

  /// Evaluate order n derivatives of all basis functions at given point in cell
  virtual void evaluate_basis_derivatives_all(unsigned int n,
                                              double* values,
                                              const double* coordinates,
                                              const ufc::cell& c) const
  {
    throw std::runtime_error("The vectorised version of evaluate_basis_derivatives() is not yet implemented.");
  }

  /// Evaluate linear functional for dof i on the function f
  virtual double evaluate_dof(unsigned int i,
                              const ufc::function& f,
                              const ufc::cell& c) const
  {
    // The reference points, direction and weights:
    const static double X[30][1][3] = {{{0, 0, 0}}, {{1, 0, 0}}, {{0, 1, 0}}, {{0, 0, 1}}, {{0, 0.5, 0.5}}, {{0.5, 0, 0.5}}, {{0.5, 0.5, 0}}, {{0, 0, 0.5}}, {{0, 0.5, 0}}, {{0.5, 0, 0}}, {{0, 0, 0}}, {{1, 0, 0}}, {{0, 1, 0}}, {{0, 0, 1}}, {{0, 0.5, 0.5}}, {{0.5, 0, 0.5}}, {{0.5, 0.5, 0}}, {{0, 0, 0.5}}, {{0, 0.5, 0}}, {{0.5, 0, 0}}, {{0, 0, 0}}, {{1, 0, 0}}, {{0, 1, 0}}, {{0, 0, 1}}, {{0, 0.5, 0.5}}, {{0.5, 0, 0.5}}, {{0.5, 0.5, 0}}, {{0, 0, 0.5}}, {{0, 0.5, 0}}, {{0.5, 0, 0}}};
    const static double W[30][1] = {{1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}, {1}};
    const static double D[30][1][3] = {{{1, 0, 0}}, {{1, 0, 0}}, {{1, 0, 0}}, {{1, 0, 0}}, {{1, 0, 0}}, {{1, 0, 0}}, {{1, 0, 0}}, {{1, 0, 0}}, {{1, 0, 0}}, {{1, 0, 0}}, {{0, 1, 0}}, {{0, 1, 0}}, {{0, 1, 0}}, {{0, 1, 0}}, {{0, 1, 0}}, {{0, 1, 0}}, {{0, 1, 0}}, {{0, 1, 0}}, {{0, 1, 0}}, {{0, 1, 0}}, {{0, 0, 1}}, {{0, 0, 1}}, {{0, 0, 1}}, {{0, 0, 1}}, {{0, 0, 1}}, {{0, 0, 1}}, {{0, 0, 1}}, {{0, 0, 1}}, {{0, 0, 1}}, {{0, 0, 1}}};
    
    const double * const * x = c.coordinates;
    double result = 0.0;
    // Iterate over the points:
    // Evaluate basis functions for affine mapping
    const double w0 = 1.0 - X[i][0][0] - X[i][0][1] - X[i][0][2];
    const double w1 = X[i][0][0];
    const double w2 = X[i][0][1];
    const double w3 = X[i][0][2];
    
    // Compute affine mapping y = F(X)
    double y[3];
    y[0] = w0*x[0][0] + w1*x[1][0] + w2*x[2][0] + w3*x[3][0];
    y[1] = w0*x[0][1] + w1*x[1][1] + w2*x[2][1] + w3*x[3][1];
    y[2] = w0*x[0][2] + w1*x[1][2] + w2*x[2][2] + w3*x[3][2];
    
    // Evaluate function at physical points
    double values[3];
    f.evaluate(values, y, c);
    
    // Map function values using appropriate mapping
    // Affine map: Do nothing
    
    // Note that we do not map the weights (yet).
    
    // Take directional components
    for(int k = 0; k < 3; k++)
      result += values[k]*D[i][0][k];
    // Multiply by weights 
    result *= W[i][0];
    
    return result;
  }

  /// Evaluate linear functionals for all dofs on the function f
  virtual void evaluate_dofs(double* values,
                             const ufc::function& f,
                             const ufc::cell& c) const
  {
    throw std::runtime_error("Not implemented (introduced in UFC v1.1).");
  }

  /// Interpolate vertex values from dof values
  virtual void interpolate_vertex_values(double* vertex_values,
                                         const double* dof_values,
                                         const ufc::cell& c) const
  {
    // Evaluate at vertices and use affine mapping
    vertex_values[0] = dof_values[0];
    vertex_values[1] = dof_values[1];
    vertex_values[2] = dof_values[2];
    vertex_values[3] = dof_values[3];
    // Evaluate at vertices and use affine mapping
    vertex_values[4] = dof_values[10];
    vertex_values[5] = dof_values[11];
    vertex_values[6] = dof_values[12];
    vertex_values[7] = dof_values[13];
    // Evaluate at vertices and use affine mapping
    vertex_values[8] = dof_values[20];
    vertex_values[9] = dof_values[21];
    vertex_values[10] = dof_values[22];
    vertex_values[11] = dof_values[23];
  }

  /// Return the number of sub elements (for a mixed element)
  virtual unsigned int num_sub_elements() const
  {
    return 3;
  }

  /// Create a new finite element for sub element i (for a mixed element)
  virtual ufc::finite_element* create_sub_element(unsigned int i) const
  {
    switch ( i )
    {
    case 0:
      return new VectorLaplaceGradCurlLinearForm_finite_element_1_0();
      break;
    case 1:
      return new VectorLaplaceGradCurlLinearForm_finite_element_1_1();
      break;
    case 2:
      return new VectorLaplaceGradCurlLinearForm_finite_element_1_2();
      break;
    }
    return 0;
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class VectorLaplaceGradCurlLinearForm_dof_map_0_0: public ufc::dof_map
{
private:

  unsigned int __global_dimension;

public:

  /// Constructor
  VectorLaplaceGradCurlLinearForm_dof_map_0_0() : ufc::dof_map()
  {
    __global_dimension = 0;
  }

  /// Destructor
  virtual ~VectorLaplaceGradCurlLinearForm_dof_map_0_0()
  {
    // Do nothing
  }

  /// Return a string identifying the dof map
  virtual const char* signature() const
  {
    return "FFC dof map for Lagrange finite element of degree 1 on a tetrahedron";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch ( d )
    {
    case 0:
      return true;
      break;
    case 1:
      return false;
      break;
    case 2:
      return false;
      break;
    case 3:
      return false;
      break;
    }
    return false;
  }

  /// Initialize dof map for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    __global_dimension = m.num_entities[0];
    return false;
  }

  /// Initialize dof map for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dof map for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return __global_dimension;
  }

  /// Return the dimension of the local finite element function space
  virtual unsigned int local_dimension() const
  {
    return 4;
  }

  // Return the geometric dimension of the coordinates this dof map provides
  virtual unsigned int geometric_dimension() const
  {
    throw std::runtime_error("Not implemented (introduced in UFC v1.1).");
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 3;
  }

  /// Return the number of dofs associated with each cell entity of dimension d
  virtual unsigned int num_entity_dofs(unsigned int d) const
  {
    throw std::runtime_error("Not implemented (introduced in UFC v1.1).");
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    dofs[0] = c.entity_indices[0][0];
    dofs[1] = c.entity_indices[0][1];
    dofs[2] = c.entity_indices[0][2];
    dofs[3] = c.entity_indices[0][3];
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch ( facet )
    {
    case 0:
      dofs[0] = 1;
      dofs[1] = 2;
      dofs[2] = 3;
      break;
    case 1:
      dofs[0] = 0;
      dofs[1] = 2;
      dofs[2] = 3;
      break;
    case 2:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 3;
      break;
    case 3:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 2;
      break;
    }
  }

  /// Tabulate the local-to-local mapping of dofs on entity (d, i)
  virtual void tabulate_entity_dofs(unsigned int* dofs,
                                    unsigned int d, unsigned int i) const
  {
    throw std::runtime_error("Not implemented (introduced in UFC v1.1).");
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    coordinates[0][0] = x[0][0];
    coordinates[0][1] = x[0][1];
    coordinates[0][2] = x[0][2];
    coordinates[1][0] = x[1][0];
    coordinates[1][1] = x[1][1];
    coordinates[1][2] = x[1][2];
    coordinates[2][0] = x[2][0];
    coordinates[2][1] = x[2][1];
    coordinates[2][2] = x[2][2];
    coordinates[3][0] = x[3][0];
    coordinates[3][1] = x[3][1];
    coordinates[3][2] = x[3][2];
  }

  /// Return the number of sub dof maps (for a mixed element)
  virtual unsigned int num_sub_dof_maps() const
  {
    return 1;
  }

  /// Create a new dof_map for sub dof map i (for a mixed element)
  virtual ufc::dof_map* create_sub_dof_map(unsigned int i) const
  {
    return new VectorLaplaceGradCurlLinearForm_dof_map_0_0();
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class VectorLaplaceGradCurlLinearForm_dof_map_0_1: public ufc::dof_map
{
private:

  unsigned int __global_dimension;

public:

  /// Constructor
  VectorLaplaceGradCurlLinearForm_dof_map_0_1() : ufc::dof_map()
  {
    __global_dimension = 0;
  }

  /// Destructor
  virtual ~VectorLaplaceGradCurlLinearForm_dof_map_0_1()
  {
    // Do nothing
  }

  /// Return a string identifying the dof map
  virtual const char* signature() const
  {
    return "FFC dof map for Nedelec finite element of degree 1 on a tetrahedron";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch ( d )
    {
    case 0:
      return false;
      break;
    case 1:
      return true;
      break;
    case 2:
      return false;
      break;
    case 3:
      return false;
      break;
    }
    return false;
  }

  /// Initialize dof map for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    __global_dimension = m.num_entities[1];
    return false;
  }

  /// Initialize dof map for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dof map for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return __global_dimension;
  }

  /// Return the dimension of the local finite element function space
  virtual unsigned int local_dimension() const
  {
    return 6;
  }

  // Return the geometric dimension of the coordinates this dof map provides
  virtual unsigned int geometric_dimension() const
  {
    throw std::runtime_error("Not implemented (introduced in UFC v1.1).");
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 3;
  }

  /// Return the number of dofs associated with each cell entity of dimension d
  virtual unsigned int num_entity_dofs(unsigned int d) const
  {
    throw std::runtime_error("Not implemented (introduced in UFC v1.1).");
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    dofs[0] = c.entity_indices[1][0];
    dofs[1] = c.entity_indices[1][1];
    dofs[2] = c.entity_indices[1][2];
    dofs[3] = c.entity_indices[1][3];
    dofs[4] = c.entity_indices[1][4];
    dofs[5] = c.entity_indices[1][5];
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch ( facet )
    {
    case 0:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 2;
      break;
    case 1:
      dofs[0] = 0;
      dofs[1] = 3;
      dofs[2] = 4;
      break;
    case 2:
      dofs[0] = 1;
      dofs[1] = 3;
      dofs[2] = 5;
      break;
    case 3:
      dofs[0] = 2;
      dofs[1] = 4;
      dofs[2] = 5;
      break;
    }
  }

  /// Tabulate the local-to-local mapping of dofs on entity (d, i)
  virtual void tabulate_entity_dofs(unsigned int* dofs,
                                    unsigned int d, unsigned int i) const
  {
    throw std::runtime_error("Not implemented (introduced in UFC v1.1).");
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    coordinates[0][0] = 0.5*x[2][0] + 0.5*x[3][0];
    coordinates[0][1] = 0.5*x[2][1] + 0.5*x[3][1];
    coordinates[0][2] = 0.5*x[2][2] + 0.5*x[3][2];
    coordinates[1][0] = 0.5*x[1][0] + 0.5*x[3][0];
    coordinates[1][1] = 0.5*x[1][1] + 0.5*x[3][1];
    coordinates[1][2] = 0.5*x[1][2] + 0.5*x[3][2];
    coordinates[2][0] = 0.5*x[1][0] + 0.5*x[2][0];
    coordinates[2][1] = 0.5*x[1][1] + 0.5*x[2][1];
    coordinates[2][2] = 0.5*x[1][2] + 0.5*x[2][2];
    coordinates[3][0] = 0.5*x[0][0] + 0.5*x[3][0];
    coordinates[3][1] = 0.5*x[0][1] + 0.5*x[3][1];
    coordinates[3][2] = 0.5*x[0][2] + 0.5*x[3][2];
    coordinates[4][0] = 0.5*x[0][0] + 0.5*x[2][0];
    coordinates[4][1] = 0.5*x[0][1] + 0.5*x[2][1];
    coordinates[4][2] = 0.5*x[0][2] + 0.5*x[2][2];
    coordinates[5][0] = 0.5*x[0][0] + 0.5*x[1][0];
    coordinates[5][1] = 0.5*x[0][1] + 0.5*x[1][1];
    coordinates[5][2] = 0.5*x[0][2] + 0.5*x[1][2];
  }

  /// Return the number of sub dof maps (for a mixed element)
  virtual unsigned int num_sub_dof_maps() const
  {
    return 1;
  }

  /// Create a new dof_map for sub dof map i (for a mixed element)
  virtual ufc::dof_map* create_sub_dof_map(unsigned int i) const
  {
    return new VectorLaplaceGradCurlLinearForm_dof_map_0_1();
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class VectorLaplaceGradCurlLinearForm_dof_map_0: public ufc::dof_map
{
private:

  unsigned int __global_dimension;

public:

  /// Constructor
  VectorLaplaceGradCurlLinearForm_dof_map_0() : ufc::dof_map()
  {
    __global_dimension = 0;
  }

  /// Destructor
  virtual ~VectorLaplaceGradCurlLinearForm_dof_map_0()
  {
    // Do nothing
  }

  /// Return a string identifying the dof map
  virtual const char* signature() const
  {
    return "FFC dof map for Mixed finite element: [Lagrange finite element of degree 1 on a tetrahedron, Nedelec finite element of degree 1 on a tetrahedron]";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch ( d )
    {
    case 0:
      return true;
      break;
    case 1:
      return true;
      break;
    case 2:
      return false;
      break;
    case 3:
      return false;
      break;
    }
    return false;
  }

  /// Initialize dof map for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    __global_dimension = m.num_entities[0] + m.num_entities[1];
    return false;
  }

  /// Initialize dof map for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dof map for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return __global_dimension;
  }

  /// Return the dimension of the local finite element function space
  virtual unsigned int local_dimension() const
  {
    return 10;
  }

  // Return the geometric dimension of the coordinates this dof map provides
  virtual unsigned int geometric_dimension() const
  {
    throw std::runtime_error("Not implemented (introduced in UFC v1.1).");
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 6;
  }

  /// Return the number of dofs associated with each cell entity of dimension d
  virtual unsigned int num_entity_dofs(unsigned int d) const
  {
    throw std::runtime_error("Not implemented (introduced in UFC v1.1).");
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    dofs[0] = c.entity_indices[0][0];
    dofs[1] = c.entity_indices[0][1];
    dofs[2] = c.entity_indices[0][2];
    dofs[3] = c.entity_indices[0][3];
    unsigned int offset = m.num_entities[0];
    dofs[4] = offset + c.entity_indices[1][0];
    dofs[5] = offset + c.entity_indices[1][1];
    dofs[6] = offset + c.entity_indices[1][2];
    dofs[7] = offset + c.entity_indices[1][3];
    dofs[8] = offset + c.entity_indices[1][4];
    dofs[9] = offset + c.entity_indices[1][5];
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch ( facet )
    {
    case 0:
      dofs[0] = 1;
      dofs[1] = 2;
      dofs[2] = 3;
      dofs[3] = 4;
      dofs[4] = 5;
      dofs[5] = 6;
      break;
    case 1:
      dofs[0] = 0;
      dofs[1] = 2;
      dofs[2] = 3;
      dofs[3] = 4;
      dofs[4] = 7;
      dofs[5] = 8;
      break;
    case 2:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 3;
      dofs[3] = 5;
      dofs[4] = 7;
      dofs[5] = 9;
      break;
    case 3:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 2;
      dofs[3] = 6;
      dofs[4] = 8;
      dofs[5] = 9;
      break;
    }
  }

  /// Tabulate the local-to-local mapping of dofs on entity (d, i)
  virtual void tabulate_entity_dofs(unsigned int* dofs,
                                    unsigned int d, unsigned int i) const
  {
    throw std::runtime_error("Not implemented (introduced in UFC v1.1).");
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    coordinates[0][0] = x[0][0];
    coordinates[0][1] = x[0][1];
    coordinates[0][2] = x[0][2];
    coordinates[1][0] = x[1][0];
    coordinates[1][1] = x[1][1];
    coordinates[1][2] = x[1][2];
    coordinates[2][0] = x[2][0];
    coordinates[2][1] = x[2][1];
    coordinates[2][2] = x[2][2];
    coordinates[3][0] = x[3][0];
    coordinates[3][1] = x[3][1];
    coordinates[3][2] = x[3][2];
    coordinates[4][0] = 0.5*x[2][0] + 0.5*x[3][0];
    coordinates[4][1] = 0.5*x[2][1] + 0.5*x[3][1];
    coordinates[4][2] = 0.5*x[2][2] + 0.5*x[3][2];
    coordinates[5][0] = 0.5*x[1][0] + 0.5*x[3][0];
    coordinates[5][1] = 0.5*x[1][1] + 0.5*x[3][1];
    coordinates[5][2] = 0.5*x[1][2] + 0.5*x[3][2];
    coordinates[6][0] = 0.5*x[1][0] + 0.5*x[2][0];
    coordinates[6][1] = 0.5*x[1][1] + 0.5*x[2][1];
    coordinates[6][2] = 0.5*x[1][2] + 0.5*x[2][2];
    coordinates[7][0] = 0.5*x[0][0] + 0.5*x[3][0];
    coordinates[7][1] = 0.5*x[0][1] + 0.5*x[3][1];
    coordinates[7][2] = 0.5*x[0][2] + 0.5*x[3][2];
    coordinates[8][0] = 0.5*x[0][0] + 0.5*x[2][0];
    coordinates[8][1] = 0.5*x[0][1] + 0.5*x[2][1];
    coordinates[8][2] = 0.5*x[0][2] + 0.5*x[2][2];
    coordinates[9][0] = 0.5*x[0][0] + 0.5*x[1][0];
    coordinates[9][1] = 0.5*x[0][1] + 0.5*x[1][1];
    coordinates[9][2] = 0.5*x[0][2] + 0.5*x[1][2];
  }

  /// Return the number of sub dof maps (for a mixed element)
  virtual unsigned int num_sub_dof_maps() const
  {
    return 2;
  }

  /// Create a new dof_map for sub dof map i (for a mixed element)
  virtual ufc::dof_map* create_sub_dof_map(unsigned int i) const
  {
    switch ( i )
    {
    case 0:
      return new VectorLaplaceGradCurlLinearForm_dof_map_0_0();
      break;
    case 1:
      return new VectorLaplaceGradCurlLinearForm_dof_map_0_1();
      break;
    }
    return 0;
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class VectorLaplaceGradCurlLinearForm_dof_map_1_0: public ufc::dof_map
{
private:

  unsigned int __global_dimension;

public:

  /// Constructor
  VectorLaplaceGradCurlLinearForm_dof_map_1_0() : ufc::dof_map()
  {
    __global_dimension = 0;
  }

  /// Destructor
  virtual ~VectorLaplaceGradCurlLinearForm_dof_map_1_0()
  {
    // Do nothing
  }

  /// Return a string identifying the dof map
  virtual const char* signature() const
  {
    return "FFC dof map for Lagrange finite element of degree 2 on a tetrahedron";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch ( d )
    {
    case 0:
      return true;
      break;
    case 1:
      return true;
      break;
    case 2:
      return false;
      break;
    case 3:
      return false;
      break;
    }
    return false;
  }

  /// Initialize dof map for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    __global_dimension = m.num_entities[0] + m.num_entities[1];
    return false;
  }

  /// Initialize dof map for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dof map for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return __global_dimension;
  }

  /// Return the dimension of the local finite element function space
  virtual unsigned int local_dimension() const
  {
    return 10;
  }

  // Return the geometric dimension of the coordinates this dof map provides
  virtual unsigned int geometric_dimension() const
  {
    throw std::runtime_error("Not implemented (introduced in UFC v1.1).");
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 6;
  }

  /// Return the number of dofs associated with each cell entity of dimension d
  virtual unsigned int num_entity_dofs(unsigned int d) const
  {
    throw std::runtime_error("Not implemented (introduced in UFC v1.1).");
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    dofs[0] = c.entity_indices[0][0];
    dofs[1] = c.entity_indices[0][1];
    dofs[2] = c.entity_indices[0][2];
    dofs[3] = c.entity_indices[0][3];
    unsigned int offset = m.num_entities[0];
    dofs[4] = offset + c.entity_indices[1][0];
    dofs[5] = offset + c.entity_indices[1][1];
    dofs[6] = offset + c.entity_indices[1][2];
    dofs[7] = offset + c.entity_indices[1][3];
    dofs[8] = offset + c.entity_indices[1][4];
    dofs[9] = offset + c.entity_indices[1][5];
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch ( facet )
    {
    case 0:
      dofs[0] = 1;
      dofs[1] = 2;
      dofs[2] = 3;
      dofs[3] = 4;
      dofs[4] = 5;
      dofs[5] = 6;
      break;
    case 1:
      dofs[0] = 0;
      dofs[1] = 2;
      dofs[2] = 3;
      dofs[3] = 4;
      dofs[4] = 7;
      dofs[5] = 8;
      break;
    case 2:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 3;
      dofs[3] = 5;
      dofs[4] = 7;
      dofs[5] = 9;
      break;
    case 3:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 2;
      dofs[3] = 6;
      dofs[4] = 8;
      dofs[5] = 9;
      break;
    }
  }

  /// Tabulate the local-to-local mapping of dofs on entity (d, i)
  virtual void tabulate_entity_dofs(unsigned int* dofs,
                                    unsigned int d, unsigned int i) const
  {
    throw std::runtime_error("Not implemented (introduced in UFC v1.1).");
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    coordinates[0][0] = x[0][0];
    coordinates[0][1] = x[0][1];
    coordinates[0][2] = x[0][2];
    coordinates[1][0] = x[1][0];
    coordinates[1][1] = x[1][1];
    coordinates[1][2] = x[1][2];
    coordinates[2][0] = x[2][0];
    coordinates[2][1] = x[2][1];
    coordinates[2][2] = x[2][2];
    coordinates[3][0] = x[3][0];
    coordinates[3][1] = x[3][1];
    coordinates[3][2] = x[3][2];
    coordinates[4][0] = 0.5*x[2][0] + 0.5*x[3][0];
    coordinates[4][1] = 0.5*x[2][1] + 0.5*x[3][1];
    coordinates[4][2] = 0.5*x[2][2] + 0.5*x[3][2];
    coordinates[5][0] = 0.5*x[1][0] + 0.5*x[3][0];
    coordinates[5][1] = 0.5*x[1][1] + 0.5*x[3][1];
    coordinates[5][2] = 0.5*x[1][2] + 0.5*x[3][2];
    coordinates[6][0] = 0.5*x[1][0] + 0.5*x[2][0];
    coordinates[6][1] = 0.5*x[1][1] + 0.5*x[2][1];
    coordinates[6][2] = 0.5*x[1][2] + 0.5*x[2][2];
    coordinates[7][0] = 0.5*x[0][0] + 0.5*x[3][0];
    coordinates[7][1] = 0.5*x[0][1] + 0.5*x[3][1];
    coordinates[7][2] = 0.5*x[0][2] + 0.5*x[3][2];
    coordinates[8][0] = 0.5*x[0][0] + 0.5*x[2][0];
    coordinates[8][1] = 0.5*x[0][1] + 0.5*x[2][1];
    coordinates[8][2] = 0.5*x[0][2] + 0.5*x[2][2];
    coordinates[9][0] = 0.5*x[0][0] + 0.5*x[1][0];
    coordinates[9][1] = 0.5*x[0][1] + 0.5*x[1][1];
    coordinates[9][2] = 0.5*x[0][2] + 0.5*x[1][2];
  }

  /// Return the number of sub dof maps (for a mixed element)
  virtual unsigned int num_sub_dof_maps() const
  {
    return 1;
  }

  /// Create a new dof_map for sub dof map i (for a mixed element)
  virtual ufc::dof_map* create_sub_dof_map(unsigned int i) const
  {
    return new VectorLaplaceGradCurlLinearForm_dof_map_1_0();
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class VectorLaplaceGradCurlLinearForm_dof_map_1_1: public ufc::dof_map
{
private:

  unsigned int __global_dimension;

public:

  /// Constructor
  VectorLaplaceGradCurlLinearForm_dof_map_1_1() : ufc::dof_map()
  {
    __global_dimension = 0;
  }

  /// Destructor
  virtual ~VectorLaplaceGradCurlLinearForm_dof_map_1_1()
  {
    // Do nothing
  }

  /// Return a string identifying the dof map
  virtual const char* signature() const
  {
    return "FFC dof map for Lagrange finite element of degree 2 on a tetrahedron";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch ( d )
    {
    case 0:
      return true;
      break;
    case 1:
      return true;
      break;
    case 2:
      return false;
      break;
    case 3:
      return false;
      break;
    }
    return false;
  }

  /// Initialize dof map for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    __global_dimension = m.num_entities[0] + m.num_entities[1];
    return false;
  }

  /// Initialize dof map for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dof map for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return __global_dimension;
  }

  /// Return the dimension of the local finite element function space
  virtual unsigned int local_dimension() const
  {
    return 10;
  }

  // Return the geometric dimension of the coordinates this dof map provides
  virtual unsigned int geometric_dimension() const
  {
    throw std::runtime_error("Not implemented (introduced in UFC v1.1).");
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 6;
  }

  /// Return the number of dofs associated with each cell entity of dimension d
  virtual unsigned int num_entity_dofs(unsigned int d) const
  {
    throw std::runtime_error("Not implemented (introduced in UFC v1.1).");
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    dofs[0] = c.entity_indices[0][0];
    dofs[1] = c.entity_indices[0][1];
    dofs[2] = c.entity_indices[0][2];
    dofs[3] = c.entity_indices[0][3];
    unsigned int offset = m.num_entities[0];
    dofs[4] = offset + c.entity_indices[1][0];
    dofs[5] = offset + c.entity_indices[1][1];
    dofs[6] = offset + c.entity_indices[1][2];
    dofs[7] = offset + c.entity_indices[1][3];
    dofs[8] = offset + c.entity_indices[1][4];
    dofs[9] = offset + c.entity_indices[1][5];
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch ( facet )
    {
    case 0:
      dofs[0] = 1;
      dofs[1] = 2;
      dofs[2] = 3;
      dofs[3] = 4;
      dofs[4] = 5;
      dofs[5] = 6;
      break;
    case 1:
      dofs[0] = 0;
      dofs[1] = 2;
      dofs[2] = 3;
      dofs[3] = 4;
      dofs[4] = 7;
      dofs[5] = 8;
      break;
    case 2:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 3;
      dofs[3] = 5;
      dofs[4] = 7;
      dofs[5] = 9;
      break;
    case 3:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 2;
      dofs[3] = 6;
      dofs[4] = 8;
      dofs[5] = 9;
      break;
    }
  }

  /// Tabulate the local-to-local mapping of dofs on entity (d, i)
  virtual void tabulate_entity_dofs(unsigned int* dofs,
                                    unsigned int d, unsigned int i) const
  {
    throw std::runtime_error("Not implemented (introduced in UFC v1.1).");
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    coordinates[0][0] = x[0][0];
    coordinates[0][1] = x[0][1];
    coordinates[0][2] = x[0][2];
    coordinates[1][0] = x[1][0];
    coordinates[1][1] = x[1][1];
    coordinates[1][2] = x[1][2];
    coordinates[2][0] = x[2][0];
    coordinates[2][1] = x[2][1];
    coordinates[2][2] = x[2][2];
    coordinates[3][0] = x[3][0];
    coordinates[3][1] = x[3][1];
    coordinates[3][2] = x[3][2];
    coordinates[4][0] = 0.5*x[2][0] + 0.5*x[3][0];
    coordinates[4][1] = 0.5*x[2][1] + 0.5*x[3][1];
    coordinates[4][2] = 0.5*x[2][2] + 0.5*x[3][2];
    coordinates[5][0] = 0.5*x[1][0] + 0.5*x[3][0];
    coordinates[5][1] = 0.5*x[1][1] + 0.5*x[3][1];
    coordinates[5][2] = 0.5*x[1][2] + 0.5*x[3][2];
    coordinates[6][0] = 0.5*x[1][0] + 0.5*x[2][0];
    coordinates[6][1] = 0.5*x[1][1] + 0.5*x[2][1];
    coordinates[6][2] = 0.5*x[1][2] + 0.5*x[2][2];
    coordinates[7][0] = 0.5*x[0][0] + 0.5*x[3][0];
    coordinates[7][1] = 0.5*x[0][1] + 0.5*x[3][1];
    coordinates[7][2] = 0.5*x[0][2] + 0.5*x[3][2];
    coordinates[8][0] = 0.5*x[0][0] + 0.5*x[2][0];
    coordinates[8][1] = 0.5*x[0][1] + 0.5*x[2][1];
    coordinates[8][2] = 0.5*x[0][2] + 0.5*x[2][2];
    coordinates[9][0] = 0.5*x[0][0] + 0.5*x[1][0];
    coordinates[9][1] = 0.5*x[0][1] + 0.5*x[1][1];
    coordinates[9][2] = 0.5*x[0][2] + 0.5*x[1][2];
  }

  /// Return the number of sub dof maps (for a mixed element)
  virtual unsigned int num_sub_dof_maps() const
  {
    return 1;
  }

  /// Create a new dof_map for sub dof map i (for a mixed element)
  virtual ufc::dof_map* create_sub_dof_map(unsigned int i) const
  {
    return new VectorLaplaceGradCurlLinearForm_dof_map_1_1();
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class VectorLaplaceGradCurlLinearForm_dof_map_1_2: public ufc::dof_map
{
private:

  unsigned int __global_dimension;

public:

  /// Constructor
  VectorLaplaceGradCurlLinearForm_dof_map_1_2() : ufc::dof_map()
  {
    __global_dimension = 0;
  }

  /// Destructor
  virtual ~VectorLaplaceGradCurlLinearForm_dof_map_1_2()
  {
    // Do nothing
  }

  /// Return a string identifying the dof map
  virtual const char* signature() const
  {
    return "FFC dof map for Lagrange finite element of degree 2 on a tetrahedron";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch ( d )
    {
    case 0:
      return true;
      break;
    case 1:
      return true;
      break;
    case 2:
      return false;
      break;
    case 3:
      return false;
      break;
    }
    return false;
  }

  /// Initialize dof map for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    __global_dimension = m.num_entities[0] + m.num_entities[1];
    return false;
  }

  /// Initialize dof map for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dof map for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return __global_dimension;
  }

  /// Return the dimension of the local finite element function space
  virtual unsigned int local_dimension() const
  {
    return 10;
  }

  // Return the geometric dimension of the coordinates this dof map provides
  virtual unsigned int geometric_dimension() const
  {
    throw std::runtime_error("Not implemented (introduced in UFC v1.1).");
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 6;
  }

  /// Return the number of dofs associated with each cell entity of dimension d
  virtual unsigned int num_entity_dofs(unsigned int d) const
  {
    throw std::runtime_error("Not implemented (introduced in UFC v1.1).");
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    dofs[0] = c.entity_indices[0][0];
    dofs[1] = c.entity_indices[0][1];
    dofs[2] = c.entity_indices[0][2];
    dofs[3] = c.entity_indices[0][3];
    unsigned int offset = m.num_entities[0];
    dofs[4] = offset + c.entity_indices[1][0];
    dofs[5] = offset + c.entity_indices[1][1];
    dofs[6] = offset + c.entity_indices[1][2];
    dofs[7] = offset + c.entity_indices[1][3];
    dofs[8] = offset + c.entity_indices[1][4];
    dofs[9] = offset + c.entity_indices[1][5];
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch ( facet )
    {
    case 0:
      dofs[0] = 1;
      dofs[1] = 2;
      dofs[2] = 3;
      dofs[3] = 4;
      dofs[4] = 5;
      dofs[5] = 6;
      break;
    case 1:
      dofs[0] = 0;
      dofs[1] = 2;
      dofs[2] = 3;
      dofs[3] = 4;
      dofs[4] = 7;
      dofs[5] = 8;
      break;
    case 2:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 3;
      dofs[3] = 5;
      dofs[4] = 7;
      dofs[5] = 9;
      break;
    case 3:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 2;
      dofs[3] = 6;
      dofs[4] = 8;
      dofs[5] = 9;
      break;
    }
  }

  /// Tabulate the local-to-local mapping of dofs on entity (d, i)
  virtual void tabulate_entity_dofs(unsigned int* dofs,
                                    unsigned int d, unsigned int i) const
  {
    throw std::runtime_error("Not implemented (introduced in UFC v1.1).");
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    coordinates[0][0] = x[0][0];
    coordinates[0][1] = x[0][1];
    coordinates[0][2] = x[0][2];
    coordinates[1][0] = x[1][0];
    coordinates[1][1] = x[1][1];
    coordinates[1][2] = x[1][2];
    coordinates[2][0] = x[2][0];
    coordinates[2][1] = x[2][1];
    coordinates[2][2] = x[2][2];
    coordinates[3][0] = x[3][0];
    coordinates[3][1] = x[3][1];
    coordinates[3][2] = x[3][2];
    coordinates[4][0] = 0.5*x[2][0] + 0.5*x[3][0];
    coordinates[4][1] = 0.5*x[2][1] + 0.5*x[3][1];
    coordinates[4][2] = 0.5*x[2][2] + 0.5*x[3][2];
    coordinates[5][0] = 0.5*x[1][0] + 0.5*x[3][0];
    coordinates[5][1] = 0.5*x[1][1] + 0.5*x[3][1];
    coordinates[5][2] = 0.5*x[1][2] + 0.5*x[3][2];
    coordinates[6][0] = 0.5*x[1][0] + 0.5*x[2][0];
    coordinates[6][1] = 0.5*x[1][1] + 0.5*x[2][1];
    coordinates[6][2] = 0.5*x[1][2] + 0.5*x[2][2];
    coordinates[7][0] = 0.5*x[0][0] + 0.5*x[3][0];
    coordinates[7][1] = 0.5*x[0][1] + 0.5*x[3][1];
    coordinates[7][2] = 0.5*x[0][2] + 0.5*x[3][2];
    coordinates[8][0] = 0.5*x[0][0] + 0.5*x[2][0];
    coordinates[8][1] = 0.5*x[0][1] + 0.5*x[2][1];
    coordinates[8][2] = 0.5*x[0][2] + 0.5*x[2][2];
    coordinates[9][0] = 0.5*x[0][0] + 0.5*x[1][0];
    coordinates[9][1] = 0.5*x[0][1] + 0.5*x[1][1];
    coordinates[9][2] = 0.5*x[0][2] + 0.5*x[1][2];
  }

  /// Return the number of sub dof maps (for a mixed element)
  virtual unsigned int num_sub_dof_maps() const
  {
    return 1;
  }

  /// Create a new dof_map for sub dof map i (for a mixed element)
  virtual ufc::dof_map* create_sub_dof_map(unsigned int i) const
  {
    return new VectorLaplaceGradCurlLinearForm_dof_map_1_2();
  }

};

/// This class defines the interface for a local-to-global mapping of
/// degrees of freedom (dofs).

class VectorLaplaceGradCurlLinearForm_dof_map_1: public ufc::dof_map
{
private:

  unsigned int __global_dimension;

public:

  /// Constructor
  VectorLaplaceGradCurlLinearForm_dof_map_1() : ufc::dof_map()
  {
    __global_dimension = 0;
  }

  /// Destructor
  virtual ~VectorLaplaceGradCurlLinearForm_dof_map_1()
  {
    // Do nothing
  }

  /// Return a string identifying the dof map
  virtual const char* signature() const
  {
    return "FFC dof map for Mixed finite element: [Lagrange finite element of degree 2 on a tetrahedron, Lagrange finite element of degree 2 on a tetrahedron, Lagrange finite element of degree 2 on a tetrahedron]";
  }

  /// Return true iff mesh entities of topological dimension d are needed
  virtual bool needs_mesh_entities(unsigned int d) const
  {
    switch ( d )
    {
    case 0:
      return true;
      break;
    case 1:
      return true;
      break;
    case 2:
      return false;
      break;
    case 3:
      return false;
      break;
    }
    return false;
  }

  /// Initialize dof map for mesh (return true iff init_cell() is needed)
  virtual bool init_mesh(const ufc::mesh& m)
  {
    __global_dimension = 3*m.num_entities[0] + 3*m.num_entities[1];
    return false;
  }

  /// Initialize dof map for given cell
  virtual void init_cell(const ufc::mesh& m,
                         const ufc::cell& c)
  {
    // Do nothing
  }

  /// Finish initialization of dof map for cells
  virtual void init_cell_finalize()
  {
    // Do nothing
  }

  /// Return the dimension of the global finite element function space
  virtual unsigned int global_dimension() const
  {
    return __global_dimension;
  }

  /// Return the dimension of the local finite element function space
  virtual unsigned int local_dimension() const
  {
    return 30;
  }

  // Return the geometric dimension of the coordinates this dof map provides
  virtual unsigned int geometric_dimension() const
  {
    throw std::runtime_error("Not implemented (introduced in UFC v1.1).");
  }

  /// Return the number of dofs on each cell facet
  virtual unsigned int num_facet_dofs() const
  {
    return 18;
  }

  /// Return the number of dofs associated with each cell entity of dimension d
  virtual unsigned int num_entity_dofs(unsigned int d) const
  {
    throw std::runtime_error("Not implemented (introduced in UFC v1.1).");
  }

  /// Tabulate the local-to-global mapping of dofs on a cell
  virtual void tabulate_dofs(unsigned int* dofs,
                             const ufc::mesh& m,
                             const ufc::cell& c) const
  {
    dofs[0] = c.entity_indices[0][0];
    dofs[1] = c.entity_indices[0][1];
    dofs[2] = c.entity_indices[0][2];
    dofs[3] = c.entity_indices[0][3];
    unsigned int offset = m.num_entities[0];
    dofs[4] = offset + c.entity_indices[1][0];
    dofs[5] = offset + c.entity_indices[1][1];
    dofs[6] = offset + c.entity_indices[1][2];
    dofs[7] = offset + c.entity_indices[1][3];
    dofs[8] = offset + c.entity_indices[1][4];
    dofs[9] = offset + c.entity_indices[1][5];
    offset = offset + m.num_entities[1];
    dofs[10] = offset + c.entity_indices[0][0];
    dofs[11] = offset + c.entity_indices[0][1];
    dofs[12] = offset + c.entity_indices[0][2];
    dofs[13] = offset + c.entity_indices[0][3];
    offset = offset + m.num_entities[0];
    dofs[14] = offset + c.entity_indices[1][0];
    dofs[15] = offset + c.entity_indices[1][1];
    dofs[16] = offset + c.entity_indices[1][2];
    dofs[17] = offset + c.entity_indices[1][3];
    dofs[18] = offset + c.entity_indices[1][4];
    dofs[19] = offset + c.entity_indices[1][5];
    offset = offset + m.num_entities[1];
    dofs[20] = offset + c.entity_indices[0][0];
    dofs[21] = offset + c.entity_indices[0][1];
    dofs[22] = offset + c.entity_indices[0][2];
    dofs[23] = offset + c.entity_indices[0][3];
    offset = offset + m.num_entities[0];
    dofs[24] = offset + c.entity_indices[1][0];
    dofs[25] = offset + c.entity_indices[1][1];
    dofs[26] = offset + c.entity_indices[1][2];
    dofs[27] = offset + c.entity_indices[1][3];
    dofs[28] = offset + c.entity_indices[1][4];
    dofs[29] = offset + c.entity_indices[1][5];
  }

  /// Tabulate the local-to-local mapping from facet dofs to cell dofs
  virtual void tabulate_facet_dofs(unsigned int* dofs,
                                   unsigned int facet) const
  {
    switch ( facet )
    {
    case 0:
      dofs[0] = 1;
      dofs[1] = 2;
      dofs[2] = 3;
      dofs[3] = 4;
      dofs[4] = 5;
      dofs[5] = 6;
      dofs[6] = 11;
      dofs[7] = 12;
      dofs[8] = 13;
      dofs[9] = 14;
      dofs[10] = 15;
      dofs[11] = 16;
      dofs[12] = 21;
      dofs[13] = 22;
      dofs[14] = 23;
      dofs[15] = 24;
      dofs[16] = 25;
      dofs[17] = 26;
      break;
    case 1:
      dofs[0] = 0;
      dofs[1] = 2;
      dofs[2] = 3;
      dofs[3] = 4;
      dofs[4] = 7;
      dofs[5] = 8;
      dofs[6] = 10;
      dofs[7] = 12;
      dofs[8] = 13;
      dofs[9] = 14;
      dofs[10] = 17;
      dofs[11] = 18;
      dofs[12] = 20;
      dofs[13] = 22;
      dofs[14] = 23;
      dofs[15] = 24;
      dofs[16] = 27;
      dofs[17] = 28;
      break;
    case 2:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 3;
      dofs[3] = 5;
      dofs[4] = 7;
      dofs[5] = 9;
      dofs[6] = 10;
      dofs[7] = 11;
      dofs[8] = 13;
      dofs[9] = 15;
      dofs[10] = 17;
      dofs[11] = 19;
      dofs[12] = 20;
      dofs[13] = 21;
      dofs[14] = 23;
      dofs[15] = 25;
      dofs[16] = 27;
      dofs[17] = 29;
      break;
    case 3:
      dofs[0] = 0;
      dofs[1] = 1;
      dofs[2] = 2;
      dofs[3] = 6;
      dofs[4] = 8;
      dofs[5] = 9;
      dofs[6] = 10;
      dofs[7] = 11;
      dofs[8] = 12;
      dofs[9] = 16;
      dofs[10] = 18;
      dofs[11] = 19;
      dofs[12] = 20;
      dofs[13] = 21;
      dofs[14] = 22;
      dofs[15] = 26;
      dofs[16] = 28;
      dofs[17] = 29;
      break;
    }
  }

  /// Tabulate the local-to-local mapping of dofs on entity (d, i)
  virtual void tabulate_entity_dofs(unsigned int* dofs,
                                    unsigned int d, unsigned int i) const
  {
    throw std::runtime_error("Not implemented (introduced in UFC v1.1).");
  }

  /// Tabulate the coordinates of all dofs on a cell
  virtual void tabulate_coordinates(double** coordinates,
                                    const ufc::cell& c) const
  {
    const double * const * x = c.coordinates;
    coordinates[0][0] = x[0][0];
    coordinates[0][1] = x[0][1];
    coordinates[0][2] = x[0][2];
    coordinates[1][0] = x[1][0];
    coordinates[1][1] = x[1][1];
    coordinates[1][2] = x[1][2];
    coordinates[2][0] = x[2][0];
    coordinates[2][1] = x[2][1];
    coordinates[2][2] = x[2][2];
    coordinates[3][0] = x[3][0];
    coordinates[3][1] = x[3][1];
    coordinates[3][2] = x[3][2];
    coordinates[4][0] = 0.5*x[2][0] + 0.5*x[3][0];
    coordinates[4][1] = 0.5*x[2][1] + 0.5*x[3][1];
    coordinates[4][2] = 0.5*x[2][2] + 0.5*x[3][2];
    coordinates[5][0] = 0.5*x[1][0] + 0.5*x[3][0];
    coordinates[5][1] = 0.5*x[1][1] + 0.5*x[3][1];
    coordinates[5][2] = 0.5*x[1][2] + 0.5*x[3][2];
    coordinates[6][0] = 0.5*x[1][0] + 0.5*x[2][0];
    coordinates[6][1] = 0.5*x[1][1] + 0.5*x[2][1];
    coordinates[6][2] = 0.5*x[1][2] + 0.5*x[2][2];
    coordinates[7][0] = 0.5*x[0][0] + 0.5*x[3][0];
    coordinates[7][1] = 0.5*x[0][1] + 0.5*x[3][1];
    coordinates[7][2] = 0.5*x[0][2] + 0.5*x[3][2];
    coordinates[8][0] = 0.5*x[0][0] + 0.5*x[2][0];
    coordinates[8][1] = 0.5*x[0][1] + 0.5*x[2][1];
    coordinates[8][2] = 0.5*x[0][2] + 0.5*x[2][2];
    coordinates[9][0] = 0.5*x[0][0] + 0.5*x[1][0];
    coordinates[9][1] = 0.5*x[0][1] + 0.5*x[1][1];
    coordinates[9][2] = 0.5*x[0][2] + 0.5*x[1][2];
    coordinates[10][0] = x[0][0];
    coordinates[10][1] = x[0][1];
    coordinates[10][2] = x[0][2];
    coordinates[11][0] = x[1][0];
    coordinates[11][1] = x[1][1];
    coordinates[11][2] = x[1][2];
    coordinates[12][0] = x[2][0];
    coordinates[12][1] = x[2][1];
    coordinates[12][2] = x[2][2];
    coordinates[13][0] = x[3][0];
    coordinates[13][1] = x[3][1];
    coordinates[13][2] = x[3][2];
    coordinates[14][0] = 0.5*x[2][0] + 0.5*x[3][0];
    coordinates[14][1] = 0.5*x[2][1] + 0.5*x[3][1];
    coordinates[14][2] = 0.5*x[2][2] + 0.5*x[3][2];
    coordinates[15][0] = 0.5*x[1][0] + 0.5*x[3][0];
    coordinates[15][1] = 0.5*x[1][1] + 0.5*x[3][1];
    coordinates[15][2] = 0.5*x[1][2] + 0.5*x[3][2];
    coordinates[16][0] = 0.5*x[1][0] + 0.5*x[2][0];
    coordinates[16][1] = 0.5*x[1][1] + 0.5*x[2][1];
    coordinates[16][2] = 0.5*x[1][2] + 0.5*x[2][2];
    coordinates[17][0] = 0.5*x[0][0] + 0.5*x[3][0];
    coordinates[17][1] = 0.5*x[0][1] + 0.5*x[3][1];
    coordinates[17][2] = 0.5*x[0][2] + 0.5*x[3][2];
    coordinates[18][0] = 0.5*x[0][0] + 0.5*x[2][0];
    coordinates[18][1] = 0.5*x[0][1] + 0.5*x[2][1];
    coordinates[18][2] = 0.5*x[0][2] + 0.5*x[2][2];
    coordinates[19][0] = 0.5*x[0][0] + 0.5*x[1][0];
    coordinates[19][1] = 0.5*x[0][1] + 0.5*x[1][1];
    coordinates[19][2] = 0.5*x[0][2] + 0.5*x[1][2];
    coordinates[20][0] = x[0][0];
    coordinates[20][1] = x[0][1];
    coordinates[20][2] = x[0][2];
    coordinates[21][0] = x[1][0];
    coordinates[21][1] = x[1][1];
    coordinates[21][2] = x[1][2];
    coordinates[22][0] = x[2][0];
    coordinates[22][1] = x[2][1];
    coordinates[22][2] = x[2][2];
    coordinates[23][0] = x[3][0];
    coordinates[23][1] = x[3][1];
    coordinates[23][2] = x[3][2];
    coordinates[24][0] = 0.5*x[2][0] + 0.5*x[3][0];
    coordinates[24][1] = 0.5*x[2][1] + 0.5*x[3][1];
    coordinates[24][2] = 0.5*x[2][2] + 0.5*x[3][2];
    coordinates[25][0] = 0.5*x[1][0] + 0.5*x[3][0];
    coordinates[25][1] = 0.5*x[1][1] + 0.5*x[3][1];
    coordinates[25][2] = 0.5*x[1][2] + 0.5*x[3][2];
    coordinates[26][0] = 0.5*x[1][0] + 0.5*x[2][0];
    coordinates[26][1] = 0.5*x[1][1] + 0.5*x[2][1];
    coordinates[26][2] = 0.5*x[1][2] + 0.5*x[2][2];
    coordinates[27][0] = 0.5*x[0][0] + 0.5*x[3][0];
    coordinates[27][1] = 0.5*x[0][1] + 0.5*x[3][1];
    coordinates[27][2] = 0.5*x[0][2] + 0.5*x[3][2];
    coordinates[28][0] = 0.5*x[0][0] + 0.5*x[2][0];
    coordinates[28][1] = 0.5*x[0][1] + 0.5*x[2][1];
    coordinates[28][2] = 0.5*x[0][2] + 0.5*x[2][2];
    coordinates[29][0] = 0.5*x[0][0] + 0.5*x[1][0];
    coordinates[29][1] = 0.5*x[0][1] + 0.5*x[1][1];
    coordinates[29][2] = 0.5*x[0][2] + 0.5*x[1][2];
  }

  /// Return the number of sub dof maps (for a mixed element)
  virtual unsigned int num_sub_dof_maps() const
  {
    return 3;
  }

  /// Create a new dof_map for sub dof map i (for a mixed element)
  virtual ufc::dof_map* create_sub_dof_map(unsigned int i) const
  {
    switch ( i )
    {
    case 0:
      return new VectorLaplaceGradCurlLinearForm_dof_map_1_0();
      break;
    case 1:
      return new VectorLaplaceGradCurlLinearForm_dof_map_1_1();
      break;
    case 2:
      return new VectorLaplaceGradCurlLinearForm_dof_map_1_2();
      break;
    }
    return 0;
  }

};

/// This class defines the interface for the tabulation of the cell
/// tensor corresponding to the local contribution to a form from
/// the integral over a cell.

class VectorLaplaceGradCurlLinearForm_cell_integral_0: public ufc::cell_integral
{
public:

  /// Constructor
  VectorLaplaceGradCurlLinearForm_cell_integral_0() : ufc::cell_integral()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~VectorLaplaceGradCurlLinearForm_cell_integral_0()
  {
    // Do nothing
  }

  /// Tabulate the tensor for the contribution from a local cell
  virtual void tabulate_tensor(double* A,
                               const double * const * w,
                               const ufc::cell& c) const
  {
    // Extract vertex coordinates
    const double * const * x = c.coordinates;
    
    // Compute Jacobian of affine map from reference cell
    const double J_00 = x[1][0] - x[0][0];
    const double J_01 = x[2][0] - x[0][0];
    const double J_02 = x[3][0] - x[0][0];
    const double J_10 = x[1][1] - x[0][1];
    const double J_11 = x[2][1] - x[0][1];
    const double J_12 = x[3][1] - x[0][1];
    const double J_20 = x[1][2] - x[0][2];
    const double J_21 = x[2][2] - x[0][2];
    const double J_22 = x[3][2] - x[0][2];
      
    // Compute sub determinants
    const double d_00 = J_11*J_22 - J_12*J_21;
    const double d_01 = J_12*J_20 - J_10*J_22;
    const double d_02 = J_10*J_21 - J_11*J_20;
    
    const double d_10 = J_02*J_21 - J_01*J_22;
    const double d_11 = J_00*J_22 - J_02*J_20;
    const double d_12 = J_01*J_20 - J_00*J_21;
    
    const double d_20 = J_01*J_12 - J_02*J_11;
    const double d_21 = J_02*J_10 - J_00*J_12;
    const double d_22 = J_00*J_11 - J_01*J_10;
      
    // Compute determinant of Jacobian
    double detJ = J_00*d_00 + J_10*d_10 + J_20*d_20;
      
    // Compute inverse of Jacobian
    const double Jinv_00 = d_00 / detJ;
    const double Jinv_01 = d_10 / detJ;
    const double Jinv_02 = d_20 / detJ;
    const double Jinv_10 = d_01 / detJ;
    const double Jinv_11 = d_11 / detJ;
    const double Jinv_12 = d_21 / detJ;
    const double Jinv_20 = d_02 / detJ;
    const double Jinv_21 = d_12 / detJ;
    const double Jinv_22 = d_22 / detJ;
    
    // Set scale factor
    const double det = std::abs(detJ);
    
    // Compute coefficients
    const double c0_0_0_0 = w[0][0];
    const double c0_0_0_1 = w[0][1];
    const double c0_0_0_2 = w[0][2];
    const double c0_0_0_3 = w[0][3];
    const double c0_0_0_4 = w[0][4];
    const double c0_0_0_5 = w[0][5];
    const double c0_0_0_6 = w[0][6];
    const double c0_0_0_7 = w[0][7];
    const double c0_0_0_8 = w[0][8];
    const double c0_0_0_9 = w[0][9];
    const double c0_0_0_10 = w[0][10];
    const double c0_0_0_11 = w[0][11];
    const double c0_0_0_12 = w[0][12];
    const double c0_0_0_13 = w[0][13];
    const double c0_0_0_14 = w[0][14];
    const double c0_0_0_15 = w[0][15];
    const double c0_0_0_16 = w[0][16];
    const double c0_0_0_17 = w[0][17];
    const double c0_0_0_18 = w[0][18];
    const double c0_0_0_19 = w[0][19];
    const double c0_0_0_20 = w[0][20];
    const double c0_0_0_21 = w[0][21];
    const double c0_0_0_22 = w[0][22];
    const double c0_0_0_23 = w[0][23];
    const double c0_0_0_24 = w[0][24];
    const double c0_0_0_25 = w[0][25];
    const double c0_0_0_26 = w[0][26];
    const double c0_0_0_27 = w[0][27];
    const double c0_0_0_28 = w[0][28];
    const double c0_0_0_29 = w[0][29];
    
    // Compute geometry tensors
    const double G0_0_0_0 = det*c0_0_0_0*Jinv_00;
    const double G0_0_1_0 = det*c0_0_0_1*Jinv_00;
    const double G0_0_2_0 = det*c0_0_0_2*Jinv_00;
    const double G0_0_3_0 = det*c0_0_0_3*Jinv_00;
    const double G0_0_4_0 = det*c0_0_0_4*Jinv_00;
    const double G0_0_5_0 = det*c0_0_0_5*Jinv_00;
    const double G0_0_6_0 = det*c0_0_0_6*Jinv_00;
    const double G0_0_7_0 = det*c0_0_0_7*Jinv_00;
    const double G0_0_8_0 = det*c0_0_0_8*Jinv_00;
    const double G0_0_9_0 = det*c0_0_0_9*Jinv_00;
    const double G0_0_10_1 = det*c0_0_0_10*Jinv_01;
    const double G0_0_11_1 = det*c0_0_0_11*Jinv_01;
    const double G0_0_12_1 = det*c0_0_0_12*Jinv_01;
    const double G0_0_13_1 = det*c0_0_0_13*Jinv_01;
    const double G0_0_14_1 = det*c0_0_0_14*Jinv_01;
    const double G0_0_15_1 = det*c0_0_0_15*Jinv_01;
    const double G0_0_16_1 = det*c0_0_0_16*Jinv_01;
    const double G0_0_17_1 = det*c0_0_0_17*Jinv_01;
    const double G0_0_18_1 = det*c0_0_0_18*Jinv_01;
    const double G0_0_19_1 = det*c0_0_0_19*Jinv_01;
    const double G0_0_20_2 = det*c0_0_0_20*Jinv_02;
    const double G0_0_21_2 = det*c0_0_0_21*Jinv_02;
    const double G0_0_22_2 = det*c0_0_0_22*Jinv_02;
    const double G0_0_23_2 = det*c0_0_0_23*Jinv_02;
    const double G0_0_24_2 = det*c0_0_0_24*Jinv_02;
    const double G0_0_25_2 = det*c0_0_0_25*Jinv_02;
    const double G0_0_26_2 = det*c0_0_0_26*Jinv_02;
    const double G0_0_27_2 = det*c0_0_0_27*Jinv_02;
    const double G0_0_28_2 = det*c0_0_0_28*Jinv_02;
    const double G0_0_29_2 = det*c0_0_0_29*Jinv_02;
    const double G0_1_0_0 = det*c0_0_0_0*Jinv_10;
    const double G0_1_1_0 = det*c0_0_0_1*Jinv_10;
    const double G0_1_2_0 = det*c0_0_0_2*Jinv_10;
    const double G0_1_3_0 = det*c0_0_0_3*Jinv_10;
    const double G0_1_4_0 = det*c0_0_0_4*Jinv_10;
    const double G0_1_5_0 = det*c0_0_0_5*Jinv_10;
    const double G0_1_6_0 = det*c0_0_0_6*Jinv_10;
    const double G0_1_7_0 = det*c0_0_0_7*Jinv_10;
    const double G0_1_8_0 = det*c0_0_0_8*Jinv_10;
    const double G0_1_9_0 = det*c0_0_0_9*Jinv_10;
    const double G0_1_10_1 = det*c0_0_0_10*Jinv_11;
    const double G0_1_11_1 = det*c0_0_0_11*Jinv_11;
    const double G0_1_12_1 = det*c0_0_0_12*Jinv_11;
    const double G0_1_13_1 = det*c0_0_0_13*Jinv_11;
    const double G0_1_14_1 = det*c0_0_0_14*Jinv_11;
    const double G0_1_15_1 = det*c0_0_0_15*Jinv_11;
    const double G0_1_16_1 = det*c0_0_0_16*Jinv_11;
    const double G0_1_17_1 = det*c0_0_0_17*Jinv_11;
    const double G0_1_18_1 = det*c0_0_0_18*Jinv_11;
    const double G0_1_19_1 = det*c0_0_0_19*Jinv_11;
    const double G0_1_20_2 = det*c0_0_0_20*Jinv_12;
    const double G0_1_21_2 = det*c0_0_0_21*Jinv_12;
    const double G0_1_22_2 = det*c0_0_0_22*Jinv_12;
    const double G0_1_23_2 = det*c0_0_0_23*Jinv_12;
    const double G0_1_24_2 = det*c0_0_0_24*Jinv_12;
    const double G0_1_25_2 = det*c0_0_0_25*Jinv_12;
    const double G0_1_26_2 = det*c0_0_0_26*Jinv_12;
    const double G0_1_27_2 = det*c0_0_0_27*Jinv_12;
    const double G0_1_28_2 = det*c0_0_0_28*Jinv_12;
    const double G0_1_29_2 = det*c0_0_0_29*Jinv_12;
    const double G0_2_0_0 = det*c0_0_0_0*Jinv_20;
    const double G0_2_1_0 = det*c0_0_0_1*Jinv_20;
    const double G0_2_2_0 = det*c0_0_0_2*Jinv_20;
    const double G0_2_3_0 = det*c0_0_0_3*Jinv_20;
    const double G0_2_4_0 = det*c0_0_0_4*Jinv_20;
    const double G0_2_5_0 = det*c0_0_0_5*Jinv_20;
    const double G0_2_6_0 = det*c0_0_0_6*Jinv_20;
    const double G0_2_7_0 = det*c0_0_0_7*Jinv_20;
    const double G0_2_8_0 = det*c0_0_0_8*Jinv_20;
    const double G0_2_9_0 = det*c0_0_0_9*Jinv_20;
    const double G0_2_10_1 = det*c0_0_0_10*Jinv_21;
    const double G0_2_11_1 = det*c0_0_0_11*Jinv_21;
    const double G0_2_12_1 = det*c0_0_0_12*Jinv_21;
    const double G0_2_13_1 = det*c0_0_0_13*Jinv_21;
    const double G0_2_14_1 = det*c0_0_0_14*Jinv_21;
    const double G0_2_15_1 = det*c0_0_0_15*Jinv_21;
    const double G0_2_16_1 = det*c0_0_0_16*Jinv_21;
    const double G0_2_17_1 = det*c0_0_0_17*Jinv_21;
    const double G0_2_18_1 = det*c0_0_0_18*Jinv_21;
    const double G0_2_19_1 = det*c0_0_0_19*Jinv_21;
    const double G0_2_20_2 = det*c0_0_0_20*Jinv_22;
    const double G0_2_21_2 = det*c0_0_0_21*Jinv_22;
    const double G0_2_22_2 = det*c0_0_0_22*Jinv_22;
    const double G0_2_23_2 = det*c0_0_0_23*Jinv_22;
    const double G0_2_24_2 = det*c0_0_0_24*Jinv_22;
    const double G0_2_25_2 = det*c0_0_0_25*Jinv_22;
    const double G0_2_26_2 = det*c0_0_0_26*Jinv_22;
    const double G0_2_27_2 = det*c0_0_0_27*Jinv_22;
    const double G0_2_28_2 = det*c0_0_0_28*Jinv_22;
    const double G0_2_29_2 = det*c0_0_0_29*Jinv_22;
    
    // Compute element tensor
    A[0] = 0;
    A[1] = 0;
    A[2] = 0;
    A[3] = 0;
    A[4] = 0.002777777778*G0_1_0_0 + 0.002777777778*G0_1_1_0 + 0.002777777778*G0_1_2_0 - 0.01111111111*G0_1_4_0 - 0.01111111111*G0_1_5_0 - 0.005555555556*G0_1_6_0 - 0.01111111111*G0_1_7_0 - 0.005555555556*G0_1_8_0 - 0.005555555556*G0_1_9_0 + 0.002777777778*G0_1_10_1 + 0.002777777778*G0_1_11_1 + 0.002777777778*G0_1_12_1 - 0.01111111111*G0_1_14_1 - 0.01111111111*G0_1_15_1 - 0.005555555556*G0_1_16_1 - 0.01111111111*G0_1_17_1 - 0.005555555556*G0_1_18_1 - 0.005555555556*G0_1_19_1 + 0.002777777778*G0_1_20_2 + 0.002777777778*G0_1_21_2 + 0.002777777778*G0_1_22_2 - 0.01111111111*G0_1_24_2 - 0.01111111111*G0_1_25_2 - 0.005555555556*G0_1_26_2 - 0.01111111111*G0_1_27_2 - 0.005555555556*G0_1_28_2 - 0.005555555556*G0_1_29_2 - 0.002777777778*G0_2_0_0 - 0.002777777778*G0_2_1_0 - 0.002777777778*G0_2_3_0 + 0.01111111111*G0_2_4_0 + 0.005555555556*G0_2_5_0 + 0.01111111111*G0_2_6_0 + 0.005555555556*G0_2_7_0 + 0.01111111111*G0_2_8_0 + 0.005555555556*G0_2_9_0 - 0.002777777778*G0_2_10_1 - 0.002777777778*G0_2_11_1 - 0.002777777778*G0_2_13_1 + 0.01111111111*G0_2_14_1 + 0.005555555556*G0_2_15_1 + 0.01111111111*G0_2_16_1 + 0.005555555556*G0_2_17_1 + 0.01111111111*G0_2_18_1 + 0.005555555556*G0_2_19_1 - 0.002777777778*G0_2_20_2 - 0.002777777778*G0_2_21_2 - 0.002777777778*G0_2_23_2 + 0.01111111111*G0_2_24_2 + 0.005555555556*G0_2_25_2 + 0.01111111111*G0_2_26_2 + 0.005555555556*G0_2_27_2 + 0.01111111111*G0_2_28_2 + 0.005555555556*G0_2_29_2;
    A[5] = 0.002777777778*G0_0_0_0 + 0.002777777778*G0_0_1_0 + 0.002777777778*G0_0_2_0 - 0.01111111111*G0_0_4_0 - 0.01111111111*G0_0_5_0 - 0.005555555556*G0_0_6_0 - 0.01111111111*G0_0_7_0 - 0.005555555556*G0_0_8_0 - 0.005555555556*G0_0_9_0 + 0.002777777778*G0_0_10_1 + 0.002777777778*G0_0_11_1 + 0.002777777778*G0_0_12_1 - 0.01111111111*G0_0_14_1 - 0.01111111111*G0_0_15_1 - 0.005555555556*G0_0_16_1 - 0.01111111111*G0_0_17_1 - 0.005555555556*G0_0_18_1 - 0.005555555556*G0_0_19_1 + 0.002777777778*G0_0_20_2 + 0.002777777778*G0_0_21_2 + 0.002777777778*G0_0_22_2 - 0.01111111111*G0_0_24_2 - 0.01111111111*G0_0_25_2 - 0.005555555556*G0_0_26_2 - 0.01111111111*G0_0_27_2 - 0.005555555556*G0_0_28_2 - 0.005555555556*G0_0_29_2 - 0.002777777778*G0_2_0_0 - 0.002777777778*G0_2_2_0 - 0.002777777778*G0_2_3_0 + 0.005555555556*G0_2_4_0 + 0.01111111111*G0_2_5_0 + 0.01111111111*G0_2_6_0 + 0.005555555556*G0_2_7_0 + 0.005555555556*G0_2_8_0 + 0.01111111111*G0_2_9_0 - 0.002777777778*G0_2_10_1 - 0.002777777778*G0_2_12_1 - 0.002777777778*G0_2_13_1 + 0.005555555556*G0_2_14_1 + 0.01111111111*G0_2_15_1 + 0.01111111111*G0_2_16_1 + 0.005555555556*G0_2_17_1 + 0.005555555556*G0_2_18_1 + 0.01111111111*G0_2_19_1 - 0.002777777778*G0_2_20_2 - 0.002777777778*G0_2_22_2 - 0.002777777778*G0_2_23_2 + 0.005555555556*G0_2_24_2 + 0.01111111111*G0_2_25_2 + 0.01111111111*G0_2_26_2 + 0.005555555556*G0_2_27_2 + 0.005555555556*G0_2_28_2 + 0.01111111111*G0_2_29_2;
    A[6] = 0.002777777778*G0_0_0_0 + 0.002777777778*G0_0_1_0 + 0.002777777778*G0_0_3_0 - 0.01111111111*G0_0_4_0 - 0.005555555556*G0_0_5_0 - 0.01111111111*G0_0_6_0 - 0.005555555556*G0_0_7_0 - 0.01111111111*G0_0_8_0 - 0.005555555556*G0_0_9_0 + 0.002777777778*G0_0_10_1 + 0.002777777778*G0_0_11_1 + 0.002777777778*G0_0_13_1 - 0.01111111111*G0_0_14_1 - 0.005555555556*G0_0_15_1 - 0.01111111111*G0_0_16_1 - 0.005555555556*G0_0_17_1 - 0.01111111111*G0_0_18_1 - 0.005555555556*G0_0_19_1 + 0.002777777778*G0_0_20_2 + 0.002777777778*G0_0_21_2 + 0.002777777778*G0_0_23_2 - 0.01111111111*G0_0_24_2 - 0.005555555556*G0_0_25_2 - 0.01111111111*G0_0_26_2 - 0.005555555556*G0_0_27_2 - 0.01111111111*G0_0_28_2 - 0.005555555556*G0_0_29_2 - 0.002777777778*G0_1_0_0 - 0.002777777778*G0_1_2_0 - 0.002777777778*G0_1_3_0 + 0.005555555556*G0_1_4_0 + 0.01111111111*G0_1_5_0 + 0.01111111111*G0_1_6_0 + 0.005555555556*G0_1_7_0 + 0.005555555556*G0_1_8_0 + 0.01111111111*G0_1_9_0 - 0.002777777778*G0_1_10_1 - 0.002777777778*G0_1_12_1 - 0.002777777778*G0_1_13_1 + 0.005555555556*G0_1_14_1 + 0.01111111111*G0_1_15_1 + 0.01111111111*G0_1_16_1 + 0.005555555556*G0_1_17_1 + 0.005555555556*G0_1_18_1 + 0.01111111111*G0_1_19_1 - 0.002777777778*G0_1_20_2 - 0.002777777778*G0_1_22_2 - 0.002777777778*G0_1_23_2 + 0.005555555556*G0_1_24_2 + 0.01111111111*G0_1_25_2 + 0.01111111111*G0_1_26_2 + 0.005555555556*G0_1_27_2 + 0.005555555556*G0_1_28_2 + 0.01111111111*G0_1_29_2;
    A[7] = -0.002777777778*G0_0_0_0 - 0.002777777778*G0_0_1_0 - 0.002777777778*G0_0_2_0 + 0.01111111111*G0_0_4_0 + 0.01111111111*G0_0_5_0 + 0.005555555556*G0_0_6_0 + 0.01111111111*G0_0_7_0 + 0.005555555556*G0_0_8_0 + 0.005555555556*G0_0_9_0 - 0.002777777778*G0_0_10_1 - 0.002777777778*G0_0_11_1 - 0.002777777778*G0_0_12_1 + 0.01111111111*G0_0_14_1 + 0.01111111111*G0_0_15_1 + 0.005555555556*G0_0_16_1 + 0.01111111111*G0_0_17_1 + 0.005555555556*G0_0_18_1 + 0.005555555556*G0_0_19_1 - 0.002777777778*G0_0_20_2 - 0.002777777778*G0_0_21_2 - 0.002777777778*G0_0_22_2 + 0.01111111111*G0_0_24_2 + 0.01111111111*G0_0_25_2 + 0.005555555556*G0_0_26_2 + 0.01111111111*G0_0_27_2 + 0.005555555556*G0_0_28_2 + 0.005555555556*G0_0_29_2 - 0.002777777778*G0_1_0_0 - 0.002777777778*G0_1_1_0 - 0.002777777778*G0_1_2_0 + 0.01111111111*G0_1_4_0 + 0.01111111111*G0_1_5_0 + 0.005555555556*G0_1_6_0 + 0.01111111111*G0_1_7_0 + 0.005555555556*G0_1_8_0 + 0.005555555556*G0_1_9_0 - 0.002777777778*G0_1_10_1 - 0.002777777778*G0_1_11_1 - 0.002777777778*G0_1_12_1 + 0.01111111111*G0_1_14_1 + 0.01111111111*G0_1_15_1 + 0.005555555556*G0_1_16_1 + 0.01111111111*G0_1_17_1 + 0.005555555556*G0_1_18_1 + 0.005555555556*G0_1_19_1 - 0.002777777778*G0_1_20_2 - 0.002777777778*G0_1_21_2 - 0.002777777778*G0_1_22_2 + 0.01111111111*G0_1_24_2 + 0.01111111111*G0_1_25_2 + 0.005555555556*G0_1_26_2 + 0.01111111111*G0_1_27_2 + 0.005555555556*G0_1_28_2 + 0.005555555556*G0_1_29_2 - 0.002777777778*G0_2_0_0 - 0.005555555556*G0_2_1_0 - 0.005555555556*G0_2_2_0 - 0.002777777778*G0_2_3_0 + 0.01666666667*G0_2_4_0 + 0.01666666667*G0_2_5_0 + 0.01111111111*G0_2_6_0 + 0.02222222222*G0_2_7_0 + 0.01666666667*G0_2_8_0 + 0.01666666667*G0_2_9_0 - 0.002777777778*G0_2_10_1 - 0.005555555556*G0_2_11_1 - 0.005555555556*G0_2_12_1 - 0.002777777778*G0_2_13_1 + 0.01666666667*G0_2_14_1 + 0.01666666667*G0_2_15_1 + 0.01111111111*G0_2_16_1 + 0.02222222222*G0_2_17_1 + 0.01666666667*G0_2_18_1 + 0.01666666667*G0_2_19_1 - 0.002777777778*G0_2_20_2 - 0.005555555556*G0_2_21_2 - 0.005555555556*G0_2_22_2 - 0.002777777778*G0_2_23_2 + 0.01666666667*G0_2_24_2 + 0.01666666667*G0_2_25_2 + 0.01111111111*G0_2_26_2 + 0.02222222222*G0_2_27_2 + 0.01666666667*G0_2_28_2 + 0.01666666667*G0_2_29_2;
    A[8] = -0.002777777778*G0_0_0_0 - 0.002777777778*G0_0_1_0 - 0.002777777778*G0_0_3_0 + 0.01111111111*G0_0_4_0 + 0.005555555556*G0_0_5_0 + 0.01111111111*G0_0_6_0 + 0.005555555556*G0_0_7_0 + 0.01111111111*G0_0_8_0 + 0.005555555556*G0_0_9_0 - 0.002777777778*G0_0_10_1 - 0.002777777778*G0_0_11_1 - 0.002777777778*G0_0_13_1 + 0.01111111111*G0_0_14_1 + 0.005555555556*G0_0_15_1 + 0.01111111111*G0_0_16_1 + 0.005555555556*G0_0_17_1 + 0.01111111111*G0_0_18_1 + 0.005555555556*G0_0_19_1 - 0.002777777778*G0_0_20_2 - 0.002777777778*G0_0_21_2 - 0.002777777778*G0_0_23_2 + 0.01111111111*G0_0_24_2 + 0.005555555556*G0_0_25_2 + 0.01111111111*G0_0_26_2 + 0.005555555556*G0_0_27_2 + 0.01111111111*G0_0_28_2 + 0.005555555556*G0_0_29_2 - 0.002777777778*G0_1_0_0 - 0.005555555556*G0_1_1_0 - 0.002777777778*G0_1_2_0 - 0.005555555556*G0_1_3_0 + 0.01666666667*G0_1_4_0 + 0.01111111111*G0_1_5_0 + 0.01666666667*G0_1_6_0 + 0.01666666667*G0_1_7_0 + 0.02222222222*G0_1_8_0 + 0.01666666667*G0_1_9_0 - 0.002777777778*G0_1_10_1 - 0.005555555556*G0_1_11_1 - 0.002777777778*G0_1_12_1 - 0.005555555556*G0_1_13_1 + 0.01666666667*G0_1_14_1 + 0.01111111111*G0_1_15_1 + 0.01666666667*G0_1_16_1 + 0.01666666667*G0_1_17_1 + 0.02222222222*G0_1_18_1 + 0.01666666667*G0_1_19_1 - 0.002777777778*G0_1_20_2 - 0.005555555556*G0_1_21_2 - 0.002777777778*G0_1_22_2 - 0.005555555556*G0_1_23_2 + 0.01666666667*G0_1_24_2 + 0.01111111111*G0_1_25_2 + 0.01666666667*G0_1_26_2 + 0.01666666667*G0_1_27_2 + 0.02222222222*G0_1_28_2 + 0.01666666667*G0_1_29_2 - 0.002777777778*G0_2_0_0 - 0.002777777778*G0_2_1_0 - 0.002777777778*G0_2_3_0 + 0.01111111111*G0_2_4_0 + 0.005555555556*G0_2_5_0 + 0.01111111111*G0_2_6_0 + 0.005555555556*G0_2_7_0 + 0.01111111111*G0_2_8_0 + 0.005555555556*G0_2_9_0 - 0.002777777778*G0_2_10_1 - 0.002777777778*G0_2_11_1 - 0.002777777778*G0_2_13_1 + 0.01111111111*G0_2_14_1 + 0.005555555556*G0_2_15_1 + 0.01111111111*G0_2_16_1 + 0.005555555556*G0_2_17_1 + 0.01111111111*G0_2_18_1 + 0.005555555556*G0_2_19_1 - 0.002777777778*G0_2_20_2 - 0.002777777778*G0_2_21_2 - 0.002777777778*G0_2_23_2 + 0.01111111111*G0_2_24_2 + 0.005555555556*G0_2_25_2 + 0.01111111111*G0_2_26_2 + 0.005555555556*G0_2_27_2 + 0.01111111111*G0_2_28_2 + 0.005555555556*G0_2_29_2;
    A[9] = -0.002777777778*G0_0_0_0 - 0.002777777778*G0_0_1_0 - 0.005555555556*G0_0_2_0 - 0.005555555556*G0_0_3_0 + 0.01111111111*G0_0_4_0 + 0.01666666667*G0_0_5_0 + 0.01666666667*G0_0_6_0 + 0.01666666667*G0_0_7_0 + 0.01666666667*G0_0_8_0 + 0.02222222222*G0_0_9_0 - 0.002777777778*G0_0_10_1 - 0.002777777778*G0_0_11_1 - 0.005555555556*G0_0_12_1 - 0.005555555556*G0_0_13_1 + 0.01111111111*G0_0_14_1 + 0.01666666667*G0_0_15_1 + 0.01666666667*G0_0_16_1 + 0.01666666667*G0_0_17_1 + 0.01666666667*G0_0_18_1 + 0.02222222222*G0_0_19_1 - 0.002777777778*G0_0_20_2 - 0.002777777778*G0_0_21_2 - 0.005555555556*G0_0_22_2 - 0.005555555556*G0_0_23_2 + 0.01111111111*G0_0_24_2 + 0.01666666667*G0_0_25_2 + 0.01666666667*G0_0_26_2 + 0.01666666667*G0_0_27_2 + 0.01666666667*G0_0_28_2 + 0.02222222222*G0_0_29_2 - 0.002777777778*G0_1_0_0 - 0.002777777778*G0_1_2_0 - 0.002777777778*G0_1_3_0 + 0.005555555556*G0_1_4_0 + 0.01111111111*G0_1_5_0 + 0.01111111111*G0_1_6_0 + 0.005555555556*G0_1_7_0 + 0.005555555556*G0_1_8_0 + 0.01111111111*G0_1_9_0 - 0.002777777778*G0_1_10_1 - 0.002777777778*G0_1_12_1 - 0.002777777778*G0_1_13_1 + 0.005555555556*G0_1_14_1 + 0.01111111111*G0_1_15_1 + 0.01111111111*G0_1_16_1 + 0.005555555556*G0_1_17_1 + 0.005555555556*G0_1_18_1 + 0.01111111111*G0_1_19_1 - 0.002777777778*G0_1_20_2 - 0.002777777778*G0_1_22_2 - 0.002777777778*G0_1_23_2 + 0.005555555556*G0_1_24_2 + 0.01111111111*G0_1_25_2 + 0.01111111111*G0_1_26_2 + 0.005555555556*G0_1_27_2 + 0.005555555556*G0_1_28_2 + 0.01111111111*G0_1_29_2 - 0.002777777778*G0_2_0_0 - 0.002777777778*G0_2_2_0 - 0.002777777778*G0_2_3_0 + 0.005555555556*G0_2_4_0 + 0.01111111111*G0_2_5_0 + 0.01111111111*G0_2_6_0 + 0.005555555556*G0_2_7_0 + 0.005555555556*G0_2_8_0 + 0.01111111111*G0_2_9_0 - 0.002777777778*G0_2_10_1 - 0.002777777778*G0_2_12_1 - 0.002777777778*G0_2_13_1 + 0.005555555556*G0_2_14_1 + 0.01111111111*G0_2_15_1 + 0.01111111111*G0_2_16_1 + 0.005555555556*G0_2_17_1 + 0.005555555556*G0_2_18_1 + 0.01111111111*G0_2_19_1 - 0.002777777778*G0_2_20_2 - 0.002777777778*G0_2_22_2 - 0.002777777778*G0_2_23_2 + 0.005555555556*G0_2_24_2 + 0.01111111111*G0_2_25_2 + 0.01111111111*G0_2_26_2 + 0.005555555556*G0_2_27_2 + 0.005555555556*G0_2_28_2 + 0.01111111111*G0_2_29_2;
  }

};

/// This class defines the interface for the assembly of the global
/// tensor corresponding to a form with r + n arguments, that is, a
/// mapping
///
///     a : V1 x V2 x ... Vr x W1 x W2 x ... x Wn -> R
///
/// with arguments v1, v2, ..., vr, w1, w2, ..., wn. The rank r
/// global tensor A is defined by
///
///     A = a(V1, V2, ..., Vr, w1, w2, ..., wn),
///
/// where each argument Vj represents the application to the
/// sequence of basis functions of Vj and w1, w2, ..., wn are given
/// fixed functions (coefficients).

class VectorLaplaceGradCurlLinearForm: public ufc::form
{
public:

  /// Constructor
  VectorLaplaceGradCurlLinearForm() : ufc::form()
  {
    // Do nothing
  }

  /// Destructor
  virtual ~VectorLaplaceGradCurlLinearForm()
  {
    // Do nothing
  }

  /// Return a string identifying the form
  virtual const char* signature() const
  {
    return "w0_a1(dXa0/dxa2) | vi0[a0 + 1]*va1[a2]*dX(0)";
  }

  /// Return the rank of the global tensor (r)
  virtual unsigned int rank() const
  {
    return 1;
  }

  /// Return the number of coefficients (n)
  virtual unsigned int num_coefficients() const
  {
    return 1;
  }

  /// Return the number of cell integrals
  virtual unsigned int num_cell_integrals() const
  {
    return 1;
  }
  
  /// Return the number of exterior facet integrals
  virtual unsigned int num_exterior_facet_integrals() const
  {
    return 0;
  }
  
  /// Return the number of interior facet integrals
  virtual unsigned int num_interior_facet_integrals() const
  {
    return 0;
  }
    
  /// Create a new finite element for argument function i
  virtual ufc::finite_element* create_finite_element(unsigned int i) const
  {
    switch ( i )
    {
    case 0:
      return new VectorLaplaceGradCurlLinearForm_finite_element_0();
      break;
    case 1:
      return new VectorLaplaceGradCurlLinearForm_finite_element_1();
      break;
    }
    return 0;
  }
  
  /// Create a new dof map for argument function i
  virtual ufc::dof_map* create_dof_map(unsigned int i) const
  {
    switch ( i )
    {
    case 0:
      return new VectorLaplaceGradCurlLinearForm_dof_map_0();
      break;
    case 1:
      return new VectorLaplaceGradCurlLinearForm_dof_map_1();
      break;
    }
    return 0;
  }

  /// Create a new cell integral on sub domain i
  virtual ufc::cell_integral* create_cell_integral(unsigned int i) const
  {
    return new VectorLaplaceGradCurlLinearForm_cell_integral_0();
  }

  /// Create a new exterior facet integral on sub domain i
  virtual ufc::exterior_facet_integral* create_exterior_facet_integral(unsigned int i) const
  {
    return 0;
  }

  /// Create a new interior facet integral on sub domain i
  virtual ufc::interior_facet_integral* create_interior_facet_integral(unsigned int i) const
  {
    return 0;
  }

};

#endif
